/* Copyright (C) 2003-2012 BOSSY Emmanuel.

This file is part of the SimSonic suite.

SimSonic is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

SimSonic is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with SimSonic.  If not, see <http://www.gnu.org/licenses/>. */

#define VERSION "3D.2012.11.20"
/* Version base sur la 3D.2012.11.19

*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <omp.h>

#include "SimSonic3D.h"

#include "PMLFaceVitesses.c"
#include "PMLFaceContraintes.c"
#include "PMLAreteVitesses.c"
#include "PMLAreteContraintes.c"
#include "PMLCoinVitesses.c"
#include "PMLCoinContraintes.c"



#define max(a, b)  (((a) > (b)) ? (a) : (b))

int main(int argc, char * argv[])
{
    /* Various flags and initialization. */
    int FoundMapFileName=0;

    int i; /* indice de boucles*/


    /* Strings for handling file names. */
    char MapFileName[500]="";
    char ParameterFileName[500]="";
    char InitialConditionsFileName[500]="";
    char PhotoacousticMapFileName[500]="";
    char VersionFileName[500]="";
    char SkipFileName[500]="";
    char DoneFileName[500]="";

    //char NomCarte[500];
    //char InitialConditionsFileName[500];

    //char ParameterFileName[500];
    //char CurrentLine[500];







    /* Strings used to read lines in text files. */
    char CurrentLine[500]="";

    /* pointers for handling files*/

    FILE* ParameterFile=NULL;
    FILE* MapFile=NULL;
    FILE* InitialConditionsFile=NULL;
    FILE* VersionFile=NULL;
    FILE* SkipFile=NULL;
    FILE* DoneFile=NULL;





    /********************************************************************/

    /*tests if program is called with at least one argument. If so, retrieves the working directory. If not, outputs a file with version info*/

    if(argc == 1)
    {

        /* write a flag file with version info in file name and file content */
        strcpy(VersionFileName,Repertoire) ;
        strcat(VersionFileName,VERSION) ;
        strcat(VersionFileName,""".ver") ;
        if((VersionFile = fopen(VersionFileName,"w+t")) != NULL)
        {
            fprintf(VersionFile,"%s",VERSION);
            fclose(VersionFile) ;
        }
        LiberationMemoire() ;
        exit(1) ;
    }
    else
        strcpy(Repertoire,argv[1]) ;


    /* looks for skip file */
    strcpy(SkipFileName,Repertoire) ;
    strcat(SkipFileName,"""skip") ;
    if((SkipFile = fopen(SkipFileName,"rb")) != NULL)
    {
        printf("skip file found...\n") ;
        exit(1) ;
    }


    /* write a flag file with version info in file name and file content */
    strcpy(VersionFileName,Repertoire) ;
    strcat(VersionFileName,VERSION) ;
    strcat(VersionFileName,""".ver") ;
    if((VersionFile = fopen(VersionFileName,"w+t")) != NULL)
    {
        fprintf(VersionFile,"%s",VERSION);
        fclose(VersionFile) ;
    }


    TableLegerete = (double*) calloc(256,sizeof(double));
    TableC11=(double*) calloc(256,sizeof(double));
    TableC22=(double*) calloc(256,sizeof(double));
    TableC33=(double*) calloc(256,sizeof(double));
    TableC12=(double*) calloc(256,sizeof(double));
    TableC23=(double*) calloc(256,sizeof(double));
    TableC31=(double*) calloc(256,sizeof(double));
    TableC44=(double*) calloc(256,sizeof(double));
    TableC55=(double*) calloc(256,sizeof(double));
    TableC66=(double*) calloc(256,sizeof(double));
    TableQModel = (double*) calloc(256,sizeof(double)) ;


    /* Opens Parametres.ini2D to look for map file name. */
    strcpy(ParameterFileName,Repertoire);
    strcat(ParameterFileName,"""Parametres.ini3D");

    if((ParameterFile=fopen(ParameterFileName,"rb"))==NULL)
    {
        strcpy(ParameterFileName,Repertoire);
        strcat(ParameterFileName,"""Parameters.ini3D");

        if((ParameterFile=fopen(ParameterFileName,"rb"))==NULL)
        {

            printf("Parameters.ini3D NOT FOUND ...\n");
            exit(1);
        }
    }
    /* Looks for a map filename (optional for the user). */
    fseek(ParameterFile,0L,SEEK_SET);
    while(fgets(CurrentLine,500,ParameterFile)!=NULL)
        if(strstr(CurrentLine,"Carte map3D file name")!=NULL)
        {
            /* Loads the map's filename */
            sscanf(CurrentLine+31,"%s",MapFileName);

            if(!strcmp(MapFileName,""))
                FoundMapFileName=0;
            else
                FoundMapFileName=1;
        }

    /* Close Parametres.ini2D after initial search for map's filename. */
    fclose(ParameterFile);

    /* Use default filename "carte.map2D" if no map's filename was found. */
    if(!FoundMapFileName)
    {
        strcpy(MapFileName,Repertoire);
        strcat(MapFileName,"""carte.map3D");
    }

    /* Opens map file. */
    if((MapFile=fopen(MapFileName,"rb"))==NULL)
    {

        strcpy(MapFileName,Repertoire);
        strcat(MapFileName,"""Geometry.map3D");

        if((MapFile=fopen(MapFileName,"rb"))==NULL)
        {

            printf("Problem opening map file: File not found...\n");
            LiberationMemoire();
            exit(1);
        }
    }
    /*Lecture des dimensions de la simulation*/
    fread(&X,sizeof(int),1,MapFile);
    fread(&Y,sizeof(int),1,MapFile);
    fread(&Z,sizeof(int),1,MapFile);

    /* Lecture des parametres de la simumlation*/
    LectureParametres();

    /*	 Allocation des grilles, les tailles tant  prsent connues*/
    AllocationGrilles();
    AdressageBuffer();

    /*	 Lecture et fermeture fichier de la carte morphologique de paramtres*/
    fread(Carte,sizeof(PREC_CARTE),X*(Y)*(Z),MapFile);
    fclose( MapFile );


    /*Looks for initial conditions, and reads if they exist*/
    strcpy(InitialConditionsFileName,Repertoire);
    strcat(InitialConditionsFileName,"""V1_000.snp3D");
    if(((InitialConditionsFile = fopen(InitialConditionsFileName, "rb" )) != NULL))
    {
        LectureConditionsInitiales(InitialConditionsFileName,Vx,X+1,Y,Z);
    }
    strcpy(InitialConditionsFileName,Repertoire);
    strcat(InitialConditionsFileName,"""V2_000.snp3D");
    if(((InitialConditionsFile = fopen(InitialConditionsFileName, "rb" )) != NULL))
    {
        LectureConditionsInitiales(InitialConditionsFileName,Vy,X,Y+1,Z);
    }
    strcpy(InitialConditionsFileName,Repertoire);
    strcat(InitialConditionsFileName,"""V3_000.snp3D");
    if(((InitialConditionsFile = fopen(InitialConditionsFileName, "rb" )) != NULL))
    {
        LectureConditionsInitiales(InitialConditionsFileName,Vz,X,Y,Z+1);
    }
    strcpy(InitialConditionsFileName,Repertoire);
    strcat(InitialConditionsFileName,"""T11_000.snp3D");
    if(((InitialConditionsFile = fopen(InitialConditionsFileName, "rb" )) != NULL))
    {
        LectureConditionsInitiales(InitialConditionsFileName,Txx,X,Y,Z);
    }
    strcpy(InitialConditionsFileName,Repertoire);
    strcat(InitialConditionsFileName,"""T22_000.snp3D");
    if(((InitialConditionsFile = fopen(InitialConditionsFileName, "rb" )) != NULL))
    {
        LectureConditionsInitiales(InitialConditionsFileName,Tyy,X,Y,Z);
    }
    strcpy(InitialConditionsFileName,Repertoire);
    strcat(InitialConditionsFileName,"""T33_000.snp3D");
    if(((InitialConditionsFile = fopen(InitialConditionsFileName, "rb" )) != NULL))
    {
        LectureConditionsInitiales(InitialConditionsFileName,Tzz,X,Y,Z);
    }
    strcpy(InitialConditionsFileName,Repertoire);
    strcat(InitialConditionsFileName,"""T12_000.snp3D");
    if(((InitialConditionsFile = fopen(InitialConditionsFileName, "rb" )) != NULL))
    {
        LectureConditionsInitiales(InitialConditionsFileName,Txy,X+1,Y+1,Z);
    }
    strcpy(InitialConditionsFileName,Repertoire);
    strcat(InitialConditionsFileName,"""T23_000.snp3D");
    if(((InitialConditionsFile = fopen(InitialConditionsFileName, "rb" )) != NULL))
    {
        LectureConditionsInitiales(InitialConditionsFileName,Tyz,X,Y+1,Z+1);
    }
    strcpy(InitialConditionsFileName,Repertoire);
    strcat(InitialConditionsFileName,"""T31_000.snp3D");
    if(((InitialConditionsFile = fopen(InitialConditionsFileName, "rb" )) != NULL))
    {
        LectureConditionsInitiales(InitialConditionsFileName,Tzx,X+1,Y,Z+1);
    }
    strcpy(InitialConditionsFileName,Repertoire);
    strcat(InitialConditionsFileName,"""Vx0.snp3D");
    if(((InitialConditionsFile = fopen(InitialConditionsFileName, "rb" )) != NULL))
    {
        LectureConditionsInitiales(InitialConditionsFileName,Vx,X+1,Y,Z);
    }
    strcpy(InitialConditionsFileName,Repertoire);
    strcat(InitialConditionsFileName,"""Vy0.snp3D");
    if(((InitialConditionsFile = fopen(InitialConditionsFileName, "rb" )) != NULL))
    {
        LectureConditionsInitiales(InitialConditionsFileName,Vy,X,Y+1,Z);
    }
    strcpy(InitialConditionsFileName,Repertoire);
    strcat(InitialConditionsFileName,"""Vz0.snp3D");
    if(((InitialConditionsFile = fopen(InitialConditionsFileName, "rb" )) != NULL))
    {
        LectureConditionsInitiales(InitialConditionsFileName,Vz,X,Y,Z+1);
    }
    strcpy(InitialConditionsFileName,Repertoire);
    strcat(InitialConditionsFileName,"""Txx0.snp3D");
    if(((InitialConditionsFile = fopen(InitialConditionsFileName, "rb" )) != NULL))
    {
        LectureConditionsInitiales(InitialConditionsFileName,Txx,X,Y,Z);
    }
    strcpy(InitialConditionsFileName,Repertoire);
    strcat(InitialConditionsFileName,"""Tyy0.snp3D");
    if(((InitialConditionsFile = fopen(InitialConditionsFileName, "rb" )) != NULL))
    {
        LectureConditionsInitiales(InitialConditionsFileName,Tyy,X,Y,Z);
    }
    strcpy(InitialConditionsFileName,Repertoire);
    strcat(InitialConditionsFileName,"""Tzz0.snp3D");
    if(((InitialConditionsFile = fopen(InitialConditionsFileName, "rb" )) != NULL))
    {
        LectureConditionsInitiales(InitialConditionsFileName,Tzz,X,Y,Z);
    }
    strcpy(InitialConditionsFileName,Repertoire);
    strcat(InitialConditionsFileName,"""Txy0.snp3D");
    if(((InitialConditionsFile = fopen(InitialConditionsFileName, "rb" )) != NULL))
    {
        LectureConditionsInitiales(InitialConditionsFileName,Txy,X+1,Y+1,Z);
    }
    strcpy(InitialConditionsFileName,Repertoire);
    strcat(InitialConditionsFileName,"""Tyz0.snp3D");
    if(((InitialConditionsFile = fopen(InitialConditionsFileName, "rb" )) != NULL))
    {
        LectureConditionsInitiales(InitialConditionsFileName,Tyz,X,Y+1,Z+1);
    }
    strcpy(InitialConditionsFileName,Repertoire);
    strcat(InitialConditionsFileName,"""Tzx0.snp3D");
    if(((InitialConditionsFile = fopen(InitialConditionsFileName, "rb" )) != NULL))
    {
        LectureConditionsInitiales(InitialConditionsFileName,Tzx,X+1,Y,Z+1);
    }
    /***********************************************************************/


    /*	Dfinition du vecteur attenuation */
#pragma omp parallel for schedule(guided)
    for (i=0; i<(2*W+1); i++)
    {
        AttenuationPML[i]=8.63E-2*((double)CoeffPML)*VmaxPML/(((double)W)*H)*((double)(i*i))/((double)(4*W*W));
        /*		// le coeff 8.63E-2 correspond  1/2*3/2*ln(10)/20 ..cf p.416 INRIA)*/
    }


    /* write a skip file */
    if(IsSkipFileCreated)
        if((SkipFile = fopen(SkipFileName,"w+t")) != NULL)
        {
            fclose(SkipFile) ;
        }


    /*	 Boucle principale de calcul */

    TempsDebut = time(NULL);
    NumeroBloc=0;
    NumeroPlan=0;

    printf("Launching %s\n",Repertoire);
    AffichageStartTime();

    ReMiseAZeroTampons();
    for (IndiceTemps=0; IndiceTemps<IndiceTempsMax; IndiceTemps++)
    {

        /*		 Etape de calcul sur les vitesse*/

        /*********************************************/
        /*Calculation of displacement velocity fields*/
        /*********************************************/

        CalculVitesses();
        CalculBordsVitesses();
        SourcesVitesses();

        /*********************************************/
        /********* Calculation of stress fields ******/
        /*********************************************/

        CalculContraintes();
        CalculBordsContraintes();
        SourcesContraintes();


        /*********************************************/
        /********* Calculation of optional fields ******/
        /*********************************************/

        if(SnapshotUx||SnapshotUy||SnapshotUz)
            ComputeDisplacement();


        /*********************************************/
        /****** Files recording and various display **/
        /*********************************************/

        if(!((IndiceTemps)%RcvDwnSmplng))
        {
            MAJMatricesReception();
        }

        if(NumberSelectedSnapshotsBlocs)
        {
            if(IndiceTemps==SnapshotsBlocsTimeSelection[NumeroBloc])
            {
                NumeroBloc++;
                SauvegardeSnapshotBlocs(NumeroBloc);
            }
        }
        else if(!((IndiceTemps + 1)%IndiceTempsBloc))
        {
            NumeroBloc++;
            SauvegardeSnapshotBlocs(NumeroBloc);
        }


        if(NumberSelectedSnapshotsPlans)
        {
            if(IndiceTemps==SnapshotsPlansTimeSelection[NumeroPlan])
            {
                NumeroPlan++;
                SauvegardeSnapshotPlans(NumeroPlan);
            }
        }
        else if(!((IndiceTemps + 1)%IndiceTempsPlan))
        {
            NumeroPlan++;
            SauvegardeSnapshotPlans(NumeroPlan);
        }


        /*	Sauvegarde des recepteurs */
        if(!((IndiceTemps+1)%IndiceTempsRecepteurs))
        {
            SauvegardeRecepteurs();
        }

        /* if necessary, displays time info to console*/
        if(argc > 2)
            if(!((IndiceTemps)%IndiceTempsAffichageTempsRestant))
                AffichageTempsRestant() ;

        if(argc == 4)
        {
            if(IsSkipFileCreated)
                remove(SkipFileName);
            LiberationMemoire() ;
            exit(1) ;
        }
    }
    AffichageEndTime();
    AffichageDureeSimul();


    SauvegardeRecepteurs();
    LiberationMemoire();


    /* write a "done" file */
    strcpy(DoneFileName,Repertoire) ;
    strcat(DoneFileName,"""done") ;
    if(IsDoneFileCreated)
        if((DoneFile = fopen(DoneFileName,"w+t")) != NULL)
        {
            fclose(DoneFile) ;
        }

    return 0;
}


/*  Fonctions  */

void ComputeDisplacement(void)
{

    int DimX,DimY,DimZ ;
    int i;

    if(SnapshotUx)
    {

        DimX = X +1;
        DimY = Y ;
        DimZ = Z ;

#pragma omp parallel for schedule(guided)
        for(i = 0 ; i < DimX*DimY*DimZ  ; i++)
            Ux[i]+=DeltaT*Vx[i];
    }
    if(SnapshotUy)
    {

        DimX = X ;
        DimY = Y +1;
        DimZ = Z ;


#pragma omp parallel for schedule(guided)
        for(i = 0 ; i < DimX*DimY*DimZ  ; i++)
            Uy[i]+=DeltaT*Vy[i];
    }
    if(SnapshotUz)
    {

        DimX = X;
        DimY = Y ;
        DimZ = Z +1;


#pragma omp parallel for schedule(guided)
        for(i = 0 ; i < DimX*DimY*DimZ  ; i++)
            Uz[i]+=DeltaT*Vz[i];
    }
}

void CalculVitesses(void)
{
    int i,j,k; /* indice (x,y,z)*/
    double Legerete;  /*Parametre effectif de lgeret*/
    double FacteurQModel ;

#pragma omp parallel for private(j,k,Legerete) schedule(guided)

    for (i=1; i<X; i++)
    {
        for (j=0; j<Y; j++)
        {
            for (k=0; k<Z; k++)
            {

                Legerete=2./(1./TableLegerete[Carte[(i-1)*(Y)*(Z)+(j)*(Z)+(k)]]+1./TableLegerete[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k)]]);

                Vx[(i)*(Y)*(Z)+(j)*(Z)+(k)] += DeltaTsurH*Legerete*(
                                                   (Txx[(i)*(Y)*(Z)+(j)*(Z)+(k)]					-	Txx[(i-1)*(Y)*(Z)+(j)*(Z)+(k)]		)	+
                                                   (Txy[(i)*(Y+1)*(Z)+(j+1)*(Z)+(k)]			-	Txy[(i)*(Y+1)*(Z)+(j)*(Z)+(k)]		)	+
                                                   (Tzx[(i)*(Y)*(Z+1)+(j)*(Z+1)+(k+1)]		-	Tzx[(i)*(Y)*(Z+1)+(j)*(Z+1)+(k)]	)	);
            }
        }
    }

#pragma omp parallel for private(j,k,Legerete) schedule(guided)

    for (i=0; i<X; i++)
    {
        for (j=1; j<Y; j++)
        {
            for (k=0; k<Z; k++)
            {

                Legerete=2./(1./TableLegerete[Carte[(i)*(Y)*(Z)+(j-1)*(Z)+(k)]]+1./TableLegerete[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k)]]);

                Vy[(i)*(Y+1)*(Z)+(j)*(Z)+(k)] += DeltaTsurH*Legerete*(
                                                     (Txy[(i+1)*(Y+1)*(Z)+(j)*(Z)+(k)]			-	Txy[(i)*(Y+1)*(Z)+(j)*(Z)+(k)]		)	+
                                                     (Tyy[(i)*(Y)*(Z)+(j)*(Z)+(k)]					-	Tyy[(i)*(Y)*(Z)+(j-1)*(Z)+(k)]		)	+
                                                     (Tyz[(i)*(Y+1)*(Z+1)+(j)*(Z+1)+(k+1)]	-	Tyz[(i)*(Y+1)*(Z+1)+(j)*(Z+1)+(k)])	);
            }
        }
    }

#pragma omp parallel for private(j,k,Legerete) schedule(guided)

    for (i=0; i<X; i++)
    {
        for (j=0; j<Y; j++)
        {
            for (k=1; k<Z; k++)
            {

                Legerete=2./(1./TableLegerete[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k-1)]]+1./TableLegerete[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k)]]);

                Vz[(i)*(Y)*(Z+1)+(j)*(Z+1)+(k)] += DeltaTsurH*Legerete*(
                                                       (Tzx[(i+1)*(Y)*(Z+1)+(j)*(Z+1)+(k)]		-	Tzx[(i)*(Y)*(Z+1)+(j)*(Z+1)+(k)]	)	+
                                                       (Tyz[(i)*(Y+1)*(Z+1)+(j+1)*(Z+1)+(k)]	-	Tyz[(i)*(Y+1)*(Z+1)+(j)*(Z+1)+(k)])	+
                                                       (Tzz[(i)*(Y)*(Z)+(j)*(Z)+(k)]					-	Tzz[(i)*(Y)*(Z)+(j)*(Z)+(k-1)]		)	);
            }
        }
    }


    /**************************************************************************/
    /*********************** Attenuating Factor (QModel) **********************/
    /**************************************************************************/

    if(TypeAbsorption == 2)
    {

#pragma omp parallel for private(j,k,FacteurQModel) schedule(guided)

        for (i=1; i<X; i++)
        {
            for (j=0; j<Y; j++)
            {
                for (k=0; k<Z; k++)
                {

                    FacteurQModel = sqrt(TableQModel[Carte[(i-1)*(Y)*(Z)+(j)*(Z)+(k)]]*TableQModel[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k)]]) ;
                    Vx[(i)*(Y)*(Z)+(j)*(Z)+(k)] *= FacteurQModel;
                }
            }
        }

#pragma omp parallel for private(j,k,FacteurQModel) schedule(guided)

        for (i=0; i<X; i++)
        {
            for (j=1; j<Y; j++)
            {
                for (k=0; k<Z; k++)
                {

                    FacteurQModel = sqrt(TableQModel[Carte[(i)*(Y)*(Z)+(j-1)*(Z)+(k)]]*TableQModel[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k)]]) ;

                    Vy[(i)*(Y+1)*(Z)+(j)*(Z)+(k)] *= FacteurQModel;
                }
            }
        }

#pragma omp parallel for private(j,k,FacteurQModel) schedule(guided)

        for (i=0; i<X; i++)
        {
            for (j=0; j<Y; j++)
            {
                for (k=1; k<Z; k++)
                {

                    FacteurQModel = sqrt(TableQModel[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k-1)]]*TableQModel[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k)]]) ;

                    Vz[(i)*(Y)*(Z+1)+(j)*(Z+1)+(k)] *= FacteurQModel;
                }
            }
        }
    }
}


void CalculContraintes()
{
    int i,j,k;
    double C44,C55,C66;
    double FacteurQModel;

#pragma omp parallel for private(j,k) schedule(guided)

    for (i=0; i<X; i++)
    {
        for (j=0; j<Y; j++)
        {
            for (k=0; k<Z; k++)
            {

                Txx[(i)*(Y)*(Z)+(j)*(Z)+(k)] += DeltaTsurH*(
                                                    TableC11[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k)]]*(Vx[(i+1)*(Y)*(Z)+(j)*(Z)+(k)]			-	Vx[(i)*(Y)*(Z)+(j)*(Z)+(k)]		)	+
                                                    TableC12[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k)]]*(Vy[(i)*(Y+1)*(Z)+(j+1)*(Z)+(k)]		-	Vy[(i)*(Y+1)*(Z)+(j)*(Z)+(k)]	)	+
                                                    TableC31[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k)]]*(Vz[(i)*(Y)*(Z+1)+(j)*(Z+1)+(k+1)]	-	Vz[(i)*(Y)*(Z+1)+(j)*(Z+1)+(k)]));

                Tyy[(i)*(Y)*(Z)+(j)*(Z)+(k)] += DeltaTsurH*(
                                                    TableC12[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k)]]*(Vx[(i+1)*(Y)*(Z)+(j)*(Z)+(k)]			-	Vx[(i)*(Y)*(Z)+(j)*(Z)+(k)]		)	+
                                                    TableC22[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k)]]*(Vy[(i)*(Y+1)*(Z)+(j+1)*(Z)+(k)]		-	Vy[(i)*(Y+1)*(Z)+(j)*(Z)+(k)]	)	+
                                                    TableC23[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k)]]*(Vz[(i)*(Y)*(Z+1)+(j)*(Z+1)+(k+1)]	-	Vz[(i)*(Y)*(Z+1)+(j)*(Z+1)+(k)]));

                Tzz[(i)*(Y)*(Z)+(j)*(Z)+(k)] += DeltaTsurH*(
                                                    TableC31[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k)]]*(Vx[(i+1)*(Y)*(Z)+(j)*(Z)+(k)]			-	Vx[(i)*(Y)*(Z)+(j)*(Z)+(k)]		)	+
                                                    TableC23[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k)]]*(Vy[(i)*(Y+1)*(Z)+(j+1)*(Z)+(k)]		-	Vy[(i)*(Y+1)*(Z)+(j)*(Z)+(k)]	)	+
                                                    TableC33[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k)]]*(Vz[(i)*(Y)*(Z+1)+(j)*(Z+1)+(k+1)]	-	Vz[(i)*(Y)*(Z+1)+(j)*(Z+1)+(k)]));


            }
        }
    }

#pragma omp parallel for private(j,k,C66) schedule(guided)

    for (i=1; i<X; i++)
    {
        for (j=1; j<Y; j++)
        {
            for (k=0; k<Z; k++)
            {

                if (TableC66[Carte[(i-1)*(Y)*(Z)+(j-1)*(Z)+(k)]]*
                        TableC66[Carte[(i-1)*(Y)*(Z)+(j)*(Z)+(k)]]*
                        TableC66[Carte[(i)*(Y)*(Z)+(j-1)*(Z)+(k)]]*
                        TableC66[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k)]])
                {

                    C66=4./(1./TableC66[Carte[(i-1)*(Y)*(Z)+(j-1)*(Z)+(k)]]+
                            1./TableC66[Carte[(i-1)*(Y)*(Z)+(j)*(Z)+(k)]]+
                            1./TableC66[Carte[(i)*(Y)*(Z)+(j-1)*(Z)+(k)]]+
                            1./TableC66[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k)]]);
                }
                else
                {
                    C66=0.0;
                }

                Txy[(i)*(Y+1)*(Z)+(j)*(Z)+(k)] += DeltaTsurH*C66*(
                                                      (Vx[(i)*(Y)*(Z)+(j)*(Z)+(k)]		-	Vx[(i)*(Y)*(Z)+(j-1)*(Z)+(k)]		)	+
                                                      (Vy[(i)*(Y+1)*(Z)+(j)*(Z)+(k)]	-	Vy[(i-1)*(Y+1)*(Z)+(j)*(Z)+(k)]	)	);

            }
        }
    }

#pragma omp parallel for private(j,k,C44) schedule(guided)

    for (i=0; i<X; i++)
    {
        for (j=1; j<Y; j++)
        {
            for (k=1; k<Z; k++)
            {

                if (TableC44[Carte[(i)*(Y)*(Z)+(j-1)*(Z)+(k-1)]]*
                        TableC44[Carte[(i)*(Y)*(Z)+(j-1)*(Z)+(k)]]*
                        TableC44[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k-1)]]*
                        TableC44[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k)]])
                {

                    C44=4./(1./TableC44[Carte[(i)*(Y)*(Z)+(j-1)*(Z)+(k-1)]]+
                            1./TableC44[Carte[(i)*(Y)*(Z)+(j-1)*(Z)+(k)]]+
                            1./TableC44[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k-1)]]+
                            1./TableC44[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k)]]);
                }
                else
                {
                    C44=0.0;
                }

                Tyz[(i)*(Y+1)*(Z+1)+(j)*(Z+1)+(k)] += DeltaTsurH*C44*(
                        (Vy[(i)*(Y+1)*(Z)+(j)*(Z)+(k)]		-	Vy[(i)*(Y+1)*(Z)+(j)*(Z)+(k-1)]		)	+
                        (Vz[(i)*(Y)*(Z+1)+(j)*(Z+1)+(k)]	-	Vz[(i)*(Y)*(Z+1)+(j-1)*(Z+1)+(k)]	)	);
            }
        }
    }

#pragma omp parallel for private(j,k,C55) schedule(guided)

    for (i=1; i<X; i++)
    {
        for (j=0; j<Y; j++)
        {
            for (k=1; k<Z; k++)
            {

                if (TableC55[Carte[(i-1)*(Y)*(Z)+(j)*(Z)+(k-1)]]*
                        TableC55[Carte[(i-1)*(Y)*(Z)+(j)*(Z)+(k)]]*
                        TableC55[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k-1)]]*
                        TableC55[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k)]])
                {

                    C55=4./(1./TableC55[Carte[(i-1)*(Y)*(Z)+(j)*(Z)+(k-1)]]+
                            1./TableC55[Carte[(i-1)*(Y)*(Z)+(j)*(Z)+(k)]]+
                            1./TableC55[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k-1)]]+
                            1./TableC55[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k)]]);
                }
                else
                {
                    C55=0.0;
                }

                Tzx[(i)*(Y)*(Z+1)+(j)*(Z+1)+(k)] += DeltaTsurH*C55*(
                                                        (Vx[(i)*(Y)*(Z)+(j)*(Z)+(k)]			-	Vx[(i)*(Y)*(Z)+(j)*(Z)+(k-1)]			)	+
                                                        (Vz[(i)*(Y)*(Z+1)+(j)*(Z+1)+(k)]	-	Vz[(i-1)*(Y)*(Z+1)+(j)*(Z+1)+(k)]	)	);
            }
        }
    }

    /**************************************************************************/
    /*********************** Attenuating Factor (QModel) **********************/
    /**************************************************************************/

    if(TypeAbsorption == 2)
    {

#pragma omp parallel for private(j,k) schedule(guided)

        for (i=0; i<X; i++)
        {
            for (j=0; j<Y; j++)
            {
                for (k=0; k<Z; k++)
                {

                    Txx[(i)*(Y)*(Z)+(j)*(Z)+(k)] *= TableQModel[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k)]];

                    Tyy[(i)*(Y)*(Z)+(j)*(Z)+(k)] *= TableQModel[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k)]];

                    Tzz[(i)*(Y)*(Z)+(j)*(Z)+(k)] *= TableQModel[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k)]];
                }
            }
        }

#pragma omp parallel for private(j,k,FacteurQModel) schedule(guided)

        for (i=1; i<X; i++)
        {
            for (j=1; j<Y; j++)
            {
                for (k=0; k<Z; k++)
                {


                    FacteurQModel = sqrt(sqrt(TableQModel[Carte[(i-1)*(Y)*(Z)+(j-1)*(Z)+(k)]]*
                                              TableQModel[Carte[(i-1)*(Y)*(Z)+(j)*(Z)+(k)]]*
                                              TableQModel[Carte[(i)*(Y)*(Z)+(j-1)*(Z)+(k)]]*
                                              TableQModel[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k)]])) ;

                    Txy[(i)*(Y+1)*(Z)+(j)*(Z)+(k)] *= FacteurQModel;
                }
            }
        }

#pragma omp parallel for private(j,k,FacteurQModel) schedule(guided)

        for (i=0; i<X; i++)
        {
            for (j=1; j<Y; j++)
            {
                for (k=1; k<Z; k++)
                {

                    FacteurQModel = sqrt(sqrt(TableQModel[Carte[(i)*(Y)*(Z)+(j-1)*(Z)+(k-1)]]*
                                              TableQModel[Carte[(i)*(Y)*(Z)+(j-1)*(Z)+(k)]]*
                                              TableQModel[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k-1)]]*
                                              TableQModel[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k)]])) ;

                    Tyz[(i)*(Y+1)*(Z+1)+(j)*(Z+1)+(k)] *= FacteurQModel;
                }
            }
        }

#pragma omp parallel for private(j,k,FacteurQModel) schedule(guided)

        for (i=1; i<X; i++)
        {
            for (j=0; j<Y; j++)
            {
                for (k=1; k<Z; k++)
                {

                    FacteurQModel = sqrt(sqrt(TableQModel[Carte[(i-1)*(Y)*(Z)+(j)*(Z)+(k-1)]]*
                                              TableQModel[Carte[(i-1)*(Y)*(Z)+(j)*(Z)+(k)]]*
                                              TableQModel[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k-1)]]*
                                              TableQModel[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k)]])) ;

                    Tzx[(i)*(Y)*(Z+1)+(j)*(Z+1)+(k)] *= FacteurQModel;
                }
            }
        }
    }
}

/*
void AffichageTempsRestant(void)
{
    int Heures, Minutes, Secondes,i;

    TempsCourant = time(NULL);
    TempsRestant= difftime(TempsCourant,TempsDebut)*((double)(IndiceTempsMax+1)/(double)(IndiceTemps+1)-1);
    Heures=(int)(TempsRestant/3600);
    Minutes=(int)((TempsRestant-3600*Heures)/60);
    Secondes= (int)(TempsRestant-3600*Heures-60*Minutes);
    for(i=0; i<80; i++)
        printf("\b");
    printf("Time: %.1f/%.1f us \t",IndiceTemps*DeltaT,IndiceTempsMax*DeltaT);
    printf("Step: %i/%i \t",IndiceTemps,IndiceTempsMax);
    printf("Computation time: %3ih %2imin %2isec ",Heures, Minutes,Secondes);
}*/


void AffichageTempsRestant(void)
{
    int Heures, Minutes, Secondes ;
#ifdef WIN32
    int i;
#endif

    TempsCourant = time(NULL) ;
    TempsRestant = difftime(TempsCourant,TempsDebut)*((double)(IndiceTempsMax+1)/(double)(IndiceTemps+1)-1) ;
    Heures = (int)(TempsRestant/3600) ;
    Minutes = (int)((TempsRestant-3600*Heures)/60) ;
    Secondes = (int)(TempsRestant-3600*Heures-60*Minutes) ;
#ifdef WIN32
    for(i = 0 ; i < 80 ; i++)
        printf("\b") ;
#else
    printf("\n") ;
#endif

    printf("Time: %.1f/%.1f us \t",IndiceTemps*DeltaT,IndiceTempsMax*DeltaT) ;
    printf("Step: %i/%i \t",IndiceTemps,IndiceTempsMax) ;
    printf("Computation time: %3ih %2imin %2isec ",Heures, Minutes,Secondes) ;
}



void AffichageDureeSimul(void)
{
    int Heures, Minutes, Secondes;
    TempsCourant = time(NULL);
    TempsRestant= difftime(TempsCourant,TempsDebut);
    Heures=(int)(TempsRestant/3600);
    Minutes=(int)((TempsRestant-3600*Heures)/60);
    Secondes= (int)(TempsRestant-3600*Heures-60*Minutes);
    printf("\nTotal computation time: %3ih %2imin %2isec \n",Heures, Minutes,Secondes);
}

void AffichageStartTime(void)
{

    time_t rawtime;
    struct tm * timeinfo;

    time ( &rawtime );
    timeinfo = localtime ( &rawtime );
    printf ( "Started on : %s", asctime (timeinfo) );
}

void AffichageEndTime(void)
{

    time_t rawtime;
    struct tm * timeinfo;

    time ( &rawtime );
    timeinfo = localtime ( &rawtime );
    printf ( "\nEnded on   : %s", asctime (timeinfo) );
}


void LectureSignal(struct MATRICE_EMISSION *MatriceCourante, int NbPtsSignal)
{

    FILE *Fich=NULL;

    if((Fich = fopen(MatriceCourante->NomSignal, "rb" )) == NULL)
    {
        printf("Problem opening %s :file not found!\n",MatriceCourante->NomSignal);
        LiberationMemoire();
        exit(1);
    }
    /*	Lecture des dimensions de la simulation dans les 2 premiers octets de carte.map*/
    fread(&MatriceCourante->DurationPt,sizeof(int),1,Fich);

    if (NbPtsSignal<MatriceCourante->DurationPt)
    {
        printf("Emission signals will be truncated, as longer than simulation time...\n");
        MatriceCourante->DurationPt=NbPtsSignal;

    }
    /*	 Lecture et fermeture fichier du signal*/
    fread(MatriceCourante->Signal,sizeof(double),MatriceCourante->DurationPt,Fich);
    fclose( Fich );

}


void LectureParametresMatriceReception(FILE *Fich,char *CurrentLine,
                                       struct MATRICE_RECEPTION *MatriceCourante)
{
    fgets(CurrentLine,500,Fich);
    sscanf(CurrentLine,"%s",MatriceCourante->Nom);
    fgets(CurrentLine,500,Fich);
    //sscanf(CurrentLine,"%c",&MatriceCourante->PlanI);
    sscanf(CurrentLine,"%c %i",&MatriceCourante->PlanI,&MatriceCourante->ActiveMedium);
    fgets(CurrentLine,500,Fich);
    sscanf(CurrentLine,"%i %i %i",
           &MatriceCourante->XMat,&MatriceCourante->YMat,&MatriceCourante->ZMat);
    fgets(CurrentLine,500,Fich);
    sscanf(CurrentLine,"%i %i %i",
           &MatriceCourante->NbElementsJ,&MatriceCourante->PitchJ,&MatriceCourante->LargeurJ);
    fgets(CurrentLine,500,Fich);
    sscanf(CurrentLine,"%i %i %i",
           &MatriceCourante->NbElementsK,&MatriceCourante->PitchK,&MatriceCourante->LargeurK);
}

int ValiditeMatriceReception(struct MATRICE_RECEPTION *MatriceCourante,int LimX,int LimY,int LimZ)
{
    int Test=1;/*default for with A,B,C,U,V,W types, no security for these...*/
    switch (MatriceCourante->PlanI)
    {
    case 'X':
    case '1':
        Test=((MatriceCourante->XMat<LimX)&&
              ((MatriceCourante->YMat+(MatriceCourante->NbElementsJ-1)*MatriceCourante->PitchJ
                +(MatriceCourante->LargeurJ-1))<LimY)&&
              ((MatriceCourante->ZMat+(MatriceCourante->NbElementsK-1)*MatriceCourante->PitchK
                +(MatriceCourante->LargeurK-1))<LimZ));
        break;
    case 'Y':
    case '2':
        Test=((MatriceCourante->YMat<LimY)&&
              ((MatriceCourante->ZMat+(MatriceCourante->NbElementsJ-1)*MatriceCourante->PitchJ
                +(MatriceCourante->LargeurJ-1))<LimZ)&&
              ((MatriceCourante->XMat+(MatriceCourante->NbElementsK-1)*MatriceCourante->PitchK
                +(MatriceCourante->LargeurK-1))<LimX));
        break;
    case 'Z':
    case '3':
        Test=((MatriceCourante->ZMat<LimZ)&&
              ((MatriceCourante->XMat+(MatriceCourante->NbElementsJ-1)*MatriceCourante->PitchJ
                +(MatriceCourante->LargeurJ-1))<LimX)&&
              ((MatriceCourante->YMat+(MatriceCourante->NbElementsK-1)*MatriceCourante->PitchK
                +(MatriceCourante->LargeurK-1))<LimY));
        break;
    }
    return Test;
}

void LectureParametresMatriceEmission(FILE *Fich, char *CurrentLine,
                                      struct MATRICE_EMISSION *MatriceCourante)
{

    MatriceCourante->Signal = (double *) calloc (IndiceTempsMax,sizeof(double));


    fgets(CurrentLine,500,Fich);
    sscanf(CurrentLine,"%i",&MatriceCourante->TypeSignal);
    if (MatriceCourante->TypeSignal)
    {
        if((MatriceCourante->TypeSignal>0))
        {
            sscanf(CurrentLine,"%i %lf %lf %lf",&MatriceCourante->TypeSignal,
                   &MatriceCourante->Freq,&MatriceCourante->T0,&MatriceCourante->Bandwidth);
            GenerationSignal(MatriceCourante->TypeSignal,MatriceCourante->Freq,
                             MatriceCourante->T0,MatriceCourante->Bandwidth,MatriceCourante->Signal);
        }
        else
        {
            /* add been added to keep compatiblity while extending folder management. Would not do it the
            	same way from scratch....*/
            char TempFileName[500]="";

            sscanf(CurrentLine,"%i %s",&MatriceCourante->TypeSignal,MatriceCourante->NomSignal);


            strcpy(TempFileName,MatriceCourante->NomSignal) ;
            strcpy(MatriceCourante->NomSignal,Repertoire) ;
            strcat(MatriceCourante->NomSignal,TempFileName) ;

            LectureSignal(MatriceCourante,IndiceTempsMax) ;
        }
    }
    else
    {
        sscanf(CurrentLine,"%i %s",&MatriceCourante->TypeSignal,MatriceCourante->NomSignal);
        LectureSignal(MatriceCourante,IndiceTempsMax);
    }
    fgets(CurrentLine,500,Fich);
    sscanf(CurrentLine,"%c",&MatriceCourante->PlanI);
    fgets(CurrentLine,500,Fich);
    sscanf(CurrentLine,"%i %i %i",
           &MatriceCourante->XMat,&MatriceCourante->YMat,&MatriceCourante->ZMat);
    fgets(CurrentLine,500,Fich);
    sscanf(CurrentLine,"%i %i %i %i %i %lf",
           &MatriceCourante->NbElementsJ,&MatriceCourante->PitchJ,&MatriceCourante->LargeurJ,
           &MatriceCourante->ApodisationJ,&MatriceCourante->FocalisationJ,&MatriceCourante->DirectionJ);
    fgets(CurrentLine,500,Fich);
    sscanf(CurrentLine,"%i %i %i %i %i %lf",
           &MatriceCourante->NbElementsK,&MatriceCourante->PitchK,&MatriceCourante->LargeurK,
           &MatriceCourante->ApodisationK,&MatriceCourante->FocalisationK,&MatriceCourante->DirectionK);
    fgets(CurrentLine,500,Fich);
    sscanf(CurrentLine,"%lf %lf",&MatriceCourante->Focale,&MatriceCourante->VitesseMilieuEmission);
}


void LectureParametresMatriceSourceFile(FILE *Fich, char *CurrentLine,
                                        struct MATRICE_RECEPTION *MatriceCourante)
{

    FILE *ArraySourceFile=NULL;
    double unused;

    fgets(CurrentLine,500,Fich);
    sscanf(CurrentLine,"%s",MatriceCourante->Nom);

    if((ArraySourceFile=fopen(MatriceCourante->Nom,"rb"))==NULL)
    {
        printf("ArraySourceFile NOT FOUND...\n");
        exit(1);
    }
    else
    {

        fread(&MatriceCourante->PlanI,sizeof(char),1,ArraySourceFile);
        fread(&MatriceCourante->NbElementsJ,sizeof(int),1,ArraySourceFile);
        fread(&MatriceCourante->NbElementsK,sizeof(int),1,ArraySourceFile);
        fread(&MatriceCourante->XMat,sizeof(int),1,ArraySourceFile);
        fread(&MatriceCourante->YMat,sizeof(int),1,ArraySourceFile);
        fread(&MatriceCourante->ZMat,sizeof(int),1,ArraySourceFile);

        fread(&MatriceCourante->LargeurJ,sizeof(int),1,ArraySourceFile);
        fread(&MatriceCourante->LargeurK,sizeof(int),1,ArraySourceFile);


        fread(&MatriceCourante->PitchJ,sizeof(int),1,ArraySourceFile);
        fread(&MatriceCourante->PitchK,sizeof(int),1,ArraySourceFile);


        fread(&unused,sizeof(double),1,ArraySourceFile);

        fread(&MatriceCourante->DurationPt,sizeof(int),1,ArraySourceFile);

        MatriceCourante->Signaux=(double *) calloc (
                                     MatriceCourante->DurationPt*MatriceCourante->NbElementsJ*MatriceCourante->NbElementsK,
                                     sizeof(double));


        fread(&unused,sizeof(double),1,ArraySourceFile);
        fread(MatriceCourante->Signaux,sizeof(double),MatriceCourante->DurationPt*MatriceCourante->NbElementsJ*MatriceCourante->NbElementsK,ArraySourceFile);

        fclose(ArraySourceFile );
    }

}


int ValiditeMatriceEmission(struct MATRICE_EMISSION *MatriceCourante,int LimX,int LimY,int LimZ)
{
    int Test=0;
    switch (MatriceCourante->PlanI)
    {
    case 'X':
    case '1':
        Test=((MatriceCourante->XMat<LimX)&&
              ((MatriceCourante->YMat+(MatriceCourante->NbElementsJ-1)*MatriceCourante->PitchJ
                +(MatriceCourante->LargeurJ-1))<LimY)&&
              ((MatriceCourante->ZMat+(MatriceCourante->NbElementsK-1)*MatriceCourante->PitchK
                +(MatriceCourante->LargeurK-1))<LimZ));
        break;
    case 'Y':
    case '2':
        Test=((MatriceCourante->YMat<LimY)&&
              ((MatriceCourante->ZMat+(MatriceCourante->NbElementsJ-1)*MatriceCourante->PitchJ
                +(MatriceCourante->LargeurJ-1))<LimZ)&&
              ((MatriceCourante->XMat+(MatriceCourante->NbElementsK-1)*MatriceCourante->PitchK
                +(MatriceCourante->LargeurK-1))<LimX));
        break;
    case 'Z':
    case '3':
        Test=((MatriceCourante->ZMat<LimZ)&&
              ((MatriceCourante->XMat+(MatriceCourante->NbElementsJ-1)*MatriceCourante->PitchJ
                +(MatriceCourante->LargeurJ-1))<LimX)&&
              ((MatriceCourante->YMat+(MatriceCourante->NbElementsK-1)*MatriceCourante->PitchK
                +(MatriceCourante->LargeurK-1))<LimY));
        break;
    }
    return Test;
}


void LectureParametres(void)
{
    struct MATRICE_EMISSION *MatriceCouranteEmission;
    struct MATRICE_RECEPTION *MatriceCouranteReception;

    char ParameterFileName[500];
    char CurrentLine[500];

    FILE *Fich=NULL;

    int i=0,IndiceMateriau,Test,NbPtsAlloues;
    double temp=0;
    double densite,C11,C22,C33,C12,C23,C31,C44,C55,C66,QModel,VitesseQModel;

    int IndiceTempsMaxRcv;
    int	IndiceTempsRcv;

    /*Default parameters, corresponding to optional parameters*/
    H=0.1;
    Vmax=1.5;
    Duree=0.0;
    CoeffDeltaT=0.99;
    TypeAbsorption=0;
    IsSkipFileCreated=0;
    IsDoneFileCreated=0;
    ConditionFaceXm=0,ConditionFaceXp=0,ConditionFaceYm=0,ConditionFaceYp=0,ConditionFaceZm=0,ConditionFaceZp=0;
    CoeffPML=80,W=20,VmaxPML=1.5;

    NumberSelectedSnapshotsBlocs=0;
    NumberSelectedSnapshotsPlans=0;


    RcvDwnSmplng=1;
    SourceType=1;
    PeriodBloc=1;
    PeriodPlan=1;
    PeriodRecepteurs=100;
    IndiceTempsAffichageTempsRestant=1;
    NbMatricesEmissionTxx=0,NbMatricesEmissionTyy=0,NbMatricesEmissionTzz=0;
    NbMatricesEmissionTxy=0,NbMatricesEmissionTyz=0,NbMatricesEmissionTzx=0;
    NbMatricesEmissionVx=0,NbMatricesEmissionVy=0,NbMatricesEmissionVz=0;
    NbMatricesReceptionTxx=0,NbMatricesReceptionTyy=0,NbMatricesReceptionTzz=0;
    NbMatricesReceptionTxy=0,NbMatricesReceptionTyz=0,NbMatricesReceptionTzx=0;
    NbMatricesReceptionVx=0,NbMatricesReceptionVy=0,NbMatricesReceptionVz=0;

    NbMatricesSourceFilesTxx=0,NbMatricesSourceFilesTyy=0,NbMatricesSourceFilesTzz=0;
    NbMatricesSourceFilesTxy=0,NbMatricesSourceFilesTyz=0,NbMatricesSourceFilesTzx=0;
    NbMatricesSourceFilesVx=0,NbMatricesSourceFilesVy=0,NbMatricesSourceFilesVz=0;


    SnapshotVx=0,SnapshotVy=0,SnapshotVz=0;
    SnapshotUx=0,SnapshotUy=0,SnapshotUz=0;
    SnapshotTxx=0,SnapshotTyy=0,SnapshotTzz=0;
    SnapshotTxy=0,SnapshotTyz=0,SnapshotTzx=0;
    SnapshotEc=0,SnapshotV=0;


    /*    SnapshotMaxV=0,SnapshotMaxVx=0,SnapshotMaxVy=0,SnapshotMaxTxx=0,SnapshotMaxTyy=0,SnapshotMaxTxy=0, SnapshotMaxEc=0;
       */



    strcpy(ParameterFileName,Repertoire);
    strcat(ParameterFileName,"""Parametres.ini3D");

    if((Fich = fopen(ParameterFileName, "rb" )) == NULL)
    {
        strcpy(ParameterFileName,Repertoire);
        strcat(ParameterFileName,"""Parameters.ini3D");

        if((Fich=fopen(ParameterFileName,"rb"))==NULL)
        {

            printf("Parameters.ini3D NOT FOUND ...\n");
            exit(1);
        }

    }




    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Vmax    ") !=NULL)
        {
            sscanf(CurrentLine+31,"%lf",&Vmax);
        }
    }

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Grid Step") !=NULL)
        {
            sscanf(CurrentLine+31,"%lf",&H);
        }
    }


    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Absorption Type") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&TypeAbsorption);
        }
    }

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Simulation Length") !=NULL)
        {
            sscanf(CurrentLine+31,"%lf",&Duree);
        }
    }


    /* for compatibility with previous versions. Fields have been changed...*/
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"DeltaT Coefficient") !=NULL)
        {
            sscanf(CurrentLine+31,"%lf",&CoeffDeltaT);
        }
    }

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"CFL Coefficient") !=NULL)
        {
            sscanf(CurrentLine+31,"%lf",&CoeffDeltaT);
        }
    }

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"3D Snapshots Record Period") !=NULL)
        {
            sscanf(CurrentLine+31,"%lf",&PeriodBloc);
        }
    }


    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"3D Data Record Frequency") !=NULL)
        {
            sscanf(CurrentLine+31,"%lf",&PeriodBloc);
            PeriodBloc=1./PeriodBloc;
        }
    }

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Signals Refresh Period") !=NULL)
        {
            sscanf(CurrentLine+31,"%lf",&PeriodRecepteurs);
        }
    }

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Receivers Record Frequency") !=NULL)
        {
            sscanf(CurrentLine+31,"%lf",&PeriodRecepteurs);
            PeriodRecepteurs=1./PeriodRecepteurs;
        }
    }

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"2D Sections Record Period") !=NULL)
        {
            sscanf(CurrentLine+31,"%lf",&PeriodPlan);
        }
    }

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"2D Sections Record Frequency") !=NULL)
        {
            sscanf(CurrentLine+31,"%lf",&PeriodPlan);
            PeriodPlan=1./PeriodPlan;
        }
    }




    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Time information step period") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&IndiceTempsAffichageTempsRestant);
        }
    }


    /*Parametres dduits des prcedents*/
    DeltaT = CoeffDeltaT*H/(sqrt(3.)*Vmax);
    DeltaTsurH = DeltaT/H;
    IndiceTempsMax = (int) (Duree/DeltaT+1);
    IndiceTempsBloc=(int)(PeriodBloc/DeltaT+1);
    IndiceTempsRecepteurs=(int)(PeriodRecepteurs/DeltaT+1);
    IndiceTempsPlan=(int)(PeriodPlan/DeltaT+1);

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of selected 3D Snapshots") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NumberSelectedSnapshotsBlocs);

            SnapshotsBlocsTimeSelection=(int *) calloc (NumberSelectedSnapshotsBlocs,sizeof(int));

            for(i=0; i<NumberSelectedSnapshotsBlocs; i++)
            {
                fscanf (Fich, "%lf", &temp);
                SnapshotsBlocsTimeSelection[i]=(int)floor(temp/DeltaT+0.5);
                //   printf ("%i \n",SnapshotsBlocsTimeSelection[i]);
            }
        }
    }

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of selected 2D Snapshots") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NumberSelectedSnapshotsPlans);

            SnapshotsPlansTimeSelection=(int *) calloc (NumberSelectedSnapshotsPlans,sizeof(int));

            for(i=0; i<NumberSelectedSnapshotsPlans; i++)
            {
                fscanf (Fich, "%lf", &temp);
                SnapshotsPlansTimeSelection[i]=(int)floor(temp/DeltaT+0.5);
                //    printf ("%i \n",SnapshotsPlansTimeSelection[i]);
            }
        }
    }


    /* Lecture des conditions de bords */

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"X1_low Boundary") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ConditionFaceXm);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"X1_high Boundary") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ConditionFaceXp);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"X2_low Boundary") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ConditionFaceYm);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"X2_high Boundary") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ConditionFaceYp);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"X3_low Boundary") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ConditionFaceZm);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"X3_high Boundary") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ConditionFaceZp);
        }
    }


    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Xm Boundary") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ConditionFaceXm);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Xp Boundary") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ConditionFaceXp);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Ym Boundary") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ConditionFaceYm);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Yp Boundary") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ConditionFaceYp);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Zm Boundary") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ConditionFaceZm);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Zp Boundary") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ConditionFaceZp);
        }
    }



    /****************************************************/


    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"PML Width W") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&W);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"PML Thickness") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&W);
        }
    }

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Vmax in PML") !=NULL)
        {
            sscanf(CurrentLine+31,"%lf",&VmaxPML);
        }
    }

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Attenuation coeff.in PML") !=NULL)
        {
            sscanf(CurrentLine+31,"%lf",&CoeffPML);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"PML Efficiency") !=NULL)
        {
            sscanf(CurrentLine+31,"%lf",&CoeffPML);
        }
    }

    if(TypeAbsorption==0)
    {
        fseek(Fich,0L,SEEK_SET);
        while(fgets(CurrentLine,500,Fich) !=NULL)
        {
            if(strstr(CurrentLine,"Starts Materials List") !=NULL)
            {
                while(strstr(fgets(CurrentLine,500,Fich),"Ends Materials List")==NULL)
                {
                    sscanf(CurrentLine,"%i %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf",
                           &IndiceMateriau,&densite,&C11,&C22,&C33,&C12,&C23,&C31,&C44,&C55,&C66);
                    TableLegerete[(PREC_CARTE)IndiceMateriau]=1./densite;
                    TableC11[(PREC_CARTE)IndiceMateriau]=C11;
                    TableC22[(PREC_CARTE)IndiceMateriau]=C22;
                    TableC33[(PREC_CARTE)IndiceMateriau]=C33;
                    TableC12[(PREC_CARTE)IndiceMateriau]=C12;
                    TableC23[(PREC_CARTE)IndiceMateriau]=C23;
                    TableC31[(PREC_CARTE)IndiceMateriau]=C31;
                    TableC44[(PREC_CARTE)IndiceMateriau]=C44;
                    TableC55[(PREC_CARTE)IndiceMateriau]=C55;
                    TableC66[(PREC_CARTE)IndiceMateriau]=C66;
                    TableQModel[(PREC_CARTE)IndiceMateriau] = 1.;
                }
            }
        }
    }

    if(TypeAbsorption!=0)
    {
        fseek(Fich,0L,SEEK_SET);
        while(fgets(CurrentLine,500,Fich) !=NULL)
        {
            if(strstr(CurrentLine,"Starts Materials List") !=NULL)
            {
                while(strstr(fgets(CurrentLine,500,Fich),"Ends Materials List")==NULL)
                {
                    sscanf(CurrentLine,"%i %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf",
                           &IndiceMateriau,&densite,&C11,&C22,&C33,&C12,&C23,&C31,&C44,&C55,&C66,&QModel,&VitesseQModel);
                    TableLegerete[(PREC_CARTE)IndiceMateriau]=1./densite;
                    TableC11[(PREC_CARTE)IndiceMateriau]=C11;
                    TableC22[(PREC_CARTE)IndiceMateriau]=C22;
                    TableC33[(PREC_CARTE)IndiceMateriau]=C33;
                    TableC12[(PREC_CARTE)IndiceMateriau]=C12;
                    TableC23[(PREC_CARTE)IndiceMateriau]=C23;
                    TableC31[(PREC_CARTE)IndiceMateriau]=C31;
                    TableC44[(PREC_CARTE)IndiceMateriau]=C44;
                    TableC55[(PREC_CARTE)IndiceMateriau]=C55;
                    TableC66[(PREC_CARTE)IndiceMateriau]=C66;
                    TableQModel[(PREC_CARTE)IndiceMateriau] = exp(-log(10)/20*QModel*VitesseQModel*DeltaT) ;
                }
            }
        }
    }


    /* Options for creating skip/done file*/

    fseek(Fich,0L,SEEK_SET) ;
    while(fgets(CurrentLine,500,Fich) != NULL)
    {
        if(strstr(CurrentLine,"Is skip file created") != NULL)
        {
            sscanf(CurrentLine+31,"%i",&IsSkipFileCreated) ;
        }
    }

    fseek(Fich,0L,SEEK_SET) ;
    while(fgets(CurrentLine,500,Fich) != NULL)
    {
        if(strstr(CurrentLine,"Is done file created") != NULL)
        {
            sscanf(CurrentLine+31,"%i",&IsDoneFileCreated) ;
        }
    }




    /* Options for recording snapshots */




    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Receivers Downsampling") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&RcvDwnSmplng);
        }
    }


    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Record 3D V Data") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&SnapshotV);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Record 3D V Snapshots") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&SnapshotV);
        }
    }

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Record 3D Ekinetic Snapshots") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&SnapshotEc);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Record 3D Ec Data") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&SnapshotEc);
        }
    }

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Record 3D V1 Snapshots") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&SnapshotVx);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Record 3D V2 Snapshots") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&SnapshotVy);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Record 3D V3 Snapshots") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&SnapshotVz);
        }
    }


    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Record 3D Vx Data") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&SnapshotVx);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Record 3D Vy Data") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&SnapshotVy);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Record 3D Vz Data") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&SnapshotVz);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Record 3D T11 Snapshots") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&SnapshotTxx);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Record 3D T22 Snapshots") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&SnapshotTyy);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Record 3D T33 Snapshots") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&SnapshotTzz);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Record 3D T12 Snapshots") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&SnapshotTxy);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Record 3D T23 Snapshots") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&SnapshotTyz);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Record 3D T31 Snapshots") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&SnapshotTzx);
        }
    }

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Record 3D Txx Data") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&SnapshotTxx);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Record 3D Tyy Data") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&SnapshotTyy);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Record 3D Tzz Data") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&SnapshotTzz);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Record 3D Txy Data") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&SnapshotTxy);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Record 3D Tyz Data") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&SnapshotTyz);
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Record 3D Tzx Data") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&SnapshotTzx);
        }
    }


    fseek(Fich,0L,SEEK_SET) ;
    while(fgets(CurrentLine,500,Fich) != NULL)
    {
        if(strstr(CurrentLine,"Record 3D U1 Snapshots") != NULL)
        {
            sscanf(CurrentLine+31,"%i",&SnapshotUx) ;
        }
    }
    fseek(Fich,0L,SEEK_SET) ;
    while(fgets(CurrentLine,500,Fich) != NULL)
    {
        if(strstr(CurrentLine,"Record 3D U2 Snapshots") != NULL)
        {
            sscanf(CurrentLine+31,"%i",&SnapshotUy) ;
        }
    }
    fseek(Fich,0L,SEEK_SET) ;
    while(fgets(CurrentLine,500,Fich) != NULL)
    {
        if(strstr(CurrentLine,"Record 3D U3 Snapshots") != NULL)
        {
            sscanf(CurrentLine+31,"%i",&SnapshotUz) ;
        }
    }
    fseek(Fich,0L,SEEK_SET) ;
    while(fgets(CurrentLine,500,Fich) != NULL)
    {
        if(strstr(CurrentLine,"Record 3D Ux Data") != NULL)
        {
            sscanf(CurrentLine+31,"%i",&SnapshotUx) ;
        }
    }
    fseek(Fich,0L,SEEK_SET) ;
    while(fgets(CurrentLine,500,Fich) != NULL)
    {
        if(strstr(CurrentLine,"Record 3D Uy Data") != NULL)
        {
            sscanf(CurrentLine+31,"%i",&SnapshotUy) ;
        }
    }
    fseek(Fich,0L,SEEK_SET) ;
    while(fgets(CurrentLine,500,Fich) != NULL)
    {
        if(strstr(CurrentLine,"Record 3D Uz Data") != NULL)
        {
            sscanf(CurrentLine+31,"%i",&SnapshotUz) ;
        }
    }


    /*****************************************************************/
    /**********Lecture des Plans  sauvegarder ***********************/
    /*****************************************************************/

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of 2D V Snapshots") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ListePlanV.NbPlan);
            if (ListePlanV.NbPlan)
            {
                ListePlanV.TypePlan = (char *) calloc (ListePlanV.NbPlan,sizeof(char));
                ListePlanV.CotePlan = (int *) calloc (ListePlanV.NbPlan,sizeof(int));
                for (i=0; i<ListePlanV.NbPlan; i++)
                {
                    fgets(CurrentLine,500,Fich);
                    sscanf(CurrentLine,"%c %i",&ListePlanV.TypePlan[i],&ListePlanV.CotePlan[i]);
                }
            }
        }
    }

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of 2D V1 Snapshots") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ListePlanVx.NbPlan);
            if (ListePlanVx.NbPlan)
            {
                ListePlanVx.TypePlan = (char *) calloc (ListePlanVx.NbPlan,sizeof(char));
                ListePlanVx.CotePlan = (int *) calloc (ListePlanVx.NbPlan,sizeof(int));
                for (i=0; i<ListePlanVx.NbPlan; i++)
                {
                    fgets(CurrentLine,500,Fich);
                    sscanf(CurrentLine,"%c %i",&ListePlanVx.TypePlan[i],&ListePlanVx.CotePlan[i]);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of 2D V2 Snapshots") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ListePlanVy.NbPlan);
            if (ListePlanVy.NbPlan)
            {
                ListePlanVy.TypePlan = (char *) calloc (ListePlanVy.NbPlan,sizeof(char));
                ListePlanVy.CotePlan = (int *) calloc (ListePlanVy.NbPlan,sizeof(int));
                for (i=0; i<ListePlanVy.NbPlan; i++)
                {
                    fgets(CurrentLine,500,Fich);
                    sscanf(CurrentLine,"%c %i",&ListePlanVy.TypePlan[i],&ListePlanVy.CotePlan[i]);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of 2D V3 Snapshots") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ListePlanVz.NbPlan);
            if (ListePlanVz.NbPlan)
            {
                ListePlanVz.TypePlan = (char *) calloc (ListePlanVz.NbPlan,sizeof(char));
                ListePlanVz.CotePlan = (int *) calloc (ListePlanVz.NbPlan,sizeof(int));
                for (i=0; i<ListePlanVz.NbPlan; i++)
                {
                    fgets(CurrentLine,500,Fich);
                    sscanf(CurrentLine,"%c %i",&ListePlanVz.TypePlan[i],&ListePlanVz.CotePlan[i]);
                }
            }
        }
    }

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of 2D Ekinetic Snapshots") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ListePlanEc.NbPlan);
            if (ListePlanEc.NbPlan)
            {
                ListePlanEc.TypePlan = (char *) calloc (ListePlanEc.NbPlan,sizeof(char));
                ListePlanEc.CotePlan = (int *) calloc (ListePlanEc.NbPlan,sizeof(int));
                for (i=0; i<ListePlanEc.NbPlan; i++)
                {
                    fgets(CurrentLine,500,Fich);
                    sscanf(CurrentLine,"%c %i",&ListePlanEc.TypePlan[i],&ListePlanEc.CotePlan[i]);
                }
            }
        }
    }

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of 2D T11 Snapshots") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ListePlanTxx.NbPlan);
            if (ListePlanTxx.NbPlan)
            {
                ListePlanTxx.TypePlan = (char *) calloc (ListePlanTxx.NbPlan,sizeof(char));
                ListePlanTxx.CotePlan = (int *) calloc (ListePlanTxx.NbPlan,sizeof(int));
                for (i=0; i<ListePlanTxx.NbPlan; i++)
                {
                    fgets(CurrentLine,500,Fich);
                    sscanf(CurrentLine,"%c %i",&ListePlanTxx.TypePlan[i],&ListePlanTxx.CotePlan[i]);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of 2D T22 Snapshots") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ListePlanTyy.NbPlan);
            if (ListePlanTyy.NbPlan)
            {
                ListePlanTyy.TypePlan = (char *) calloc (ListePlanTyy.NbPlan,sizeof(char));
                ListePlanTyy.CotePlan = (int *) calloc (ListePlanTyy.NbPlan,sizeof(int));
                for (i=0; i<ListePlanTyy.NbPlan; i++)
                {
                    fgets(CurrentLine,500,Fich);
                    sscanf(CurrentLine,"%c %i",&ListePlanTyy.TypePlan[i],&ListePlanTyy.CotePlan[i]);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of 2D T33 Snapshots") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ListePlanTzz.NbPlan);
            if (ListePlanTzz.NbPlan)
            {
                ListePlanTzz.TypePlan = (char *) calloc (ListePlanTzz.NbPlan,sizeof(char));
                ListePlanTzz.CotePlan = (int *) calloc (ListePlanTzz.NbPlan,sizeof(int));
                for (i=0; i<ListePlanTzz.NbPlan; i++)
                {
                    fgets(CurrentLine,500,Fich);
                    sscanf(CurrentLine,"%c %i",&ListePlanTzz.TypePlan[i],&ListePlanTzz.CotePlan[i]);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of 2D T12 Snapshots") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ListePlanTxy.NbPlan);
            if (ListePlanTxy.NbPlan)
            {
                ListePlanTxy.TypePlan = (char *) calloc (ListePlanTxy.NbPlan,sizeof(char));
                ListePlanTxy.CotePlan = (int *) calloc (ListePlanTxy.NbPlan,sizeof(int));
                for (i=0; i<ListePlanTxy.NbPlan; i++)
                {
                    fgets(CurrentLine,500,Fich);
                    sscanf(CurrentLine,"%c %i",&ListePlanTxy.TypePlan[i],&ListePlanTxy.CotePlan[i]);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of 2D T23 Snapshots") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ListePlanTyz.NbPlan);
            if (ListePlanTyz.NbPlan)
            {
                ListePlanTyz.TypePlan = (char *) calloc (ListePlanTyz.NbPlan,sizeof(char));
                ListePlanTyz.CotePlan = (int *) calloc (ListePlanTyz.NbPlan,sizeof(int));
                for (i=0; i<ListePlanTyz.NbPlan; i++)
                {
                    fgets(CurrentLine,500,Fich);
                    sscanf(CurrentLine,"%c %i",&ListePlanTyz.TypePlan[i],&ListePlanTyz.CotePlan[i]);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of 2D T31 Snapshots") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ListePlanTzx.NbPlan);
            if (ListePlanTzx.NbPlan)
            {
                ListePlanTzx.TypePlan = (char *) calloc (ListePlanTzx.NbPlan,sizeof(char));
                ListePlanTzx.CotePlan = (int *) calloc (ListePlanTzx.NbPlan,sizeof(int));
                for (i=0; i<ListePlanTzx.NbPlan; i++)
                {
                    fgets(CurrentLine,500,Fich);
                    sscanf(CurrentLine,"%c %i",&ListePlanTzx.TypePlan[i],&ListePlanTzx.CotePlan[i]);
                }
            }
        }
    }




    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of recorded V sections") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ListePlanV.NbPlan);
            if (ListePlanV.NbPlan)
            {
                ListePlanV.TypePlan = (char *) calloc (ListePlanV.NbPlan,sizeof(char));
                ListePlanV.CotePlan = (int *) calloc (ListePlanV.NbPlan,sizeof(int));
                for (i=0; i<ListePlanV.NbPlan; i++)
                {
                    fgets(CurrentLine,500,Fich);
                    sscanf(CurrentLine,"%c %i",&ListePlanV.TypePlan[i],&ListePlanV.CotePlan[i]);
                }
            }
        }
    }

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of recorded Vx sections") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ListePlanVx.NbPlan);
            if (ListePlanVx.NbPlan)
            {
                ListePlanVx.TypePlan = (char *) calloc (ListePlanVx.NbPlan,sizeof(char));
                ListePlanVx.CotePlan = (int *) calloc (ListePlanVx.NbPlan,sizeof(int));
                for (i=0; i<ListePlanVx.NbPlan; i++)
                {
                    fgets(CurrentLine,500,Fich);
                    sscanf(CurrentLine,"%c %i",&ListePlanVx.TypePlan[i],&ListePlanVx.CotePlan[i]);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of recorded Vy sections") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ListePlanVy.NbPlan);
            if (ListePlanVy.NbPlan)
            {
                ListePlanVy.TypePlan = (char *) calloc (ListePlanVy.NbPlan,sizeof(char));
                ListePlanVy.CotePlan = (int *) calloc (ListePlanVy.NbPlan,sizeof(int));
                for (i=0; i<ListePlanVy.NbPlan; i++)
                {
                    fgets(CurrentLine,500,Fich);
                    sscanf(CurrentLine,"%c %i",&ListePlanVy.TypePlan[i],&ListePlanVy.CotePlan[i]);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of recorded Vz sections") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ListePlanVz.NbPlan);
            if (ListePlanVz.NbPlan)
            {
                ListePlanVz.TypePlan = (char *) calloc (ListePlanVz.NbPlan,sizeof(char));
                ListePlanVz.CotePlan = (int *) calloc (ListePlanVz.NbPlan,sizeof(int));
                for (i=0; i<ListePlanVz.NbPlan; i++)
                {
                    fgets(CurrentLine,500,Fich);
                    sscanf(CurrentLine,"%c %i",&ListePlanVz.TypePlan[i],&ListePlanVz.CotePlan[i]);
                }
            }
        }
    }

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of recorded Ec sections") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ListePlanEc.NbPlan);
            if (ListePlanEc.NbPlan)
            {
                ListePlanEc.TypePlan = (char *) calloc (ListePlanEc.NbPlan,sizeof(char));
                ListePlanEc.CotePlan = (int *) calloc (ListePlanEc.NbPlan,sizeof(int));
                for (i=0; i<ListePlanEc.NbPlan; i++)
                {
                    fgets(CurrentLine,500,Fich);
                    sscanf(CurrentLine,"%c %i",&ListePlanEc.TypePlan[i],&ListePlanEc.CotePlan[i]);
                }
            }
        }
    }

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of recorded Txx sections") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ListePlanTxx.NbPlan);
            if (ListePlanTxx.NbPlan)
            {
                ListePlanTxx.TypePlan = (char *) calloc (ListePlanTxx.NbPlan,sizeof(char));
                ListePlanTxx.CotePlan = (int *) calloc (ListePlanTxx.NbPlan,sizeof(int));
                for (i=0; i<ListePlanTxx.NbPlan; i++)
                {
                    fgets(CurrentLine,500,Fich);
                    sscanf(CurrentLine,"%c %i",&ListePlanTxx.TypePlan[i],&ListePlanTxx.CotePlan[i]);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of recorded Tyy sections") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ListePlanTyy.NbPlan);
            if (ListePlanTyy.NbPlan)
            {
                ListePlanTyy.TypePlan = (char *) calloc (ListePlanTyy.NbPlan,sizeof(char));
                ListePlanTyy.CotePlan = (int *) calloc (ListePlanTyy.NbPlan,sizeof(int));
                for (i=0; i<ListePlanTyy.NbPlan; i++)
                {
                    fgets(CurrentLine,500,Fich);
                    sscanf(CurrentLine,"%c %i",&ListePlanTyy.TypePlan[i],&ListePlanTyy.CotePlan[i]);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of recorded Tzz sections") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ListePlanTzz.NbPlan);
            if (ListePlanTzz.NbPlan)
            {
                ListePlanTzz.TypePlan = (char *) calloc (ListePlanTzz.NbPlan,sizeof(char));
                ListePlanTzz.CotePlan = (int *) calloc (ListePlanTzz.NbPlan,sizeof(int));
                for (i=0; i<ListePlanTzz.NbPlan; i++)
                {
                    fgets(CurrentLine,500,Fich);
                    sscanf(CurrentLine,"%c %i",&ListePlanTzz.TypePlan[i],&ListePlanTzz.CotePlan[i]);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of recorded Txy sections") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ListePlanTxy.NbPlan);
            if (ListePlanTxy.NbPlan)
            {
                ListePlanTxy.TypePlan = (char *) calloc (ListePlanTxy.NbPlan,sizeof(char));
                ListePlanTxy.CotePlan = (int *) calloc (ListePlanTxy.NbPlan,sizeof(int));
                for (i=0; i<ListePlanTxy.NbPlan; i++)
                {
                    fgets(CurrentLine,500,Fich);
                    sscanf(CurrentLine,"%c %i",&ListePlanTxy.TypePlan[i],&ListePlanTxy.CotePlan[i]);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of recorded Tyz sections") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ListePlanTyz.NbPlan);
            if (ListePlanTyz.NbPlan)
            {
                ListePlanTyz.TypePlan = (char *) calloc (ListePlanTyz.NbPlan,sizeof(char));
                ListePlanTyz.CotePlan = (int *) calloc (ListePlanTyz.NbPlan,sizeof(int));
                for (i=0; i<ListePlanTyz.NbPlan; i++)
                {
                    fgets(CurrentLine,500,Fich);
                    sscanf(CurrentLine,"%c %i",&ListePlanTyz.TypePlan[i],&ListePlanTyz.CotePlan[i]);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of recorded Tzx sections") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&ListePlanTzx.NbPlan);
            if (ListePlanTzx.NbPlan)
            {
                ListePlanTzx.TypePlan = (char *) calloc (ListePlanTzx.NbPlan,sizeof(char));
                ListePlanTzx.CotePlan = (int *) calloc (ListePlanTzx.NbPlan,sizeof(int));
                for (i=0; i<ListePlanTzx.NbPlan; i++)
                {
                    fgets(CurrentLine,500,Fich);
                    sscanf(CurrentLine,"%c %i",&ListePlanTzx.TypePlan[i],&ListePlanTzx.CotePlan[i]);
                }
            }
        }
    }





    /*****************************************************************/
    /**********Lecture des parametres de matrice d'EMISSION *********/
    /*****************************************************************/

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Type of Source") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&SourceType);
        }
    }


    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of T11 Emitters Arrays") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NbMatricesEmissionTxx);
            if (NbMatricesEmissionTxx)
                EmissionTxx = (struct MATRICE_EMISSION*) calloc(NbMatricesEmissionTxx,sizeof(struct MATRICE_EMISSION));
            for(i=0; i<NbMatricesEmissionTxx; i++)
            {
                MatriceCouranteEmission=&EmissionTxx[i];
                LectureParametresMatriceEmission(Fich, CurrentLine,MatriceCouranteEmission);
                Test=ValiditeMatriceEmission(MatriceCouranteEmission,X,Y,Z);
                if (!Test)
                {
                    printf("Pb on emission matrix Txx number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of T22 Emitters Arrays") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NbMatricesEmissionTyy);
            if (NbMatricesEmissionTyy)
                EmissionTyy = (struct MATRICE_EMISSION*) calloc(NbMatricesEmissionTyy,sizeof(struct MATRICE_EMISSION));
            for(i=0; i<NbMatricesEmissionTyy; i++)
            {
                MatriceCouranteEmission=&EmissionTyy[i];
                LectureParametresMatriceEmission(Fich, CurrentLine,MatriceCouranteEmission);
                Test=ValiditeMatriceEmission(MatriceCouranteEmission,X,Y,Z);
                if (!Test)
                {
                    printf("Pb on emission matrix Tyy number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of T33 Emitters Arrays") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NbMatricesEmissionTzz);
            if (NbMatricesEmissionTzz)
                EmissionTzz = (struct MATRICE_EMISSION*) calloc(NbMatricesEmissionTzz,sizeof(struct MATRICE_EMISSION));
            for(i=0; i<NbMatricesEmissionTzz; i++)
            {
                MatriceCouranteEmission=&EmissionTzz[i];
                LectureParametresMatriceEmission(Fich, CurrentLine,MatriceCouranteEmission);
                Test=ValiditeMatriceEmission(MatriceCouranteEmission,X,Y,Z);
                if (!Test)
                {
                    printf("Pb on emission matrix Tzz number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of T12 Emitters Arrays") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NbMatricesEmissionTxy);
            if (NbMatricesEmissionTxy)
                EmissionTxy = (struct MATRICE_EMISSION*) calloc(NbMatricesEmissionTxy,sizeof(struct MATRICE_EMISSION));
            for(i=0; i<NbMatricesEmissionTxy; i++)
            {
                MatriceCouranteEmission=&EmissionTxy[i];
                LectureParametresMatriceEmission(Fich, CurrentLine,MatriceCouranteEmission);
                Test=ValiditeMatriceEmission(MatriceCouranteEmission,X,Y,Z);
                if (!Test)
                {
                    printf("Pb on emission matrix Txy number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of T23 Emitters Arrays") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NbMatricesEmissionTyz);
            if (NbMatricesEmissionTyz)
                EmissionTyz = (struct MATRICE_EMISSION*) calloc(NbMatricesEmissionTyz,sizeof(struct MATRICE_EMISSION));
            for(i=0; i<NbMatricesEmissionTyz; i++)
            {
                MatriceCouranteEmission=&EmissionTyz[i];
                LectureParametresMatriceEmission(Fich, CurrentLine,MatriceCouranteEmission);
                Test=ValiditeMatriceEmission(MatriceCouranteEmission,X,Y,Z);
                if (!Test)
                {
                    printf("Pb on emission matrix Tyz number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of T31 Emitters Arrays") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NbMatricesEmissionTzx);
            if (NbMatricesEmissionTzx)
                EmissionTzx = (struct MATRICE_EMISSION*) calloc(NbMatricesEmissionTzx,sizeof(struct MATRICE_EMISSION));
            for(i=0; i<NbMatricesEmissionTzx; i++)
            {
                MatriceCouranteEmission=&EmissionTzx[i];
                LectureParametresMatriceEmission(Fich, CurrentLine,MatriceCouranteEmission);
                Test=ValiditeMatriceEmission(MatriceCouranteEmission,X,Y,Z);
                if (!Test)
                {
                    printf("Pb on emission matrix Tzx number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of V1 Emitters Arrays") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NbMatricesEmissionVx);
            if (NbMatricesEmissionVx)
                EmissionVx = (struct MATRICE_EMISSION*) calloc(NbMatricesEmissionVx,sizeof(struct MATRICE_EMISSION));
            for(i=0; i<NbMatricesEmissionVx; i++)
            {
                MatriceCouranteEmission=&EmissionVx[i];
                LectureParametresMatriceEmission(Fich, CurrentLine,MatriceCouranteEmission);
                Test=ValiditeMatriceEmission(MatriceCouranteEmission,X,Y,Z);
                if (!Test)
                {
                    printf("Pb on emission matrix Vx number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of V2 Emitters Arrays") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NbMatricesEmissionVy);
            if (NbMatricesEmissionVy)
                EmissionVy = (struct MATRICE_EMISSION*) calloc(NbMatricesEmissionVy,sizeof(struct MATRICE_EMISSION));
            for(i=0; i<NbMatricesEmissionVy; i++)
            {
                MatriceCouranteEmission=&EmissionVy[i];
                LectureParametresMatriceEmission(Fich, CurrentLine,MatriceCouranteEmission);
                Test=ValiditeMatriceEmission(MatriceCouranteEmission,X,Y,Z);
                if (!Test)
                {
                    printf("Pb on emission matrix Vy number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of V3 Emitters Arrays") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NbMatricesEmissionVz);
            if (NbMatricesEmissionVz)
                EmissionVz = (struct MATRICE_EMISSION*) calloc(NbMatricesEmissionVz,sizeof(struct MATRICE_EMISSION));
            for(i=0; i<NbMatricesEmissionVz; i++)
            {
                MatriceCouranteEmission=&EmissionVz[i];
                LectureParametresMatriceEmission(Fich, CurrentLine,MatriceCouranteEmission);
                Test=ValiditeMatriceEmission(MatriceCouranteEmission,X,Y,Z);
                if (!Test)
                {
                    printf("Pb on emission matrix Vz number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
            }
        }
    }


    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of Txx Emitting Array") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NbMatricesEmissionTxx);
            if (NbMatricesEmissionTxx)
                EmissionTxx = (struct MATRICE_EMISSION*) calloc(NbMatricesEmissionTxx,sizeof(struct MATRICE_EMISSION));
            for(i=0; i<NbMatricesEmissionTxx; i++)
            {
                MatriceCouranteEmission=&EmissionTxx[i];
                LectureParametresMatriceEmission(Fich, CurrentLine,MatriceCouranteEmission);
                Test=ValiditeMatriceEmission(MatriceCouranteEmission,X,Y,Z);
                if (!Test)
                {
                    printf("Pb on emission matrix Txx number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of Tyy Emitting Array") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NbMatricesEmissionTyy);
            if (NbMatricesEmissionTyy)
                EmissionTyy = (struct MATRICE_EMISSION*) calloc(NbMatricesEmissionTyy,sizeof(struct MATRICE_EMISSION));
            for(i=0; i<NbMatricesEmissionTyy; i++)
            {
                MatriceCouranteEmission=&EmissionTyy[i];
                LectureParametresMatriceEmission(Fich, CurrentLine,MatriceCouranteEmission);
                Test=ValiditeMatriceEmission(MatriceCouranteEmission,X,Y,Z);
                if (!Test)
                {
                    printf("Pb on emission matrix Tyy number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of Tzz Emitting Array") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NbMatricesEmissionTzz);
            if (NbMatricesEmissionTzz)
                EmissionTzz = (struct MATRICE_EMISSION*) calloc(NbMatricesEmissionTzz,sizeof(struct MATRICE_EMISSION));
            for(i=0; i<NbMatricesEmissionTzz; i++)
            {
                MatriceCouranteEmission=&EmissionTzz[i];
                LectureParametresMatriceEmission(Fich, CurrentLine,MatriceCouranteEmission);
                Test=ValiditeMatriceEmission(MatriceCouranteEmission,X,Y,Z);
                if (!Test)
                {
                    printf("Pb on emission matrix Tzz number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of Txy Emitting Array") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NbMatricesEmissionTxy);
            if (NbMatricesEmissionTxy)
                EmissionTxy = (struct MATRICE_EMISSION*) calloc(NbMatricesEmissionTxy,sizeof(struct MATRICE_EMISSION));
            for(i=0; i<NbMatricesEmissionTxy; i++)
            {
                MatriceCouranteEmission=&EmissionTxy[i];
                LectureParametresMatriceEmission(Fich, CurrentLine,MatriceCouranteEmission);
                Test=ValiditeMatriceEmission(MatriceCouranteEmission,X,Y,Z);
                if (!Test)
                {
                    printf("Pb on emission matrix Txy number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of Tyz Emitting Array") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NbMatricesEmissionTyz);
            if (NbMatricesEmissionTyz)
                EmissionTyz = (struct MATRICE_EMISSION*) calloc(NbMatricesEmissionTyz,sizeof(struct MATRICE_EMISSION));
            for(i=0; i<NbMatricesEmissionTyz; i++)
            {
                MatriceCouranteEmission=&EmissionTyz[i];
                LectureParametresMatriceEmission(Fich, CurrentLine,MatriceCouranteEmission);
                Test=ValiditeMatriceEmission(MatriceCouranteEmission,X,Y,Z);
                if (!Test)
                {
                    printf("Pb on emission matrix Tyz number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of Tzx Emitting Array") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NbMatricesEmissionTzx);
            if (NbMatricesEmissionTzx)
                EmissionTzx = (struct MATRICE_EMISSION*) calloc(NbMatricesEmissionTzx,sizeof(struct MATRICE_EMISSION));
            for(i=0; i<NbMatricesEmissionTzx; i++)
            {
                MatriceCouranteEmission=&EmissionTzx[i];
                LectureParametresMatriceEmission(Fich, CurrentLine,MatriceCouranteEmission);
                Test=ValiditeMatriceEmission(MatriceCouranteEmission,X,Y,Z);
                if (!Test)
                {
                    printf("Pb on emission matrix Tzx number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of Vx Emitting Array") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NbMatricesEmissionVx);
            if (NbMatricesEmissionVx)
                EmissionVx = (struct MATRICE_EMISSION*) calloc(NbMatricesEmissionVx,sizeof(struct MATRICE_EMISSION));
            for(i=0; i<NbMatricesEmissionVx; i++)
            {
                MatriceCouranteEmission=&EmissionVx[i];
                LectureParametresMatriceEmission(Fich, CurrentLine,MatriceCouranteEmission);
                Test=ValiditeMatriceEmission(MatriceCouranteEmission,X,Y,Z);
                if (!Test)
                {
                    printf("Pb on emission matrix Vx number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of Vy Emitting Array") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NbMatricesEmissionVy);
            if (NbMatricesEmissionVy)
                EmissionVy = (struct MATRICE_EMISSION*) calloc(NbMatricesEmissionVy,sizeof(struct MATRICE_EMISSION));
            for(i=0; i<NbMatricesEmissionVy; i++)
            {
                MatriceCouranteEmission=&EmissionVy[i];
                LectureParametresMatriceEmission(Fich, CurrentLine,MatriceCouranteEmission);
                Test=ValiditeMatriceEmission(MatriceCouranteEmission,X,Y,Z);
                if (!Test)
                {
                    printf("Pb on emission matrix Vy number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of Vz Emitting Array") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NbMatricesEmissionVz);
            if (NbMatricesEmissionVz)
                EmissionVz = (struct MATRICE_EMISSION*) calloc(NbMatricesEmissionVz,sizeof(struct MATRICE_EMISSION));
            for(i=0; i<NbMatricesEmissionVz; i++)
            {
                MatriceCouranteEmission=&EmissionVz[i];
                LectureParametresMatriceEmission(Fich, CurrentLine,MatriceCouranteEmission);
                Test=ValiditeMatriceEmission(MatriceCouranteEmission,X,Y,Z);
                if (!Test)
                {
                    printf("Pb on emission matrix Vz number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
            }
        }
    }

    /*****************************************************************/
    /**********Lecture des fichiers source de barrette d'EMISSION *********/
    /*****************************************************************/

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of T11 Array Source Files") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NbMatricesSourceFilesTxx);
            if(NbMatricesSourceFilesTxx)
                MatricesSourceFilesTxx=(struct MATRICE_RECEPTION *) calloc(NbMatricesSourceFilesTxx,sizeof(struct MATRICE_RECEPTION));
            for(i=0; i < NbMatricesSourceFilesTxx; i++)
            {
                MatriceCouranteReception=&MatricesSourceFilesTxx[i];
                LectureParametresMatriceSourceFile(Fich,CurrentLine,MatriceCouranteReception);
                Test=ValiditeMatriceReception(MatriceCouranteReception,X,Y,Z);
                if(!Test)
                {
                    printf("Pb on emission source file T11 number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of T22 Array Source Files") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NbMatricesSourceFilesTyy);
            if(NbMatricesSourceFilesTyy)
                MatricesSourceFilesTyy=(struct MATRICE_RECEPTION *) calloc(NbMatricesSourceFilesTyy,sizeof(struct MATRICE_RECEPTION));
            for(i=0; i < NbMatricesSourceFilesTyy; i++)
            {
                MatriceCouranteReception=&MatricesSourceFilesTyy[i];
                LectureParametresMatriceSourceFile(Fich,CurrentLine,MatriceCouranteReception);
                Test=ValiditeMatriceReception(MatriceCouranteReception,X,Y,Z);
                if(!Test)
                {
                    printf("Pb on emission source file T22 number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
            }
        }
    }

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of T33 Array Source Files") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NbMatricesSourceFilesTzz);
            if(NbMatricesSourceFilesTzz)
                MatricesSourceFilesTzz=(struct MATRICE_RECEPTION *) calloc(NbMatricesSourceFilesTzz,sizeof(struct MATRICE_RECEPTION));
            for(i=0; i < NbMatricesSourceFilesTzz; i++)
            {
                MatriceCouranteReception=&MatricesSourceFilesTzz[i];
                LectureParametresMatriceSourceFile(Fich,CurrentLine,MatriceCouranteReception);
                Test=ValiditeMatriceReception(MatriceCouranteReception,X,Y,Z);
                if(!Test)
                {
                    printf("Pb on emission source file T33 number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
            }
        }
    }

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of T12 Array Source Files") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NbMatricesSourceFilesTxy);
            if(NbMatricesSourceFilesTxy)
                MatricesSourceFilesTxy=(struct MATRICE_RECEPTION *) calloc(NbMatricesSourceFilesTxy,sizeof(struct MATRICE_RECEPTION));
            for(i=0; i < NbMatricesSourceFilesTxy; i++)
            {
                MatriceCouranteReception=&MatricesSourceFilesTxy[i];
                LectureParametresMatriceSourceFile(Fich,CurrentLine,MatriceCouranteReception);
                Test=ValiditeMatriceReception(MatriceCouranteReception,X+1,Y+1,Z);
                if(!Test)
                {
                    printf("Pb on emission source file T12 number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
            }
        }
    }

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of T23 Array Source Files") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NbMatricesSourceFilesTyz);
            if(NbMatricesSourceFilesTyz)
                MatricesSourceFilesTyz=(struct MATRICE_RECEPTION *) calloc(NbMatricesSourceFilesTyz,sizeof(struct MATRICE_RECEPTION));
            for(i=0; i < NbMatricesSourceFilesTyz; i++)
            {
                MatriceCouranteReception=&MatricesSourceFilesTyz[i];
                LectureParametresMatriceSourceFile(Fich,CurrentLine,MatriceCouranteReception);
                Test=ValiditeMatriceReception(MatriceCouranteReception,X,Y+1,Z+1);
                if(!Test)
                {
                    printf("Pb on emission source file T23 number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
            }
        }
    }


    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of T31 Array Source Files") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NbMatricesSourceFilesTzx);
            if(NbMatricesSourceFilesTzx)
                MatricesSourceFilesTzx=(struct MATRICE_RECEPTION *) calloc(NbMatricesSourceFilesTzx,sizeof(struct MATRICE_RECEPTION));
            for(i=0; i < NbMatricesSourceFilesTzx; i++)
            {
                MatriceCouranteReception=&MatricesSourceFilesTzx[i];
                LectureParametresMatriceSourceFile(Fich,CurrentLine,MatriceCouranteReception);
                Test=ValiditeMatriceReception(MatriceCouranteReception,X+1,Y,Z+1);
                if(!Test)
                {
                    printf("Pb on emission source file T31 number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
            }
        }
    }

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of V1 Array Source Files") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NbMatricesSourceFilesVx);
            if(NbMatricesSourceFilesVx)
                MatricesSourceFilesVx=(struct MATRICE_RECEPTION *) calloc(NbMatricesSourceFilesVx,sizeof(struct MATRICE_RECEPTION));
            for(i=0; i < NbMatricesSourceFilesVx; i++)
            {
                MatriceCouranteReception=&MatricesSourceFilesVx[i];
                LectureParametresMatriceSourceFile(Fich,CurrentLine,MatriceCouranteReception);
                Test=ValiditeMatriceReception(MatriceCouranteReception,X+1,Y,Z);
                if(!Test)
                {
                    printf("Pb on emission source file V1 number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of V2 Array Source Files") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NbMatricesSourceFilesVy);
            if(NbMatricesSourceFilesVy)
                MatricesSourceFilesVy=(struct MATRICE_RECEPTION *) calloc(NbMatricesSourceFilesVy,sizeof(struct MATRICE_RECEPTION));
            for(i=0; i < NbMatricesSourceFilesVy; i++)
            {
                MatriceCouranteReception=&MatricesSourceFilesVy[i];
                LectureParametresMatriceSourceFile(Fich,CurrentLine,MatriceCouranteReception);
                Test=ValiditeMatriceReception(MatriceCouranteReception,X,Y+1,Z);
                if(!Test)
                {
                    printf("Pb on emission source file V2 number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of V3 Array Source Files") !=NULL)
        {
            sscanf(CurrentLine+31,"%i",&NbMatricesSourceFilesVz);
            if(NbMatricesSourceFilesVz)
                MatricesSourceFilesVz=(struct MATRICE_RECEPTION *) calloc(NbMatricesSourceFilesVz,sizeof(struct MATRICE_RECEPTION));
            for(i=0; i < NbMatricesSourceFilesVz; i++)
            {
                MatriceCouranteReception=&MatricesSourceFilesVz[i];
                LectureParametresMatriceSourceFile(Fich,CurrentLine,MatriceCouranteReception);
                Test=ValiditeMatriceReception(MatriceCouranteReception,X,Y,Z+1);
                if(!Test)
                {
                    printf("Pb on emission source file V3 number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
            }
        }
    }
    /*****************************************************************/
    /**********Lecture des parametres de matrice de RECEPTION *******/
    /*****************************************************************/

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of T11 Receivers Arrays") !=NULL)
        {

            sscanf(CurrentLine+31,"%i",&NbMatricesReceptionTxx);
            if (NbMatricesReceptionTxx)
                ReceptionTxx = (struct MATRICE_RECEPTION*) calloc(NbMatricesReceptionTxx,sizeof(struct MATRICE_RECEPTION));
            for(i=0; i<NbMatricesReceptionTxx; i++)
            {
                MatriceCouranteReception=&ReceptionTxx[i];
                LectureParametresMatriceReception(Fich, CurrentLine,MatriceCouranteReception);
                Test=ValiditeMatriceReception(MatriceCouranteReception,X,Y,Z);
                if (!Test)
                {
                    printf("Pb on receiving matrix T11 number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
                NbPtsAlloues=(1+(int)((IndiceTempsMax-1)/RcvDwnSmplng))*(MatriceCouranteReception->NbElementsJ)*(MatriceCouranteReception->NbElementsK);
                MatriceCouranteReception->Signaux = (double *) calloc (NbPtsAlloues,sizeof(double));
            }
        }
    }

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of T22 Receivers Arrays") !=NULL)
        {

            sscanf(CurrentLine+31,"%i",&NbMatricesReceptionTyy);
            if (NbMatricesReceptionTyy)
                ReceptionTyy = (struct MATRICE_RECEPTION*) calloc(NbMatricesReceptionTyy,sizeof(struct MATRICE_RECEPTION));
            for(i=0; i<NbMatricesReceptionTyy; i++)
            {
                MatriceCouranteReception=&ReceptionTyy[i];
                LectureParametresMatriceReception(Fich, CurrentLine,MatriceCouranteReception);
                Test=ValiditeMatriceReception(MatriceCouranteReception,X,Y,Z);
                if (!Test)
                {
                    printf("Pb on receiving matrix T22 number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
                NbPtsAlloues=(1+(int)((IndiceTempsMax-1)/RcvDwnSmplng))*(MatriceCouranteReception->NbElementsJ)*(MatriceCouranteReception->NbElementsK);
                MatriceCouranteReception->Signaux = (double *) calloc (NbPtsAlloues,sizeof(double));
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of T33 Receivers Arrays") !=NULL)
        {

            sscanf(CurrentLine+31,"%i",&NbMatricesReceptionTzz);
            if (NbMatricesReceptionTzz)
                ReceptionTzz = (struct MATRICE_RECEPTION*) calloc(NbMatricesReceptionTzz,sizeof(struct MATRICE_RECEPTION));
            for(i=0; i<NbMatricesReceptionTzz; i++)
            {
                MatriceCouranteReception=&ReceptionTzz[i];
                LectureParametresMatriceReception(Fich, CurrentLine,MatriceCouranteReception);
                Test=ValiditeMatriceReception(MatriceCouranteReception,X,Y,Z);
                if (!Test)
                {
                    printf("Pb on receiving matrix T33 number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
                NbPtsAlloues=(1+(int)((IndiceTempsMax-1)/RcvDwnSmplng))*(MatriceCouranteReception->NbElementsJ)*(MatriceCouranteReception->NbElementsK);
                MatriceCouranteReception->Signaux = (double *) calloc (NbPtsAlloues,sizeof(double));
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of T12 Receivers Arrays") !=NULL)
        {

            sscanf(CurrentLine+31,"%i",&NbMatricesReceptionTxy);
            if (NbMatricesReceptionTxy)
                ReceptionTxy = (struct MATRICE_RECEPTION*) calloc(NbMatricesReceptionTxy,sizeof(struct MATRICE_RECEPTION));
            for(i=0; i<NbMatricesReceptionTxy; i++)
            {
                MatriceCouranteReception=&ReceptionTxy[i];
                LectureParametresMatriceReception(Fich, CurrentLine,MatriceCouranteReception);
                Test=ValiditeMatriceReception(MatriceCouranteReception,X+1,Y+1,Z);
                if (!Test)
                {
                    printf("Pb on receiving matrix T12 number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
                NbPtsAlloues=(1+(int)((IndiceTempsMax-1)/RcvDwnSmplng))*(MatriceCouranteReception->NbElementsJ)*(MatriceCouranteReception->NbElementsK);
                MatriceCouranteReception->Signaux = (double *) calloc (NbPtsAlloues,sizeof(double));
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of T23 Receivers Arrays") !=NULL)
        {

            sscanf(CurrentLine+31,"%i",&NbMatricesReceptionTyz);
            if (NbMatricesReceptionTyz)
                ReceptionTyz = (struct MATRICE_RECEPTION*) calloc(NbMatricesReceptionTyz,sizeof(struct MATRICE_RECEPTION));
            for(i=0; i<NbMatricesReceptionTyz; i++)
            {
                MatriceCouranteReception=&ReceptionTyz[i];
                LectureParametresMatriceReception(Fich, CurrentLine,MatriceCouranteReception);
                Test=ValiditeMatriceReception(MatriceCouranteReception,X,Y+1,Z+1);
                if (!Test)
                {
                    printf("Pb on receiving matrix T23 number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
                NbPtsAlloues=(1+(int)((IndiceTempsMax-1)/RcvDwnSmplng))*(MatriceCouranteReception->NbElementsJ)*(MatriceCouranteReception->NbElementsK);
                MatriceCouranteReception->Signaux = (double *) calloc (NbPtsAlloues,sizeof(double));
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of T31 Receivers Arrays") !=NULL)
        {

            sscanf(CurrentLine+31,"%i",&NbMatricesReceptionTzx);
            if (NbMatricesReceptionTzx)
                ReceptionTzx = (struct MATRICE_RECEPTION*) calloc(NbMatricesReceptionTzx,sizeof(struct MATRICE_RECEPTION));
            for(i=0; i<NbMatricesReceptionTzx; i++)
            {
                MatriceCouranteReception=&ReceptionTzx[i];
                LectureParametresMatriceReception(Fich, CurrentLine,MatriceCouranteReception);
                Test=ValiditeMatriceReception(MatriceCouranteReception,X+1,Y,Z+1);
                if (!Test)
                {
                    printf("Pb on receiving matrix T31 number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
                NbPtsAlloues=(1+(int)((IndiceTempsMax-1)/RcvDwnSmplng))*(MatriceCouranteReception->NbElementsJ)*(MatriceCouranteReception->NbElementsK);
                MatriceCouranteReception->Signaux = (double *) calloc (NbPtsAlloues,sizeof(double));
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of V1 Receivers Arrays") !=NULL)
        {

            sscanf(CurrentLine+31,"%i",&NbMatricesReceptionVx);
            if (NbMatricesReceptionVx)
                ReceptionVx = (struct MATRICE_RECEPTION*) calloc(NbMatricesReceptionVx,sizeof(struct MATRICE_RECEPTION));
            for(i=0; i<NbMatricesReceptionVx; i++)
            {
                MatriceCouranteReception=&ReceptionVx[i];
                LectureParametresMatriceReception(Fich, CurrentLine,MatriceCouranteReception);
                Test=ValiditeMatriceReception(MatriceCouranteReception,X+1,Y,Z);
                if (!Test)
                {
                    printf("Pb on receiving matrix V1 number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
                NbPtsAlloues=(1+(int)((IndiceTempsMax-1)/RcvDwnSmplng))*(MatriceCouranteReception->NbElementsJ)*(MatriceCouranteReception->NbElementsK);
                MatriceCouranteReception->Signaux = (double *) calloc (NbPtsAlloues,sizeof(double));
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of V2 Receivers Arrays") !=NULL)
        {

            sscanf(CurrentLine+31,"%i",&NbMatricesReceptionVy);
            if (NbMatricesReceptionVy)
                ReceptionVy = (struct MATRICE_RECEPTION*) calloc(NbMatricesReceptionVy,sizeof(struct MATRICE_RECEPTION));
            for(i=0; i<NbMatricesReceptionVy; i++)
            {
                MatriceCouranteReception=&ReceptionVy[i];
                LectureParametresMatriceReception(Fich, CurrentLine,MatriceCouranteReception);
                Test=ValiditeMatriceReception(MatriceCouranteReception,X,Y+1,Z);
                if (!Test)
                {
                    printf("Pb on receiving matrix V2 number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
                NbPtsAlloues=(1+(int)((IndiceTempsMax-1)/RcvDwnSmplng))*(MatriceCouranteReception->NbElementsJ)*(MatriceCouranteReception->NbElementsK);
                MatriceCouranteReception->Signaux = (double *) calloc (NbPtsAlloues,sizeof(double));
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of V3 Receivers Arrays") !=NULL)
        {

            sscanf(CurrentLine+31,"%i",&NbMatricesReceptionVz);
            if (NbMatricesReceptionVz)
                ReceptionVz = (struct MATRICE_RECEPTION*) calloc(NbMatricesReceptionVz,sizeof(struct MATRICE_RECEPTION));
            for(i=0; i<NbMatricesReceptionVz; i++)
            {
                MatriceCouranteReception=&ReceptionVz[i];
                LectureParametresMatriceReception(Fich, CurrentLine,MatriceCouranteReception);
                Test=ValiditeMatriceReception(MatriceCouranteReception,X,Y,Z+1);
                if (!Test)
                {
                    printf("Pb on receiving matrix V3 number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
                NbPtsAlloues=(1+(int)((IndiceTempsMax-1)/RcvDwnSmplng))*(MatriceCouranteReception->NbElementsJ)*(MatriceCouranteReception->NbElementsK);
                MatriceCouranteReception->Signaux = (double *) calloc (NbPtsAlloues,sizeof(double));
            }
        }
    }


    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of Txx Receiving Array") !=NULL)
        {

            sscanf(CurrentLine+31,"%i",&NbMatricesReceptionTxx);
            if (NbMatricesReceptionTxx)
                ReceptionTxx = (struct MATRICE_RECEPTION*) calloc(NbMatricesReceptionTxx,sizeof(struct MATRICE_RECEPTION));
            for(i=0; i<NbMatricesReceptionTxx; i++)
            {
                MatriceCouranteReception=&ReceptionTxx[i];
                LectureParametresMatriceReception(Fich, CurrentLine,MatriceCouranteReception);
                Test=ValiditeMatriceReception(MatriceCouranteReception,X,Y,Z);
                if (!Test)
                {
                    printf("Pb on receiving matrix Txx number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
                NbPtsAlloues=(1+(int)((IndiceTempsMax-1)/RcvDwnSmplng))*(MatriceCouranteReception->NbElementsJ)*(MatriceCouranteReception->NbElementsK);
                MatriceCouranteReception->Signaux = (double *) calloc (NbPtsAlloues,sizeof(double));
            }
        }
    }

    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of Tyy Receiving Array") !=NULL)
        {

            sscanf(CurrentLine+31,"%i",&NbMatricesReceptionTyy);
            if (NbMatricesReceptionTyy)
                ReceptionTyy = (struct MATRICE_RECEPTION*) calloc(NbMatricesReceptionTyy,sizeof(struct MATRICE_RECEPTION));
            for(i=0; i<NbMatricesReceptionTyy; i++)
            {
                MatriceCouranteReception=&ReceptionTyy[i];
                LectureParametresMatriceReception(Fich, CurrentLine,MatriceCouranteReception);
                Test=ValiditeMatriceReception(MatriceCouranteReception,X,Y,Z);
                if (!Test)
                {
                    printf("Pb on receiving matrix Tyy number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
                NbPtsAlloues=(1+(int)((IndiceTempsMax-1)/RcvDwnSmplng))*(MatriceCouranteReception->NbElementsJ)*(MatriceCouranteReception->NbElementsK);
                MatriceCouranteReception->Signaux = (double *) calloc (NbPtsAlloues,sizeof(double));
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of Tzz Receiving Array") !=NULL)
        {

            sscanf(CurrentLine+31,"%i",&NbMatricesReceptionTzz);
            if (NbMatricesReceptionTzz)
                ReceptionTzz = (struct MATRICE_RECEPTION*) calloc(NbMatricesReceptionTzz,sizeof(struct MATRICE_RECEPTION));
            for(i=0; i<NbMatricesReceptionTzz; i++)
            {
                MatriceCouranteReception=&ReceptionTzz[i];
                LectureParametresMatriceReception(Fich, CurrentLine,MatriceCouranteReception);
                Test=ValiditeMatriceReception(MatriceCouranteReception,X,Y,Z);
                if (!Test)
                {
                    printf("Pb on receiving matrix Tzz number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
                NbPtsAlloues=(1+(int)((IndiceTempsMax-1)/RcvDwnSmplng))*(MatriceCouranteReception->NbElementsJ)*(MatriceCouranteReception->NbElementsK);
                MatriceCouranteReception->Signaux = (double *) calloc (NbPtsAlloues,sizeof(double));
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of Txy Receiving Array") !=NULL)
        {

            sscanf(CurrentLine+31,"%i",&NbMatricesReceptionTxy);
            if (NbMatricesReceptionTxy)
                ReceptionTxy = (struct MATRICE_RECEPTION*) calloc(NbMatricesReceptionTxy,sizeof(struct MATRICE_RECEPTION));
            for(i=0; i<NbMatricesReceptionTxy; i++)
            {
                MatriceCouranteReception=&ReceptionTxy[i];
                LectureParametresMatriceReception(Fich, CurrentLine,MatriceCouranteReception);
                Test=ValiditeMatriceReception(MatriceCouranteReception,X+1,Y+1,Z);
                if (!Test)
                {
                    printf("Pb on receiving matrix Txy number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
                NbPtsAlloues=(1+(int)((IndiceTempsMax-1)/RcvDwnSmplng))*(MatriceCouranteReception->NbElementsJ)*(MatriceCouranteReception->NbElementsK);
                MatriceCouranteReception->Signaux = (double *) calloc (NbPtsAlloues,sizeof(double));
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of Tyz Receiving Array") !=NULL)
        {

            sscanf(CurrentLine+31,"%i",&NbMatricesReceptionTyz);
            if (NbMatricesReceptionTyz)
                ReceptionTyz = (struct MATRICE_RECEPTION*) calloc(NbMatricesReceptionTyz,sizeof(struct MATRICE_RECEPTION));
            for(i=0; i<NbMatricesReceptionTyz; i++)
            {
                MatriceCouranteReception=&ReceptionTyz[i];
                LectureParametresMatriceReception(Fich, CurrentLine,MatriceCouranteReception);
                Test=ValiditeMatriceReception(MatriceCouranteReception,X,Y+1,Z+1);
                if (!Test)
                {
                    printf("Pb on receiving matrix Tyz number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
                NbPtsAlloues=(1+(int)((IndiceTempsMax-1)/RcvDwnSmplng))*(MatriceCouranteReception->NbElementsJ)*(MatriceCouranteReception->NbElementsK);
                MatriceCouranteReception->Signaux = (double *) calloc (NbPtsAlloues,sizeof(double));
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of Tzx Receiving Array") !=NULL)
        {

            sscanf(CurrentLine+31,"%i",&NbMatricesReceptionTzx);
            if (NbMatricesReceptionTzx)
                ReceptionTzx = (struct MATRICE_RECEPTION*) calloc(NbMatricesReceptionTzx,sizeof(struct MATRICE_RECEPTION));
            for(i=0; i<NbMatricesReceptionTzx; i++)
            {
                MatriceCouranteReception=&ReceptionTzx[i];
                LectureParametresMatriceReception(Fich, CurrentLine,MatriceCouranteReception);
                Test=ValiditeMatriceReception(MatriceCouranteReception,X+1,Y,Z+1);
                if (!Test)
                {
                    printf("Pb on receiving matrix Tzx number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
                NbPtsAlloues=(1+(int)((IndiceTempsMax-1)/RcvDwnSmplng))*(MatriceCouranteReception->NbElementsJ)*(MatriceCouranteReception->NbElementsK);
                MatriceCouranteReception->Signaux = (double *) calloc (NbPtsAlloues,sizeof(double));
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of Vx Receiving Array") !=NULL)
        {

            sscanf(CurrentLine+31,"%i",&NbMatricesReceptionVx);
            if (NbMatricesReceptionVx)
                ReceptionVx = (struct MATRICE_RECEPTION*) calloc(NbMatricesReceptionVx,sizeof(struct MATRICE_RECEPTION));
            for(i=0; i<NbMatricesReceptionVx; i++)
            {
                MatriceCouranteReception=&ReceptionVx[i];
                LectureParametresMatriceReception(Fich, CurrentLine,MatriceCouranteReception);
                Test=ValiditeMatriceReception(MatriceCouranteReception,X+1,Y,Z);
                if (!Test)
                {
                    printf("Pb on receiving matrix Vx number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
                NbPtsAlloues=(1+(int)((IndiceTempsMax-1)/RcvDwnSmplng))*(MatriceCouranteReception->NbElementsJ)*(MatriceCouranteReception->NbElementsK);
                MatriceCouranteReception->Signaux = (double *) calloc (NbPtsAlloues,sizeof(double));
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of Vy Receiving Array") !=NULL)
        {

            sscanf(CurrentLine+31,"%i",&NbMatricesReceptionVy);
            if (NbMatricesReceptionVy)
                ReceptionVy = (struct MATRICE_RECEPTION*) calloc(NbMatricesReceptionVy,sizeof(struct MATRICE_RECEPTION));
            for(i=0; i<NbMatricesReceptionVy; i++)
            {
                MatriceCouranteReception=&ReceptionVy[i];
                LectureParametresMatriceReception(Fich, CurrentLine,MatriceCouranteReception);
                Test=ValiditeMatriceReception(MatriceCouranteReception,X,Y+1,Z);
                if (!Test)
                {
                    printf("Pb on receiving matrix Vy number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
                NbPtsAlloues=(1+(int)((IndiceTempsMax-1)/RcvDwnSmplng))*(MatriceCouranteReception->NbElementsJ)*(MatriceCouranteReception->NbElementsK);
                MatriceCouranteReception->Signaux = (double *) calloc (NbPtsAlloues,sizeof(double));
            }
        }
    }
    fseek(Fich,0L,SEEK_SET);
    while(fgets(CurrentLine,500,Fich) !=NULL)
    {
        if(strstr(CurrentLine,"Nb of Vz Receiving Array") !=NULL)
        {

            sscanf(CurrentLine+31,"%i",&NbMatricesReceptionVz);
            if (NbMatricesReceptionVz)
                ReceptionVz = (struct MATRICE_RECEPTION*) calloc(NbMatricesReceptionVz,sizeof(struct MATRICE_RECEPTION));
            for(i=0; i<NbMatricesReceptionVz; i++)
            {
                MatriceCouranteReception=&ReceptionVz[i];
                LectureParametresMatriceReception(Fich, CurrentLine,MatriceCouranteReception);
                Test=ValiditeMatriceReception(MatriceCouranteReception,X,Y,Z+1);
                if (!Test)
                {
                    printf("Pb on receiving matrix Vz number %i\n",i+1);
                    LiberationMemoire();
                    exit(1);
                }
                NbPtsAlloues=(1+(int)((IndiceTempsMax-1)/RcvDwnSmplng))*(MatriceCouranteReception->NbElementsJ)*(MatriceCouranteReception->NbElementsK);
                MatriceCouranteReception->Signaux = (double *) calloc (NbPtsAlloues,sizeof(double));
            }
        }
    }

    fclose( Fich );
}
void AllocationGrilles(void)
{

    int AireMax,AireIntermediaire;

    /***************************************************************************/
    /*********** Allocation des grilles du bloc principal **********************/
    /***************************************************************************/

    Vx = (PREC_GRILLE *) calloc((X+1)*(Y)*(Z),sizeof(PREC_GRILLE));
    Vy = (PREC_GRILLE *) calloc((X)*(Y+1)*(Z),sizeof(PREC_GRILLE));
    Vz = (PREC_GRILLE *) calloc((X)*(Y)*(Z+1),sizeof(PREC_GRILLE));
    Txx = (PREC_GRILLE *) calloc((X)*(Y)*(Z),sizeof(PREC_GRILLE));
    Tyy = (PREC_GRILLE *) calloc((X)*(Y)*(Z),sizeof(PREC_GRILLE));
    Tzz = (PREC_GRILLE *) calloc((X)*(Y)*(Z),sizeof(PREC_GRILLE));
    Txy = (PREC_GRILLE *) calloc((X+1)*(Y+1)*(Z),sizeof(PREC_GRILLE));
    Tyz = (PREC_GRILLE *) calloc((X)*(Y+1)*(Z+1),sizeof(PREC_GRILLE));
    Tzx = (PREC_GRILLE *) calloc((X+1)*(Y)*(Z+1),sizeof(PREC_GRILLE));

    if(SnapshotUx)
        Ux = (PREC_GRILLE *) calloc((X+1)*(Y)*(Z),sizeof(PREC_GRILLE));
    if(SnapshotUy)
        Uy = (PREC_GRILLE *) calloc((X)*(Y+1)*(Z),sizeof(PREC_GRILLE));
    if(SnapshotUz)
        Uz = (PREC_GRILLE *) calloc((X)*(Y)*(Z+1),sizeof(PREC_GRILLE));


    /***************************************************************************/
    /**************** Allocation des Faces PML *********************************/
    /***************************************************************************/


    if (ConditionFaceXp==0)
    {
        Vx_Xp_X = (PREC_GRILLE *) calloc((W)*(Y)*(Z),sizeof(PREC_GRILLE));
        Vx_Xp_YZ = (PREC_GRILLE *) calloc((W)*(Y)*(Z),sizeof(PREC_GRILLE));
        Vy_Xp_X = (PREC_GRILLE *) calloc((W+1)*(Y+1)*(Z),sizeof(PREC_GRILLE));
        Vy_Xp_YZ = (PREC_GRILLE *) calloc((W+1)*(Y+1)*(Z),sizeof(PREC_GRILLE));
        Vz_Xp_X = (PREC_GRILLE *) calloc((W+1)*(Y)*(Z+1),sizeof(PREC_GRILLE));
        Vz_Xp_YZ = (PREC_GRILLE *) calloc((W+1)*(Y)*(Z+1),sizeof(PREC_GRILLE));
        Txx_Xp_X = (PREC_GRILLE *) calloc((W+1)*(Y)*(Z),sizeof(PREC_GRILLE));
        Txx_Xp_YZ = (PREC_GRILLE *) calloc((W+1)*(Y)*(Z),sizeof(PREC_GRILLE));
        Tyy_Xp_X = (PREC_GRILLE *) calloc((W+1)*(Y)*(Z),sizeof(PREC_GRILLE));
        Tyy_Xp_YZ = (PREC_GRILLE *) calloc((W+1)*(Y)*(Z),sizeof(PREC_GRILLE));
        Tzz_Xp_X = (PREC_GRILLE *) calloc((W+1)*(Y)*(Z),sizeof(PREC_GRILLE));
        Tzz_Xp_YZ = (PREC_GRILLE *) calloc((W+1)*(Y)*(Z),sizeof(PREC_GRILLE));
        Txy_Xp_X = (PREC_GRILLE *) calloc((W)*(Y+1)*(Z),sizeof(PREC_GRILLE));
        Txy_Xp_YZ = (PREC_GRILLE *) calloc((W)*(Y+1)*(Z),sizeof(PREC_GRILLE));
        Tzx_Xp_X = (PREC_GRILLE *) calloc((W)*(Y)*(Z+1),sizeof(PREC_GRILLE));
        Tzx_Xp_YZ = (PREC_GRILLE *) calloc((W)*(Y)*(Z+1),sizeof(PREC_GRILLE));
        /*		 seule variable qui n'est calcule que par drive //  la PML*/
        Tyz_Xp = (PREC_GRILLE *) calloc((W+1)*(Y+1)*(Z+1),sizeof(PREC_GRILLE));
    }

    if (ConditionFaceXm==0)
    {
        Vx_Xm_X = (PREC_GRILLE *) calloc((W)*(Y)*(Z),sizeof(PREC_GRILLE));
        Vx_Xm_YZ = (PREC_GRILLE *) calloc((W)*(Y)*(Z),sizeof(PREC_GRILLE));
        Vy_Xm_X = (PREC_GRILLE *) calloc((W+1)*(Y+1)*(Z),sizeof(PREC_GRILLE));
        Vy_Xm_YZ = (PREC_GRILLE *) calloc((W+1)*(Y+1)*(Z),sizeof(PREC_GRILLE));
        Vz_Xm_X = (PREC_GRILLE *) calloc((W+1)*(Y)*(Z+1),sizeof(PREC_GRILLE));
        Vz_Xm_YZ = (PREC_GRILLE *) calloc((W+1)*(Y)*(Z+1),sizeof(PREC_GRILLE));
        Txx_Xm_X = (PREC_GRILLE *) calloc((W+1)*(Y)*(Z),sizeof(PREC_GRILLE));
        Txx_Xm_YZ = (PREC_GRILLE *) calloc((W+1)*(Y)*(Z),sizeof(PREC_GRILLE));
        Tyy_Xm_X = (PREC_GRILLE *) calloc((W+1)*(Y)*(Z),sizeof(PREC_GRILLE));
        Tyy_Xm_YZ = (PREC_GRILLE *) calloc((W+1)*(Y)*(Z),sizeof(PREC_GRILLE));
        Tzz_Xm_X = (PREC_GRILLE *) calloc((W+1)*(Y)*(Z),sizeof(PREC_GRILLE));
        Tzz_Xm_YZ = (PREC_GRILLE *) calloc((W+1)*(Y)*(Z),sizeof(PREC_GRILLE));
        Txy_Xm_X = (PREC_GRILLE *) calloc((W)*(Y+1)*(Z),sizeof(PREC_GRILLE));
        Txy_Xm_YZ = (PREC_GRILLE *) calloc((W)*(Y+1)*(Z),sizeof(PREC_GRILLE));
        Tzx_Xm_X = (PREC_GRILLE *) calloc((W)*(Y)*(Z+1),sizeof(PREC_GRILLE));
        Tzx_Xm_YZ = (PREC_GRILLE *) calloc((W)*(Y)*(Z+1),sizeof(PREC_GRILLE));
        /*		 seule variable qui n'est calcule que par drive //  la PML*/
        Tyz_Xm = (PREC_GRILLE *) calloc((W+1)*(Y+1)*(Z+1),sizeof(PREC_GRILLE));
    }

    if (ConditionFaceYp==0)
    {
        Vx_Yp_Y = (PREC_GRILLE *) calloc((W+1)*(Z)*(X+1),sizeof(PREC_GRILLE));
        Vx_Yp_ZX = (PREC_GRILLE *) calloc((W+1)*(Z)*(X+1),sizeof(PREC_GRILLE));
        Vy_Yp_Y = (PREC_GRILLE *) calloc((W)*(Z)*(X),sizeof(PREC_GRILLE));
        Vy_Yp_ZX = (PREC_GRILLE *) calloc((W)*(Z)*(X),sizeof(PREC_GRILLE));
        Vz_Yp_Y = (PREC_GRILLE *) calloc((W+1)*(X)*(Z+1),sizeof(PREC_GRILLE));
        Vz_Yp_ZX = (PREC_GRILLE *) calloc((W+1)*(X)*(Z+1),sizeof(PREC_GRILLE));
        Txx_Yp_Y = (PREC_GRILLE *) calloc((W+1)*(X)*(Z),sizeof(PREC_GRILLE));
        Txx_Yp_ZX = (PREC_GRILLE *) calloc((W+1)*(X)*(Z),sizeof(PREC_GRILLE));
        Tyy_Yp_Y = (PREC_GRILLE *) calloc((W+1)*(X)*(Z),sizeof(PREC_GRILLE));
        Tyy_Yp_ZX = (PREC_GRILLE *) calloc((W+1)*(X)*(Z),sizeof(PREC_GRILLE));
        Tzz_Yp_Y = (PREC_GRILLE *) calloc((W+1)*(X)*(Z),sizeof(PREC_GRILLE));
        Tzz_Yp_ZX = (PREC_GRILLE *) calloc((W+1)*(X)*(Z),sizeof(PREC_GRILLE));
        Txy_Yp_Y = (PREC_GRILLE *) calloc((W)*(X+1)*(Z),sizeof(PREC_GRILLE));
        Txy_Yp_ZX = (PREC_GRILLE *) calloc((W)*(X+1)*(Z),sizeof(PREC_GRILLE));
        Tyz_Yp_Y = (PREC_GRILLE *) calloc((W)*(X)*(Z+1),sizeof(PREC_GRILLE));
        Tyz_Yp_ZX = (PREC_GRILLE *) calloc((W)*(X)*(Z+1),sizeof(PREC_GRILLE));
        /*		 seule variable qui n'est calcule que par drive //  la PML*/
        Tzx_Yp = (PREC_GRILLE *) calloc((W+1)*(X+1)*(Z+1),sizeof(PREC_GRILLE));
    }


    if (ConditionFaceYm==0)
    {
        Vx_Ym_Y = (PREC_GRILLE *) calloc((W+1)*(Z)*(X+1),sizeof(PREC_GRILLE));
        Vx_Ym_ZX = (PREC_GRILLE *) calloc((W+1)*(Z)*(X+1),sizeof(PREC_GRILLE));
        Vy_Ym_Y = (PREC_GRILLE *) calloc((W)*(Z)*(X),sizeof(PREC_GRILLE));
        Vy_Ym_ZX = (PREC_GRILLE *) calloc((W)*(Z)*(X),sizeof(PREC_GRILLE));
        Vz_Ym_Y = (PREC_GRILLE *) calloc((W+1)*(X)*(Z+1),sizeof(PREC_GRILLE));
        Vz_Ym_ZX = (PREC_GRILLE *) calloc((W+1)*(X)*(Z+1),sizeof(PREC_GRILLE));
        Txx_Ym_Y = (PREC_GRILLE *) calloc((W+1)*(X)*(Z),sizeof(PREC_GRILLE));
        Txx_Ym_ZX = (PREC_GRILLE *) calloc((W+1)*(X)*(Z),sizeof(PREC_GRILLE));
        Tyy_Ym_Y = (PREC_GRILLE *) calloc((W+1)*(X)*(Z),sizeof(PREC_GRILLE));
        Tyy_Ym_ZX = (PREC_GRILLE *) calloc((W+1)*(X)*(Z),sizeof(PREC_GRILLE));
        Tzz_Ym_Y = (PREC_GRILLE *) calloc((W+1)*(X)*(Z),sizeof(PREC_GRILLE));
        Tzz_Ym_ZX = (PREC_GRILLE *) calloc((W+1)*(X)*(Z),sizeof(PREC_GRILLE));
        Txy_Ym_Y = (PREC_GRILLE *) calloc((W)*(X+1)*(Z),sizeof(PREC_GRILLE));
        Txy_Ym_ZX = (PREC_GRILLE *) calloc((W)*(X+1)*(Z),sizeof(PREC_GRILLE));
        Tyz_Ym_Y = (PREC_GRILLE *) calloc((W)*(X)*(Z+1),sizeof(PREC_GRILLE));
        Tyz_Ym_ZX = (PREC_GRILLE *) calloc((W)*(X)*(Z+1),sizeof(PREC_GRILLE));
        /*		 seule variable qui n'est calcule que par drive //  la PML*/
        Tzx_Ym = (PREC_GRILLE *) calloc((W+1)*(X+1)*(Z+1),sizeof(PREC_GRILLE));
    }

    if (ConditionFaceZp==0)
    {
        Vx_Zp_Z = (PREC_GRILLE *) calloc((W+1)*(Y)*(X+1),sizeof(PREC_GRILLE));
        Vx_Zp_XY = (PREC_GRILLE *) calloc((W+1)*(Y)*(X+1),sizeof(PREC_GRILLE));
        Vy_Zp_Z = (PREC_GRILLE *) calloc((W+1)*(Y+1)*(X),sizeof(PREC_GRILLE));
        Vy_Zp_XY = (PREC_GRILLE *) calloc((W+1)*(Y+1)*(X),sizeof(PREC_GRILLE));
        Vz_Zp_Z = (PREC_GRILLE *) calloc((W)*(Y)*(X),sizeof(PREC_GRILLE));
        Vz_Zp_XY = (PREC_GRILLE *) calloc((W)*(Y)*(X),sizeof(PREC_GRILLE));
        Txx_Zp_Z = (PREC_GRILLE *) calloc((W+1)*(Y)*(X),sizeof(PREC_GRILLE));
        Txx_Zp_XY = (PREC_GRILLE *) calloc((W+1)*(Y)*(X),sizeof(PREC_GRILLE));
        Tyy_Zp_Z = (PREC_GRILLE *) calloc((W+1)*(Y)*(X),sizeof(PREC_GRILLE));
        Tyy_Zp_XY = (PREC_GRILLE *) calloc((W+1)*(Y)*(X),sizeof(PREC_GRILLE));
        Tzz_Zp_Z = (PREC_GRILLE *) calloc((W+1)*(Y)*(X),sizeof(PREC_GRILLE));
        Tzz_Zp_XY = (PREC_GRILLE *) calloc((W+1)*(Y)*(X),sizeof(PREC_GRILLE));
        Tyz_Zp_Z = (PREC_GRILLE *) calloc((W)*(Y+1)*(X),sizeof(PREC_GRILLE));
        Tyz_Zp_XY = (PREC_GRILLE *) calloc((W)*(Y+1)*(X),sizeof(PREC_GRILLE));
        Tzx_Zp_Z = (PREC_GRILLE *) calloc((W)*(Y)*(X+1),sizeof(PREC_GRILLE));
        Tzx_Zp_XY = (PREC_GRILLE *) calloc((W)*(Y)*(X+1),sizeof(PREC_GRILLE));
        /*		 seule variable qui n'est calcule que par drive //  la PML*/
        Txy_Zp = (PREC_GRILLE *) calloc((W+1)*(Y+1)*(X+1),sizeof(PREC_GRILLE));
    }


    if (ConditionFaceZm==0)
    {
        Vx_Zm_Z = (PREC_GRILLE *) calloc((W+1)*(Y)*(X+1),sizeof(PREC_GRILLE));
        Vx_Zm_XY = (PREC_GRILLE *) calloc((W+1)*(Y)*(X+1),sizeof(PREC_GRILLE));
        Vy_Zm_Z = (PREC_GRILLE *) calloc((W+1)*(Y+1)*(X),sizeof(PREC_GRILLE));
        Vy_Zm_XY = (PREC_GRILLE *) calloc((W+1)*(Y+1)*(X),sizeof(PREC_GRILLE));
        Vz_Zm_Z = (PREC_GRILLE *) calloc((W)*(Y)*(X),sizeof(PREC_GRILLE));
        Vz_Zm_XY = (PREC_GRILLE *) calloc((W)*(Y)*(X),sizeof(PREC_GRILLE));
        Txx_Zm_Z = (PREC_GRILLE *) calloc((W+1)*(Y)*(X),sizeof(PREC_GRILLE));
        Txx_Zm_XY = (PREC_GRILLE *) calloc((W+1)*(Y)*(X),sizeof(PREC_GRILLE));
        Tyy_Zm_Z = (PREC_GRILLE *) calloc((W+1)*(Y)*(X),sizeof(PREC_GRILLE));
        Tyy_Zm_XY = (PREC_GRILLE *) calloc((W+1)*(Y)*(X),sizeof(PREC_GRILLE));
        Tzz_Zm_Z = (PREC_GRILLE *) calloc((W+1)*(Y)*(X),sizeof(PREC_GRILLE));
        Tzz_Zm_XY = (PREC_GRILLE *) calloc((W+1)*(Y)*(X),sizeof(PREC_GRILLE));
        Tyz_Zm_Z = (PREC_GRILLE *) calloc((W)*(Y+1)*(X),sizeof(PREC_GRILLE));
        Tyz_Zm_XY = (PREC_GRILLE *) calloc((W)*(Y+1)*(X),sizeof(PREC_GRILLE));
        Tzx_Zm_Z = (PREC_GRILLE *) calloc((W)*(Y)*(X+1),sizeof(PREC_GRILLE));
        Tzx_Zm_XY = (PREC_GRILLE *) calloc((W)*(Y)*(X+1),sizeof(PREC_GRILLE));
        /*		 seule variable qui n'est calcule que par drive //  la PML*/
        Txy_Zm = (PREC_GRILLE *) calloc((W+1)*(Y+1)*(X+1),sizeof(PREC_GRILLE));
    }

    /***************************************************************************/
    /**************** Allocation des  12 PML Aretes ****************************/
    /***************************************************************************/

    if ((ConditionFaceXp==0)&&(ConditionFaceYp==0))
    {

        Vx_XpYp_X = (PREC_GRILLE *) calloc((W)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Vx_XpYp_Y = (PREC_GRILLE *) calloc((W)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Vx_XpYp_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Vy_XpYp_X = (PREC_GRILLE *) calloc((W+1)*(W)*(Z),sizeof(PREC_GRILLE));
        Vy_XpYp_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(Z),sizeof(PREC_GRILLE));
        Vy_XpYp_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(Z),sizeof(PREC_GRILLE));
        Vz_XpYp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z+1),sizeof(PREC_GRILLE));
        Vz_XpYp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z+1),sizeof(PREC_GRILLE));
        Vz_XpYp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z+1),sizeof(PREC_GRILLE));
        Txx_XpYp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Txx_XpYp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Txx_XpYp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Tyy_XpYp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Tyy_XpYp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Tyy_XpYp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Tzz_XpYp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Tzz_XpYp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Tzz_XpYp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Txy_XpYp_X = (PREC_GRILLE *) calloc((W)*(W)*(Z),sizeof(PREC_GRILLE));
        Txy_XpYp_Y = (PREC_GRILLE *) calloc((W)*(W)*(Z),sizeof(PREC_GRILLE));
        Tyz_XpYp_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(Z+1),sizeof(PREC_GRILLE));
        Tyz_XpYp_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(Z+1),sizeof(PREC_GRILLE));
        Tzx_XpYp_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(Z+1),sizeof(PREC_GRILLE));
        Tzx_XpYp_X = (PREC_GRILLE *) calloc((W)*(W+1)*(Z+1),sizeof(PREC_GRILLE));
    }

    if ((ConditionFaceXp==0)&&(ConditionFaceYm==0))
    {

        Vx_XpYm_X = (PREC_GRILLE *) calloc((W)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Vx_XpYm_Y = (PREC_GRILLE *) calloc((W)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Vx_XpYm_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Vy_XpYm_X = (PREC_GRILLE *) calloc((W+1)*(W)*(Z),sizeof(PREC_GRILLE));
        Vy_XpYm_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(Z),sizeof(PREC_GRILLE));
        Vy_XpYm_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(Z),sizeof(PREC_GRILLE));
        Vz_XpYm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z+1),sizeof(PREC_GRILLE));
        Vz_XpYm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z+1),sizeof(PREC_GRILLE));
        Vz_XpYm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z+1),sizeof(PREC_GRILLE));
        Txx_XpYm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Txx_XpYm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Txx_XpYm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Tyy_XpYm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Tyy_XpYm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Tyy_XpYm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Tzz_XpYm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Tzz_XpYm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Tzz_XpYm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Txy_XpYm_X = (PREC_GRILLE *) calloc((W)*(W)*(Z),sizeof(PREC_GRILLE));
        Txy_XpYm_Y = (PREC_GRILLE *) calloc((W)*(W)*(Z),sizeof(PREC_GRILLE));
        Tyz_XpYm_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(Z+1),sizeof(PREC_GRILLE));
        Tyz_XpYm_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(Z+1),sizeof(PREC_GRILLE));
        Tzx_XpYm_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(Z+1),sizeof(PREC_GRILLE));
        Tzx_XpYm_X = (PREC_GRILLE *) calloc((W)*(W+1)*(Z+1),sizeof(PREC_GRILLE));
    }

    if ((ConditionFaceXm==0)&&(ConditionFaceYp==0))
    {

        Vx_XmYp_X = (PREC_GRILLE *) calloc((W)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Vx_XmYp_Y = (PREC_GRILLE *) calloc((W)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Vx_XmYp_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Vy_XmYp_X = (PREC_GRILLE *) calloc((W+1)*(W)*(Z),sizeof(PREC_GRILLE));
        Vy_XmYp_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(Z),sizeof(PREC_GRILLE));
        Vy_XmYp_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(Z),sizeof(PREC_GRILLE));
        Vz_XmYp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z+1),sizeof(PREC_GRILLE));
        Vz_XmYp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z+1),sizeof(PREC_GRILLE));
        Vz_XmYp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z+1),sizeof(PREC_GRILLE));
        Txx_XmYp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Txx_XmYp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Txx_XmYp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Tyy_XmYp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Tyy_XmYp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Tyy_XmYp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Tzz_XmYp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Tzz_XmYp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Tzz_XmYp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Txy_XmYp_X = (PREC_GRILLE *) calloc((W)*(W)*(Z),sizeof(PREC_GRILLE));
        Txy_XmYp_Y = (PREC_GRILLE *) calloc((W)*(W)*(Z),sizeof(PREC_GRILLE));
        Tyz_XmYp_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(Z+1),sizeof(PREC_GRILLE));
        Tyz_XmYp_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(Z+1),sizeof(PREC_GRILLE));
        Tzx_XmYp_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(Z+1),sizeof(PREC_GRILLE));
        Tzx_XmYp_X = (PREC_GRILLE *) calloc((W)*(W+1)*(Z+1),sizeof(PREC_GRILLE));
    }

    if ((ConditionFaceXm==0)&&(ConditionFaceYm==0))
    {

        Vx_XmYm_X = (PREC_GRILLE *) calloc((W)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Vx_XmYm_Y = (PREC_GRILLE *) calloc((W)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Vx_XmYm_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Vy_XmYm_X = (PREC_GRILLE *) calloc((W+1)*(W)*(Z),sizeof(PREC_GRILLE));
        Vy_XmYm_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(Z),sizeof(PREC_GRILLE));
        Vy_XmYm_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(Z),sizeof(PREC_GRILLE));
        Vz_XmYm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z+1),sizeof(PREC_GRILLE));
        Vz_XmYm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z+1),sizeof(PREC_GRILLE));
        Vz_XmYm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z+1),sizeof(PREC_GRILLE));
        Txx_XmYm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Txx_XmYm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Txx_XmYm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Tyy_XmYm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Tyy_XmYm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Tyy_XmYm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Tzz_XmYm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Tzz_XmYm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Tzz_XmYm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Z),sizeof(PREC_GRILLE));
        Txy_XmYm_X = (PREC_GRILLE *) calloc((W)*(W)*(Z),sizeof(PREC_GRILLE));
        Txy_XmYm_Y = (PREC_GRILLE *) calloc((W)*(W)*(Z),sizeof(PREC_GRILLE));
        Tyz_XmYm_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(Z+1),sizeof(PREC_GRILLE));
        Tyz_XmYm_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(Z+1),sizeof(PREC_GRILLE));
        Tzx_XmYm_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(Z+1),sizeof(PREC_GRILLE));
        Tzx_XmYm_X = (PREC_GRILLE *) calloc((W)*(W+1)*(Z+1),sizeof(PREC_GRILLE));
    }

    if ((ConditionFaceYp==0)&&(ConditionFaceZp==0))
    {

        Vy_YpZp_Y = (PREC_GRILLE *) calloc((W)*(W+1)*(X),sizeof(PREC_GRILLE));
        Vy_YpZp_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(X),sizeof(PREC_GRILLE));
        Vy_YpZp_X = (PREC_GRILLE *) calloc((W)*(W+1)*(X),sizeof(PREC_GRILLE));
        Vz_YpZp_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(X),sizeof(PREC_GRILLE));
        Vz_YpZp_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(X),sizeof(PREC_GRILLE));
        Vz_YpZp_X = (PREC_GRILLE *) calloc((W+1)*(W)*(X),sizeof(PREC_GRILLE));
        Vx_YpZp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X+1),sizeof(PREC_GRILLE));
        Vx_YpZp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X+1),sizeof(PREC_GRILLE));
        Vx_YpZp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X+1),sizeof(PREC_GRILLE));
        Tyy_YpZp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Tyy_YpZp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Tyy_YpZp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Tzz_YpZp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Tzz_YpZp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Tzz_YpZp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Txx_YpZp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Txx_YpZp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Txx_YpZp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Tyz_YpZp_Y = (PREC_GRILLE *) calloc((W)*(W)*(X),sizeof(PREC_GRILLE));
        Tyz_YpZp_Z = (PREC_GRILLE *) calloc((W)*(W)*(X),sizeof(PREC_GRILLE));
        Tzx_YpZp_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(X+1),sizeof(PREC_GRILLE));
        Tzx_YpZp_X = (PREC_GRILLE *) calloc((W+1)*(W)*(X+1),sizeof(PREC_GRILLE));
        Txy_YpZp_X = (PREC_GRILLE *) calloc((W)*(W+1)*(X+1),sizeof(PREC_GRILLE));
        Txy_YpZp_Y = (PREC_GRILLE *) calloc((W)*(W+1)*(X+1),sizeof(PREC_GRILLE));
    }
    if ((ConditionFaceYm==0)&&(ConditionFaceZp==0))
    {

        Vy_YmZp_Y = (PREC_GRILLE *) calloc((W)*(W+1)*(X),sizeof(PREC_GRILLE));
        Vy_YmZp_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(X),sizeof(PREC_GRILLE));
        Vy_YmZp_X = (PREC_GRILLE *) calloc((W)*(W+1)*(X),sizeof(PREC_GRILLE));
        Vz_YmZp_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(X),sizeof(PREC_GRILLE));
        Vz_YmZp_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(X),sizeof(PREC_GRILLE));
        Vz_YmZp_X = (PREC_GRILLE *) calloc((W+1)*(W)*(X),sizeof(PREC_GRILLE));
        Vx_YmZp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X+1),sizeof(PREC_GRILLE));
        Vx_YmZp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X+1),sizeof(PREC_GRILLE));
        Vx_YmZp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X+1),sizeof(PREC_GRILLE));
        Tyy_YmZp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Tyy_YmZp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Tyy_YmZp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Tzz_YmZp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Tzz_YmZp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Tzz_YmZp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Txx_YmZp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Txx_YmZp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Txx_YmZp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Tyz_YmZp_Y = (PREC_GRILLE *) calloc((W)*(W)*(X),sizeof(PREC_GRILLE));
        Tyz_YmZp_Z = (PREC_GRILLE *) calloc((W)*(W)*(X),sizeof(PREC_GRILLE));
        Tzx_YmZp_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(X+1),sizeof(PREC_GRILLE));
        Tzx_YmZp_X = (PREC_GRILLE *) calloc((W+1)*(W)*(X+1),sizeof(PREC_GRILLE));
        Txy_YmZp_X = (PREC_GRILLE *) calloc((W)*(W+1)*(X+1),sizeof(PREC_GRILLE));
        Txy_YmZp_Y = (PREC_GRILLE *) calloc((W)*(W+1)*(X+1),sizeof(PREC_GRILLE));
    }
    if ((ConditionFaceYp==0)&&(ConditionFaceZm==0))
    {

        Vy_YpZm_Y = (PREC_GRILLE *) calloc((W)*(W+1)*(X),sizeof(PREC_GRILLE));
        Vy_YpZm_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(X),sizeof(PREC_GRILLE));
        Vy_YpZm_X = (PREC_GRILLE *) calloc((W)*(W+1)*(X),sizeof(PREC_GRILLE));
        Vz_YpZm_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(X),sizeof(PREC_GRILLE));
        Vz_YpZm_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(X),sizeof(PREC_GRILLE));
        Vz_YpZm_X = (PREC_GRILLE *) calloc((W+1)*(W)*(X),sizeof(PREC_GRILLE));
        Vx_YpZm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X+1),sizeof(PREC_GRILLE));
        Vx_YpZm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X+1),sizeof(PREC_GRILLE));
        Vx_YpZm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X+1),sizeof(PREC_GRILLE));
        Tyy_YpZm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Tyy_YpZm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Tyy_YpZm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Tzz_YpZm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Tzz_YpZm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Tzz_YpZm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Txx_YpZm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Txx_YpZm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Txx_YpZm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Tyz_YpZm_Y = (PREC_GRILLE *) calloc((W)*(W)*(X),sizeof(PREC_GRILLE));
        Tyz_YpZm_Z = (PREC_GRILLE *) calloc((W)*(W)*(X),sizeof(PREC_GRILLE));
        Tzx_YpZm_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(X+1),sizeof(PREC_GRILLE));
        Tzx_YpZm_X = (PREC_GRILLE *) calloc((W+1)*(W)*(X+1),sizeof(PREC_GRILLE));
        Txy_YpZm_X = (PREC_GRILLE *) calloc((W)*(W+1)*(X+1),sizeof(PREC_GRILLE));
        Txy_YpZm_Y = (PREC_GRILLE *) calloc((W)*(W+1)*(X+1),sizeof(PREC_GRILLE));
    }
    if ((ConditionFaceYm==0)&&(ConditionFaceZm==0))
    {

        Vy_YmZm_Y = (PREC_GRILLE *) calloc((W)*(W+1)*(X),sizeof(PREC_GRILLE));
        Vy_YmZm_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(X),sizeof(PREC_GRILLE));
        Vy_YmZm_X = (PREC_GRILLE *) calloc((W)*(W+1)*(X),sizeof(PREC_GRILLE));
        Vz_YmZm_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(X),sizeof(PREC_GRILLE));
        Vz_YmZm_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(X),sizeof(PREC_GRILLE));
        Vz_YmZm_X = (PREC_GRILLE *) calloc((W+1)*(W)*(X),sizeof(PREC_GRILLE));
        Vx_YmZm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X+1),sizeof(PREC_GRILLE));
        Vx_YmZm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X+1),sizeof(PREC_GRILLE));
        Vx_YmZm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X+1),sizeof(PREC_GRILLE));
        Tyy_YmZm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Tyy_YmZm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Tyy_YmZm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Tzz_YmZm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Tzz_YmZm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Tzz_YmZm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Txx_YmZm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Txx_YmZm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Txx_YmZm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(X),sizeof(PREC_GRILLE));
        Tyz_YmZm_Y = (PREC_GRILLE *) calloc((W)*(W)*(X),sizeof(PREC_GRILLE));
        Tyz_YmZm_Z = (PREC_GRILLE *) calloc((W)*(W)*(X),sizeof(PREC_GRILLE));
        Tzx_YmZm_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(X+1),sizeof(PREC_GRILLE));
        Tzx_YmZm_X = (PREC_GRILLE *) calloc((W+1)*(W)*(X+1),sizeof(PREC_GRILLE));
        Txy_YmZm_X = (PREC_GRILLE *) calloc((W)*(W+1)*(X+1),sizeof(PREC_GRILLE));
        Txy_YmZm_Y = (PREC_GRILLE *) calloc((W)*(W+1)*(X+1),sizeof(PREC_GRILLE));
    }
    if ((ConditionFaceXp==0)&&(ConditionFaceZp==0))
    {

        Vz_ZpXp_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Vz_ZpXp_X = (PREC_GRILLE *) calloc((W)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Vz_ZpXp_Y = (PREC_GRILLE *) calloc((W)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Vx_ZpXp_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(Y),sizeof(PREC_GRILLE));
        Vx_ZpXp_X = (PREC_GRILLE *) calloc((W+1)*(W)*(Y),sizeof(PREC_GRILLE));
        Vx_ZpXp_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(Y),sizeof(PREC_GRILLE));
        Vy_ZpXp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y+1),sizeof(PREC_GRILLE));
        Vy_ZpXp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y+1),sizeof(PREC_GRILLE));
        Vy_ZpXp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y+1),sizeof(PREC_GRILLE));
        Tzz_ZpXp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Tzz_ZpXp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Tzz_ZpXp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Txx_ZpXp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Txx_ZpXp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Txx_ZpXp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Tyy_ZpXp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Tyy_ZpXp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Tyy_ZpXp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Tzx_ZpXp_Z = (PREC_GRILLE *) calloc((W)*(W)*(Y),sizeof(PREC_GRILLE));
        Tzx_ZpXp_X = (PREC_GRILLE *) calloc((W)*(W)*(Y),sizeof(PREC_GRILLE));
        Txy_ZpXp_X = (PREC_GRILLE *) calloc((W+1)*(W)*(Y+1),sizeof(PREC_GRILLE));
        Txy_ZpXp_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(Y+1),sizeof(PREC_GRILLE));
        Tyz_ZpXp_Y = (PREC_GRILLE *) calloc((W)*(W+1)*(Y+1),sizeof(PREC_GRILLE));
        Tyz_ZpXp_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(Y+1),sizeof(PREC_GRILLE));
    }
    if ((ConditionFaceXm==0)&&(ConditionFaceZp==0))
    {

        Vz_ZpXm_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Vz_ZpXm_X = (PREC_GRILLE *) calloc((W)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Vz_ZpXm_Y = (PREC_GRILLE *) calloc((W)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Vx_ZpXm_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(Y),sizeof(PREC_GRILLE));
        Vx_ZpXm_X = (PREC_GRILLE *) calloc((W+1)*(W)*(Y),sizeof(PREC_GRILLE));
        Vx_ZpXm_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(Y),sizeof(PREC_GRILLE));
        Vy_ZpXm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y+1),sizeof(PREC_GRILLE));
        Vy_ZpXm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y+1),sizeof(PREC_GRILLE));
        Vy_ZpXm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y+1),sizeof(PREC_GRILLE));
        Tzz_ZpXm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Tzz_ZpXm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Tzz_ZpXm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Txx_ZpXm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Txx_ZpXm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Txx_ZpXm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Tyy_ZpXm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Tyy_ZpXm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Tyy_ZpXm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Tzx_ZpXm_Z = (PREC_GRILLE *) calloc((W)*(W)*(Y),sizeof(PREC_GRILLE));
        Tzx_ZpXm_X = (PREC_GRILLE *) calloc((W)*(W)*(Y),sizeof(PREC_GRILLE));
        Txy_ZpXm_X = (PREC_GRILLE *) calloc((W+1)*(W)*(Y+1),sizeof(PREC_GRILLE));
        Txy_ZpXm_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(Y+1),sizeof(PREC_GRILLE));
        Tyz_ZpXm_Y = (PREC_GRILLE *) calloc((W)*(W+1)*(Y+1),sizeof(PREC_GRILLE));
        Tyz_ZpXm_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(Y+1),sizeof(PREC_GRILLE));
    }
    if ((ConditionFaceXm==0)&&(ConditionFaceZm==0))
    {

        Vz_ZmXm_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Vz_ZmXm_X = (PREC_GRILLE *) calloc((W)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Vz_ZmXm_Y = (PREC_GRILLE *) calloc((W)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Vx_ZmXm_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(Y),sizeof(PREC_GRILLE));
        Vx_ZmXm_X = (PREC_GRILLE *) calloc((W+1)*(W)*(Y),sizeof(PREC_GRILLE));
        Vx_ZmXm_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(Y),sizeof(PREC_GRILLE));
        Vy_ZmXm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y+1),sizeof(PREC_GRILLE));
        Vy_ZmXm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y+1),sizeof(PREC_GRILLE));
        Vy_ZmXm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y+1),sizeof(PREC_GRILLE));
        Tzz_ZmXm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Tzz_ZmXm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Tzz_ZmXm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Txx_ZmXm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Txx_ZmXm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Txx_ZmXm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Tyy_ZmXm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Tyy_ZmXm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Tyy_ZmXm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Tzx_ZmXm_Z = (PREC_GRILLE *) calloc((W)*(W)*(Y),sizeof(PREC_GRILLE));
        Tzx_ZmXm_X = (PREC_GRILLE *) calloc((W)*(W)*(Y),sizeof(PREC_GRILLE));
        Txy_ZmXm_X = (PREC_GRILLE *) calloc((W+1)*(W)*(Y+1),sizeof(PREC_GRILLE));
        Txy_ZmXm_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(Y+1),sizeof(PREC_GRILLE));
        Tyz_ZmXm_Y = (PREC_GRILLE *) calloc((W)*(W+1)*(Y+1),sizeof(PREC_GRILLE));
        Tyz_ZmXm_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(Y+1),sizeof(PREC_GRILLE));
    }
    if ((ConditionFaceXp==0)&&(ConditionFaceZm==0))
    {

        Vz_ZmXp_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Vz_ZmXp_X = (PREC_GRILLE *) calloc((W)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Vz_ZmXp_Y = (PREC_GRILLE *) calloc((W)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Vx_ZmXp_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(Y),sizeof(PREC_GRILLE));
        Vx_ZmXp_X = (PREC_GRILLE *) calloc((W+1)*(W)*(Y),sizeof(PREC_GRILLE));
        Vx_ZmXp_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(Y),sizeof(PREC_GRILLE));
        Vy_ZmXp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y+1),sizeof(PREC_GRILLE));
        Vy_ZmXp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y+1),sizeof(PREC_GRILLE));
        Vy_ZmXp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y+1),sizeof(PREC_GRILLE));
        Tzz_ZmXp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Tzz_ZmXp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Tzz_ZmXp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Txx_ZmXp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Txx_ZmXp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Txx_ZmXp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Tyy_ZmXp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Tyy_ZmXp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Tyy_ZmXp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(Y),sizeof(PREC_GRILLE));
        Tzx_ZmXp_Z = (PREC_GRILLE *) calloc((W)*(W)*(Y),sizeof(PREC_GRILLE));
        Tzx_ZmXp_X = (PREC_GRILLE *) calloc((W)*(W)*(Y),sizeof(PREC_GRILLE));
        Txy_ZmXp_X = (PREC_GRILLE *) calloc((W+1)*(W)*(Y+1),sizeof(PREC_GRILLE));
        Txy_ZmXp_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(Y+1),sizeof(PREC_GRILLE));
        Tyz_ZmXp_Y = (PREC_GRILLE *) calloc((W)*(W+1)*(Y+1),sizeof(PREC_GRILLE));
        Tyz_ZmXp_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(Y+1),sizeof(PREC_GRILLE));
    }

    /***************************************************************************/
    /**************** Allocation des Coins PML *********************************/
    /***************************************************************************/

    if ((ConditionFaceXp==0)&&(ConditionFaceYp==0)&&(ConditionFaceZp==0))
    {

        Vx_XpYpZp_X = (PREC_GRILLE *) calloc((W)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Vx_XpYpZp_Y = (PREC_GRILLE *) calloc((W)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Vx_XpYpZp_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Vy_XpYpZp_X = (PREC_GRILLE *) calloc((W+1)*(W)*(W+1),sizeof(PREC_GRILLE));
        Vy_XpYpZp_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(W+1),sizeof(PREC_GRILLE));
        Vy_XpYpZp_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(W+1),sizeof(PREC_GRILLE));
        Vz_XpYpZp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W),sizeof(PREC_GRILLE));
        Vz_XpYpZp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W),sizeof(PREC_GRILLE));
        Vz_XpYpZp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W),sizeof(PREC_GRILLE));
        Txx_XpYpZp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Txx_XpYpZp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Txx_XpYpZp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tyy_XpYpZp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tyy_XpYpZp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tyy_XpYpZp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tzz_XpYpZp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tzz_XpYpZp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tzz_XpYpZp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Txy_XpYpZp_X = (PREC_GRILLE *) calloc((W)*(W)*(W+1),sizeof(PREC_GRILLE));
        Txy_XpYpZp_Y = (PREC_GRILLE *) calloc((W)*(W)*(W+1),sizeof(PREC_GRILLE));
        Tyz_XpYpZp_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(W),sizeof(PREC_GRILLE));
        Tyz_XpYpZp_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(W),sizeof(PREC_GRILLE));
        Tzx_XpYpZp_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(W),sizeof(PREC_GRILLE));
        Tzx_XpYpZp_X = (PREC_GRILLE *) calloc((W)*(W+1)*(W),sizeof(PREC_GRILLE));
    }
    if ((ConditionFaceXp==0)&&(ConditionFaceYp==0)&&(ConditionFaceZm==0))
    {

        Vx_XpYpZm_X = (PREC_GRILLE *) calloc((W)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Vx_XpYpZm_Y = (PREC_GRILLE *) calloc((W)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Vx_XpYpZm_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Vy_XpYpZm_X = (PREC_GRILLE *) calloc((W+1)*(W)*(W+1),sizeof(PREC_GRILLE));
        Vy_XpYpZm_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(W+1),sizeof(PREC_GRILLE));
        Vy_XpYpZm_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(W+1),sizeof(PREC_GRILLE));
        Vz_XpYpZm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W),sizeof(PREC_GRILLE));
        Vz_XpYpZm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W),sizeof(PREC_GRILLE));
        Vz_XpYpZm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W),sizeof(PREC_GRILLE));
        Txx_XpYpZm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Txx_XpYpZm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Txx_XpYpZm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tyy_XpYpZm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tyy_XpYpZm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tyy_XpYpZm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tzz_XpYpZm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tzz_XpYpZm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tzz_XpYpZm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Txy_XpYpZm_X = (PREC_GRILLE *) calloc((W)*(W)*(W+1),sizeof(PREC_GRILLE));
        Txy_XpYpZm_Y = (PREC_GRILLE *) calloc((W)*(W)*(W+1),sizeof(PREC_GRILLE));
        Tyz_XpYpZm_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(W),sizeof(PREC_GRILLE));
        Tyz_XpYpZm_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(W),sizeof(PREC_GRILLE));
        Tzx_XpYpZm_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(W),sizeof(PREC_GRILLE));
        Tzx_XpYpZm_X = (PREC_GRILLE *) calloc((W)*(W+1)*(W),sizeof(PREC_GRILLE));
    }
    if ((ConditionFaceXp==0)&&(ConditionFaceYm==0)&&(ConditionFaceZp==0))
    {

        Vx_XpYmZp_X = (PREC_GRILLE *) calloc((W)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Vx_XpYmZp_Y = (PREC_GRILLE *) calloc((W)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Vx_XpYmZp_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Vy_XpYmZp_X = (PREC_GRILLE *) calloc((W+1)*(W)*(W+1),sizeof(PREC_GRILLE));
        Vy_XpYmZp_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(W+1),sizeof(PREC_GRILLE));
        Vy_XpYmZp_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(W+1),sizeof(PREC_GRILLE));
        Vz_XpYmZp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W),sizeof(PREC_GRILLE));
        Vz_XpYmZp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W),sizeof(PREC_GRILLE));
        Vz_XpYmZp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W),sizeof(PREC_GRILLE));
        Txx_XpYmZp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Txx_XpYmZp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Txx_XpYmZp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tyy_XpYmZp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tyy_XpYmZp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tyy_XpYmZp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tzz_XpYmZp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tzz_XpYmZp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tzz_XpYmZp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Txy_XpYmZp_X = (PREC_GRILLE *) calloc((W)*(W)*(W+1),sizeof(PREC_GRILLE));
        Txy_XpYmZp_Y = (PREC_GRILLE *) calloc((W)*(W)*(W+1),sizeof(PREC_GRILLE));
        Tyz_XpYmZp_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(W),sizeof(PREC_GRILLE));
        Tyz_XpYmZp_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(W),sizeof(PREC_GRILLE));
        Tzx_XpYmZp_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(W),sizeof(PREC_GRILLE));
        Tzx_XpYmZp_X = (PREC_GRILLE *) calloc((W)*(W+1)*(W),sizeof(PREC_GRILLE));
    }
    if ((ConditionFaceXp==0)&&(ConditionFaceYm==0)&&(ConditionFaceZm==0))
    {

        Vx_XpYmZm_X = (PREC_GRILLE *) calloc((W)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Vx_XpYmZm_Y = (PREC_GRILLE *) calloc((W)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Vx_XpYmZm_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Vy_XpYmZm_X = (PREC_GRILLE *) calloc((W+1)*(W)*(W+1),sizeof(PREC_GRILLE));
        Vy_XpYmZm_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(W+1),sizeof(PREC_GRILLE));
        Vy_XpYmZm_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(W+1),sizeof(PREC_GRILLE));
        Vz_XpYmZm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W),sizeof(PREC_GRILLE));
        Vz_XpYmZm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W),sizeof(PREC_GRILLE));
        Vz_XpYmZm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W),sizeof(PREC_GRILLE));
        Txx_XpYmZm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Txx_XpYmZm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Txx_XpYmZm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tyy_XpYmZm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tyy_XpYmZm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tyy_XpYmZm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tzz_XpYmZm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tzz_XpYmZm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tzz_XpYmZm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Txy_XpYmZm_X = (PREC_GRILLE *) calloc((W)*(W)*(W+1),sizeof(PREC_GRILLE));
        Txy_XpYmZm_Y = (PREC_GRILLE *) calloc((W)*(W)*(W+1),sizeof(PREC_GRILLE));
        Tyz_XpYmZm_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(W),sizeof(PREC_GRILLE));
        Tyz_XpYmZm_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(W),sizeof(PREC_GRILLE));
        Tzx_XpYmZm_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(W),sizeof(PREC_GRILLE));
        Tzx_XpYmZm_X = (PREC_GRILLE *) calloc((W)*(W+1)*(W),sizeof(PREC_GRILLE));
    }
    if ((ConditionFaceXm==0)&&(ConditionFaceYp==0)&&(ConditionFaceZp==0))
    {

        Vx_XmYpZp_X = (PREC_GRILLE *) calloc((W)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Vx_XmYpZp_Y = (PREC_GRILLE *) calloc((W)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Vx_XmYpZp_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Vy_XmYpZp_X = (PREC_GRILLE *) calloc((W+1)*(W)*(W+1),sizeof(PREC_GRILLE));
        Vy_XmYpZp_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(W+1),sizeof(PREC_GRILLE));
        Vy_XmYpZp_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(W+1),sizeof(PREC_GRILLE));
        Vz_XmYpZp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W),sizeof(PREC_GRILLE));
        Vz_XmYpZp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W),sizeof(PREC_GRILLE));
        Vz_XmYpZp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W),sizeof(PREC_GRILLE));
        Txx_XmYpZp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Txx_XmYpZp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Txx_XmYpZp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tyy_XmYpZp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tyy_XmYpZp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tyy_XmYpZp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tzz_XmYpZp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tzz_XmYpZp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tzz_XmYpZp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Txy_XmYpZp_X = (PREC_GRILLE *) calloc((W)*(W)*(W+1),sizeof(PREC_GRILLE));
        Txy_XmYpZp_Y = (PREC_GRILLE *) calloc((W)*(W)*(W+1),sizeof(PREC_GRILLE));
        Tyz_XmYpZp_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(W),sizeof(PREC_GRILLE));
        Tyz_XmYpZp_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(W),sizeof(PREC_GRILLE));
        Tzx_XmYpZp_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(W),sizeof(PREC_GRILLE));
        Tzx_XmYpZp_X = (PREC_GRILLE *) calloc((W)*(W+1)*(W),sizeof(PREC_GRILLE));
    }
    if ((ConditionFaceXm==0)&&(ConditionFaceYp==0)&&(ConditionFaceZm==0))
    {

        Vx_XmYpZm_X = (PREC_GRILLE *) calloc((W)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Vx_XmYpZm_Y = (PREC_GRILLE *) calloc((W)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Vx_XmYpZm_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Vy_XmYpZm_X = (PREC_GRILLE *) calloc((W+1)*(W)*(W+1),sizeof(PREC_GRILLE));
        Vy_XmYpZm_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(W+1),sizeof(PREC_GRILLE));
        Vy_XmYpZm_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(W+1),sizeof(PREC_GRILLE));
        Vz_XmYpZm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W),sizeof(PREC_GRILLE));
        Vz_XmYpZm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W),sizeof(PREC_GRILLE));
        Vz_XmYpZm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W),sizeof(PREC_GRILLE));
        Txx_XmYpZm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Txx_XmYpZm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Txx_XmYpZm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tyy_XmYpZm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tyy_XmYpZm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tyy_XmYpZm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tzz_XmYpZm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tzz_XmYpZm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tzz_XmYpZm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Txy_XmYpZm_X = (PREC_GRILLE *) calloc((W)*(W)*(W+1),sizeof(PREC_GRILLE));
        Txy_XmYpZm_Y = (PREC_GRILLE *) calloc((W)*(W)*(W+1),sizeof(PREC_GRILLE));
        Tyz_XmYpZm_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(W),sizeof(PREC_GRILLE));
        Tyz_XmYpZm_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(W),sizeof(PREC_GRILLE));
        Tzx_XmYpZm_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(W),sizeof(PREC_GRILLE));
        Tzx_XmYpZm_X = (PREC_GRILLE *) calloc((W)*(W+1)*(W),sizeof(PREC_GRILLE));
    }
    if ((ConditionFaceXm==0)&&(ConditionFaceYm==0)&&(ConditionFaceZp==0))
    {

        Vx_XmYmZp_X = (PREC_GRILLE *) calloc((W)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Vx_XmYmZp_Y = (PREC_GRILLE *) calloc((W)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Vx_XmYmZp_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Vy_XmYmZp_X = (PREC_GRILLE *) calloc((W+1)*(W)*(W+1),sizeof(PREC_GRILLE));
        Vy_XmYmZp_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(W+1),sizeof(PREC_GRILLE));
        Vy_XmYmZp_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(W+1),sizeof(PREC_GRILLE));
        Vz_XmYmZp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W),sizeof(PREC_GRILLE));
        Vz_XmYmZp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W),sizeof(PREC_GRILLE));
        Vz_XmYmZp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W),sizeof(PREC_GRILLE));
        Txx_XmYmZp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Txx_XmYmZp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Txx_XmYmZp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tyy_XmYmZp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tyy_XmYmZp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tyy_XmYmZp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tzz_XmYmZp_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tzz_XmYmZp_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tzz_XmYmZp_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Txy_XmYmZp_X = (PREC_GRILLE *) calloc((W)*(W)*(W+1),sizeof(PREC_GRILLE));
        Txy_XmYmZp_Y = (PREC_GRILLE *) calloc((W)*(W)*(W+1),sizeof(PREC_GRILLE));
        Tyz_XmYmZp_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(W),sizeof(PREC_GRILLE));
        Tyz_XmYmZp_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(W),sizeof(PREC_GRILLE));
        Tzx_XmYmZp_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(W),sizeof(PREC_GRILLE));
        Tzx_XmYmZp_X = (PREC_GRILLE *) calloc((W)*(W+1)*(W),sizeof(PREC_GRILLE));
    }
    if ((ConditionFaceXm==0)&&(ConditionFaceYm==0)&&(ConditionFaceZm==0))
    {

        Vx_XmYmZm_X = (PREC_GRILLE *) calloc((W)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Vx_XmYmZm_Y = (PREC_GRILLE *) calloc((W)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Vx_XmYmZm_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Vy_XmYmZm_X = (PREC_GRILLE *) calloc((W+1)*(W)*(W+1),sizeof(PREC_GRILLE));
        Vy_XmYmZm_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(W+1),sizeof(PREC_GRILLE));
        Vy_XmYmZm_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(W+1),sizeof(PREC_GRILLE));
        Vz_XmYmZm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W),sizeof(PREC_GRILLE));
        Vz_XmYmZm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W),sizeof(PREC_GRILLE));
        Vz_XmYmZm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W),sizeof(PREC_GRILLE));
        Txx_XmYmZm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Txx_XmYmZm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Txx_XmYmZm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tyy_XmYmZm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tyy_XmYmZm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tyy_XmYmZm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tzz_XmYmZm_X = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tzz_XmYmZm_Y = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Tzz_XmYmZm_Z = (PREC_GRILLE *) calloc((W+1)*(W+1)*(W+1),sizeof(PREC_GRILLE));
        Txy_XmYmZm_X = (PREC_GRILLE *) calloc((W)*(W)*(W+1),sizeof(PREC_GRILLE));
        Txy_XmYmZm_Y = (PREC_GRILLE *) calloc((W)*(W)*(W+1),sizeof(PREC_GRILLE));
        Tyz_XmYmZm_Y = (PREC_GRILLE *) calloc((W+1)*(W)*(W),sizeof(PREC_GRILLE));
        Tyz_XmYmZm_Z = (PREC_GRILLE *) calloc((W+1)*(W)*(W),sizeof(PREC_GRILLE));
        Tzx_XmYmZm_Z = (PREC_GRILLE *) calloc((W)*(W+1)*(W),sizeof(PREC_GRILLE));
        Tzx_XmYmZm_X = (PREC_GRILLE *) calloc((W)*(W+1)*(W),sizeof(PREC_GRILLE));
    }

    /***************************************************************************/
    /****************** Allocation des Faces Tampons  **************************/
    /***************************************************************************/

    AireMax = (max(max(X,Y),Z)+1)*(DeuxiemeMax(X,Y,Z)+1);
    AireIntermediaire = (max(max(X,Y),Z)+1)*(W+1);
    AireMax = max(AireMax,(W+1)*(W+1));
    AireIntermediaire = max(AireIntermediaire,(W+1)*(W+1));

    Carte_Tampon = (PREC_CARTE *) calloc(AireMax,sizeof(PREC_CARTE));
    Buffer1AireMax = (PREC_GRILLE *) calloc(AireMax,sizeof(PREC_GRILLE));
    Buffer2AireMax = (PREC_GRILLE *) calloc(AireMax,sizeof(PREC_GRILLE));
    Buffer01AireIntermediaire =  (PREC_GRILLE *) calloc(AireIntermediaire,sizeof(PREC_GRILLE));
    Buffer02AireIntermediaire =  (PREC_GRILLE *) calloc(AireIntermediaire,sizeof(PREC_GRILLE));
    Buffer03AireIntermediaire =  (PREC_GRILLE *) calloc(AireIntermediaire,sizeof(PREC_GRILLE));
    Buffer04AireIntermediaire =  (PREC_GRILLE *) calloc(AireIntermediaire,sizeof(PREC_GRILLE));
    Buffer05AireIntermediaire =  (PREC_GRILLE *) calloc(AireIntermediaire,sizeof(PREC_GRILLE));
    Buffer06AireIntermediaire =  (PREC_GRILLE *) calloc(AireIntermediaire,sizeof(PREC_GRILLE));
    Buffer07AireIntermediaire =  (PREC_GRILLE *) calloc(AireIntermediaire,sizeof(PREC_GRILLE));
    Buffer08AireIntermediaire =  (PREC_GRILLE *) calloc(AireIntermediaire,sizeof(PREC_GRILLE));

    Carte = (PREC_CARTE *) calloc(X*(Y)*(Z),sizeof(PREC_CARTE));
    AttenuationPML =(double*) calloc (2*W+1,sizeof(double));

}
void LiberationMemoire(void)
{

    int i;

    /***************************************************************************/
    /***************************************************************************/
    /************************ Liberation du BLOC PRINCIPAL *********************/
    /***************************************************************************/
    /***************************************************************************/

    free(Vx);
    free(Vy);
    free(Vz);

    free(Txx);
    free(Tyy);
    free(Tzz);
    free(Txy);
    free(Tyz);
    free(Tzx);

    free(Carte);

    if(SnapshotUx)
        free(Ux);
    if(SnapshotUy)
        free(Uy);
    if(SnapshotUz)
        free(Uz);


    /***************************************************************************/
    /***************************************************************************/
    /************************ Liberation des PML *******************************/
    /***************************************************************************/
    /***************************************************************************/


    if (ConditionFaceXp==0)
    {
        free(Vx_Xp_X);
        free(Vx_Xp_YZ);
        free(Vy_Xp_X);
        free(Vy_Xp_YZ);
        free(Vz_Xp_X);
        free(Vz_Xp_YZ);
        free(Txx_Xp_X);
        free(Txx_Xp_YZ);
        free(Tyy_Xp_X);
        free(Tyy_Xp_YZ);
        free(Tzz_Xp_X);
        free(Tzz_Xp_YZ);
        free(Txy_Xp_X);
        free(Txy_Xp_YZ);
        free(Tzx_Xp_X);
        free(Tzx_Xp_YZ);
        /*		 seule variable qui n'est calcule que par drive //  la PML*/
        free(Tyz_Xp);
    }

    if (ConditionFaceXm==0)
    {
        free(Vx_Xm_X);
        free(Vx_Xm_YZ);
        free(Vy_Xm_X);
        free(Vy_Xm_YZ);
        free(Vz_Xm_X);
        free(Vz_Xm_YZ);
        free(Txx_Xm_X);
        free(Txx_Xm_YZ);
        free(Tyy_Xm_X);
        free(Tyy_Xm_YZ);
        free(Tzz_Xm_X);
        free(Tzz_Xm_YZ);
        free(Txy_Xm_X);
        free(Txy_Xm_YZ);
        free(Tzx_Xm_X);
        free(Tzx_Xm_YZ);
        /*		 seule variable qui n'est calcule que par drive //  la PML*/
        free(Tyz_Xm);
    }

    if (ConditionFaceYp==0)
    {
        free(Vx_Yp_Y);
        free(Vx_Yp_ZX);
        free(Vy_Yp_Y);
        free(Vy_Yp_ZX);
        free(Vz_Yp_Y);
        free(Vz_Yp_ZX);
        free(Txx_Yp_Y);
        free(Txx_Yp_ZX);
        free(Tyy_Yp_Y);
        free(Tyy_Yp_ZX);
        free(Tzz_Yp_Y);
        free(Tzz_Yp_ZX);
        free(Txy_Yp_Y);
        free(Txy_Yp_ZX);
        free(Tyz_Yp_Y);
        free(Tyz_Yp_ZX);
        /*		 seule variable qui n'est calcule que par drive //  la PML*/
        free(Tzx_Yp);
    }

    if (ConditionFaceYm==0)
    {
        free(Vx_Ym_Y);
        free(Vx_Ym_ZX);
        free(Vy_Ym_Y);
        free(Vy_Ym_ZX);
        free(Vz_Ym_Y);
        free(Vz_Ym_ZX);
        free(Txx_Ym_Y);
        free(Txx_Ym_ZX);
        free(Tyy_Ym_Y);
        free(Tyy_Ym_ZX);
        free(Tzz_Ym_Y);
        free(Tzz_Ym_ZX);
        free(Txy_Ym_Y);
        free(Txy_Ym_ZX);
        free(Tyz_Ym_Y);
        free(Tyz_Ym_ZX);
        /*		 seule variable qui n'est calcule que par drive //  la PML*/
        free(Tzx_Ym);
    }


    if (ConditionFaceZp==0)
    {
        free(Vx_Zp_Z);
        free(Vx_Zp_XY);
        free(Vy_Zp_Z);
        free(Vy_Zp_XY);
        free(Vz_Zp_Z);
        free(Vz_Zp_XY);
        free(Txx_Zp_Z);
        free(Txx_Zp_XY);
        free(Tyy_Zp_Z);
        free(Tyy_Zp_XY);
        free(Tzz_Zp_Z);
        free(Tzz_Zp_XY);
        free(Tzx_Zp_Z);
        free(Tzx_Zp_XY);
        free(Tyz_Zp_Z);
        free(Tyz_Zp_XY);
        /*		 seule variable qui n'est calcule que par drive //  la PML*/
        free(Txy_Zp);
    }

    if (ConditionFaceZm==0)
    {
        free(Vx_Zm_Z);
        free(Vx_Zm_XY);
        free(Vy_Zm_Z);
        free(Vy_Zm_XY);
        free(Vz_Zm_Z);
        free(Vz_Zm_XY);
        free(Txx_Zm_Z);
        free(Txx_Zm_XY);
        free(Tyy_Zm_Z);
        free(Tyy_Zm_XY);
        free(Tzz_Zm_Z);
        free(Tzz_Zm_XY);
        free(Tzx_Zm_Z);
        free(Tzx_Zm_XY);
        free(Tyz_Zm_Z);
        free(Tyz_Zm_XY);
        /*		 seule variable qui n'est calcule que par drive //  la PML*/
        free(Txy_Zm);
    }

    /***************************************************************************/
    /************************ Liberation des aretes ****************************/
    /***************************************************************************/

    if ((ConditionFaceXp==0)&&(ConditionFaceYp==0))
    {

        free(Vx_XpYp_X);
        free(Vx_XpYp_Y);
        free(Vx_XpYp_Z);
        free(Vy_XpYp_X);
        free(Vy_XpYp_Y);
        free(Vy_XpYp_Z);
        free(Vz_XpYp_X);
        free(Vz_XpYp_Y);
        free(Vz_XpYp_Z);
        free(Txx_XpYp_X);
        free(Txx_XpYp_Y);
        free(Txx_XpYp_Z);
        free(Tyy_XpYp_X);
        free(Tyy_XpYp_Y);
        free(Tyy_XpYp_Z);
        free(Tzz_XpYp_X);
        free(Tzz_XpYp_Y);
        free(Tzz_XpYp_Z);
        free(Txy_XpYp_X);
        free(Txy_XpYp_Y);
        free(Tyz_XpYp_Y);
        free(Tyz_XpYp_Z);
        free(Tzx_XpYp_Z);
        free(Tzx_XpYp_X);
    }
    if ((ConditionFaceXp==0)&&(ConditionFaceYm==0))
    {

        free(Vx_XpYm_X);
        free(Vx_XpYm_Y);
        free(Vx_XpYm_Z);
        free(Vy_XpYm_X);
        free(Vy_XpYm_Y);
        free(Vy_XpYm_Z);
        free(Vz_XpYm_X);
        free(Vz_XpYm_Y);
        free(Vz_XpYm_Z);
        free(Txx_XpYm_X);
        free(Txx_XpYm_Y);
        free(Txx_XpYm_Z);
        free(Tyy_XpYm_X);
        free(Tyy_XpYm_Y);
        free(Tyy_XpYm_Z);
        free(Tzz_XpYm_X);
        free(Tzz_XpYm_Y);
        free(Tzz_XpYm_Z);
        free(Txy_XpYm_X);
        free(Txy_XpYm_Y);
        free(Tyz_XpYm_Y);
        free(Tyz_XpYm_Z);
        free(Tzx_XpYm_Z);
        free(Tzx_XpYm_X);
    }
    if ((ConditionFaceXm==0)&&(ConditionFaceYp==0))
    {

        free(Vx_XmYp_X);
        free(Vx_XmYp_Y);
        free(Vx_XmYp_Z);
        free(Vy_XmYp_X);
        free(Vy_XmYp_Y);
        free(Vy_XmYp_Z);
        free(Vz_XmYp_X);
        free(Vz_XmYp_Y);
        free(Vz_XmYp_Z);
        free(Txx_XmYp_X);
        free(Txx_XmYp_Y);
        free(Txx_XmYp_Z);
        free(Tyy_XmYp_X);
        free(Tyy_XmYp_Y);
        free(Tyy_XmYp_Z);
        free(Tzz_XmYp_X);
        free(Tzz_XmYp_Y);
        free(Tzz_XmYp_Z);
        free(Txy_XmYp_X);
        free(Txy_XmYp_Y);
        free(Tyz_XmYp_Y);
        free(Tyz_XmYp_Z);
        free(Tzx_XmYp_Z);
        free(Tzx_XmYp_X);
    }
    if ((ConditionFaceXm==0)&&(ConditionFaceYm==0))
    {

        free(Vx_XmYm_X);
        free(Vx_XmYm_Y);
        free(Vx_XmYm_Z);
        free(Vy_XmYm_X);
        free(Vy_XmYm_Y);
        free(Vy_XmYm_Z);
        free(Vz_XmYm_X);
        free(Vz_XmYm_Y);
        free(Vz_XmYm_Z);
        free(Txx_XmYm_X);
        free(Txx_XmYm_Y);
        free(Txx_XmYm_Z);
        free(Tyy_XmYm_X);
        free(Tyy_XmYm_Y);
        free(Tyy_XmYm_Z);
        free(Tzz_XmYm_X);
        free(Tzz_XmYm_Y);
        free(Tzz_XmYm_Z);
        free(Txy_XmYm_X);
        free(Txy_XmYm_Y);
        free(Tyz_XmYm_Y);
        free(Tyz_XmYm_Z);
        free(Tzx_XmYm_Z);
        free(Tzx_XmYm_X);
    }

    if ((ConditionFaceYp==0)&&(ConditionFaceZp==0))
    {

        free(Vy_YpZp_Y);
        free(Vy_YpZp_Z);
        free(Vy_YpZp_X);
        free(Vz_YpZp_Y);
        free(Vz_YpZp_Z);
        free(Vz_YpZp_X);
        free(Vx_YpZp_Y);
        free(Vx_YpZp_Z);
        free(Vx_YpZp_X);
        free(Tyy_YpZp_Y);
        free(Tyy_YpZp_Z);
        free(Tyy_YpZp_X);
        free(Tzz_YpZp_Y);
        free(Tzz_YpZp_Z);
        free(Tzz_YpZp_X);
        free(Txx_YpZp_Y);
        free(Txx_YpZp_Z);
        free(Txx_YpZp_X);
        free(Tyz_YpZp_Y);
        free(Tyz_YpZp_Z);
        free(Tzx_YpZp_Z);
        free(Tzx_YpZp_X);
        free(Txy_YpZp_X);
        free(Txy_YpZp_Y);
    }
    if ((ConditionFaceYp==0)&&(ConditionFaceZm==0))
    {

        free(Vy_YpZm_Y);
        free(Vy_YpZm_Z);
        free(Vy_YpZm_X);
        free(Vz_YpZm_Y);
        free(Vz_YpZm_Z);
        free(Vz_YpZm_X);
        free(Vx_YpZm_Y);
        free(Vx_YpZm_Z);
        free(Vx_YpZm_X);
        free(Tyy_YpZm_Y);
        free(Tyy_YpZm_Z);
        free(Tyy_YpZm_X);
        free(Tzz_YpZm_Y);
        free(Tzz_YpZm_Z);
        free(Tzz_YpZm_X);
        free(Txx_YpZm_Y);
        free(Txx_YpZm_Z);
        free(Txx_YpZm_X);
        free(Tyz_YpZm_Y);
        free(Tyz_YpZm_Z);
        free(Tzx_YpZm_Z);
        free(Tzx_YpZm_X);
        free(Txy_YpZm_X);
        free(Txy_YpZm_Y);
    }
    if ((ConditionFaceYm==0)&&(ConditionFaceZp==0))
    {

        free(Vy_YmZp_Y);
        free(Vy_YmZp_Z);
        free(Vy_YmZp_X);
        free(Vz_YmZp_Y);
        free(Vz_YmZp_Z);
        free(Vz_YmZp_X);
        free(Vx_YmZp_Y);
        free(Vx_YmZp_Z);
        free(Vx_YmZp_X);
        free(Tyy_YmZp_Y);
        free(Tyy_YmZp_Z);
        free(Tyy_YmZp_X);
        free(Tzz_YmZp_Y);
        free(Tzz_YmZp_Z);
        free(Tzz_YmZp_X);
        free(Txx_YmZp_Y);
        free(Txx_YmZp_Z);
        free(Txx_YmZp_X);
        free(Tyz_YmZp_Y);
        free(Tyz_YmZp_Z);
        free(Tzx_YmZp_Z);
        free(Tzx_YmZp_X);
        free(Txy_YmZp_X);
        free(Txy_YmZp_Y);
    }
    if ((ConditionFaceYm==0)&&(ConditionFaceZm==0))
    {

        free(Vy_YmZm_Y);
        free(Vy_YmZm_Z);
        free(Vy_YmZm_X);
        free(Vz_YmZm_Y);
        free(Vz_YmZm_Z);
        free(Vz_YmZm_X);
        free(Vx_YmZm_Y);
        free(Vx_YmZm_Z);
        free(Vx_YmZm_X);
        free(Tyy_YmZm_Y);
        free(Tyy_YmZm_Z);
        free(Tyy_YmZm_X);
        free(Tzz_YmZm_Y);
        free(Tzz_YmZm_Z);
        free(Tzz_YmZm_X);
        free(Txx_YmZm_Y);
        free(Txx_YmZm_Z);
        free(Txx_YmZm_X);
        free(Tyz_YmZm_Y);
        free(Tyz_YmZm_Z);
        free(Tzx_YmZm_Z);
        free(Tzx_YmZm_X);
        free(Txy_YmZm_X);
        free(Txy_YmZm_Y);
    }
    if ((ConditionFaceZp==0)&&(ConditionFaceXp==0))
    {

        free(Vz_ZpXp_Z);
        free(Vz_ZpXp_X);
        free(Vz_ZpXp_Y);
        free(Vx_ZpXp_Z);
        free(Vx_ZpXp_X);
        free(Vx_ZpXp_Y);
        free(Vy_ZpXp_Z);
        free(Vy_ZpXp_X);
        free(Vy_ZpXp_Y);
        free(Tzz_ZpXp_Z);
        free(Tzz_ZpXp_X);
        free(Tzz_ZpXp_Y);
        free(Txx_ZpXp_Z);
        free(Txx_ZpXp_X);
        free(Txx_ZpXp_Y);
        free(Tyy_ZpXp_Z);
        free(Tyy_ZpXp_X);
        free(Tyy_ZpXp_Y);
        free(Tzx_ZpXp_Z);
        free(Tzx_ZpXp_X);
        free(Txy_ZpXp_X);
        free(Txy_ZpXp_Y);
        free(Tyz_ZpXp_Y);
        free(Tyz_ZpXp_Z);
    }
    if ((ConditionFaceZp==0)&&(ConditionFaceXm==0))
    {

        free(Vz_ZpXm_Z);
        free(Vz_ZpXm_X);
        free(Vz_ZpXm_Y);
        free(Vx_ZpXm_Z);
        free(Vx_ZpXm_X);
        free(Vx_ZpXm_Y);
        free(Vy_ZpXm_Z);
        free(Vy_ZpXm_X);
        free(Vy_ZpXm_Y);
        free(Tzz_ZpXm_Z);
        free(Tzz_ZpXm_X);
        free(Tzz_ZpXm_Y);
        free(Txx_ZpXm_Z);
        free(Txx_ZpXm_X);
        free(Txx_ZpXm_Y);
        free(Tyy_ZpXm_Z);
        free(Tyy_ZpXm_X);
        free(Tyy_ZpXm_Y);
        free(Tzx_ZpXm_Z);
        free(Tzx_ZpXm_X);
        free(Txy_ZpXm_X);
        free(Txy_ZpXm_Y);
        free(Tyz_ZpXm_Y);
        free(Tyz_ZpXm_Z);
    }
    if ((ConditionFaceZm==0)&&(ConditionFaceXp==0))
    {

        free(Vz_ZmXp_Z);
        free(Vz_ZmXp_X);
        free(Vz_ZmXp_Y);
        free(Vx_ZmXp_Z);
        free(Vx_ZmXp_X);
        free(Vx_ZmXp_Y);
        free(Vy_ZmXp_Z);
        free(Vy_ZmXp_X);
        free(Vy_ZmXp_Y);
        free(Tzz_ZmXp_Z);
        free(Tzz_ZmXp_X);
        free(Tzz_ZmXp_Y);
        free(Txx_ZmXp_Z);
        free(Txx_ZmXp_X);
        free(Txx_ZmXp_Y);
        free(Tyy_ZmXp_Z);
        free(Tyy_ZmXp_X);
        free(Tyy_ZmXp_Y);
        free(Tzx_ZmXp_Z);
        free(Tzx_ZmXp_X);
        free(Txy_ZmXp_X);
        free(Txy_ZmXp_Y);
        free(Tyz_ZmXp_Y);
        free(Tyz_ZmXp_Z);
    }
    if ((ConditionFaceZm==0)&&(ConditionFaceXm==0))
    {

        free(Vz_ZmXm_Z);
        free(Vz_ZmXm_X);
        free(Vz_ZmXm_Y);
        free(Vx_ZmXm_Z);
        free(Vx_ZmXm_X);
        free(Vx_ZmXm_Y);
        free(Vy_ZmXm_Z);
        free(Vy_ZmXm_X);
        free(Vy_ZmXm_Y);
        free(Tzz_ZmXm_Z);
        free(Tzz_ZmXm_X);
        free(Tzz_ZmXm_Y);
        free(Txx_ZmXm_Z);
        free(Txx_ZmXm_X);
        free(Txx_ZmXm_Y);
        free(Tyy_ZmXm_Z);
        free(Tyy_ZmXm_X);
        free(Tyy_ZmXm_Y);
        free(Tzx_ZmXm_Z);
        free(Tzx_ZmXm_X);
        free(Txy_ZmXm_X);
        free(Txy_ZmXm_Y);
        free(Tyz_ZmXm_Y);
        free(Tyz_ZmXm_Z);
    }

    /***************************************************************************/
    /************************ Liberation des aretes ****************************/
    /***************************************************************************/
    if ((ConditionFaceXp==0)&&(ConditionFaceYp==0)&&(ConditionFaceZp==0))
    {

        free(Vx_XpYpZp_X);
        free(Vx_XpYpZp_Y);
        free(Vx_XpYpZp_Z);
        free(Vy_XpYpZp_X);
        free(Vy_XpYpZp_Y);
        free(Vy_XpYpZp_Z);
        free(Vz_XpYpZp_X);
        free(Vz_XpYpZp_Y);
        free(Vz_XpYpZp_Z);
        free(Txx_XpYpZp_X);
        free(Txx_XpYpZp_Y);
        free(Txx_XpYpZp_Z);
        free(Tyy_XpYpZp_X);
        free(Tyy_XpYpZp_Y);
        free(Tyy_XpYpZp_Z);
        free(Tzz_XpYpZp_X);
        free(Tzz_XpYpZp_Y);
        free(Tzz_XpYpZp_Z);
        free(Txy_XpYpZp_X);
        free(Txy_XpYpZp_Y);
        free(Tyz_XpYpZp_Y);
        free(Tyz_XpYpZp_Z);
        free(Tzx_XpYpZp_Z);
        free(Tzx_XpYpZp_X);
    }
    if ((ConditionFaceXp==0)&&(ConditionFaceYp==0)&&(ConditionFaceZm==0))
    {

        free(Vx_XpYpZm_X);
        free(Vx_XpYpZm_Y);
        free(Vx_XpYpZm_Z);
        free(Vy_XpYpZm_X);
        free(Vy_XpYpZm_Y);
        free(Vy_XpYpZm_Z);
        free(Vz_XpYpZm_X);
        free(Vz_XpYpZm_Y);
        free(Vz_XpYpZm_Z);
        free(Txx_XpYpZm_X);
        free(Txx_XpYpZm_Y);
        free(Txx_XpYpZm_Z);
        free(Tyy_XpYpZm_X);
        free(Tyy_XpYpZm_Y);
        free(Tyy_XpYpZm_Z);
        free(Tzz_XpYpZm_X);
        free(Tzz_XpYpZm_Y);
        free(Tzz_XpYpZm_Z);
        free(Txy_XpYpZm_X);
        free(Txy_XpYpZm_Y);
        free(Tyz_XpYpZm_Y);
        free(Tyz_XpYpZm_Z);
        free(Tzx_XpYpZm_Z);
        free(Tzx_XpYpZm_X);
    }
    if ((ConditionFaceXp==0)&&(ConditionFaceYm==0)&&(ConditionFaceZp==0))
    {

        free(Vx_XpYmZp_X);
        free(Vx_XpYmZp_Y);
        free(Vx_XpYmZp_Z);
        free(Vy_XpYmZp_X);
        free(Vy_XpYmZp_Y);
        free(Vy_XpYmZp_Z);
        free(Vz_XpYmZp_X);
        free(Vz_XpYmZp_Y);
        free(Vz_XpYmZp_Z);
        free(Txx_XpYmZp_X);
        free(Txx_XpYmZp_Y);
        free(Txx_XpYmZp_Z);
        free(Tyy_XpYmZp_X);
        free(Tyy_XpYmZp_Y);
        free(Tyy_XpYmZp_Z);
        free(Tzz_XpYmZp_X);
        free(Tzz_XpYmZp_Y);
        free(Tzz_XpYmZp_Z);
        free(Txy_XpYmZp_X);
        free(Txy_XpYmZp_Y);
        free(Tyz_XpYmZp_Y);
        free(Tyz_XpYmZp_Z);
        free(Tzx_XpYmZp_Z);
        free(Tzx_XpYmZp_X);
    }
    if ((ConditionFaceXp==0)&&(ConditionFaceYm==0)&&(ConditionFaceZm==0))
    {

        free(Vx_XpYmZm_X);
        free(Vx_XpYmZm_Y);
        free(Vx_XpYmZm_Z);
        free(Vy_XpYmZm_X);
        free(Vy_XpYmZm_Y);
        free(Vy_XpYmZm_Z);
        free(Vz_XpYmZm_X);
        free(Vz_XpYmZm_Y);
        free(Vz_XpYmZm_Z);
        free(Txx_XpYmZm_X);
        free(Txx_XpYmZm_Y);
        free(Txx_XpYmZm_Z);
        free(Tyy_XpYmZm_X);
        free(Tyy_XpYmZm_Y);
        free(Tyy_XpYmZm_Z);
        free(Tzz_XpYmZm_X);
        free(Tzz_XpYmZm_Y);
        free(Tzz_XpYmZm_Z);
        free(Txy_XpYmZm_X);
        free(Txy_XpYmZm_Y);
        free(Tyz_XpYmZm_Y);
        free(Tyz_XpYmZm_Z);
        free(Tzx_XpYmZm_Z);
        free(Tzx_XpYmZm_X);
    }
    if ((ConditionFaceXm==0)&&(ConditionFaceYp==0)&&(ConditionFaceZp==0))
    {

        free(Vx_XmYpZp_X);
        free(Vx_XmYpZp_Y);
        free(Vx_XmYpZp_Z);
        free(Vy_XmYpZp_X);
        free(Vy_XmYpZp_Y);
        free(Vy_XmYpZp_Z);
        free(Vz_XmYpZp_X);
        free(Vz_XmYpZp_Y);
        free(Vz_XmYpZp_Z);
        free(Txx_XmYpZp_X);
        free(Txx_XmYpZp_Y);
        free(Txx_XmYpZp_Z);
        free(Tyy_XmYpZp_X);
        free(Tyy_XmYpZp_Y);
        free(Tyy_XmYpZp_Z);
        free(Tzz_XmYpZp_X);
        free(Tzz_XmYpZp_Y);
        free(Tzz_XmYpZp_Z);
        free(Txy_XmYpZp_X);
        free(Txy_XmYpZp_Y);
        free(Tyz_XmYpZp_Y);
        free(Tyz_XmYpZp_Z);
        free(Tzx_XmYpZp_Z);
        free(Tzx_XmYpZp_X);
    }
    if ((ConditionFaceXm==0)&&(ConditionFaceYp==0)&&(ConditionFaceZm==0))
    {

        free(Vx_XmYpZm_X);
        free(Vx_XmYpZm_Y);
        free(Vx_XmYpZm_Z);
        free(Vy_XmYpZm_X);
        free(Vy_XmYpZm_Y);
        free(Vy_XmYpZm_Z);
        free(Vz_XmYpZm_X);
        free(Vz_XmYpZm_Y);
        free(Vz_XmYpZm_Z);
        free(Txx_XmYpZm_X);
        free(Txx_XmYpZm_Y);
        free(Txx_XmYpZm_Z);
        free(Tyy_XmYpZm_X);
        free(Tyy_XmYpZm_Y);
        free(Tyy_XmYpZm_Z);
        free(Tzz_XmYpZm_X);
        free(Tzz_XmYpZm_Y);
        free(Tzz_XmYpZm_Z);
        free(Txy_XmYpZm_X);
        free(Txy_XmYpZm_Y);
        free(Tyz_XmYpZm_Y);
        free(Tyz_XmYpZm_Z);
        free(Tzx_XmYpZm_Z);
        free(Tzx_XmYpZm_X);
    }
    if ((ConditionFaceXm==0)&&(ConditionFaceYm==0)&&(ConditionFaceZp==0))
    {

        free(Vx_XmYmZp_X);
        free(Vx_XmYmZp_Y);
        free(Vx_XmYmZp_Z);
        free(Vy_XmYmZp_X);
        free(Vy_XmYmZp_Y);
        free(Vy_XmYmZp_Z);
        free(Vz_XmYmZp_X);
        free(Vz_XmYmZp_Y);
        free(Vz_XmYmZp_Z);
        free(Txx_XmYmZp_X);
        free(Txx_XmYmZp_Y);
        free(Txx_XmYmZp_Z);
        free(Tyy_XmYmZp_X);
        free(Tyy_XmYmZp_Y);
        free(Tyy_XmYmZp_Z);
        free(Tzz_XmYmZp_X);
        free(Tzz_XmYmZp_Y);
        free(Tzz_XmYmZp_Z);
        free(Txy_XmYmZp_X);
        free(Txy_XmYmZp_Y);
        free(Tyz_XmYmZp_Y);
        free(Tyz_XmYmZp_Z);
        free(Tzx_XmYmZp_Z);
        free(Tzx_XmYmZp_X);
    }
    if ((ConditionFaceXm==0)&&(ConditionFaceYm==0)&&(ConditionFaceZm==0))
    {

        free(Vx_XmYmZm_X);
        free(Vx_XmYmZm_Y);
        free(Vx_XmYmZm_Z);
        free(Vy_XmYmZm_X);
        free(Vy_XmYmZm_Y);
        free(Vy_XmYmZm_Z);
        free(Vz_XmYmZm_X);
        free(Vz_XmYmZm_Y);
        free(Vz_XmYmZm_Z);
        free(Txx_XmYmZm_X);
        free(Txx_XmYmZm_Y);
        free(Txx_XmYmZm_Z);
        free(Tyy_XmYmZm_X);
        free(Tyy_XmYmZm_Y);
        free(Tyy_XmYmZm_Z);
        free(Tzz_XmYmZm_X);
        free(Tzz_XmYmZm_Y);
        free(Tzz_XmYmZm_Z);
        free(Txy_XmYmZm_X);
        free(Txy_XmYmZm_Y);
        free(Tyz_XmYmZm_Y);
        free(Tyz_XmYmZm_Z);
        free(Tzx_XmYmZm_Z);
        free(Tzx_XmYmZm_X);
    }

    /***************************************************************************/
    /***************************************************************************/
    /************************ Liberation du reste... ***************************/
    /***************************************************************************/
    /***************************************************************************/

    free(TableLegerete);
    free(AttenuationPML);

    free(TableC11);
    free(TableC22);
    free(TableC33);
    free(TableC12);
    free(TableC23);
    free(TableC31);
    free(TableC44);
    free(TableC55);
    free(TableC66);

    free(Buffer1AireMax);
    free(Buffer2AireMax);
    free(Carte_Tampon);
    free(Buffer01AireIntermediaire);
    free(Buffer02AireIntermediaire);
    free(Buffer03AireIntermediaire);
    free(Buffer04AireIntermediaire);
    free(Buffer05AireIntermediaire);
    free(Buffer06AireIntermediaire);
    free(Buffer07AireIntermediaire);
    free(Buffer08AireIntermediaire);

    free(ListePlanV.TypePlan);
    free(ListePlanV.CotePlan);
    free(ListePlanEc.TypePlan);
    free(ListePlanEc.CotePlan);
    free(ListePlanVx.TypePlan);
    free(ListePlanVx.CotePlan);
    free(ListePlanVy.TypePlan);
    free(ListePlanVy.CotePlan);
    free(ListePlanVz.TypePlan);
    free(ListePlanVz.CotePlan);
    free(ListePlanTxx.TypePlan);
    free(ListePlanTxx.CotePlan);
    free(ListePlanTyy.TypePlan);
    free(ListePlanTyy.CotePlan);
    free(ListePlanTzz.TypePlan);
    free(ListePlanTzz.CotePlan);
    free(ListePlanTxy.TypePlan);
    free(ListePlanTxy.CotePlan);
    free(ListePlanTyz.TypePlan);
    free(ListePlanTyz.CotePlan);
    free(ListePlanTzx.TypePlan);
    free(ListePlanTzx.CotePlan);


    for(i=0; i<NbMatricesEmissionTxx; i++)
    {
        free(EmissionTxx[i].Signal);
    }
    free(EmissionTxx);

    for(i=0; i<NbMatricesEmissionTyy; i++)
    {
        free(EmissionTyy[i].Signal);
    }
    free(EmissionTyy);

    for(i=0; i<NbMatricesEmissionTzz; i++)
    {
        free(EmissionTzz[i].Signal);
    }
    free(EmissionTzz);

    for(i=0; i<NbMatricesEmissionTxy; i++)
    {
        free(EmissionTxy[i].Signal);
    }
    free(EmissionTxy);

    for(i=0; i<NbMatricesEmissionTyz; i++)
    {
        free(EmissionTyz[i].Signal);
    }
    free(EmissionTyz);

    for(i=0; i<NbMatricesEmissionTzx; i++)
    {
        free(EmissionTzx[i].Signal);
    }
    free(EmissionTzx);

    for(i=0; i<NbMatricesEmissionVx; i++)
    {
        free(EmissionVx[i].Signal);
    }
    free(EmissionVx);

    for(i=0; i<NbMatricesEmissionVy; i++)
    {
        free(EmissionVy[i].Signal);
    }
    free(EmissionVy);

    for(i=0; i<NbMatricesEmissionVz; i++)
    {
        free(EmissionVz[i].Signal);
    }
    free(EmissionVz);

    for(i=0; i<NbMatricesReceptionTxx; i++)
    {
        free(ReceptionTxx[i].Signaux);
    }
    free(ReceptionTxx);

    for(i=0; i<NbMatricesReceptionTyy; i++)
    {
        free(ReceptionTyy[i].Signaux);
    }
    free(ReceptionTyy);

    for(i=0; i<NbMatricesReceptionTzz; i++)
    {
        free(ReceptionTzz[i].Signaux);
    }
    free(ReceptionTzz);

    for(i=0; i<NbMatricesReceptionTxy; i++)
    {
        free(ReceptionTxy[i].Signaux);
    }
    free(ReceptionTxy);

    for(i=0; i<NbMatricesReceptionTyz; i++)
    {
        free(ReceptionTyz[i].Signaux);
    }
    free(ReceptionTyz);

    for(i=0; i<NbMatricesReceptionTzx; i++)
    {
        free(ReceptionTzx[i].Signaux);
    }
    free(ReceptionTzx);

    for(i=0; i<NbMatricesReceptionVx; i++)
    {
        free(ReceptionVx[i].Signaux);
    }
    free(ReceptionVx);

    for(i=0; i<NbMatricesReceptionVy; i++)
    {
        free(ReceptionVy[i].Signaux);
    }
    free(ReceptionVy);

    for(i=0; i<NbMatricesReceptionVz; i++)
    {
        free(ReceptionVz[i].Signaux);
    }
    free(ReceptionVz);


}

void SauvegardeRecepteurs(void)
{

    int NumeroMatrice;
    FILE *Fich=NULL;
    char NomFich[200];
    char NomComplet[200];
    int NbPoints;
    double DeltaT_Rcv;
    int IndiceTempsMaxRcv;
    DeltaT_Rcv=DeltaT*RcvDwnSmplng;
    IndiceTempsMaxRcv=1+((int)((IndiceTempsMax-1)/RcvDwnSmplng));

    for (NumeroMatrice=0; NumeroMatrice<NbMatricesReceptionTxx; NumeroMatrice++)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"");
        strcat(NomFich,ReceptionTxx[NumeroMatrice].Nom);
        strcat(NomFich,"_T11.rcv3D");
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        fwrite(&ReceptionTxx[NumeroMatrice].PlanI,sizeof(char),1,Fich);
        fwrite(&ReceptionTxx[NumeroMatrice].NbElementsJ,sizeof(int),1,Fich);
        fwrite(&ReceptionTxx[NumeroMatrice].NbElementsK,sizeof(int),1,Fich);
        fwrite(&ReceptionTxx[NumeroMatrice].XMat,sizeof(int),1,Fich);
        fwrite(&ReceptionTxx[NumeroMatrice].YMat,sizeof(int),1,Fich);
        fwrite(&ReceptionTxx[NumeroMatrice].ZMat,sizeof(int),1,Fich);
        fwrite(&ReceptionTxx[NumeroMatrice].LargeurJ,sizeof(int),1,Fich);
        fwrite(&ReceptionTxx[NumeroMatrice].LargeurK,sizeof(int),1,Fich);
        fwrite(&ReceptionTxx[NumeroMatrice].PitchJ,sizeof(int),1,Fich);
        fwrite(&ReceptionTxx[NumeroMatrice].PitchK,sizeof(int),1,Fich);
        fwrite(&H,sizeof(double),1,Fich);
        fwrite(&IndiceTempsMaxRcv,sizeof(int),1,Fich);
        fwrite(&DeltaT_Rcv,sizeof(double),1,Fich);
        NbPoints=ReceptionTxx[NumeroMatrice].NbElementsJ*ReceptionTxx[NumeroMatrice].NbElementsK*IndiceTempsMaxRcv;
        fwrite(ReceptionTxx[NumeroMatrice].Signaux,sizeof(double),NbPoints,Fich);
        fclose( Fich );


    }
    for (NumeroMatrice=0; NumeroMatrice<NbMatricesReceptionTyy; NumeroMatrice++)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"");
        strcat(NomFich,ReceptionTyy[NumeroMatrice].Nom);
        strcat(NomFich,"_T22.rcv3D");
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        fwrite(&ReceptionTyy[NumeroMatrice].PlanI,sizeof(char),1,Fich);
        fwrite(&ReceptionTyy[NumeroMatrice].NbElementsJ,sizeof(int),1,Fich);
        fwrite(&ReceptionTyy[NumeroMatrice].NbElementsK,sizeof(int),1,Fich);
        fwrite(&ReceptionTyy[NumeroMatrice].XMat,sizeof(int),1,Fich);
        fwrite(&ReceptionTyy[NumeroMatrice].YMat,sizeof(int),1,Fich);
        fwrite(&ReceptionTyy[NumeroMatrice].ZMat,sizeof(int),1,Fich);
        fwrite(&ReceptionTyy[NumeroMatrice].LargeurJ,sizeof(int),1,Fich);
        fwrite(&ReceptionTyy[NumeroMatrice].LargeurK,sizeof(int),1,Fich);
        fwrite(&ReceptionTyy[NumeroMatrice].PitchJ,sizeof(int),1,Fich);
        fwrite(&ReceptionTyy[NumeroMatrice].PitchK,sizeof(int),1,Fich);
        fwrite(&H,sizeof(double),1,Fich);
        fwrite(&IndiceTempsMaxRcv,sizeof(int),1,Fich);
        fwrite(&DeltaT_Rcv,sizeof(double),1,Fich);
        NbPoints=ReceptionTyy[NumeroMatrice].NbElementsJ*ReceptionTyy[NumeroMatrice].NbElementsK*IndiceTempsMaxRcv;
        fwrite(ReceptionTyy[NumeroMatrice].Signaux,sizeof(double),NbPoints,Fich);
        fclose( Fich );
    }
    for (NumeroMatrice=0; NumeroMatrice<NbMatricesReceptionTzz; NumeroMatrice++)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"");
        strcat(NomFich,ReceptionTzz[NumeroMatrice].Nom);
        strcat(NomFich,"_T33.rcv3D");
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        fwrite(&ReceptionTzz[NumeroMatrice].PlanI,sizeof(char),1,Fich);
        fwrite(&ReceptionTzz[NumeroMatrice].NbElementsJ,sizeof(int),1,Fich);
        fwrite(&ReceptionTzz[NumeroMatrice].NbElementsK,sizeof(int),1,Fich);
        fwrite(&ReceptionTzz[NumeroMatrice].XMat,sizeof(int),1,Fich);
        fwrite(&ReceptionTzz[NumeroMatrice].YMat,sizeof(int),1,Fich);
        fwrite(&ReceptionTzz[NumeroMatrice].ZMat,sizeof(int),1,Fich);
        fwrite(&ReceptionTzz[NumeroMatrice].LargeurJ,sizeof(int),1,Fich);
        fwrite(&ReceptionTzz[NumeroMatrice].LargeurK,sizeof(int),1,Fich);
        fwrite(&ReceptionTzz[NumeroMatrice].PitchJ,sizeof(int),1,Fich);
        fwrite(&ReceptionTzz[NumeroMatrice].PitchK,sizeof(int),1,Fich);
        fwrite(&H,sizeof(double),1,Fich);
        fwrite(&IndiceTempsMaxRcv,sizeof(int),1,Fich);
        fwrite(&DeltaT_Rcv,sizeof(double),1,Fich);
        NbPoints=ReceptionTzz[NumeroMatrice].NbElementsJ*ReceptionTzz[NumeroMatrice].NbElementsK*IndiceTempsMaxRcv;
        fwrite(ReceptionTzz[NumeroMatrice].Signaux,sizeof(double),NbPoints,Fich);
        fclose( Fich );
    }
    for (NumeroMatrice=0; NumeroMatrice<NbMatricesReceptionTxy; NumeroMatrice++)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"");
        strcat(NomFich,ReceptionTxy[NumeroMatrice].Nom);
        strcat(NomFich,"_T12.rcv3D");
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        fwrite(&ReceptionTxy[NumeroMatrice].PlanI,sizeof(char),1,Fich);
        fwrite(&ReceptionTxy[NumeroMatrice].NbElementsJ,sizeof(int),1,Fich);
        fwrite(&ReceptionTxy[NumeroMatrice].NbElementsK,sizeof(int),1,Fich);
        fwrite(&ReceptionTxy[NumeroMatrice].XMat,sizeof(int),1,Fich);
        fwrite(&ReceptionTxy[NumeroMatrice].YMat,sizeof(int),1,Fich);
        fwrite(&ReceptionTxy[NumeroMatrice].ZMat,sizeof(int),1,Fich);
        fwrite(&ReceptionTxy[NumeroMatrice].LargeurJ,sizeof(int),1,Fich);
        fwrite(&ReceptionTxy[NumeroMatrice].LargeurK,sizeof(int),1,Fich);
        fwrite(&ReceptionTxy[NumeroMatrice].PitchJ,sizeof(int),1,Fich);
        fwrite(&ReceptionTxy[NumeroMatrice].PitchK,sizeof(int),1,Fich);
        fwrite(&H,sizeof(double),1,Fich);
        fwrite(&IndiceTempsMaxRcv,sizeof(int),1,Fich);
        fwrite(&DeltaT_Rcv,sizeof(double),1,Fich);
        NbPoints=ReceptionTxy[NumeroMatrice].NbElementsJ*ReceptionTxy[NumeroMatrice].NbElementsK*IndiceTempsMaxRcv;
        fwrite(ReceptionTxy[NumeroMatrice].Signaux,sizeof(double),NbPoints,Fich);
        fclose( Fich );
    }
    for (NumeroMatrice=0; NumeroMatrice<NbMatricesReceptionTyz; NumeroMatrice++)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"");
        strcat(NomFich,ReceptionTyz[NumeroMatrice].Nom);
        strcat(NomFich,"_T23.rcv3D");
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        fwrite(&ReceptionTyz[NumeroMatrice].PlanI,sizeof(char),1,Fich);
        fwrite(&ReceptionTyz[NumeroMatrice].NbElementsJ,sizeof(int),1,Fich);
        fwrite(&ReceptionTyz[NumeroMatrice].NbElementsK,sizeof(int),1,Fich);
        fwrite(&ReceptionTyz[NumeroMatrice].XMat,sizeof(int),1,Fich);
        fwrite(&ReceptionTyz[NumeroMatrice].YMat,sizeof(int),1,Fich);
        fwrite(&ReceptionTyz[NumeroMatrice].ZMat,sizeof(int),1,Fich);
        fwrite(&ReceptionTyz[NumeroMatrice].LargeurJ,sizeof(int),1,Fich);
        fwrite(&ReceptionTyz[NumeroMatrice].LargeurK,sizeof(int),1,Fich);
        fwrite(&ReceptionTyz[NumeroMatrice].PitchJ,sizeof(int),1,Fich);
        fwrite(&ReceptionTyz[NumeroMatrice].PitchK,sizeof(int),1,Fich);
        fwrite(&H,sizeof(double),1,Fich);
        fwrite(&IndiceTempsMaxRcv,sizeof(int),1,Fich);
        fwrite(&DeltaT_Rcv,sizeof(double),1,Fich);
        NbPoints=ReceptionTyz[NumeroMatrice].NbElementsJ*ReceptionTyz[NumeroMatrice].NbElementsK*IndiceTempsMaxRcv;
        fwrite(ReceptionTyz[NumeroMatrice].Signaux,sizeof(double),NbPoints,Fich);
        fclose( Fich );
    }
    for (NumeroMatrice=0; NumeroMatrice<NbMatricesReceptionTzx; NumeroMatrice++)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"");
        strcat(NomFich,ReceptionTzx[NumeroMatrice].Nom);
        strcat(NomFich,"_T31.rcv3D");
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        fwrite(&ReceptionTzx[NumeroMatrice].PlanI,sizeof(char),1,Fich);
        fwrite(&ReceptionTzx[NumeroMatrice].NbElementsJ,sizeof(int),1,Fich);
        fwrite(&ReceptionTzx[NumeroMatrice].NbElementsK,sizeof(int),1,Fich);
        fwrite(&ReceptionTzx[NumeroMatrice].XMat,sizeof(int),1,Fich);
        fwrite(&ReceptionTzx[NumeroMatrice].YMat,sizeof(int),1,Fich);
        fwrite(&ReceptionTzx[NumeroMatrice].ZMat,sizeof(int),1,Fich);
        fwrite(&ReceptionTzx[NumeroMatrice].LargeurJ,sizeof(int),1,Fich);
        fwrite(&ReceptionTzx[NumeroMatrice].LargeurK,sizeof(int),1,Fich);
        fwrite(&ReceptionTzx[NumeroMatrice].PitchJ,sizeof(int),1,Fich);
        fwrite(&ReceptionTzx[NumeroMatrice].PitchK,sizeof(int),1,Fich);
        fwrite(&H,sizeof(double),1,Fich);
        fwrite(&IndiceTempsMaxRcv,sizeof(int),1,Fich);
        fwrite(&DeltaT_Rcv,sizeof(double),1,Fich);
        NbPoints=ReceptionTzx[NumeroMatrice].NbElementsJ*ReceptionTzx[NumeroMatrice].NbElementsK*IndiceTempsMaxRcv;
        fwrite(ReceptionTzx[NumeroMatrice].Signaux,sizeof(double),NbPoints,Fich);
        fclose( Fich );
    }
    for (NumeroMatrice=0; NumeroMatrice<NbMatricesReceptionVx; NumeroMatrice++)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"");
        strcat(NomFich,ReceptionVx[NumeroMatrice].Nom);
        strcat(NomFich,"_V1.rcv3D");
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        fwrite(&ReceptionVx[NumeroMatrice].PlanI,sizeof(char),1,Fich);
        fwrite(&ReceptionVx[NumeroMatrice].NbElementsJ,sizeof(int),1,Fich);
        fwrite(&ReceptionVx[NumeroMatrice].NbElementsK,sizeof(int),1,Fich);
        fwrite(&ReceptionVx[NumeroMatrice].XMat,sizeof(int),1,Fich);
        fwrite(&ReceptionVx[NumeroMatrice].YMat,sizeof(int),1,Fich);
        fwrite(&ReceptionVx[NumeroMatrice].ZMat,sizeof(int),1,Fich);
        fwrite(&ReceptionVx[NumeroMatrice].LargeurJ,sizeof(int),1,Fich);
        fwrite(&ReceptionVx[NumeroMatrice].LargeurK,sizeof(int),1,Fich);
        fwrite(&ReceptionVx[NumeroMatrice].PitchJ,sizeof(int),1,Fich);
        fwrite(&ReceptionVx[NumeroMatrice].PitchK,sizeof(int),1,Fich);
        fwrite(&H,sizeof(double),1,Fich);
        fwrite(&IndiceTempsMaxRcv,sizeof(int),1,Fich);
        fwrite(&DeltaT_Rcv,sizeof(double),1,Fich);
        NbPoints=ReceptionVx[NumeroMatrice].NbElementsJ*ReceptionVx[NumeroMatrice].NbElementsK*IndiceTempsMaxRcv;
        fwrite(ReceptionVx[NumeroMatrice].Signaux,sizeof(double),NbPoints,Fich);
        fclose( Fich );
    }
    for (NumeroMatrice=0; NumeroMatrice<NbMatricesReceptionVy; NumeroMatrice++)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"");
        strcat(NomFich,ReceptionVy[NumeroMatrice].Nom);
        strcat(NomFich,"_V2.rcv3D");
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        fwrite(&ReceptionVy[NumeroMatrice].PlanI,sizeof(char),1,Fich);
        fwrite(&ReceptionVy[NumeroMatrice].NbElementsJ,sizeof(int),1,Fich);
        fwrite(&ReceptionVy[NumeroMatrice].NbElementsK,sizeof(int),1,Fich);
        fwrite(&ReceptionVy[NumeroMatrice].XMat,sizeof(int),1,Fich);
        fwrite(&ReceptionVy[NumeroMatrice].YMat,sizeof(int),1,Fich);
        fwrite(&ReceptionVy[NumeroMatrice].ZMat,sizeof(int),1,Fich);
        fwrite(&ReceptionVy[NumeroMatrice].LargeurJ,sizeof(int),1,Fich);
        fwrite(&ReceptionVy[NumeroMatrice].LargeurK,sizeof(int),1,Fich);
        fwrite(&ReceptionVy[NumeroMatrice].PitchJ,sizeof(int),1,Fich);
        fwrite(&ReceptionVy[NumeroMatrice].PitchK,sizeof(int),1,Fich);
        fwrite(&H,sizeof(double),1,Fich);
        fwrite(&IndiceTempsMaxRcv,sizeof(int),1,Fich);
        fwrite(&DeltaT_Rcv,sizeof(double),1,Fich);
        NbPoints=ReceptionVy[NumeroMatrice].NbElementsJ*ReceptionVy[NumeroMatrice].NbElementsK*IndiceTempsMaxRcv;
        fwrite(ReceptionVy[NumeroMatrice].Signaux,sizeof(double),NbPoints,Fich);
        fclose( Fich );
    }
    for (NumeroMatrice=0; NumeroMatrice<NbMatricesReceptionVz; NumeroMatrice++)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"");
        strcat(NomFich,ReceptionVz[NumeroMatrice].Nom);
        strcat(NomFich,"_V3.rcv3D");
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        fwrite(&ReceptionVz[NumeroMatrice].PlanI,sizeof(char),1,Fich);
        fwrite(&ReceptionVz[NumeroMatrice].NbElementsJ,sizeof(int),1,Fich);
        fwrite(&ReceptionVz[NumeroMatrice].NbElementsK,sizeof(int),1,Fich);
        fwrite(&ReceptionVz[NumeroMatrice].XMat,sizeof(int),1,Fich);
        fwrite(&ReceptionVz[NumeroMatrice].YMat,sizeof(int),1,Fich);
        fwrite(&ReceptionVz[NumeroMatrice].ZMat,sizeof(int),1,Fich);
        fwrite(&ReceptionVz[NumeroMatrice].LargeurJ,sizeof(int),1,Fich);
        fwrite(&ReceptionVz[NumeroMatrice].LargeurK,sizeof(int),1,Fich);
        fwrite(&ReceptionVz[NumeroMatrice].PitchJ,sizeof(int),1,Fich);
        fwrite(&ReceptionVz[NumeroMatrice].PitchK,sizeof(int),1,Fich);
        fwrite(&H,sizeof(double),1,Fich);
        fwrite(&IndiceTempsMaxRcv,sizeof(int),1,Fich);
        fwrite(&DeltaT_Rcv,sizeof(double),1,Fich);
        NbPoints=ReceptionVz[NumeroMatrice].NbElementsJ*ReceptionVz[NumeroMatrice].NbElementsK*IndiceTempsMaxRcv;
        fwrite(ReceptionVz[NumeroMatrice].Signaux,sizeof(double),NbPoints,Fich);
        fclose( Fich );
    }
}


void SauvegardeSnapshotPlans(int NumeroPlan)
{

    FILE *Fich=NULL;
    char NomFich[200];
    char NomComplet[200];
    int j,k;
    int Boucle;
    int DimJ,DimK,DimX,DimY,DimZ;
    float Temp;
    double when;


    for(Boucle=0; Boucle<ListePlanV.NbPlan; Boucle++)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"""SliceV_X%c_%i_%0.3i.snp2D",ListePlanV.TypePlan[Boucle],ListePlanV.CotePlan[Boucle],NumeroPlan);
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        DimX=X;
        DimY=Y;
        DimZ=Z;
        switch(ListePlanV.TypePlan[Boucle])
        {
        case 'X':
        case '1':
            DimJ=DimY;
            DimK=DimZ;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    /*  Temp=(float)(sqrt(Vx[(ListePlanV.CotePlan[Boucle])*(Y)*(Z)+j*(Z)+k]*Vx[(ListePlanV.CotePlan[Boucle])*(Y)*(Z)+j*(Z)+k]+
                                        Vy[(ListePlanV.CotePlan[Boucle])*(Y+1)*(Z)+j*(Z)+k]*Vy[(ListePlanV.CotePlan[Boucle])*(Y+1)*(Z)+j*(Z)+k]+
                                        Vz[(ListePlanV.CotePlan[Boucle])*(Y)*(Z+1)+j*(Z+1)+k]*Vz[(ListePlanV.CotePlan[Boucle])*(Y)*(Z+1)+j*(Z+1)+k]));*/
                    Temp=(float)sqrt(
                             0.25*(Vx[(ListePlanV.CotePlan[Boucle])*(Y)*(Z)+j*(Z)+k]+Vx[(ListePlanV.CotePlan[Boucle]+1)*(Y)*(Z)+j*(Z)+k])
                             *(Vx[(ListePlanV.CotePlan[Boucle])*(Y)*(Z)+j*(Z)+k]+Vx[(ListePlanV.CotePlan[Boucle]+1)*(Y)*(Z)+j*(Z)+k])
                             +
                             0.25*(Vy[(ListePlanV.CotePlan[Boucle])*(Y+1)*(Z)+j*(Z)+k]+Vy[(ListePlanV.CotePlan[Boucle])*(Y+1)*(Z)+(j+1)*(Z)+k])
                             *(Vy[(ListePlanV.CotePlan[Boucle])*(Y+1)*(Z)+j*(Z)+k]+Vy[(ListePlanV.CotePlan[Boucle])*(Y+1)*(Z)+(j+1)*(Z)+k])
                             +
                             0.25*(Vz[(ListePlanV.CotePlan[Boucle])*(Y)*(Z+1)+j*(Z+1)+k]+Vz[(ListePlanV.CotePlan[Boucle])*(Y)*(Z+1)+j*(Z+1)+k+1])
                             *(Vz[(ListePlanV.CotePlan[Boucle])*(Y)*(Z+1)+j*(Z+1)+k]+Vz[(ListePlanV.CotePlan[Boucle])*(Y)*(Z+1)+j*(Z+1)+k+1])
                         );
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        case 'Y':
        case '2':
            DimJ=DimZ;
            DimK=DimX;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)sqrt(
                             0.25*(Vx[k*(Y)*(Z)+(ListePlanV.CotePlan[Boucle])*(Z)+j]+Vx[(k+1)*(Y)*(Z)+(ListePlanV.CotePlan[Boucle])*(Z)+j])
                             *(Vx[k*(Y)*(Z)+(ListePlanV.CotePlan[Boucle])*(Z)+j]+Vx[(k+1)*(Y)*(Z)+(ListePlanV.CotePlan[Boucle])*(Z)+j])
                             +
                             0.25*(Vy[k*(Y+1)*(Z)+(ListePlanV.CotePlan[Boucle])*(Z)+j]+Vy[k*(Y+1)*(Z)+(ListePlanV.CotePlan[Boucle]+1)*(Z)+j])
                             *(Vy[k*(Y+1)*(Z)+(ListePlanV.CotePlan[Boucle])*(Z)+j]+Vy[k*(Y+1)*(Z)+(ListePlanV.CotePlan[Boucle]+1)*(Z)+j])
                             +
                             0.25*(Vz[k*(Y)*(Z+1)+(ListePlanV.CotePlan[Boucle])*(Z+1)+j]+Vz[k*(Y)*(Z+1)+(ListePlanV.CotePlan[Boucle])*(Z+1)+j+1])
                             *(Vz[k*(Y)*(Z+1)+(ListePlanV.CotePlan[Boucle])*(Z+1)+j]+Vz[k*(Y)*(Z+1)+(ListePlanV.CotePlan[Boucle])*(Z+1)+j+1])
                         );
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        case 'Z':
        case '3':
            DimJ=DimX;
            DimK=DimY;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    /*                    Temp=(float)(sqrt(Vx[j*(Y)*(Z)+k*(Z)+(ListePlanV.CotePlan[Boucle])]*Vx[j*(Y)*(Z)+k*(Z)+(ListePlanV.CotePlan[Boucle])]+
                                                          Vy[j*(Y+1)*(Z)+k*(Z)+(ListePlanV.CotePlan[Boucle])]*Vy[j*(Y+1)*(Z)+k*(Z)+(ListePlanV.CotePlan[Boucle])]+
                                                          Vz[j*(Y)*(Z+1)+k*(Z+1)+(ListePlanV.CotePlan[Boucle])]*Vz[j*(Y)*(Z+1)+k*(Z+1)+(ListePlanV.CotePlan[Boucle])]));*/
                    Temp=(float)sqrt(
                             0.25*(Vx[j*(Y)*(Z)+k*(Z)+(ListePlanV.CotePlan[Boucle])]+Vx[(j+1)*(Y)*(Z)+k*(Z)+(ListePlanV.CotePlan[Boucle])])
                             *(Vx[j*(Y)*(Z)+k*(Z)+(ListePlanV.CotePlan[Boucle])]+Vx[(j+1)*(Y)*(Z)+k*(Z)+(ListePlanV.CotePlan[Boucle])])
                             +
                             0.25*(Vy[j*(Y+1)*(Z)+k*(Z)+(ListePlanV.CotePlan[Boucle])]+Vy[j*(Y+1)*(Z)+(k+1)*(Z)+(ListePlanV.CotePlan[Boucle])])
                             *(Vy[j*(Y+1)*(Z)+k*(Z)+(ListePlanV.CotePlan[Boucle])]+Vy[j*(Y+1)*(Z)+(k+1)*(Z)+(ListePlanV.CotePlan[Boucle])])
                             +
                             0.25*(Vz[j*(Y)*(Z+1)+k*(Z+1)+(ListePlanV.CotePlan[Boucle])]+Vz[j*(Y)*(Z+1)+k*(Z+1)+(ListePlanV.CotePlan[Boucle]+1)])
                             *(Vz[j*(Y)*(Z+1)+k*(Z+1)+(ListePlanV.CotePlan[Boucle])]+Vz[j*(Y)*(Z+1)+k*(Z+1)+(ListePlanV.CotePlan[Boucle]+1)])
                         );

                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        }
        fclose(Fich);
    }

    for(Boucle=0; Boucle<ListePlanEc.NbPlan; Boucle++)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"""SliceEc_X%c_%i_%0.3i.snp2D",ListePlanEc.TypePlan[Boucle],ListePlanEc.CotePlan[Boucle],NumeroPlan);
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        DimX=X;
        DimY=Y;
        DimZ=Z;
        switch(ListePlanEc.TypePlan[Boucle])
        {
        case 'X':
        case '1':
            DimJ=DimY;
            DimK=DimZ;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)sqrt(
                             0.25*(Vx[(ListePlanV.CotePlan[Boucle])*(Y)*(Z)+j*(Z)+k]+Vx[(ListePlanV.CotePlan[Boucle]+1)*(Y)*(Z)+j*(Z)+k])
                             *(Vx[(ListePlanV.CotePlan[Boucle])*(Y)*(Z)+j*(Z)+k]+Vx[(ListePlanV.CotePlan[Boucle]+1)*(Y)*(Z)+j*(Z)+k])
                             +
                             0.25*(Vy[(ListePlanV.CotePlan[Boucle])*(Y+1)*(Z)+j*(Z)+k]+Vy[(ListePlanV.CotePlan[Boucle])*(Y+1)*(Z)+(j+1)*(Z)+k])
                             *(Vy[(ListePlanV.CotePlan[Boucle])*(Y+1)*(Z)+j*(Z)+k]+Vy[(ListePlanV.CotePlan[Boucle])*(Y+1)*(Z)+(j+1)*(Z)+k])
                             +
                             0.25*(Vz[(ListePlanV.CotePlan[Boucle])*(Y)*(Z+1)+j*(Z+1)+k]+Vz[(ListePlanV.CotePlan[Boucle])*(Y)*(Z+1)+j*(Z+1)+k+1])
                             *(Vz[(ListePlanV.CotePlan[Boucle])*(Y)*(Z+1)+j*(Z+1)+k]+Vz[(ListePlanV.CotePlan[Boucle])*(Y)*(Z+1)+j*(Z+1)+k+1])
                         );

                    Temp=(float)(0.5/TableLegerete[Carte[(ListePlanEc.CotePlan[Boucle])*(Y)*(Z)+(j)*(Z)+(k)]])*Temp*Temp;
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        case 'Y':
        case '2':
            DimJ=DimZ;
            DimK=DimX;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);

            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)sqrt(
                             0.25*(Vx[k*(Y)*(Z)+(ListePlanV.CotePlan[Boucle])*(Z)+j]+Vx[(k+1)*(Y)*(Z)+(ListePlanV.CotePlan[Boucle])*(Z)+j])
                             *(Vx[k*(Y)*(Z)+(ListePlanV.CotePlan[Boucle])*(Z)+j]+Vx[(k+1)*(Y)*(Z)+(ListePlanV.CotePlan[Boucle])*(Z)+j])
                             +
                             0.25*(Vy[k*(Y+1)*(Z)+(ListePlanV.CotePlan[Boucle])*(Z)+j]+Vy[k*(Y+1)*(Z)+(ListePlanV.CotePlan[Boucle]+1)*(Z)+j])
                             *(Vy[k*(Y+1)*(Z)+(ListePlanV.CotePlan[Boucle])*(Z)+j]+Vy[k*(Y+1)*(Z)+(ListePlanV.CotePlan[Boucle]+1)*(Z)+j])
                             +
                             0.25*(Vz[k*(Y)*(Z+1)+(ListePlanV.CotePlan[Boucle])*(Z+1)+j]+Vz[k*(Y)*(Z+1)+(ListePlanV.CotePlan[Boucle])*(Z+1)+j+1])
                             *(Vz[k*(Y)*(Z+1)+(ListePlanV.CotePlan[Boucle])*(Z+1)+j]+Vz[k*(Y)*(Z+1)+(ListePlanV.CotePlan[Boucle])*(Z+1)+j+1])
                         );
                    Temp=(float)(0.5/TableLegerete[Carte[(k)*(Y)*(Z)+(ListePlanEc.CotePlan[Boucle])*(Z)+(j)]])*Temp*Temp;
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        case 'Z':
        case '3':
            DimJ=DimX;
            DimK=DimY;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)sqrt(
                             0.25*(Vx[j*(Y)*(Z)+k*(Z)+(ListePlanV.CotePlan[Boucle])]+Vx[(j+1)*(Y)*(Z)+k*(Z)+(ListePlanV.CotePlan[Boucle])])
                             *(Vx[j*(Y)*(Z)+k*(Z)+(ListePlanV.CotePlan[Boucle])]+Vx[(j+1)*(Y)*(Z)+k*(Z)+(ListePlanV.CotePlan[Boucle])])
                             +
                             0.25*(Vy[j*(Y+1)*(Z)+k*(Z)+(ListePlanV.CotePlan[Boucle])]+Vy[j*(Y+1)*(Z)+(k+1)*(Z)+(ListePlanV.CotePlan[Boucle])])
                             *(Vy[j*(Y+1)*(Z)+k*(Z)+(ListePlanV.CotePlan[Boucle])]+Vy[j*(Y+1)*(Z)+(k+1)*(Z)+(ListePlanV.CotePlan[Boucle])])
                             +
                             0.25*(Vz[j*(Y)*(Z+1)+k*(Z+1)+(ListePlanV.CotePlan[Boucle])]+Vz[j*(Y)*(Z+1)+k*(Z+1)+(ListePlanV.CotePlan[Boucle]+1)])
                             *(Vz[j*(Y)*(Z+1)+k*(Z+1)+(ListePlanV.CotePlan[Boucle])]+Vz[j*(Y)*(Z+1)+k*(Z+1)+(ListePlanV.CotePlan[Boucle]+1)])
                         );
                    Temp=(float)(0.5/TableLegerete[Carte[(j)*(Y)*(Z)+(k)*(Z)+(ListePlanEc.CotePlan[Boucle])]])*Temp*Temp;
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        }
        fclose(Fich);
    }

    for(Boucle=0; Boucle<ListePlanVx.NbPlan; Boucle++)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"""SliceV1_X%c_%i_%0.3i.snp2D",ListePlanVx.TypePlan[Boucle],ListePlanVx.CotePlan[Boucle],NumeroPlan);
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        DimX=X+1;
        DimY=Y;
        DimZ=Z;
        switch(ListePlanVx.TypePlan[Boucle])
        {
        case 'X':
        case '1':
            DimJ=DimY;
            DimK=DimZ;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)(Vx[(ListePlanVx.CotePlan[Boucle])*DimY*DimZ+(j)*DimZ+k]);
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        case 'Y':
        case '2':
            DimJ=DimZ;
            DimK=X;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)(Vx[(k)*DimY*DimZ+(ListePlanVx.CotePlan[Boucle])*DimZ+j]);
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        case 'Z':
        case '3':
            DimJ=X;
            DimK=DimY;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)(Vx[(j)*DimY*DimZ+(k)*DimZ+ListePlanVx.CotePlan[Boucle]]);
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        }
        fclose(Fich);
    }
    for(Boucle=0; Boucle<ListePlanVy.NbPlan; Boucle++)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"""SliceV2_X%c_%i_%0.3i.snp2D",ListePlanVy.TypePlan[Boucle],ListePlanVy.CotePlan[Boucle],NumeroPlan);
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        DimX=X;
        DimY=Y+1;
        DimZ=Z;
        switch(ListePlanVy.TypePlan[Boucle])
        {
        case 'X':
        case '1':
            DimJ=DimY;
            DimK=DimZ;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)(Vy[(ListePlanVy.CotePlan[Boucle])*DimY*DimZ+(j)*DimZ+k]);
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        case 'Y':
        case '2':
            DimJ=DimZ;
            DimK=X;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)(Vy[(k)*DimY*DimZ+(ListePlanVy.CotePlan[Boucle])*DimZ+j]);
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        case 'Z':
        case '3':
            DimJ=X;
            DimK=DimY;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)(Vy[(j)*DimY*DimZ+(k)*DimZ+ListePlanVy.CotePlan[Boucle]]);
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        }
        fclose(Fich);
    }
    for(Boucle=0; Boucle<ListePlanVz.NbPlan; Boucle++)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"""SliceV3_X%c_%i_%0.3i.snp2D",ListePlanVz.TypePlan[Boucle],ListePlanVz.CotePlan[Boucle],NumeroPlan);
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        DimX=X;
        DimY=Y;
        DimZ=Z+1;
        switch(ListePlanVz.TypePlan[Boucle])
        {
        case 'X':
        case '1':
            DimJ=DimY;
            DimK=DimZ;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)(Vz[(ListePlanVz.CotePlan[Boucle])*DimY*DimZ+(j)*DimZ+k]);
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        case 'Y':
        case '2':
            DimJ=DimZ;
            DimK=DimX;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)(Vz[(k)*DimY*DimZ+(ListePlanVz.CotePlan[Boucle])*DimZ+j]);
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        case 'Z':
        case '3':
            DimJ=DimX;
            DimK=DimY;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)(Vz[(j)*DimY*DimZ+(k)*DimZ+ListePlanVz.CotePlan[Boucle]]);
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        }
        fclose(Fich);
    }
    for(Boucle=0; Boucle<ListePlanTxx.NbPlan; Boucle++)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"""SliceT11_X%c_%i_%0.3i.snp2D",ListePlanTxx.TypePlan[Boucle],ListePlanTxx.CotePlan[Boucle],NumeroPlan);
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        DimX=X;
        DimY=Y;
        DimZ=Z;
        switch(ListePlanTxx.TypePlan[Boucle])
        {
        case 'X':
        case '1':
            DimJ=DimY;
            DimK=DimZ;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)(Txx[(ListePlanTxx.CotePlan[Boucle])*DimY*DimZ+(j)*DimZ+k]);
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        case 'Y':
        case '2':
            DimJ=DimZ;
            DimK=DimX;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)(Txx[(k)*DimY*DimZ+(ListePlanTxx.CotePlan[Boucle])*DimZ+j]);
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        case 'Z':
        case '3':
            DimJ=DimX;
            DimK=DimY;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)(Txx[(j)*DimY*DimZ+(k)*DimZ+ListePlanTxx.CotePlan[Boucle]]);
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        }
        fclose(Fich);
    }
    for(Boucle=0; Boucle<ListePlanTyy.NbPlan; Boucle++)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"""SliceT22_X%c_%i_%0.3i.snp2D",ListePlanTyy.TypePlan[Boucle],ListePlanTyy.CotePlan[Boucle],NumeroPlan);
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        DimX=X;
        DimY=Y;
        DimZ=Z;
        switch(ListePlanTyy.TypePlan[Boucle])
        {
        case 'X':
        case '1':
            DimJ=DimY;
            DimK=DimZ;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)(Tyy[(ListePlanTyy.CotePlan[Boucle])*DimY*DimZ+(j)*DimZ+k]);
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        case 'Y':
        case '2':
            DimJ=DimZ;
            DimK=X;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)(Tyy[(k)*DimY*DimZ+(ListePlanTyy.CotePlan[Boucle])*DimZ+j]);
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        case 'Z':
        case '3':
            DimJ=X;
            DimK=DimY;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)(Tyy[(j)*DimY*DimZ+(k)*DimZ+ListePlanTyy.CotePlan[Boucle]]);
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        }
        fclose(Fich);
    }
    for(Boucle=0; Boucle<ListePlanTzz.NbPlan; Boucle++)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"""SliceT33_X%c_%i_%0.3i.snp2D",ListePlanTzz.TypePlan[Boucle],ListePlanTzz.CotePlan[Boucle],NumeroPlan);
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        DimX=X;
        DimY=Y;
        DimZ=Z;
        switch(ListePlanTzz.TypePlan[Boucle])
        {
        case 'X':
        case '1':
            DimJ=DimY;
            DimK=DimZ;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)(Tzz[(ListePlanTzz.CotePlan[Boucle])*DimY*DimZ+(j)*DimZ+k]);
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        case 'Y':
        case '2':
            DimJ=DimZ;
            DimK=X;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)(Tzz[(k)*DimY*DimZ+(ListePlanTzz.CotePlan[Boucle])*DimZ+j]);
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        case 'Z':
        case '3':
            DimJ=X;
            DimK=DimY;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)(Tzz[(j)*DimY*DimZ+(k)*DimZ+ListePlanTzz.CotePlan[Boucle]]);
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        }
        fclose(Fich);
    }
    for(Boucle=0; Boucle<ListePlanTxy.NbPlan; Boucle++)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"""SliceT12_X%c_%i_%0.3i.snp2D",ListePlanTxy.TypePlan[Boucle],ListePlanTxy.CotePlan[Boucle],NumeroPlan);
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        DimX=X+1;
        DimY=Y+1;
        DimZ=Z;
        switch(ListePlanTxy.TypePlan[Boucle])
        {
        case 'X':
        case '1':
            DimJ=DimY;
            DimK=DimZ;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)(Txy[(ListePlanTxy.CotePlan[Boucle])*DimY*DimZ+(j)*DimZ+k]);
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        case 'Y':
        case '2':
            DimJ=DimZ;
            DimK=X;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)(Txy[(k)*DimY*DimZ+(ListePlanTxy.CotePlan[Boucle])*DimZ+j]);
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        case 'Z':
        case '3':
            DimJ=X;
            DimK=DimY;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)(Txy[(j)*DimY*DimZ+(k)*DimZ+ListePlanTxy.CotePlan[Boucle]]);
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        }
        fclose(Fich);
    }
    for(Boucle=0; Boucle<ListePlanTyz.NbPlan; Boucle++)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"""SliceT23_X%c_%i_%0.3i.snp2D",ListePlanTyz.TypePlan[Boucle],ListePlanTyz.CotePlan[Boucle],NumeroPlan);
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        DimX=X;
        DimY=Y+1;
        DimZ=Z+1;
        switch(ListePlanTyz.TypePlan[Boucle])
        {
        case 'X':
        case '1':
            DimJ=DimY;
            DimK=DimZ;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)(Tyz[(ListePlanTyz.CotePlan[Boucle])*DimY*DimZ+(j)*DimZ+k]);
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        case 'Y':
        case '2':
            DimJ=DimZ;
            DimK=X;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)(Tyz[(k)*DimY*DimZ+(ListePlanTyz.CotePlan[Boucle])*DimZ+j]);
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        case 'Z':
        case '3':
            DimJ=X;
            DimK=DimY;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)(Tyz[(j)*DimY*DimZ+(k)*DimZ+ListePlanTyz.CotePlan[Boucle]]);
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        }
        fclose(Fich);
    }
    for(Boucle=0; Boucle<ListePlanTzx.NbPlan; Boucle++)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"""SliceT31_X%c_%i_%0.3i.snp2D",ListePlanTzx.TypePlan[Boucle],ListePlanTzx.CotePlan[Boucle],NumeroPlan);
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        DimX=X+1;
        DimY=Y;
        DimZ=Z+1;
        switch(ListePlanTzx.TypePlan[Boucle])
        {
        case 'X':
        case '1':
            DimJ=DimY;
            DimK=DimZ;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)(Tzx[(ListePlanTzx.CotePlan[Boucle])*DimY*DimZ+(j)*DimZ+k]);
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        case 'Y':
        case '2':
            DimJ=DimZ;
            DimK=X;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)(Tzx[(k)*DimY*DimZ+(ListePlanTzx.CotePlan[Boucle])*DimZ+j]);
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        case 'Z':
        case '3':
            DimJ=X;
            DimK=DimY;
            fwrite(&DimJ,sizeof(int),1,Fich);
            fwrite(&DimK,sizeof(int),1,Fich);
            when = ((double)IndiceTemps)*DeltaT;
            fwrite(&when,sizeof(double),1,Fich);
            fwrite(&H,sizeof(double),1,Fich);
            fwrite(&DeltaT,sizeof(double),1,Fich);
            for(j=0; j<DimJ; j++)
            {
                for(k=0; k<DimK; k++)
                {
                    Temp=(float)(Tzx[(j)*DimY*DimZ+(k)*DimZ+ListePlanTzx.CotePlan[Boucle]]);
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
            break;
        }
        fclose(Fich);
    }
}

void SauvegardeSnapshotBlocs(int NumeroBloc)
{

    FILE *Fich=NULL;
    char NomFich[200];
    char NomComplet[200];
    int i,j,k;
    int DimX,DimY,DimZ;
    float Temp;
    double when;

    if(SnapshotTyy)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"""T22_%0.3i.snp3D",NumeroBloc);
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        DimX=X;
        DimY=Y;
        DimZ=Z;
        fwrite(&X,sizeof(int),1,Fich);
        fwrite(&Y,sizeof(int),1,Fich);
        fwrite(&Z,sizeof(int),1,Fich);
        when = ((double)IndiceTemps)*DeltaT;
        fwrite(&when,sizeof(double),1,Fich);
        fwrite(&H,sizeof(double),1,Fich);
        fwrite(&DeltaT,sizeof(double),1,Fich);
        for(i=0; i<(DimX)*(DimY)*(DimZ); i++)
        {
            Temp=(float)(Tyy[i]);
            fwrite(&Temp,sizeof(float),1,Fich);
        }
        fclose( Fich );
    }
    if(SnapshotTxx)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"""T11_%0.3i.snp3D",NumeroBloc);
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        DimX=X;
        DimY=Y;
        DimZ=Z;
        fwrite(&X,sizeof(int),1,Fich);
        fwrite(&Y,sizeof(int),1,Fich);
        fwrite(&Z,sizeof(int),1,Fich);
        when = ((double)IndiceTemps)*DeltaT;
        fwrite(&when,sizeof(double),1,Fich);
        fwrite(&H,sizeof(double),1,Fich);
        fwrite(&DeltaT,sizeof(double),1,Fich);
        for(i=0; i<(DimX)*(DimY)*(DimZ); i++)
        {
            Temp=(float)(Txx[i]);
            fwrite(&Temp,sizeof(float),1,Fich);
        }
        fclose( Fich );
    }
    if(SnapshotTzz)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"""T33_%0.3i.snp3D",NumeroBloc);
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        DimX=X;
        DimY=Y;
        DimZ=Z;
        fwrite(&X,sizeof(int),1,Fich);
        fwrite(&Y,sizeof(int),1,Fich);
        fwrite(&Z,sizeof(int),1,Fich);
        when = ((double)IndiceTemps)*DeltaT;
        fwrite(&when,sizeof(double),1,Fich);
        fwrite(&H,sizeof(double),1,Fich);
        fwrite(&DeltaT,sizeof(double),1,Fich);
        for(i=0; i<(DimX)*(DimY)*(DimZ); i++)
        {
            Temp=(float)(Tzz[i]);
            fwrite(&Temp,sizeof(float),1,Fich);
        }
        fclose( Fich );
    }
    if(SnapshotTxy)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"""T12_%0.3i.snp3D",NumeroBloc);
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        DimX=X+1;
        DimY=Y+1;
        DimZ=Z;
        fwrite(&DimX,sizeof(int),1,Fich);
        fwrite(&DimY,sizeof(int),1,Fich);
        fwrite(&DimZ,sizeof(int),1,Fich);
        when = ((double)IndiceTemps)*DeltaT;
        fwrite(&when,sizeof(double),1,Fich);
        fwrite(&H,sizeof(double),1,Fich);
        fwrite(&DeltaT,sizeof(double),1,Fich);
        for(i=0; i<(DimX)*(DimY)*(DimZ); i++)
        {
            Temp=(float)(Txy[i]);
            fwrite(&Temp,sizeof(float),1,Fich);
        }
        fclose( Fich );
    }
    if(SnapshotTyz)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"""T23_%0.3i.snp3D",NumeroBloc);
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        DimX=X;
        DimY=Y+1;
        DimZ=Z+1;
        fwrite(&DimX,sizeof(int),1,Fich);
        fwrite(&DimY,sizeof(int),1,Fich);
        fwrite(&DimZ,sizeof(int),1,Fich);
        when = ((double)IndiceTemps)*DeltaT;
        fwrite(&when,sizeof(double),1,Fich);
        fwrite(&H,sizeof(double),1,Fich);
        fwrite(&DeltaT,sizeof(double),1,Fich);
        for(i=0; i<(DimX)*(DimY)*(DimZ); i++)
        {
            Temp=(float)(Tyz[i]);
            fwrite(&Temp,sizeof(float),1,Fich);
        }
        fclose( Fich );
    }
    if(SnapshotTzx)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"""T31_%0.3i.snp3D",NumeroBloc);
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        DimX=X+1;
        DimY=Y;
        DimZ=Z+1;
        fwrite(&DimX,sizeof(int),1,Fich);
        fwrite(&DimY,sizeof(int),1,Fich);
        fwrite(&DimZ,sizeof(int),1,Fich);
        when = ((double)IndiceTemps)*DeltaT;
        fwrite(&when,sizeof(double),1,Fich);
        fwrite(&H,sizeof(double),1,Fich);
        fwrite(&DeltaT,sizeof(double),1,Fich);
        for(i=0; i<(DimX)*(DimY)*(DimZ); i++)
        {
            Temp=(float)(Tzx[i]);
            fwrite(&Temp,sizeof(float),1,Fich);
        }
        fclose( Fich );
    }
    if(SnapshotVx)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"""V1_%0.3i.snp3D",NumeroBloc);
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        DimX=X+1;
        DimY=Y;
        DimZ=Z;
        fwrite(&DimX,sizeof(int),1,Fich);
        fwrite(&DimY,sizeof(int),1,Fich);
        fwrite(&DimZ,sizeof(int),1,Fich);
        when = ((double)IndiceTemps)*DeltaT;
        fwrite(&when,sizeof(double),1,Fich);
        fwrite(&H,sizeof(double),1,Fich);
        fwrite(&DeltaT,sizeof(double),1,Fich);
        for(i=0; i<(DimX)*(DimY)*(DimZ); i++)
        {
            Temp=(float)(Vx[i]);
            fwrite(&Temp,sizeof(float),1,Fich);
        }
        fclose( Fich );
    }

    if(SnapshotUx)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"""U1_%0.3i.snp3D",NumeroBloc);
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        DimX=X+1;
        DimY=Y;
        DimZ=Z;
        fwrite(&DimX,sizeof(int),1,Fich);
        fwrite(&DimY,sizeof(int),1,Fich);
        fwrite(&DimZ,sizeof(int),1,Fich);
        when = ((double)IndiceTemps)*DeltaT;
        fwrite(&when,sizeof(double),1,Fich);
        fwrite(&H,sizeof(double),1,Fich);
        fwrite(&DeltaT,sizeof(double),1,Fich);
        for(i=0; i<(DimX)*(DimY)*(DimZ); i++)
        {
            Temp=(float)(Ux[i]);
            fwrite(&Temp,sizeof(float),1,Fich);
        }
        fclose( Fich );
    }

    if(SnapshotVy)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"""V2_%0.3i.snp3D",NumeroBloc);
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        DimX=X;
        DimY=Y+1;
        DimZ=Z;
        fwrite(&DimX,sizeof(int),1,Fich);
        fwrite(&DimY,sizeof(int),1,Fich);
        fwrite(&DimZ,sizeof(int),1,Fich);
        when = ((double)IndiceTemps)*DeltaT;
        fwrite(&when,sizeof(double),1,Fich);
        fwrite(&H,sizeof(double),1,Fich);
        fwrite(&DeltaT,sizeof(double),1,Fich);
        for(i=0; i<(DimX)*(DimY)*(DimZ); i++)
        {
            Temp=(float)(Vy[i]);
            fwrite(&Temp,sizeof(float),1,Fich);
        }
        fclose( Fich );
    }
    if(SnapshotUy)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"""U2_%0.3i.snp3D",NumeroBloc);
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        DimX=X;
        DimY=Y+1;
        DimZ=Z;
        fwrite(&DimX,sizeof(int),1,Fich);
        fwrite(&DimY,sizeof(int),1,Fich);
        fwrite(&DimZ,sizeof(int),1,Fich);
        when = ((double)IndiceTemps)*DeltaT;
        fwrite(&when,sizeof(double),1,Fich);
        fwrite(&H,sizeof(double),1,Fich);
        fwrite(&DeltaT,sizeof(double),1,Fich);
        for(i=0; i<(DimX)*(DimY)*(DimZ); i++)
        {
            Temp=(float)(Uy[i]);
            fwrite(&Temp,sizeof(float),1,Fich);
        }
        fclose( Fich );
    }
    if(SnapshotVz)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"""V3_%0.3i.snp3D",NumeroBloc);
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        DimX=X;
        DimY=Y;
        DimZ=Z+1;
        fwrite(&DimX,sizeof(int),1,Fich);
        fwrite(&DimY,sizeof(int),1,Fich);
        fwrite(&DimZ,sizeof(int),1,Fich);
        when = ((double)IndiceTemps)*DeltaT;
        fwrite(&when,sizeof(double),1,Fich);
        fwrite(&H,sizeof(double),1,Fich);
        fwrite(&DeltaT,sizeof(double),1,Fich);
        for(i=0; i<(DimX)*(DimY)*(DimZ); i++)
        {
            Temp=(float)(Vz[i]);
            fwrite(&Temp,sizeof(float),1,Fich);
        }
        fclose( Fich );
    }
    if(SnapshotUz)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"""U3_%0.3i.snp3D",NumeroBloc);
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        DimX=X;
        DimY=Y;
        DimZ=Z+1;
        fwrite(&DimX,sizeof(int),1,Fich);
        fwrite(&DimY,sizeof(int),1,Fich);
        fwrite(&DimZ,sizeof(int),1,Fich);
        when = ((double)IndiceTemps)*DeltaT;
        fwrite(&when,sizeof(double),1,Fich);
        fwrite(&H,sizeof(double),1,Fich);
        fwrite(&DeltaT,sizeof(double),1,Fich);
        for(i=0; i<(DimX)*(DimY)*(DimZ); i++)
        {
            Temp=(float)(Uz[i]);
            fwrite(&Temp,sizeof(float),1,Fich);
        }
        fclose( Fich );
    }
    if(SnapshotV)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"""V_%0.3i.snp3D",NumeroBloc);
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        DimX=X;
        DimY=Y;
        DimZ=Z;
        fwrite(&DimX,sizeof(int),1,Fich);
        fwrite(&DimY,sizeof(int),1,Fich);
        fwrite(&DimZ,sizeof(int),1,Fich);
        when = ((double)IndiceTemps)*DeltaT;
        fwrite(&when,sizeof(double),1,Fich);
        fwrite(&H,sizeof(double),1,Fich);
        fwrite(&DeltaT,sizeof(double),1,Fich);
        for(i=0; i<DimX; i++)
        {
            for(j=0; j<DimY; j++)
            {
                for(k=0; k<DimZ; k++)
                {
                    Temp=(float)(sqrt(Vx[i*(Y)*(Z)+j*(Z)+k]*Vx[i*(Y)*(Z)+j*(Z)+k]+
                                      Vy[i*(Y+1)*(Z)+j*(Z)+k]*Vy[i*(Y+1)*(Z)+j*(Z)+k]+
                                      Vz[i*(Y)*(Z+1)+j*(Z+1)+k]*Vz[i*(Y)*(Z+1)+j*(Z+1)+k]));
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
        }
        fclose( Fich );
    }
    if(SnapshotEc)
    {
        sprintf(NomComplet,Repertoire);
        sprintf(NomFich,"""Ec_%0.3i.snp3D",NumeroBloc);
        strcat(NomComplet,NomFich);
        Fich = fopen(NomComplet, "w+b" );
        DimX=X;
        DimY=Y;
        DimZ=Z;
        fwrite(&DimX,sizeof(int),1,Fich);
        fwrite(&DimY,sizeof(int),1,Fich);
        fwrite(&DimZ,sizeof(int),1,Fich);
        when = ((double)IndiceTemps)*DeltaT;
        fwrite(&when,sizeof(double),1,Fich);
        fwrite(&H,sizeof(double),1,Fich);
        fwrite(&DeltaT,sizeof(double),1,Fich);
        for(i=0; i<DimX; i++)
        {
            for(j=0; j<DimY; j++)
            {
                for(k=0; k<DimZ; k++)
                {
                    Temp=(float)(sqrt(Vx[i*(Y)*(Z)+j*(Z)+k]*Vx[i*(Y)*(Z)+j*(Z)+k]+
                                      Vy[i*(Y+1)*(Z)+j*(Z)+k]*Vy[i*(Y+1)*(Z)+j*(Z)+k]+
                                      Vz[i*(Y)*(Z+1)+j*(Z+1)+k]*Vz[i*(Y)*(Z+1)+j*(Z+1)+k]));
                    Temp=(float)(1./TableLegerete[Carte[(i)*(Y)*(Z)+(j)*(Z)+(k)]])*Temp*Temp;
                    fwrite(&Temp,sizeof(float),1,Fich);
                }
            }
        }
        fclose( Fich );
    }
}

void SourcesContraintes(void)
{

    int NumeroMatrice;
    struct MATRICE_EMISSION *MatriceCourante;
    struct MATRICE_RECEPTION *MatriceCouranteSourceFile;

    for(NumeroMatrice=0; NumeroMatrice < NbMatricesSourceFilesTxx; NumeroMatrice++)
    {
        MatriceCouranteSourceFile=&MatricesSourceFilesTxx[NumeroMatrice];
        ActivationMatriceSourceFile(Txx,MatriceCouranteSourceFile,X,Y,Z);
    }

    for(NumeroMatrice=0; NumeroMatrice < NbMatricesSourceFilesTyy; NumeroMatrice++)
    {
        MatriceCouranteSourceFile=&MatricesSourceFilesTyy[NumeroMatrice];
        ActivationMatriceSourceFile(Tyy,MatriceCouranteSourceFile,X,Y,Z);
    }

    for(NumeroMatrice=0; NumeroMatrice < NbMatricesSourceFilesTzz; NumeroMatrice++)
    {
        MatriceCouranteSourceFile=&MatricesSourceFilesTzz[NumeroMatrice];
        ActivationMatriceSourceFile(Tzz,MatriceCouranteSourceFile,X,Y,Z);
    }

    for(NumeroMatrice=0; NumeroMatrice < NbMatricesSourceFilesTxy; NumeroMatrice++)
    {
        MatriceCouranteSourceFile=&MatricesSourceFilesTxy[NumeroMatrice];
        ActivationMatriceSourceFile(Txy,MatriceCouranteSourceFile,X+1,Y+1,Z);
    }

    for(NumeroMatrice=0; NumeroMatrice < NbMatricesSourceFilesTyz; NumeroMatrice++)
    {
        MatriceCouranteSourceFile=&MatricesSourceFilesTyz[NumeroMatrice];
        ActivationMatriceSourceFile(Tyz,MatriceCouranteSourceFile,X,Y+1,Z+1);
    }

    for(NumeroMatrice=0; NumeroMatrice < NbMatricesSourceFilesTzx; NumeroMatrice++)
    {
        MatriceCouranteSourceFile=&MatricesSourceFilesTzx[NumeroMatrice];
        ActivationMatriceSourceFile(Tzx,MatriceCouranteSourceFile,X+1,Y,Z+1);
    }


    for(NumeroMatrice=0; NumeroMatrice<NbMatricesEmissionTxx; NumeroMatrice++)
    {
        MatriceCourante=&EmissionTxx[NumeroMatrice];
        ActivationMatriceEmission(Txx,MatriceCourante,X,Y,Z);
    }
    for(NumeroMatrice=0; NumeroMatrice<NbMatricesEmissionTyy; NumeroMatrice++)
    {
        MatriceCourante=&EmissionTyy[NumeroMatrice];
        ActivationMatriceEmission(Tyy,MatriceCourante,X,Y,Z);
    }
    for(NumeroMatrice=0; NumeroMatrice<NbMatricesEmissionTzz; NumeroMatrice++)
    {
        MatriceCourante=&EmissionTzz[NumeroMatrice];
        ActivationMatriceEmission(Tzz,MatriceCourante,X,Y,Z);
    }
    for(NumeroMatrice=0; NumeroMatrice<NbMatricesEmissionTxy; NumeroMatrice++)
    {
        MatriceCourante=&EmissionTxy[NumeroMatrice];
        ActivationMatriceEmission(Txy,MatriceCourante,X+1,Y+1,Z);
    }
    for(NumeroMatrice=0; NumeroMatrice<NbMatricesEmissionTyz; NumeroMatrice++)
    {
        MatriceCourante=&EmissionTyz[NumeroMatrice];
        ActivationMatriceEmission(Tyz,MatriceCourante,X,Y+1,Z+1);
    }
    for(NumeroMatrice=0; NumeroMatrice<NbMatricesEmissionTzx; NumeroMatrice++)
    {
        MatriceCourante=&EmissionTzx[NumeroMatrice];
        ActivationMatriceEmission(Tzx,MatriceCourante,X+1,Y,Z+1);
    }
}

void SourcesVitesses(void)
{

    int NumeroMatrice;
    struct MATRICE_EMISSION *MatriceCourante;
    struct MATRICE_RECEPTION *MatriceCouranteSourceFile;



    for(NumeroMatrice=0; NumeroMatrice < NbMatricesSourceFilesVx; NumeroMatrice++)
    {
        MatriceCouranteSourceFile=&MatricesSourceFilesVx[NumeroMatrice];
        ActivationMatriceSourceFile(Vx,MatriceCouranteSourceFile,X+1,Y,Z);
    }

    for(NumeroMatrice=0; NumeroMatrice < NbMatricesSourceFilesVy; NumeroMatrice++)
    {
        MatriceCouranteSourceFile=&MatricesSourceFilesVy[NumeroMatrice];
        ActivationMatriceSourceFile(Vy,MatriceCouranteSourceFile,X,Y+1,Z);
    }

    for(NumeroMatrice=0; NumeroMatrice < NbMatricesSourceFilesVz; NumeroMatrice++)
    {
        MatriceCouranteSourceFile=&MatricesSourceFilesVz[NumeroMatrice];
        ActivationMatriceSourceFile(Vz,MatriceCouranteSourceFile,X,Y,Z+1);
    }


    for(NumeroMatrice=0; NumeroMatrice<NbMatricesEmissionVx; NumeroMatrice++)
    {
        MatriceCourante=&EmissionVx[NumeroMatrice];
        ActivationMatriceEmission(Vx,MatriceCourante,X+1,Y,Z);
    }
    for(NumeroMatrice=0; NumeroMatrice<NbMatricesEmissionVy; NumeroMatrice++)
    {
        MatriceCourante=&EmissionVy[NumeroMatrice];
        ActivationMatriceEmission(Vy,MatriceCourante,X,Y+1,Z);
    }
    for(NumeroMatrice=0; NumeroMatrice<NbMatricesEmissionVz; NumeroMatrice++)
    {
        MatriceCourante=&EmissionVz[NumeroMatrice];
        ActivationMatriceEmission(Vz,MatriceCourante,X,Y,Z+1);
    }
}

void GenerationSignal(int TypeSignal, double Freq, double T0, double Bandwidth,double *Signal)
{

    int i,i0;
    double argument,Maximum;

    i0=(int)(T0/DeltaT);

    Maximum=0.0;

    switch (TypeSignal)
    {
    case 1:
        for(i=0; i<IndiceTempsMax; i++)
        {
            argument=2*Bandwidth*Freq*(double)(i-i0)*DeltaT;
            Signal[i]=sin(2*3.1415*Freq*(double)(i-i0)*DeltaT)*exp(-argument*argument);
        }
        break;
    case 2: /*Signal  moyenne nulle, de primitive  moyenne non nulle*/
        for(i=0; i<IndiceTempsMax; i++)
        {
            argument=Freq*(double)(i-i0)*DeltaT;
            Signal[i]=-(i-i0)*exp(-argument*argument);
        }
        break;
    case 10: /*Signal  moyenne nulle, de primitive  moyenne non nulle*/
        for(i=0; i<IndiceTempsMax; i++)
        {
            argument=4.44*Freq*(double)(i-i0)*DeltaT;
            Signal[i]=-(i-i0)*exp(-argument*argument);
        }
        break;
    case 11:/* Driv du prcedent*/
        for(i=0; i<IndiceTempsMax; i++)
        {
            argument=3.116*Freq*(double)(i-i0)*DeltaT;
            Signal[i]=-(i-i0)*exp(-argument*argument);
        }
        for(i=0; i<(IndiceTempsMax-1); i++) /*/ drivation*/
        {
            Signal[i]=Signal[i+1]-Signal[i];
        }
        break;
    case 12:/* Driv du prcedent*/
        for(i=0; i<IndiceTempsMax; i++)
        {
            argument=2.5*Freq*(double)(i-i0)*DeltaT;
            Signal[i]=-(i-i0)*exp(-argument*argument);
        }
        for(i=0; i<(IndiceTempsMax-2); i++) /*/ drivation deux fois...*/
        {
            Signal[i]=Signal[i+2]+Signal[i]-2*Signal[i+1];
        }
        break;
    case 13:/*Driv du prcedent*/
        for(i=0; i<IndiceTempsMax; i++)
        {
            argument=2.2*Freq*(double)(i-i0)*DeltaT;
            Signal[i]=-(i-i0)*exp(-argument*argument);
        }
        for(i=0; i<(IndiceTempsMax-3); i++) /* drivation trois fois...*/
        {
            Signal[i]=Signal[i+3]-3*Signal[i+2]+3*Signal[i+1]-Signal[i];
        }
        break;
    case 14:/* Driv du prcedent*/
        for(i=0; i<IndiceTempsMax; i++)
        {
            argument=2.05*Freq*(double)(i-i0)*DeltaT;
            Signal[i]=-(i-i0)*exp(-argument*argument);
        }
        for(i=0; i<(IndiceTempsMax-4); i++) /* drivation trois fois...*/
        {
            Signal[i]=Signal[i+4]-4*Signal[i+3]+6*Signal[i+2]-4*Signal[i+1]+Signal[i];
        }
        break;
    }

    /*	   Normalisation du signal */
    Maximum = MaxAbsSignal(Signal,IndiceTempsMax);

    if(!(Maximum==0))
        for(i=0; i<IndiceTempsMax; i++)
        {

            Signal[i] = Signal[i]/Maximum;
        }
    /**/
}

double MaxAbsSignal(double* Signal,int IndiceTempsMax)
{
    int i;
    double Maximum=0;

    for(i=0; i<IndiceTempsMax; i++)
    {
        Maximum = max(Maximum,fabs(Signal[i]));
    }
    return Maximum;
}

void MiseAJourContraintesTamponFacePML(const char *TypeFace)
{
    int i,j,k;

    if (!strcmp(TypeFace,"Xp"))
    {
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<Y; j++)
        {
            for(k=0; k<Z; k++)
            {
                Carte_Tampon[j*(Z)+k] = Carte[(X-1)*(Y)*(Z)+j*(Z)+k];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<Y; j++)
        {
            for(k=0; k<(Z+1); k++)
            {
                Tki_BufferFaceBlocI[j*(Z+1)+k]= Tzx[(X)*(Y)*(Z+1)+j*(Z+1)+k];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(Y+1); j++)
        {
            for(k=0; k<(Z); k++)
            {
                Tij_BufferFaceBlocI[j*(Z)+k]= Txy[(X)*(Y+1)*(Z)+j*(Z)+k];
            }
        }
        /************************************************************************/
        /**************  Face  sur PML Arete XY *********************************/
        /************************************************************************/
        if ((ConditionFaceYp==0))
        {

#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(k=0; k<(Z+1); k++)
                {
                    Tki_BufferFaceAreteJp[i*(Z+1)+k] =	Tzx_XpYp_Z[i*(W+1)*(Z+1)+k]	+
                                                        Tzx_XpYp_X[i*(W+1)*(Z+1)+k];
                    /*ce type de variable ne sert pas dans les tampons... a virer*/
                }
            }


#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(k=0; k<(Z); k++)
                {
                    Tjj_BufferFaceAreteJp[i*(Z)+k] =	Tyy_XpYp_X[i*(W+1)*(Z)+k]	+
                                                        Tyy_XpYp_Y[i*(W+1)*(Z)+k]	+
                                                        Tyy_XpYp_Z[i*(W+1)*(Z)+k];
                }
            }
        }
        if ((ConditionFaceYm==0))
        {

#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(k=0; k<(Z+1); k++)
                {
                    Tki_BufferFaceAreteJm[i*(Z+1)+k] =	Tzx_XpYm_Z[i*(W+1)*(Z+1)+k]	+
                                                        Tzx_XpYm_X[i*(W+1)*(Z+1)+k];
                }
            }


#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(k=0; k<(Z); k++)
                {
                    Tjj_BufferFaceAreteJm[i*(Z)+k] =	Tyy_XpYm_X[i*(W+1)*(Z)+k]	+
                                                        Tyy_XpYm_Y[i*(W+1)*(Z)+k]	+
                                                        Tyy_XpYm_Z[i*(W+1)*(Z)+k];
                }
            }
        }
        /************************************************************************/
        /**************  Face  sur PML Arete ZX *********************************/
        /************************************************************************/
        if ((ConditionFaceZp==0))
        {

#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(Y+1); j++)
                {
                    Tij_BufferFaceAreteKp[i*(Y+1)+j] =	Txy_ZpXp_X[i*(Y+1)+j]	+
                                                        Txy_ZpXp_Y[i*(Y+1)+j];
                    /*ce type de variable ne sert pas dans les tampons... a virer*/
                }
            }

#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(Y); j++)
                {
                    Tkk_BufferFaceAreteKp[i*(Y)+j] =	Tzz_ZpXp_X[i*(Y)+j]	+
                                                        Tzz_ZpXp_Y[i*(Y)+j]	+
                                                        Tzz_ZpXp_Z[i*(Y)+j];
                }
            }
        }
        if ((ConditionFaceZm==0))
        {
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(Y+1); j++)
                {
                    Tij_BufferFaceAreteKm[i*(Y+1)+j] =	Txy_ZmXp_X[i*(Y+1)+j]	+
                                                        Txy_ZmXp_Y[i*(Y+1)+j];
                }
            }

#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(Y); j++)
                {
                    Tkk_BufferFaceAreteKm[i*(Y)+j] =	Tzz_ZmXp_X[i*(Y)+j]	+
                                                        Tzz_ZmXp_Y[i*(Y)+j]	+
                                                        Tzz_ZmXp_Z[i*(Y)+j];
                }
            }
        }
    }

    if (!strcmp(TypeFace,"Xm"))
    {
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<Y; j++)
        {
            for(k=0; k<Z; k++)
            {
                Carte_Tampon[j*(Z)+k] = Carte[(0)*(Y)*(Z)+j*(Z)+k];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<Y; j++)
        {
            for(k=0; k<(Z+1); k++)
            {
                Tki_BufferFaceBlocI[j*(Z+1)+k]= Tzx[(0)*(Y)*(Z+1)+j*(Z+1)+k];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(Y+1); j++)
        {
            for(k=0; k<(Z); k++)
            {
                Tij_BufferFaceBlocI[j*(Z)+k]= Txy[(0)*(Y+1)*(Z)+j*(Z)+k];
            }
        }
        /************************************************************************/
        /**************  Face  sur PML Arete XY *********************************/
        /************************************************************************/
        if ((ConditionFaceYp==0))
        {

#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(k=0; k<(Z+1); k++)
                {
                    Tki_BufferFaceAreteJp[i*(Z+1)+k] =	Tzx_XmYp_Z[i*(W+1)*(Z+1)+k]	+
                                                        Tzx_XmYp_X[i*(W+1)*(Z+1)+k];
                }
            }


#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(k=0; k<(Z); k++)
                {
                    Tjj_BufferFaceAreteJp[i*(Z)+k] =	Tyy_XmYp_X[i*(W+1)*(Z)+k]	+
                                                        Tyy_XmYp_Y[i*(W+1)*(Z)+k]	+
                                                        Tyy_XmYp_Z[i*(W+1)*(Z)+k];
                }
            }
        }

        if ((ConditionFaceYm==0))
        {
#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(k=0; k<(Z+1); k++)
                {
                    Tki_BufferFaceAreteJm[i*(Z+1)+k] =	Tzx_XmYm_Z[i*(W+1)*(Z+1)+k]	+
                                                        Tzx_XmYm_X[i*(W+1)*(Z+1)+k];
                }
            }


#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(k=0; k<(Z); k++)
                {
                    Tjj_BufferFaceAreteJm[i*(Z)+k] =	Tyy_XmYm_X[i*(W+1)*(Z)+k]	+
                                                        Tyy_XmYm_Y[i*(W+1)*(Z)+k]	+
                                                        Tyy_XmYm_Z[i*(W+1)*(Z)+k];
                }
            }
        }
        /************************************************************************/
        /**************  Face  sur PML Arete ZX *********************************/
        /************************************************************************/
        if ((ConditionFaceZp==0))
        {
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(Y+1); j++)
                {
                    Tij_BufferFaceAreteKp[i*(Y+1)+j] =	Txy_ZpXm_X[i*(Y+1)+j]	+
                                                        Txy_ZpXm_Y[i*(Y+1)+j];
                }
            }

#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(Y); j++)
                {
                    Tkk_BufferFaceAreteKp[i*(Y)+j] =	Tzz_ZpXm_X[i*(Y)+j]	+
                                                        Tzz_ZpXm_Y[i*(Y)+j]	+
                                                        Tzz_ZpXm_Z[i*(Y)+j];
                }
            }
        }
        if ((ConditionFaceZm==0))
        {
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(Y+1); j++)
                {
                    Tij_BufferFaceAreteKm[i*(Y+1)+j] =	Txy_ZmXm_X[i*(Y+1)+j]	+
                                                        Txy_ZmXm_Y[i*(Y+1)+j];
                }
            }

#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(Y); j++)
                {
                    Tkk_BufferFaceAreteKm[i*(Y)+j] =	Tzz_ZmXm_X[i*(Y)+j]	+
                                                        Tzz_ZmXm_Y[i*(Y)+j]	+
                                                        Tzz_ZmXm_Z[i*(Y)+j];
                }
            }
        }
    }

    if (!strcmp(TypeFace,"Yp"))
    {

#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<Z; j++)
        {
            for(k=0; k<X; k++)
            {
                Carte_Tampon[j*(X)+k] = Carte[k*(Y)*(Z)+(Y-1)*(Z)+j];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<Z; j++)
        {
            for(k=0; k<(X+1); k++)
            {
                Tki_BufferFaceBlocI[j*(X+1)+k]= Txy[k*(Y+1)*(Z)+(Y)*(Z)+j];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(Z+1); j++)
        {
            for(k=0; k<(X); k++)
            {
                Tij_BufferFaceBlocI[j*(X)+k]= Tyz[k*(Y+1)*(Z+1)+(Y)*(Z+1)+j];
            }
        }
        /************************************************************************/
        /**************  Face  sur PML Arete YZ *********************************/
        /************************************************************************/
        if ((ConditionFaceZp==0))
        {
#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(k=0; k<(X+1); k++)
                {
                    Tki_BufferFaceAreteJp[i*(X+1)+k] =	Txy_YpZp_X[i*(W+1)*(X+1)+k] +
                                                        Txy_YpZp_Y[i*(W+1)*(X+1)+k];
                }
            }

#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(k=0; k<(X); k++)
                {
                    Tjj_BufferFaceAreteJp[i*(X)+k] =	Tzz_YpZp_X[i*(W+1)*(X)+k]	+
                                                        Tzz_YpZp_Y[i*(W+1)*(X)+k]	+
                                                        Tzz_YpZp_Z[i*(W+1)*(X)+k];

                }
            }
        }

        if ((ConditionFaceZm==0))
        {
#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(k=0; k<(X+1); k++)
                {
                    Tki_BufferFaceAreteJm[i*(X+1)+k] =	Txy_YpZm_X[i*(W+1)*(X+1)+k] +
                                                        Txy_YpZm_Y[i*(W+1)*(X+1)+k];
                }
            }

#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(k=0; k<(X); k++)
                {
                    Tjj_BufferFaceAreteJm[i*(X)+k] =	Tzz_YpZm_X[i*(W+1)*(X)+k]	+
                                                        Tzz_YpZm_Y[i*(W+1)*(X)+k]	+
                                                        Tzz_YpZm_Z[i*(W+1)*(X)+k];

                }
            }
        }
        /************************************************************************/
        /**************  Face  sur PML Arete XY *********************************/
        /************************************************************************/
        if ((ConditionFaceXp==0))
        {
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(Z+1); j++)
                {
                    Tij_BufferFaceAreteKp[i*(Z+1)+j] =	Tyz_XpYp_Y[i*(Z+1)+j]	+
                                                        Tyz_XpYp_Z[i*(Z+1)+j];

                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(Z); j++)
                {
                    Tkk_BufferFaceAreteKp[i*(Z)+j] =	Txx_XpYp_X[i*(Z)+j]	+
                                                        Txx_XpYp_Y[i*(Z)+j]	+
                                                        Txx_XpYp_Z[i*(Z)+j];

                }
            }
        }
        if ((ConditionFaceXm==0))
        {
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(Z+1); j++)
                {
                    Tij_BufferFaceAreteKm[i*(Z+1)+j] =	Tyz_XmYp_Y[i*(Z+1)+j]	+
                                                        Tyz_XmYp_Z[i*(Z+1)+j];

                }
            }

#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(Z); j++)
                {
                    Tkk_BufferFaceAreteKm[i*(Z)+j] =	Txx_XmYp_X[i*(Z)+j]	+
                                                        Txx_XmYp_Y[i*(Z)+j]	+
                                                        Txx_XmYp_Z[i*(Z)+j];

                }
            }
        }
    }

    if (!strcmp(TypeFace,"Ym"))
    {

#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<Z; j++)
        {
            for(k=0; k<X; k++)
            {
                Carte_Tampon[j*(X)+k] = Carte[k*(Y)*(Z)+(0)*(Z)+j];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<Z; j++)
        {
            for(k=0; k<(X+1); k++)
            {
                Tki_BufferFaceBlocI[j*(X+1)+k]= Txy[k*(Y+1)*(Z)+(0)*(Z)+j];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(Z+1); j++)
        {
            for(k=0; k<(X); k++)
            {
                Tij_BufferFaceBlocI[j*(X)+k]= Tyz[k*(Y+1)*(Z+1)+(0)*(Z+1)+j];
            }
        }
        /************************************************************************/
        /**************  Face  sur PML Arete YZ *********************************/
        /************************************************************************/
        if ((ConditionFaceZp==0))
        {
#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(k=0; k<(X+1); k++)
                {
                    Tki_BufferFaceAreteJp[i*(X+1)+k] =	Txy_YmZp_X[i*(W+1)*(X+1)+k] +
                                                        Txy_YmZp_Y[i*(W+1)*(X+1)+k];
                }
            }

#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(k=0; k<(X); k++)
                {
                    Tjj_BufferFaceAreteJp[i*(X)+k] =	Tzz_YmZp_X[i*(W+1)*(X)+k]	+
                                                        Tzz_YmZp_Y[i*(W+1)*(X)+k]	+
                                                        Tzz_YmZp_Z[i*(W+1)*(X)+k];

                }
            }
        }
        if ((ConditionFaceZm==0))
        {
#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(k=0; k<(X+1); k++)
                {
                    Tki_BufferFaceAreteJm[i*(X+1)+k] =	Txy_YmZm_X[i*(W+1)*(X+1)+k] +
                                                        Txy_YmZm_Y[i*(W+1)*(X+1)+k];
                }
            }

#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(k=0; k<(X); k++)
                {
                    Tjj_BufferFaceAreteJm[i*(X)+k] =	Tzz_YmZm_X[i*(W+1)*(X)+k]	+
                                                        Tzz_YmZm_Y[i*(W+1)*(X)+k]	+
                                                        Tzz_YmZm_Z[i*(W+1)*(X)+k];

                }
            }
        }
        /************************************************************************/
        /**************  Face  sur PML Arete XY *********************************/
        /************************************************************************/
        if ((ConditionFaceXp==0))
        {
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(Z+1); j++)
                {
                    Tij_BufferFaceAreteKp[i*(Z+1)+j] =	Tyz_XpYm_Y[i*(Z+1)+j]	+
                                                        Tyz_XpYm_Z[i*(Z+1)+j];

                }
            }

#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(Z); j++)
                {
                    Tkk_BufferFaceAreteKp[i*(Z)+j] =	Txx_XpYm_X[i*(Z)+j]	+
                                                        Txx_XpYm_Y[i*(Z)+j]	+
                                                        Txx_XpYm_Z[i*(Z)+j];

                }
            }
        }
        if ((ConditionFaceXm==0))
        {
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(Z+1); j++)
                {
                    Tij_BufferFaceAreteKm[i*(Z+1)+j] =	Tyz_XmYm_Y[i*(Z+1)+j]	+
                                                        Tyz_XmYm_Z[i*(Z+1)+j];

                }
            }

#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(Z); j++)
                {
                    Tkk_BufferFaceAreteKm[i*(Z)+j] =	Txx_XmYm_X[i*(Z)+j]	+
                                                        Txx_XmYm_Y[i*(Z)+j]	+
                                                        Txx_XmYm_Z[i*(Z)+j];

                }
            }
        }
    }

    if (!strcmp(TypeFace,"Zp"))
    {
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<X; j++)
        {
            for(k=0; k<Y; k++)
            {
                Carte_Tampon[j*(Y)+k] = Carte[j*(Y)*(Z)+k*(Z)+(Z-1)];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<X; j++)
        {
            for(k=0; k<(Y+1); k++)
            {
                Tki_BufferFaceBlocI[j*(Y+1)+k]= Tyz[j*(Y+1)*(Z+1)+k*(Z+1)+(Z)];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(X+1); j++)
        {
            for(k=0; k<(Y); k++)
            {
                Tij_BufferFaceBlocI[j*(Y)+k]= Tzx[j*(Y)*(Z+1)+k*(Z+1)+(Z)];
            }
        }
        /************************************************************************/
        /**************  Face  sur PML Arete ZX *********************************/
        /************************************************************************/

        if ((ConditionFaceXp==0))
        {
#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(k=0; k<(Y+1); k++)
                {
                    Tki_BufferFaceAreteJp[i*(Y+1)+k] =	Tyz_ZpXp_Y[i*(W+1)*(Y+1)+k]	+
                                                        Tyz_ZpXp_Z[i*(W+1)*(Y+1)+k];
                }
            }


#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(k=0; k<(Y); k++)
                {
                    Tjj_BufferFaceAreteJp[i*(Y)+k] =	Txx_ZpXp_X[i*(W+1)*(Y)+k]	+
                                                        Txx_ZpXp_Y[i*(W+1)*(Y)+k]	+
                                                        Txx_ZpXp_Z[i*(W+1)*(Y)+k];
                }
            }
        }
        if ((ConditionFaceXm==0))
        {
#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(k=0; k<(Y+1); k++)
                {
                    Tki_BufferFaceAreteJm[i*(Y+1)+k] =	Tyz_ZpXm_Y[i*(W+1)*(Y+1)+k]	+
                                                        Tyz_ZpXm_Z[i*(W+1)*(Y+1)+k];
                }
            }


#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(k=0; k<(Y); k++)
                {
                    Tjj_BufferFaceAreteJm[i*(Y)+k] =	Txx_ZpXm_X[i*(W+1)*(Y)+k]	+
                                                        Txx_ZpXm_Y[i*(W+1)*(Y)+k]	+
                                                        Txx_ZpXm_Z[i*(W+1)*(Y)+k];
                }
            }
        }
        /************************************************************************/
        /**************  Face  sur PML Arete YZ *********************************/
        /************************************************************************/

        if ((ConditionFaceYp==0))
        {
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(X+1); j++)
                {
                    Tij_BufferFaceAreteKp[i*(X+1)+j] =	Tzx_YpZp_X[i*(X+1)+j]	+
                                                        Tzx_YpZp_Z[i*(X+1)+j];
                }
            }

#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(X); j++)
                {
                    Tkk_BufferFaceAreteKp[i*(X)+j] =	Tyy_YpZp_X[i*(X)+j]	+
                                                        Tyy_YpZp_Y[i*(X)+j]	+
                                                        Tyy_YpZp_Z[i*(X)+j];
                }
            }
        }

        if ((ConditionFaceYm==0))
        {
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(X+1); j++)
                {
                    Tij_BufferFaceAreteKm[i*(X+1)+j] =	Tzx_YmZp_X[i*(X+1)+j]	+
                                                        Tzx_YmZp_Z[i*(X+1)+j];
                }
            }

#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(X); j++)
                {
                    Tkk_BufferFaceAreteKm[i*(X)+j] =	Tyy_YmZp_X[i*(X)+j]	+
                                                        Tyy_YmZp_Y[i*(X)+j]	+
                                                        Tyy_YmZp_Z[i*(X)+j];
                }
            }
        }
    }


    if (!strcmp(TypeFace,"Zm"))
    {
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<X; j++)
        {
            for(k=0; k<Y; k++)
            {
                Carte_Tampon[j*(Y)+k] = Carte[j*(Y)*(Z)+k*(Z)+(0)];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<X; j++)
        {
            for(k=0; k<(Y+1); k++)
            {
                Tki_BufferFaceBlocI[j*(Y+1)+k]= Tyz[j*(Y+1)*(Z+1)+k*(Z+1)+(0)];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(X+1); j++)
        {
            for(k=0; k<(Y); k++)
            {
                Tij_BufferFaceBlocI[j*(Y)+k]= Tzx[j*(Y)*(Z+1)+k*(Z+1)+(0)];
            }
        }
        /************************************************************************/
        /**************  Face  sur PML Arete ZX *********************************/
        /************************************************************************/
        if ((ConditionFaceXp==0))
        {
#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(k=0; k<(Y+1); k++)
                {
                    Tki_BufferFaceAreteJp[i*(Y+1)+k] =	Tyz_ZmXp_Y[i*(W+1)*(Y+1)+k]	+
                                                        Tyz_ZmXp_Z[i*(W+1)*(Y+1)+k];
                }
            }


#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(k=0; k<(Y); k++)
                {
                    Tjj_BufferFaceAreteJp[i*(Y)+k] =	Txx_ZmXp_X[i*(W+1)*(Y)+k]	+
                                                        Txx_ZmXp_Y[i*(W+1)*(Y)+k]	+
                                                        Txx_ZmXp_Z[i*(W+1)*(Y)+k];
                }
            }
        }
        if ((ConditionFaceXm==0))
        {
#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(k=0; k<(Y+1); k++)
                {
                    Tki_BufferFaceAreteJm[i*(Y+1)+k] =	Tyz_ZmXm_Y[i*(W+1)*(Y+1)+k]	+
                                                        Tyz_ZmXm_Z[i*(W+1)*(Y+1)+k];
                }
            }


#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(k=0; k<(Y); k++)
                {
                    Tjj_BufferFaceAreteJm[i*(Y)+k] =	Txx_ZmXm_X[i*(W+1)*(Y)+k]	+
                                                        Txx_ZmXm_Y[i*(W+1)*(Y)+k]	+
                                                        Txx_ZmXm_Z[i*(W+1)*(Y)+k];
                }
            }
        }
        /************************************************************************/
        /**************  Face  sur PML Arete YZ *********************************/
        /************************************************************************/

        if ((ConditionFaceYp==0))
        {
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(X+1); j++)
                {
                    Tij_BufferFaceAreteKp[i*(X+1)+j] =	Tzx_YpZm_X[i*(X+1)+j]	+
                                                        Tzx_YpZm_Z[i*(X+1)+j];
                }
            }

#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(X); j++)
                {
                    Tkk_BufferFaceAreteKp[i*(X)+j] =	Tyy_YpZm_X[i*(X)+j]	+
                                                        Tyy_YpZm_Y[i*(X)+j]	+
                                                        Tyy_YpZm_Z[i*(X)+j];
                }
            }
        }
        if ((ConditionFaceYm==0))
        {
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(X+1); j++)
                {
                    Tij_BufferFaceAreteKm[i*(X+1)+j] =	Tzx_YmZm_X[i*(X+1)+j]	+
                                                        Tzx_YmZm_Z[i*(X+1)+j];
                }
            }

#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(X); j++)
                {
                    Tkk_BufferFaceAreteKm[i*(X)+j] =	Tyy_YmZm_X[i*(X)+j]	+
                                                        Tyy_YmZm_Y[i*(X)+j]	+
                                                        Tyy_YmZm_Z[i*(X)+j];
                }
            }
        }
    }
}

void MiseAJourContraintesTamponAretePML(const char *TypeArete)
{
    int i,j,k;

    if (!strcmp(TypeArete,"XpYp"))
    {

#pragma omp parallel for schedule(guided)
        for(k=0; k<Z; k++)
        {
            Carte_Tampon[k] = Carte[(X-1)*(Y)*(Z)+(Y-1)*(Z)+k];
        }
        /**************  Face J sur PML Face I *********************************/

#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W); i++)
        {
            for(k=0; k<(Z); k++)
            {
                Tij_BufferAreteFaceI[i*(Z)+k] =	Txy_Xp_X[i*(Y+1)*(Z)+Y*(Z)+k]	+
                                                Txy_Xp_YZ[i*(Y+1)*(Z)+Y*(Z)+k];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(Z+1); k++)
            {
                Tjk_BufferAreteFaceI[i*(Z+1)+k] =	Tyz_Xp[i*(Y+1)*(Z+1)+Y*(Z+1)+k];
            }
        }

        /**************  Face I sur PML Face J *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W); j++)
        {
            for(k=0; k<(Z); k++)
            {
                Tij_BufferAreteFaceJ[j*(Z)+k] =	Txy_Yp_Y[j*(Z)*(X+1)+k*(X+1)+X]	+
                                                Txy_Yp_ZX[j*(Z)*(X+1)+k*(X+1)+X];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(Z+1); k++)
            {
                Tki_BufferAreteFaceJ[j*(Z+1)+k] =	Tzx_Yp[j*(Z+1)*(X+1)+k*(X+1)+X];
            }
        }
        if ((ConditionFaceZp==0))
        {
            /**************  Face Kp sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Tkk_BufferAreteCoinKp[i*(W+1)+j]=	Tzz_XpYpZp_X[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                        Tzz_XpYpZp_Y[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                        Tzz_XpYpZp_Z[i*(W+1)*(W+1)+j*(W+1)+0];
                }
            }
        }
        if ((ConditionFaceZm==0))
        {
            /**************  Face Km sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Tkk_BufferAreteCoinKm[i*(W+1)+j]=	Tzz_XpYpZm_X[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                        Tzz_XpYpZm_Y[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                        Tzz_XpYpZm_Z[i*(W+1)*(W+1)+j*(W+1)+0];
                }
            }
        }
    }
    if (!strcmp(TypeArete,"XmYm"))
    {

#pragma omp parallel for schedule(guided)
        for(k=0; k<Z; k++)
        {
            Carte_Tampon[k] = Carte[(0)*(Y)*(Z)+(0)*(Z)+k];
        }
        /**************  Face J sur PML Face I *********************************/

#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W); i++)
        {
            for(k=0; k<(Z); k++)
            {
                Tij_BufferAreteFaceI[i*(Z)+k] =	Txy_Xm_X[i*(Y+1)*(Z)+0*(Z)+k]	+
                                                Txy_Xm_YZ[i*(Y+1)*(Z)+0*(Z)+k];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(Z+1); k++)
            {
                Tjk_BufferAreteFaceI[i*(Z+1)+k] =	Tyz_Xm[i*(Y+1)*(Z+1)+0*(Z+1)+k];
            }
        }

        /**************  Face I sur PML Face J *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W); j++)
        {
            for(k=0; k<(Z); k++)
            {
                Tij_BufferAreteFaceJ[j*(Z)+k] =	Txy_Ym_Y[j*(Z)*(X+1)+k*(X+1)+0]	+
                                                Txy_Ym_ZX[j*(Z)*(X+1)+k*(X+1)+0];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(Z+1); k++)
            {
                Tki_BufferAreteFaceJ[j*(Z+1)+k] =	Tzx_Ym[j*(Z+1)*(X+1)+k*(X+1)+0];
            }
        }
        if ((ConditionFaceZp==0))
        {
            /**************  Face Kp sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Tkk_BufferAreteCoinKp[i*(W+1)+j]=	Tzz_XmYmZp_X[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                        Tzz_XmYmZp_Y[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                        Tzz_XmYmZp_Z[i*(W+1)*(W+1)+j*(W+1)+0];
                }
            }
        }
        if ((ConditionFaceZm==0))
        {
            /**************  Face Km sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Tkk_BufferAreteCoinKm[i*(W+1)+j]=	Tzz_XmYmZm_X[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                        Tzz_XmYmZm_Y[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                        Tzz_XmYmZm_Z[i*(W+1)*(W+1)+j*(W+1)+0];
                }
            }
        }
    }

    if (!strcmp(TypeArete,"XmYp"))
    {

#pragma omp parallel for schedule(guided)
        for(k=0; k<Z; k++)
        {
            Carte_Tampon[k] = Carte[(0)*(Y)*(Z)+(Y-1)*(Z)+k];
        }
        /**************  Face J sur PML Face I *********************************/

#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W); i++)
        {
            for(k=0; k<(Z); k++)
            {
                Tij_BufferAreteFaceI[i*(Z)+k] =	Txy_Xm_X[i*(Y+1)*(Z)+Y*(Z)+k]	+
                                                Txy_Xm_YZ[i*(Y+1)*(Z)+Y*(Z)+k];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(Z+1); k++)
            {
                Tjk_BufferAreteFaceI[i*(Z+1)+k] =	Tyz_Xm[i*(Y+1)*(Z+1)+Y*(Z+1)+k];
            }
        }

        /**************  Face I sur PML Face J *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W); j++)
        {
            for(k=0; k<(Z); k++)
            {
                Tij_BufferAreteFaceJ[j*(Z)+k] =	Txy_Yp_Y[j*(Z)*(X+1)+k*(X+1)+0]	+
                                                Txy_Yp_ZX[j*(Z)*(X+1)+k*(X+1)+0];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(Z+1); k++)
            {
                Tki_BufferAreteFaceJ[j*(Z+1)+k] =	Tzx_Yp[j*(Z+1)*(X+1)+k*(X+1)+0];
            }
        }
        if ((ConditionFaceZp==0))
        {
            /**************  Face Kp sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Tkk_BufferAreteCoinKp[i*(W+1)+j]=	Tzz_XmYpZp_X[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                        Tzz_XmYpZp_Y[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                        Tzz_XmYpZp_Z[i*(W+1)*(W+1)+j*(W+1)+0];
                }
            }
        }
        if ((ConditionFaceZm==0))
        {
            /**************  Face Km sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Tkk_BufferAreteCoinKm[i*(W+1)+j]=	Tzz_XmYpZm_X[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                        Tzz_XmYpZm_Y[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                        Tzz_XmYpZm_Z[i*(W+1)*(W+1)+j*(W+1)+0];
                }
            }
        }
    }
    if (!strcmp(TypeArete,"XpYm"))
    {

#pragma omp parallel for schedule(guided)
        for(k=0; k<Z; k++)
        {
            Carte_Tampon[k] = Carte[(X-1)*(Y)*(Z)+(0)*(Z)+k];
        }
        /**************  Face J sur PML Face I *********************************/

#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W); i++)
        {
            for(k=0; k<(Z); k++)
            {
                Tij_BufferAreteFaceI[i*(Z)+k] =	Txy_Xp_X[i*(Y+1)*(Z)+0*(Z)+k]	+
                                                Txy_Xp_YZ[i*(Y+1)*(Z)+0*(Z)+k];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(Z+1); k++)
            {
                Tjk_BufferAreteFaceI[i*(Z+1)+k] =	Tyz_Xp[i*(Y+1)*(Z+1)+0*(Z+1)+k];
            }
        }

        /**************  Face I sur PML Face J *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W); j++)
        {
            for(k=0; k<(Z); k++)
            {
                Tij_BufferAreteFaceJ[j*(Z)+k] =	Txy_Ym_Y[j*(Z)*(X+1)+k*(X+1)+X]	+
                                                Txy_Ym_ZX[j*(Z)*(X+1)+k*(X+1)+X];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(Z+1); k++)
            {
                Tki_BufferAreteFaceJ[j*(Z+1)+k] =	Tzx_Ym[j*(Z+1)*(X+1)+k*(X+1)+X];
            }
        }
        if ((ConditionFaceZp==0))
        {
            /**************  Face Kp sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Tkk_BufferAreteCoinKp[i*(W+1)+j]=	Tzz_XpYmZp_X[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                        Tzz_XpYmZp_Y[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                        Tzz_XpYmZp_Z[i*(W+1)*(W+1)+j*(W+1)+0];
                }
            }
        }
        if ((ConditionFaceZm==0))
        {
            /**************  Face Km sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Tkk_BufferAreteCoinKm[i*(W+1)+j]=	Tzz_XpYmZm_X[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                        Tzz_XpYmZm_Y[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                        Tzz_XpYmZm_Z[i*(W+1)*(W+1)+j*(W+1)+0];
                }
            }
        }
    }
    /**************************************************************************/
    /**************************************************************************/
    /*********** Aretes perpendiculaire au plan YZ ****************************/
    /**************************************************************************/
    /**************************************************************************/

    if (!strcmp(TypeArete,"YpZp"))
    {

#pragma omp parallel for schedule(guided)
        for(k=0; k<X; k++)
        {
            Carte_Tampon[k] = Carte[k*(Y)*(Z)+(Y-1)*(Z)+Z-1];
        }
        /**************  Face J sur PML Face I *********************************/

#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W); i++)
        {
            for(k=0; k<(X); k++)
            {
                Tij_BufferAreteFaceI[i*(X)+k] =	Tyz_Yp_Y[i*(Z+1)*(X)+Z*(X)+k]	+
                                                Tyz_Yp_ZX[i*(Z+1)*(X)+Z*(X)+k];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(X+1); k++)
            {
                Tjk_BufferAreteFaceI[i*(X+1)+k] =	Tzx_Yp[i*(Z+1)*(X+1)+Z*(X+1)+k];
            }
        }

        /**************  Face I sur PML Face J *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W); j++)
        {
            for(k=0; k<(X); k++)
            {
                Tij_BufferAreteFaceJ[j*(X)+k] =	Tyz_Zp_Z[j*(X)*(Y+1)+k*(Y+1)+Y]	+
                                                Tyz_Zp_XY[j*(X)*(Y+1)+k*(Y+1)+Y];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(X+1); k++)
            {
                Tki_BufferAreteFaceJ[j*(X+1)+k] =	Txy_Zp[j*(X+1)*(Y+1)+k*(Y+1)+Y];
            }
        }
        if ((ConditionFaceXp==0))
        {
            /**************  Face Kp sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Tkk_BufferAreteCoinKp[i*(W+1)+j]=	Txx_XpYpZp_X[0*(W+1)*(W+1)+i*(W+1)+j] +
                                                        Txx_XpYpZp_Y[0*(W+1)*(W+1)+i*(W+1)+j] +
                                                        Txx_XpYpZp_Z[0*(W+1)*(W+1)+i*(W+1)+j];
                }
            }
        }
        if ((ConditionFaceXm==0))
        {
            /**************  Face Km sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Tkk_BufferAreteCoinKm[i*(W+1)+j]=	Txx_XmYpZp_X[0*(W+1)*(W+1)+i*(W+1)+j] +
                                                        Txx_XmYpZp_Y[0*(W+1)*(W+1)+i*(W+1)+j] +
                                                        Txx_XmYpZp_Z[0*(W+1)*(W+1)+i*(W+1)+j];
                }
            }
        }
    }
    if (!strcmp(TypeArete,"YmZp"))
    {

#pragma omp parallel for schedule(guided)
        for(k=0; k<X; k++)
        {
            Carte_Tampon[k] = Carte[k*(Y)*(Z)+(0)*(Z)+Z-1];
        }
        /**************  Face J sur PML Face I *********************************/

#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W); i++)
        {
            for(k=0; k<(X); k++)
            {
                Tij_BufferAreteFaceI[i*(X)+k] =	Tyz_Ym_Y[i*(Z+1)*(X)+Z*(X)+k]	+
                                                Tyz_Ym_ZX[i*(Z+1)*(X)+Z*(X)+k];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(X+1); k++)
            {
                Tjk_BufferAreteFaceI[i*(X+1)+k] =	Tzx_Ym[i*(Z+1)*(X+1)+Z*(X+1)+k];
            }
        }

        /**************  Face I sur PML Face J *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W); j++)
        {
            for(k=0; k<(X); k++)
            {
                Tij_BufferAreteFaceJ[j*(X)+k] =	Tyz_Zp_Z[j*(X)*(Y+1)+k*(Y+1)+0]	+
                                                Tyz_Zp_XY[j*(X)*(Y+1)+k*(Y+1)+0];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(X+1); k++)
            {
                Tki_BufferAreteFaceJ[j*(X+1)+k] =	Txy_Zp[j*(X+1)*(Y+1)+k*(Y+1)+0];
            }
        }
        if ((ConditionFaceXp==0))
        {
            /**************  Face Kp sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Tkk_BufferAreteCoinKp[i*(W+1)+j]=	Txx_XpYmZp_X[0*(W+1)*(W+1)+i*(W+1)+j] +
                                                        Txx_XpYmZp_Y[0*(W+1)*(W+1)+i*(W+1)+j] +
                                                        Txx_XpYmZp_Z[0*(W+1)*(W+1)+i*(W+1)+j];
                }
            }
        }
        if ((ConditionFaceXm==0))
        {
            /**************  Face Km sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Tkk_BufferAreteCoinKm[i*(W+1)+j]=	Txx_XmYmZp_X[0*(W+1)*(W+1)+i*(W+1)+j] +
                                                        Txx_XmYmZp_Y[0*(W+1)*(W+1)+i*(W+1)+j] +
                                                        Txx_XmYmZp_Z[0*(W+1)*(W+1)+i*(W+1)+j];
                }
            }
        }
    }
    if (!strcmp(TypeArete,"YpZm"))
    {

#pragma omp parallel for schedule(guided)
        for(k=0; k<X; k++)
        {
            Carte_Tampon[k] = Carte[k*(Y)*(Z)+(Y-1)*(Z)+0];
        }
        /**************  Face J sur PML Face I *********************************/

#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W); i++)
        {
            for(k=0; k<(X); k++)
            {
                Tij_BufferAreteFaceI[i*(X)+k] =	Tyz_Yp_Y[i*(Z+1)*(X)+0*(X)+k]	+
                                                Tyz_Yp_ZX[i*(Z+1)*(X)+0*(X)+k];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(X+1); k++)
            {
                Tjk_BufferAreteFaceI[i*(X+1)+k] =	Tzx_Yp[i*(Z+1)*(X+1)+0*(X+1)+k];
            }
        }

        /**************  Face I sur PML Face J *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W); j++)
        {
            for(k=0; k<(X); k++)
            {
                Tij_BufferAreteFaceJ[j*(X)+k] =	Tyz_Zm_Z[j*(X)*(Y+1)+k*(Y+1)+Y]	+
                                                Tyz_Zm_XY[j*(X)*(Y+1)+k*(Y+1)+Y];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(X+1); k++)
            {
                Tki_BufferAreteFaceJ[j*(X+1)+k] =	Txy_Zm[j*(X+1)*(Y+1)+k*(Y+1)+Y];
            }
        }
        if ((ConditionFaceXp==0))
        {
            /**************  Face Kp sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Tkk_BufferAreteCoinKp[i*(W+1)+j]=	Txx_XpYpZm_X[0*(W+1)*(W+1)+i*(W+1)+j] +
                                                        Txx_XpYpZm_Y[0*(W+1)*(W+1)+i*(W+1)+j] +
                                                        Txx_XpYpZm_Z[0*(W+1)*(W+1)+i*(W+1)+j];
                }
            }
        }
        if ((ConditionFaceXm==0))
        {
            /**************  Face Km sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Tkk_BufferAreteCoinKm[i*(W+1)+j]=	Txx_XmYpZm_X[0*(W+1)*(W+1)+i*(W+1)+j] +
                                                        Txx_XmYpZm_Y[0*(W+1)*(W+1)+i*(W+1)+j] +
                                                        Txx_XmYpZm_Z[0*(W+1)*(W+1)+i*(W+1)+j];
                }
            }
        }
    }

    if (!strcmp(TypeArete,"YmZm"))
    {

#pragma omp parallel for schedule(guided)
        for(k=0; k<X; k++)
        {
            Carte_Tampon[k] = Carte[k*(Y)*(Z)+(0)*(Z)+0];
        }
        /**************  Face J sur PML Face I *********************************/

#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W); i++)
        {
            for(k=0; k<(X); k++)
            {
                Tij_BufferAreteFaceI[i*(X)+k] =	Tyz_Ym_Y[i*(Z+1)*(X)+0*(X)+k]	+
                                                Tyz_Ym_ZX[i*(Z+1)*(X)+0*(X)+k];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(X+1); k++)
            {
                Tjk_BufferAreteFaceI[i*(X+1)+k] =	Tzx_Ym[i*(Z+1)*(X+1)+0*(X+1)+k];
            }
        }

        /**************  Face I sur PML Face J *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W); j++)
        {
            for(k=0; k<(X); k++)
            {
                Tij_BufferAreteFaceJ[j*(X)+k] =	Tyz_Zm_Z[j*(X)*(Y+1)+k*(Y+1)+0]	+
                                                Tyz_Zm_XY[j*(X)*(Y+1)+k*(Y+1)+0];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(X+1); k++)
            {
                Tki_BufferAreteFaceJ[j*(X+1)+k] =	Txy_Zm[j*(X+1)*(Y+1)+k*(Y+1)+0];
            }
        }
        if ((ConditionFaceXp==0))
        {
            /**************  Face Kp sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Tkk_BufferAreteCoinKp[i*(W+1)+j]=	Txx_XpYmZm_X[0*(W+1)*(W+1)+i*(W+1)+j] +
                                                        Txx_XpYmZm_Y[0*(W+1)*(W+1)+i*(W+1)+j] +
                                                        Txx_XpYmZm_Z[0*(W+1)*(W+1)+i*(W+1)+j];
                }
            }
        }
        if ((ConditionFaceXm==0))
        {
            /**************  Face Km sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Tkk_BufferAreteCoinKm[i*(W+1)+j]=	Txx_XmYmZm_X[0*(W+1)*(W+1)+i*(W+1)+j] +
                                                        Txx_XmYmZm_Y[0*(W+1)*(W+1)+i*(W+1)+j] +
                                                        Txx_XmYmZm_Z[0*(W+1)*(W+1)+i*(W+1)+j];
                }
            }
        }
    }
    /**************************************************************************/
    /*********** Aretes perpendiculaire au plan ZX ****************************/
    /**************************************************************************/
    /**************************************************************************/

    if (!strcmp(TypeArete,"ZpXp"))
    {

#pragma omp parallel for schedule(guided)
        for(k=0; k<Y; k++)
        {
            Carte_Tampon[k] = Carte[(X-1)*(Y)*(Z)+(k)*(Z)+(Z-1)];
        }
        /**************  Face J sur PML Face I *********************************/

#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W); i++)
        {
            for(k=0; k<(Y); k++)
            {
                Tij_BufferAreteFaceI[i*(Y)+k] =	Tzx_Zp_Z[i*(X+1)*(Y)+X*(Y)+k]	+
                                                Tzx_Zp_XY[i*(X+1)*(Y)+X*(Y)+k];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(Y+1); k++)
            {
                Tjk_BufferAreteFaceI[i*(Y+1)+k] =	Txy_Zp[i*(X+1)*(Y+1)+X*(Y+1)+k];
            }
        }

        /**************  Face I sur PML Face J *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W); j++)
        {
            for(k=0; k<(Y); k++)
            {
                Tij_BufferAreteFaceJ[j*(Y)+k] =	Tzx_Xp_X[j*(Y)*(Z+1)+k*(Z+1)+Z]	+
                                                Tzx_Xp_YZ[j*(Y)*(Z+1)+k*(Z+1)+Z];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(Y+1); k++)
            {
                Tki_BufferAreteFaceJ[j*(Y+1)+k] =	Tyz_Xp[j*(Y+1)*(Z+1)+k*(Z+1)+Z];
            }
        }
        if ((ConditionFaceYp==0))
        {
            /**************  Face Kp sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Tkk_BufferAreteCoinKp[i*(W+1)+j]=	Tyy_XpYpZp_X[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                        Tyy_XpYpZp_Y[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                        Tyy_XpYpZp_Z[j*(W+1)*(W+1)+0*(W+1)+i];
                }
            }
        }
        if ((ConditionFaceYm==0))
        {
            /**************  Face Km sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Tkk_BufferAreteCoinKm[i*(W+1)+j]=	Tyy_XpYmZp_X[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                        Tyy_XpYmZp_Y[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                        Tyy_XpYmZp_Z[j*(W+1)*(W+1)+0*(W+1)+i];
                }
            }
        }
    }
    if (!strcmp(TypeArete,"ZpXm"))
    {

#pragma omp parallel for schedule(guided)
        for(k=0; k<Y; k++)
        {
            Carte_Tampon[k] = Carte[(0)*(Y)*(Z)+(k)*(Z)+(Z-1)];
        }
        /**************  Face J sur PML Face I *********************************/

#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W); i++)
        {
            for(k=0; k<(Y); k++)
            {
                Tij_BufferAreteFaceI[i*(Y)+k] =	Tzx_Zp_Z[i*(X+1)*(Y)+0*(Y)+k]	+
                                                Tzx_Zp_XY[i*(X+1)*(Y)+0*(Y)+k];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(Y+1); k++)
            {
                Tjk_BufferAreteFaceI[i*(Y+1)+k] =	Txy_Zp[i*(X+1)*(Y+1)+0*(Y+1)+k];
            }
        }

        /**************  Face I sur PML Face J *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W); j++)
        {
            for(k=0; k<(Y); k++)
            {
                Tij_BufferAreteFaceJ[j*(Y)+k] =	Tzx_Xm_X[j*(Y)*(Z+1)+k*(Z+1)+Z]	+
                                                Tzx_Xm_YZ[j*(Y)*(Z+1)+k*(Z+1)+Z];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(Y+1); k++)
            {
                Tki_BufferAreteFaceJ[j*(Y+1)+k] =	Tyz_Xm[j*(Y+1)*(Z+1)+k*(Z+1)+Z];
            }
        }
        if ((ConditionFaceYp==0))
        {
            /**************  Face Kp sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Tkk_BufferAreteCoinKp[i*(W+1)+j]=	Tyy_XmYpZp_X[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                        Tyy_XmYpZp_Y[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                        Tyy_XmYpZp_Z[j*(W+1)*(W+1)+0*(W+1)+i];
                }
            }
        }
        if ((ConditionFaceYm==0))
        {
            /**************  Face Km sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Tkk_BufferAreteCoinKm[i*(W+1)+j]=	Tyy_XmYmZp_X[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                        Tyy_XmYmZp_Y[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                        Tyy_XmYmZp_Z[j*(W+1)*(W+1)+0*(W+1)+i];
                }
            }
        }
    }

    if (!strcmp(TypeArete,"ZmXp"))
    {

#pragma omp parallel for schedule(guided)
        for(k=0; k<Y; k++)
        {
            Carte_Tampon[k] = Carte[(X-1)*(Y)*(Z)+(k)*(Z)+(0)];
        }
        /**************  Face J sur PML Face I *********************************/

#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W); i++)
        {
            for(k=0; k<(Y); k++)
            {
                Tij_BufferAreteFaceI[i*(Y)+k] =	Tzx_Zm_Z[i*(X+1)*(Y)+X*(Y)+k]	+
                                                Tzx_Zm_XY[i*(X+1)*(Y)+X*(Y)+k];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(Y+1); k++)
            {
                Tjk_BufferAreteFaceI[i*(Y+1)+k] =	Txy_Zm[i*(X+1)*(Y+1)+X*(Y+1)+k];
            }
        }

        /**************  Face I sur PML Face J *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W); j++)
        {
            for(k=0; k<(Y); k++)
            {
                Tij_BufferAreteFaceJ[j*(Y)+k] =	Tzx_Xp_X[j*(Y)*(Z+1)+k*(Z+1)+0]	+
                                                Tzx_Xp_YZ[j*(Y)*(Z+1)+k*(Z+1)+0];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(Y+1); k++)
            {
                Tki_BufferAreteFaceJ[j*(Y+1)+k] =	Tyz_Xp[j*(Y+1)*(Z+1)+k*(Z+1)+0];
            }
        }
        if ((ConditionFaceYp==0))
        {
            /**************  Face Kp sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Tkk_BufferAreteCoinKp[i*(W+1)+j]=	Tyy_XpYpZm_X[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                        Tyy_XpYpZm_Y[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                        Tyy_XpYpZm_Z[j*(W+1)*(W+1)+0*(W+1)+i];
                }
            }
        }
        if ((ConditionFaceYm==0))
        {
            /**************  Face Km sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Tkk_BufferAreteCoinKm[i*(W+1)+j]=	Tyy_XpYmZm_X[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                        Tyy_XpYmZm_Y[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                        Tyy_XpYmZm_Z[j*(W+1)*(W+1)+0*(W+1)+i];
                }
            }
        }
    }
    if (!strcmp(TypeArete,"ZmXm"))
    {

#pragma omp parallel for schedule(guided)
        for(k=0; k<Y; k++)
        {
            Carte_Tampon[k] = Carte[(0)*(Y)*(Z)+(k)*(Z)+(0)];
        }
        /**************  Face J sur PML Face I *********************************/

#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W); i++)
        {
            for(k=0; k<(Y); k++)
            {
                Tij_BufferAreteFaceI[i*(Y)+k] =	Tzx_Zm_Z[i*(X+1)*(Y)+0*(Y)+k]	+
                                                Tzx_Zm_XY[i*(X+1)*(Y)+0*(Y)+k];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(Y+1); k++)
            {
                Tjk_BufferAreteFaceI[i*(Y+1)+k] =	Txy_Zm[i*(X+1)*(Y+1)+0*(Y+1)+k];
            }
        }

        /**************  Face I sur PML Face J *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W); j++)
        {
            for(k=0; k<(Y); k++)
            {
                Tij_BufferAreteFaceJ[j*(Y)+k] =	Tzx_Xm_X[j*(Y)*(Z+1)+k*(Z+1)+0]	+
                                                Tzx_Xm_YZ[j*(Y)*(Z+1)+k*(Z+1)+0];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(Y+1); k++)
            {
                Tki_BufferAreteFaceJ[j*(Y+1)+k] =	Tyz_Xm[j*(Y+1)*(Z+1)+k*(Z+1)+0];
            }
        }
        if ((ConditionFaceYp==0))
        {
            /**************  Face Kp sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Tkk_BufferAreteCoinKp[i*(W+1)+j]=	Tyy_XmYpZm_X[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                        Tyy_XmYpZm_Y[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                        Tyy_XmYpZm_Z[j*(W+1)*(W+1)+0*(W+1)+i];
                }
            }
        }
        if ((ConditionFaceYm==0))
        {
            /**************  Face Km sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Tkk_BufferAreteCoinKm[i*(W+1)+j]=	Tyy_XmYmZm_X[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                        Tyy_XmYmZm_Y[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                        Tyy_XmYmZm_Z[j*(W+1)*(W+1)+0*(W+1)+i];
                }
            }
        }
    }
}

void MiseAJourVitessesTamponAretePML(const char *TypeArete)
{
    int i,j,k;

    if (!strcmp(TypeArete,"XpYp"))
    {

        TableCii = TableC11;
        TableCjj = TableC22;
        TableCkk = TableC33;
        TableCij = TableC12;
        TableCjk = TableC23;
        TableCki = TableC31;
        TableCjkjk = TableC44;
        TableCkiki = TableC55;
        TableCijij = TableC66;

#pragma omp parallel for schedule(guided)
        for(k=0; k<Z; k++)
        {
            Carte_Tampon[k] = Carte[(X-1)*(Y)*(Z)+(Y-1)*(Z)+k];
        }
        /**************  Face J sur PML Face I *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(Z); k++)
            {
                Vj_BufferAreteFaceI[i*(Z)+k] =	Vy_Xp_X[i*(Y+1)*(Z)+Y*(Z)+k]	+
                                                Vy_Xp_YZ[i*(Y+1)*(Z)+Y*(Z)+k];
            }
        }
        /**************  Face I sur PML Face J *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(Z); k++)
            {
                Vi_BufferAreteFaceJ[j*(Z)+k] =	Vx_Yp_Y[j*(Z)*(X+1)+k*(X+1)+X]	+
                                                Vx_Yp_ZX[j*(Z)*(X+1)+k*(X+1)+X];
            }
        }
        if ((ConditionFaceZp==0))
        {
            /**************  Face Kp sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Vi_BufferAreteCoinKp[i*(W+1)+j] = Vx_XpYpZp_X[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                      Vx_XpYpZp_Y[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                      Vx_XpYpZp_Z[i*(W+1)*(W+1)+j*(W+1)+0];
                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W); j++)
                {
                    Vj_BufferAreteCoinKp[i*(W)+j] = Vy_XpYpZp_X[i*(W)*(W+1)+j*(W+1)+0] +
                                                    Vy_XpYpZp_Y[i*(W)*(W+1)+j*(W+1)+0] +
                                                    Vy_XpYpZp_Z[i*(W)*(W+1)+j*(W+1)+0];
                }
            }
        }

        if ((ConditionFaceZm==0))
        {
            /**************  Face Km sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Vi_BufferAreteCoinKm[i*(W+1)+j] = Vx_XpYpZm_X[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                      Vx_XpYpZm_Y[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                      Vx_XpYpZm_Z[i*(W+1)*(W+1)+j*(W+1)+0];
                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W); j++)
                {
                    Vj_BufferAreteCoinKm[i*(W)+j] = Vy_XpYpZm_X[i*(W)*(W+1)+j*(W+1)+0] +
                                                    Vy_XpYpZm_Y[i*(W)*(W+1)+j*(W+1)+0] +
                                                    Vy_XpYpZm_Z[i*(W)*(W+1)+j*(W+1)+0];
                }
            }
        }
    }

    if (!strcmp(TypeArete,"XmYm"))
    {

        TableCii = TableC11;
        TableCjj = TableC22;
        TableCkk = TableC33;
        TableCij = TableC12;
        TableCjk = TableC23;
        TableCki = TableC31;
        TableCjkjk = TableC44;
        TableCkiki = TableC55;
        TableCijij = TableC66;

#pragma omp parallel for schedule(guided)
        for(k=0; k<Z; k++)
        {
            Carte_Tampon[k] = Carte[(0)*(Y)*(Z)+(0)*(Z)+k];
        }
        /**************  Face J sur PML Face I *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(Z); k++)
            {
                Vj_BufferAreteFaceI[i*(Z)+k] =	Vy_Xm_X[i*(Y+1)*(Z)+0*(Z)+k]	+
                                                Vy_Xm_YZ[i*(Y+1)*(Z)+0*(Z)+k];
            }
        }
        /**************  Face I sur PML Face J *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(Z); k++)
            {
                Vi_BufferAreteFaceJ[j*(Z)+k] =	Vx_Ym_Y[j*(Z)*(X+1)+k*(X+1)+0]	+
                                                Vx_Ym_ZX[j*(Z)*(X+1)+k*(X+1)+0];
            }
        }
        if ((ConditionFaceZp==0))
        {
            /**************  Face Kp sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Vi_BufferAreteCoinKp[i*(W+1)+j] = Vx_XmYmZp_X[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                      Vx_XmYmZp_Y[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                      Vx_XmYmZp_Z[i*(W+1)*(W+1)+j*(W+1)+0];
                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W); j++)
                {
                    Vj_BufferAreteCoinKp[i*(W)+j] = Vy_XmYmZp_X[i*(W)*(W+1)+j*(W+1)+0] +
                                                    Vy_XmYmZp_Y[i*(W)*(W+1)+j*(W+1)+0] +
                                                    Vy_XmYmZp_Z[i*(W)*(W+1)+j*(W+1)+0];
                }
            }
        }

        if ((ConditionFaceZm==0))
        {
            /**************  Face Km sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Vi_BufferAreteCoinKm[i*(W+1)+j] = Vx_XmYmZm_X[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                      Vx_XmYmZm_Y[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                      Vx_XmYmZm_Z[i*(W+1)*(W+1)+j*(W+1)+0];
                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W); j++)
                {
                    Vj_BufferAreteCoinKm[i*(W)+j] = Vy_XmYmZm_X[i*(W)*(W+1)+j*(W+1)+0] +
                                                    Vy_XmYmZm_Y[i*(W)*(W+1)+j*(W+1)+0] +
                                                    Vy_XmYmZm_Z[i*(W)*(W+1)+j*(W+1)+0];
                }
            }
        }

    }
    if (!strcmp(TypeArete,"XmYp"))
    {

        TableCii = TableC11;
        TableCjj = TableC22;
        TableCkk = TableC33;
        TableCij = TableC12;
        TableCjk = TableC23;
        TableCki = TableC31;
        TableCjkjk = TableC44;
        TableCkiki = TableC55;
        TableCijij = TableC66;

#pragma omp parallel for schedule(guided)
        for(k=0; k<Z; k++)
        {
            Carte_Tampon[k] = Carte[(0)*(Y)*(Z)+(Y-1)*(Z)+k];
        }
        /**************  Face J sur PML Face I *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(Z); k++)
            {
                Vj_BufferAreteFaceI[i*(Z)+k] =	Vy_Xm_X[i*(Y+1)*(Z)+Y*(Z)+k]	+
                                                Vy_Xm_YZ[i*(Y+1)*(Z)+Y*(Z)+k];
            }
        }
        /**************  Face I sur PML Face J *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(Z); k++)
            {
                Vi_BufferAreteFaceJ[j*(Z)+k] =	Vx_Yp_Y[j*(Z)*(X+1)+k*(X+1)+0]	+
                                                Vx_Yp_ZX[j*(Z)*(X+1)+k*(X+1)+0];
            }
        }
        if ((ConditionFaceZp==0))
        {
            /**************  Face Kp sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Vi_BufferAreteCoinKp[i*(W+1)+j] = Vx_XmYpZp_X[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                      Vx_XmYpZp_Y[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                      Vx_XmYpZp_Z[i*(W+1)*(W+1)+j*(W+1)+0];
                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W); j++)
                {
                    Vj_BufferAreteCoinKp[i*(W)+j] = Vy_XmYpZp_X[i*(W)*(W+1)+j*(W+1)+0] +
                                                    Vy_XmYpZp_Y[i*(W)*(W+1)+j*(W+1)+0] +
                                                    Vy_XmYpZp_Z[i*(W)*(W+1)+j*(W+1)+0];
                }
            }
        }

        if ((ConditionFaceZm==0))
        {
            /**************  Face Km sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Vi_BufferAreteCoinKm[i*(W+1)+j] = Vx_XmYpZm_X[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                      Vx_XmYpZm_Y[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                      Vx_XmYpZm_Z[i*(W+1)*(W+1)+j*(W+1)+0];
                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W); j++)
                {
                    Vj_BufferAreteCoinKm[i*(W)+j] = Vy_XmYpZm_X[i*(W)*(W+1)+j*(W+1)+0] +
                                                    Vy_XmYpZm_Y[i*(W)*(W+1)+j*(W+1)+0] +
                                                    Vy_XmYpZm_Z[i*(W)*(W+1)+j*(W+1)+0];
                }
            }
        }
    }
    if (!strcmp(TypeArete,"XpYm"))
    {

        TableCii = TableC11;
        TableCjj = TableC22;
        TableCkk = TableC33;
        TableCij = TableC12;
        TableCjk = TableC23;
        TableCki = TableC31;
        TableCjkjk = TableC44;
        TableCkiki = TableC55;
        TableCijij = TableC66;

#pragma omp parallel for schedule(guided)
        for(k=0; k<Z; k++)
        {
            Carte_Tampon[k] = Carte[(X-1)*(Y)*(Z)+(0)*(Z)+k];
        }
        /**************  Face J sur PML Face I *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(Z); k++)
            {
                Vj_BufferAreteFaceI[i*(Z)+k] =	Vy_Xp_X[i*(Y+1)*(Z)+0*(Z)+k]	+
                                                Vy_Xp_YZ[i*(Y+1)*(Z)+0*(Z)+k];
            }
        }
        /**************  Face I sur PML Face J *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(Z); k++)
            {
                Vi_BufferAreteFaceJ[j*(Z)+k] =	Vx_Ym_Y[j*(Z)*(X+1)+k*(X+1)+X]	+
                                                Vx_Ym_ZX[j*(Z)*(X+1)+k*(X+1)+X];
            }
        }
        if ((ConditionFaceZp==0))
        {
            /**************  Face Kp sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Vi_BufferAreteCoinKp[i*(W+1)+j] = Vx_XpYmZp_X[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                      Vx_XpYmZp_Y[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                      Vx_XpYmZp_Z[i*(W+1)*(W+1)+j*(W+1)+0];
                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W); j++)
                {
                    Vj_BufferAreteCoinKp[i*(W)+j] = Vy_XpYmZp_X[i*(W)*(W+1)+j*(W+1)+0] +
                                                    Vy_XpYmZp_Y[i*(W)*(W+1)+j*(W+1)+0] +
                                                    Vy_XpYmZp_Z[i*(W)*(W+1)+j*(W+1)+0];
                }
            }
        }

        if ((ConditionFaceZm==0))
        {
            /**************  Face Km sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Vi_BufferAreteCoinKm[i*(W+1)+j] = Vx_XpYmZm_X[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                      Vx_XpYmZm_Y[i*(W+1)*(W+1)+j*(W+1)+0] +
                                                      Vx_XpYmZm_Z[i*(W+1)*(W+1)+j*(W+1)+0];
                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W); j++)
                {
                    Vj_BufferAreteCoinKm[i*(W)+j] = Vy_XpYmZm_X[i*(W)*(W+1)+j*(W+1)+0] +
                                                    Vy_XpYmZm_Y[i*(W)*(W+1)+j*(W+1)+0] +
                                                    Vy_XpYmZm_Z[i*(W)*(W+1)+j*(W+1)+0];
                }
            }
        }

    }
    /**************************************************************************/
    /**************************************************************************/
    /*********** Aretes perpendiculaire au plan YZ ****************************/
    /**************************************************************************/
    /**************************************************************************/

    if (!strcmp(TypeArete,"YpZp"))
    {

        TableCii = TableC22;
        TableCjj = TableC33;
        TableCkk = TableC11;
        TableCij = TableC23;
        TableCjk = TableC31;
        TableCki = TableC12;
        TableCjkjk = TableC55;
        TableCkiki = TableC66;
        TableCijij = TableC44;

#pragma omp parallel for schedule(guided)
        for(k=0; k<X; k++)
        {
            Carte_Tampon[k] = Carte[(k)*(Y)*(Z)+(Y-1)*(Z)+Z-1];
        }
        /**************  Face J sur PML Face I *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(X); k++)
            {
                Vj_BufferAreteFaceI[i*(X)+k] =	Vz_Yp_Y[i*(Z+1)*(X)+Z*(X)+k]	+
                                                Vz_Yp_ZX[i*(Z+1)*(X)+Z*(X)+k];
            }
        }
        /**************  Face I sur PML Face J *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(X); k++)
            {
                Vi_BufferAreteFaceJ[j*(X)+k] =	Vy_Zp_Z[j*(X)*(Y+1)+k*(Y+1)+Y]	+
                                                Vy_Zp_XY[j*(X)*(Y+1)+k*(Y+1)+Y];
            }
        }
        if ((ConditionFaceXp==0))
        {
            /**************  Face Kp sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Vi_BufferAreteCoinKp[i*(W+1)+j] = Vy_XpYpZp_X[0*(W)*(W+1)+i*(W+1)+j] +
                                                      Vy_XpYpZp_Y[0*(W)*(W+1)+i*(W+1)+j] +
                                                      Vy_XpYpZp_Z[0*(W)*(W+1)+i*(W+1)+j];
                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W); j++)
                {
                    Vj_BufferAreteCoinKp[i*(W)+j] = Vz_XpYpZp_X[0*(W+1)*(W)+i*(W)+j] +
                                                    Vz_XpYpZp_Y[0*(W+1)*(W)+i*(W)+j] +
                                                    Vz_XpYpZp_Z[0*(W+1)*(W)+i*(W)+j];
                }
            }
        }

        if ((ConditionFaceXm==0))
        {
            /**************  Face Km sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Vi_BufferAreteCoinKm[i*(W+1)+j] = Vy_XmYpZp_X[0*(W)*(W+1)+i*(W+1)+j] +
                                                      Vy_XmYpZp_Y[0*(W)*(W+1)+i*(W+1)+j] +
                                                      Vy_XmYpZp_Z[0*(W)*(W+1)+i*(W+1)+j];
                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W); j++)
                {
                    Vj_BufferAreteCoinKm[i*(W)+j] = Vz_XmYpZp_X[0*(W+1)*(W)+i*(W)+j] +
                                                    Vz_XmYpZp_Y[0*(W+1)*(W)+i*(W)+j] +
                                                    Vz_XmYpZp_Z[0*(W+1)*(W)+i*(W)+j];
                }
            }
        }

    }
    if (!strcmp(TypeArete,"YpZm"))
    {

        TableCii = TableC22;
        TableCjj = TableC33;
        TableCkk = TableC11;
        TableCij = TableC23;
        TableCjk = TableC31;
        TableCki = TableC12;
        TableCjkjk = TableC55;
        TableCkiki = TableC66;
        TableCijij = TableC44;

#pragma omp parallel for schedule(guided)
        for(k=0; k<X; k++)
        {
            Carte_Tampon[k] = Carte[(k)*(Y)*(Z)+(Y-1)*(Z)+0];
        }
        /**************  Face J sur PML Face I *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(X); k++)
            {
                Vj_BufferAreteFaceI[i*(X)+k] =	Vz_Yp_Y[i*(Z+1)*(X)+0*(X)+k]	+
                                                Vz_Yp_ZX[i*(Z+1)*(X)+0*(X)+k];
            }
        }
        /**************  Face I sur PML Face J *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(X); k++)
            {
                Vi_BufferAreteFaceJ[j*(X)+k] =	Vy_Zm_Z[j*(X)*(Y+1)+k*(Y+1)+Y]	+
                                                Vy_Zm_XY[j*(X)*(Y+1)+k*(Y+1)+Y];
            }
        }
        if ((ConditionFaceXp==0))
        {
            /**************  Face Kp sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Vi_BufferAreteCoinKp[i*(W+1)+j] = Vy_XpYpZm_X[0*(W)*(W+1)+i*(W+1)+j] +
                                                      Vy_XpYpZm_Y[0*(W)*(W+1)+i*(W+1)+j] +
                                                      Vy_XpYpZm_Z[0*(W)*(W+1)+i*(W+1)+j];
                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W); j++)
                {
                    Vj_BufferAreteCoinKp[i*(W)+j] = Vz_XpYpZm_X[0*(W+1)*(W)+i*(W)+j] +
                                                    Vz_XpYpZm_Y[0*(W+1)*(W)+i*(W)+j] +
                                                    Vz_XpYpZm_Z[0*(W+1)*(W)+i*(W)+j];
                }
            }
        }

        if ((ConditionFaceXm==0))
        {
            /**************  Face Km sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Vi_BufferAreteCoinKm[i*(W+1)+j] = Vy_XmYpZm_X[0*(W)*(W+1)+i*(W+1)+j] +
                                                      Vy_XmYpZm_Y[0*(W)*(W+1)+i*(W+1)+j] +
                                                      Vy_XmYpZm_Z[0*(W)*(W+1)+i*(W+1)+j];
                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W); j++)
                {
                    Vj_BufferAreteCoinKm[i*(W)+j] = Vz_XmYpZm_X[0*(W+1)*(W)+i*(W)+j] +
                                                    Vz_XmYpZm_Y[0*(W+1)*(W)+i*(W)+j] +
                                                    Vz_XmYpZm_Z[0*(W+1)*(W)+i*(W)+j];
                }
            }
        }

    }
    if (!strcmp(TypeArete,"YmZp"))
    {

        TableCii = TableC22;
        TableCjj = TableC33;
        TableCkk = TableC11;
        TableCij = TableC23;
        TableCjk = TableC31;
        TableCki = TableC12;
        TableCjkjk = TableC55;
        TableCkiki = TableC66;
        TableCijij = TableC44;

#pragma omp parallel for schedule(guided)
        for(k=0; k<X; k++)
        {
            Carte_Tampon[k] = Carte[(k)*(Y)*(Z)+(0)*(Z)+Z-1];
        }
        /**************  Face J sur PML Face I *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(X); k++)
            {
                Vj_BufferAreteFaceI[i*(X)+k] =	Vz_Ym_Y[i*(Z+1)*(X)+Z*(X)+k]	+
                                                Vz_Ym_ZX[i*(Z+1)*(X)+Z*(X)+k];
            }
        }
        /**************  Face I sur PML Face J *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(X); k++)
            {
                Vi_BufferAreteFaceJ[j*(X)+k] =	Vy_Zp_Z[j*(X)*(Y+1)+k*(Y+1)+0]	+
                                                Vy_Zp_XY[j*(X)*(Y+1)+k*(Y+1)+0];
            }
        }
        if ((ConditionFaceXp==0))
        {
            /**************  Face Kp sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Vi_BufferAreteCoinKp[i*(W+1)+j] = Vy_XpYmZp_X[0*(W)*(W+1)+i*(W+1)+j] +
                                                      Vy_XpYmZp_Y[0*(W)*(W+1)+i*(W+1)+j] +
                                                      Vy_XpYmZp_Z[0*(W)*(W+1)+i*(W+1)+j];
                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W); j++)
                {
                    Vj_BufferAreteCoinKp[i*(W)+j] = Vz_XpYmZp_X[0*(W+1)*(W)+i*(W)+j] +
                                                    Vz_XpYmZp_Y[0*(W+1)*(W)+i*(W)+j] +
                                                    Vz_XpYmZp_Z[0*(W+1)*(W)+i*(W)+j];
                }
            }
        }

        if ((ConditionFaceXm==0))
        {
            /**************  Face Km sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Vi_BufferAreteCoinKm[i*(W+1)+j] = Vy_XmYmZp_X[0*(W)*(W+1)+i*(W+1)+j] +
                                                      Vy_XmYmZp_Y[0*(W)*(W+1)+i*(W+1)+j] +
                                                      Vy_XmYmZp_Z[0*(W)*(W+1)+i*(W+1)+j];
                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W); j++)
                {
                    Vj_BufferAreteCoinKm[i*(W)+j] = Vz_XmYmZp_X[0*(W+1)*(W)+i*(W)+j] +
                                                    Vz_XmYmZp_Y[0*(W+1)*(W)+i*(W)+j] +
                                                    Vz_XmYmZp_Z[0*(W+1)*(W)+i*(W)+j];
                }
            }
        }

    }
    if (!strcmp(TypeArete,"YmZm"))
    {

        TableCii = TableC22;
        TableCjj = TableC33;
        TableCkk = TableC11;
        TableCij = TableC23;
        TableCjk = TableC31;
        TableCki = TableC12;
        TableCjkjk = TableC55;
        TableCkiki = TableC66;
        TableCijij = TableC44;

#pragma omp parallel for schedule(guided)
        for(k=0; k<X; k++)
        {
            Carte_Tampon[k] = Carte[(k)*(Y)*(Z)+(0)*(Z)+0];
        }
        /**************  Face J sur PML Face I *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(X); k++)
            {
                Vj_BufferAreteFaceI[i*(X)+k] =	Vz_Ym_Y[i*(Z+1)*(X)+0*(X)+k]	+
                                                Vz_Ym_ZX[i*(Z+1)*(X)+0*(X)+k];
            }
        }
        /**************  Face I sur PML Face J *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(X); k++)
            {
                Vi_BufferAreteFaceJ[j*(X)+k] =	Vy_Zm_Z[j*(X)*(Y+1)+k*(Y+1)+0]	+
                                                Vy_Zm_XY[j*(X)*(Y+1)+k*(Y+1)+0];
            }
        }
        if ((ConditionFaceXp==0))
        {
            /**************  Face Kp sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Vi_BufferAreteCoinKp[i*(W+1)+j] = Vy_XpYmZm_X[0*(W)*(W+1)+i*(W+1)+j] +
                                                      Vy_XpYmZm_Y[0*(W)*(W+1)+i*(W+1)+j] +
                                                      Vy_XpYmZm_Z[0*(W)*(W+1)+i*(W+1)+j];
                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W); j++)
                {
                    Vj_BufferAreteCoinKp[i*(W)+j] = Vz_XpYmZm_X[0*(W+1)*(W)+i*(W)+j] +
                                                    Vz_XpYmZm_Y[0*(W+1)*(W)+i*(W)+j] +
                                                    Vz_XpYmZm_Z[0*(W+1)*(W)+i*(W)+j];
                }
            }
        }

        if ((ConditionFaceXm==0))
        {
            /**************  Face Km sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Vi_BufferAreteCoinKm[i*(W+1)+j] = Vy_XmYmZm_X[0*(W)*(W+1)+i*(W+1)+j] +
                                                      Vy_XmYmZm_Y[0*(W)*(W+1)+i*(W+1)+j] +
                                                      Vy_XmYmZm_Z[0*(W)*(W+1)+i*(W+1)+j];
                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W); j++)
                {
                    Vj_BufferAreteCoinKm[i*(W)+j] = Vz_XmYmZm_X[0*(W+1)*(W)+i*(W)+j] +
                                                    Vz_XmYmZm_Y[0*(W+1)*(W)+i*(W)+j] +
                                                    Vz_XmYmZm_Z[0*(W+1)*(W)+i*(W)+j];
                }
            }
        }

    }
    /**************************************************************************/
    /**************************************************************************/
    /*********** Aretes perpendiculaire au plan ZX ****************************/
    /**************************************************************************/
    /**************************************************************************/

    if (!strcmp(TypeArete,"ZpXp"))
    {

        TableCii = TableC33;
        TableCjj = TableC11;
        TableCkk = TableC22;
        TableCij = TableC31;
        TableCjk = TableC12;
        TableCki = TableC23;
        TableCjkjk = TableC66;
        TableCkiki = TableC44;
        TableCijij = TableC55;

#pragma omp parallel for schedule(guided)
        for(k=0; k<Y; k++)
        {
            Carte_Tampon[k] = Carte[(X-1)*(Y)*(Z)+(k)*(Z)+(Z-1)];
        }
        /**************  Face J sur PML Face I *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(Y); k++)
            {
                Vj_BufferAreteFaceI[i*(Y)+k] =	Vx_Zp_Z[i*(X+1)*(Y)+X*(Y)+k]	+
                                                Vx_Zp_XY[i*(X+1)*(Y)+X*(Y)+k];
            }
        }
        /**************  Face I sur PML Face J *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(Y); k++)
            {
                Vi_BufferAreteFaceJ[j*(Y)+k] =	Vz_Xp_X[j*(Y)*(Z+1)+k*(Z+1)+Z]	+
                                                Vz_Xp_YZ[j*(Y)*(Z+1)+k*(Z+1)+Z];
            }
        }
        if ((ConditionFaceYp==0))
        {
            /**************  Face Kp sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Vi_BufferAreteCoinKp[i*(W+1)+j] = Vz_XpYpZp_X[j*(W+1)*(W)+0*(W)+i] +
                                                      Vz_XpYpZp_Y[j*(W+1)*(W)+0*(W)+i] +
                                                      Vz_XpYpZp_Z[j*(W+1)*(W)+0*(W)+i];
                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W); j++)
                {
                    Vj_BufferAreteCoinKp[i*(W)+j] = Vx_XpYpZp_X[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                    Vx_XpYpZp_Y[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                    Vx_XpYpZp_Z[j*(W+1)*(W+1)+0*(W+1)+i];
                }
            }
        }
        if ((ConditionFaceYm==0))
        {
            /**************  Face Km sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Vi_BufferAreteCoinKm[i*(W+1)+j] = Vz_XpYmZp_X[j*(W+1)*(W)+0*(W)+i] +
                                                      Vz_XpYmZp_Y[j*(W+1)*(W)+0*(W)+i] +
                                                      Vz_XpYmZp_Z[j*(W+1)*(W)+0*(W)+i];
                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W); j++)
                {
                    Vj_BufferAreteCoinKm[i*(W)+j] = Vx_XpYmZp_X[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                    Vx_XpYmZp_Y[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                    Vx_XpYmZp_Z[j*(W+1)*(W+1)+0*(W+1)+i];
                }
            }
        }

    }
    if (!strcmp(TypeArete,"ZpXm"))
    {

        TableCii = TableC33;
        TableCjj = TableC11;
        TableCkk = TableC22;
        TableCij = TableC31;
        TableCjk = TableC12;
        TableCki = TableC23;
        TableCjkjk = TableC66;
        TableCkiki = TableC44;
        TableCijij = TableC55;

#pragma omp parallel for schedule(guided)
        for(k=0; k<Y; k++)
        {
            Carte_Tampon[k] = Carte[(0)*(Y)*(Z)+(k)*(Z)+(Z-1)];
        }
        /**************  Face J sur PML Face I *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(Y); k++)
            {
                Vj_BufferAreteFaceI[i*(Y)+k] =	Vx_Zp_Z[i*(X+1)*(Y)+0*(Y)+k]	+
                                                Vx_Zp_XY[i*(X+1)*(Y)+0*(Y)+k];
            }
        }
        /**************  Face I sur PML Face J *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(Y); k++)
            {
                Vi_BufferAreteFaceJ[j*(Y)+k] =	Vz_Xm_X[j*(Y)*(Z+1)+k*(Z+1)+Z]	+
                                                Vz_Xm_YZ[j*(Y)*(Z+1)+k*(Z+1)+Z];
            }
        }
        if ((ConditionFaceYp==0))
        {
            /**************  Face Kp sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Vi_BufferAreteCoinKp[i*(W+1)+j] = Vz_XmYpZp_X[j*(W+1)*(W)+0*(W)+i] +
                                                      Vz_XmYpZp_Y[j*(W+1)*(W)+0*(W)+i] +
                                                      Vz_XmYpZp_Z[j*(W+1)*(W)+0*(W)+i];
                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W); j++)
                {
                    Vj_BufferAreteCoinKp[i*(W)+j] = Vx_XmYpZp_X[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                    Vx_XmYpZp_Y[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                    Vx_XmYpZp_Z[j*(W+1)*(W+1)+0*(W+1)+i];
                }
            }
        }
        if ((ConditionFaceYm==0))
        {
            /**************  Face Km sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Vi_BufferAreteCoinKm[i*(W+1)+j] = Vz_XmYmZp_X[j*(W+1)*(W)+0*(W)+i] +
                                                      Vz_XmYmZp_Y[j*(W+1)*(W)+0*(W)+i] +
                                                      Vz_XmYmZp_Z[j*(W+1)*(W)+0*(W)+i];
                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W); j++)
                {
                    Vj_BufferAreteCoinKm[i*(W)+j] = Vx_XmYmZp_X[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                    Vx_XmYmZp_Y[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                    Vx_XmYmZp_Z[j*(W+1)*(W+1)+0*(W+1)+i];
                }
            }
        }

    }

    if (!strcmp(TypeArete,"ZmXp"))
    {

        TableCii = TableC33;
        TableCjj = TableC11;
        TableCkk = TableC22;
        TableCij = TableC31;
        TableCjk = TableC12;
        TableCki = TableC23;
        TableCjkjk = TableC66;
        TableCkiki = TableC44;
        TableCijij = TableC55;

#pragma omp parallel for schedule(guided)
        for(k=0; k<Y; k++)
        {
            Carte_Tampon[k] = Carte[(X-1)*(Y)*(Z)+(k)*(Z)+(0)];
        }
        /**************  Face J sur PML Face I *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(Y); k++)
            {
                Vj_BufferAreteFaceI[i*(Y)+k] =	Vx_Zm_Z[i*(X+1)*(Y)+X*(Y)+k]	+
                                                Vx_Zm_XY[i*(X+1)*(Y)+X*(Y)+k];
            }
        }
        /**************  Face I sur PML Face J *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(Y); k++)
            {
                Vi_BufferAreteFaceJ[j*(Y)+k] =	Vz_Xp_X[j*(Y)*(Z+1)+k*(Z+1)+0]	+
                                                Vz_Xp_YZ[j*(Y)*(Z+1)+k*(Z+1)+0];
            }
        }
        if ((ConditionFaceYp==0))
        {
            /**************  Face Kp sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Vi_BufferAreteCoinKp[i*(W+1)+j] = Vz_XpYpZm_X[j*(W+1)*(W)+0*(W)+i] +
                                                      Vz_XpYpZm_Y[j*(W+1)*(W)+0*(W)+i] +
                                                      Vz_XpYpZm_Z[j*(W+1)*(W)+0*(W)+i];
                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W); j++)
                {
                    Vj_BufferAreteCoinKp[i*(W)+j] = Vx_XpYpZm_X[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                    Vx_XpYpZm_Y[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                    Vx_XpYpZm_Z[j*(W+1)*(W+1)+0*(W+1)+i];
                }
            }
        }
        if ((ConditionFaceYm==0))
        {
            /**************  Face Km sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Vi_BufferAreteCoinKm[i*(W+1)+j] = Vz_XpYmZm_X[j*(W+1)*(W)+0*(W)+i] +
                                                      Vz_XpYmZm_Y[j*(W+1)*(W)+0*(W)+i] +
                                                      Vz_XpYmZm_Z[j*(W+1)*(W)+0*(W)+i];
                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W); j++)
                {
                    Vj_BufferAreteCoinKm[i*(W)+j] = Vx_XpYmZm_X[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                    Vx_XpYmZm_Y[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                    Vx_XpYmZm_Z[j*(W+1)*(W+1)+0*(W+1)+i];
                }
            }
        }

    }
    if (!strcmp(TypeArete,"ZmXm"))
    {

        TableCii = TableC33;
        TableCjj = TableC11;
        TableCkk = TableC22;
        TableCij = TableC31;
        TableCjk = TableC12;
        TableCki = TableC23;
        TableCjkjk = TableC66;
        TableCkiki = TableC44;
        TableCijij = TableC55;

#pragma omp parallel for schedule(guided)
        for(k=0; k<Y; k++)
        {
            Carte_Tampon[k] = Carte[(0)*(Y)*(Z)+(k)*(Z)+(0)];
        }
        /**************  Face J sur PML Face I *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(Y); k++)
            {
                Vj_BufferAreteFaceI[i*(Y)+k] =	Vx_Zm_Z[i*(X+1)*(Y)+0*(Y)+k]	+
                                                Vx_Zm_XY[i*(X+1)*(Y)+0*(Y)+k];
            }
        }
        /**************  Face I sur PML Face J *********************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(Y); k++)
            {
                Vi_BufferAreteFaceJ[j*(Y)+k] =	Vz_Xm_X[j*(Y)*(Z+1)+k*(Z+1)+0]	+
                                                Vz_Xm_YZ[j*(Y)*(Z+1)+k*(Z+1)+0];
            }
        }
        if ((ConditionFaceYp==0))
        {
            /**************  Face Kp sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Vi_BufferAreteCoinKp[i*(W+1)+j] = Vz_XmYpZm_X[j*(W+1)*(W)+0*(W)+i] +
                                                      Vz_XmYpZm_Y[j*(W+1)*(W)+0*(W)+i] +
                                                      Vz_XmYpZm_Z[j*(W+1)*(W)+0*(W)+i];
                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W); j++)
                {
                    Vj_BufferAreteCoinKp[i*(W)+j] = Vx_XmYpZm_X[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                    Vx_XmYpZm_Y[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                    Vx_XmYpZm_Z[j*(W+1)*(W+1)+0*(W+1)+i];
                }
            }
        }
        if ((ConditionFaceYm==0))
        {
            /**************  Face Km sur PML Coin IJK********************************/
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(W+1); j++)
                {
                    Vi_BufferAreteCoinKm[i*(W+1)+j] = Vz_XmYmZm_X[j*(W+1)*(W)+0*(W)+i] +
                                                      Vz_XmYmZm_Y[j*(W+1)*(W)+0*(W)+i] +
                                                      Vz_XmYmZm_Z[j*(W+1)*(W)+0*(W)+i];
                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(W); j++)
                {
                    Vj_BufferAreteCoinKm[i*(W)+j] = Vx_XmYmZm_X[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                    Vx_XmYmZm_Y[j*(W+1)*(W+1)+0*(W+1)+i] +
                                                    Vx_XmYmZm_Z[j*(W+1)*(W+1)+0*(W+1)+i];
                }
            }
        }
    }
}

void MiseAJourVitessesTamponFacePML(const char *TypeFace)
{

    int i,j,k;

    if (!strcmp(TypeFace,"Xp"))
    {

        TableCii = TableC11;
        TableCjj = TableC22;
        TableCkk = TableC33;
        TableCij = TableC12;
        TableCjk = TableC23;
        TableCki = TableC31;
        TableCjkjk = TableC44;
        TableCkiki = TableC55;
        TableCijij = TableC66;

#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<Y; j++)
        {
            for(k=0; k<Z; k++)
            {
                Vi_BufferFaceBlocI[j*(Z)+k] = Vx[(X)*(Y)*(Z)+j*(Z)+k];
                Carte_Tampon[j*(Z)+k] = Carte[(X-1)*(Y)*(Z)+j*(Z)+k];
            }
        }

        /************************************************************************/
        /**************  Face  sur PML Arete XY *********************************/
        /************************************************************************/
        if (ConditionFaceYp==0)
        {
#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(k=0; k<(Z); k++)
                {
                    Vi_BufferFaceAreteJp[i*(Z)+k] =	Vx_XpYp_X[i*(W+1)*(Z)+k]	+
                                                    Vx_XpYp_Y[i*(W+1)*(Z)+k]	+
                                                    Vx_XpYp_Z[i*(W+1)*(Z)+k];
                }
            }

#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(k=0; k<(Z+1); k++)
                {
                    Vk_BufferFaceAreteJp[i*(Z+1)+k] =	Vz_XpYp_X[i*(W+1)*(Z+1)+k]	+
                                                        Vz_XpYp_Y[i*(W+1)*(Z+1)+k]	+
                                                        Vz_XpYp_Z[i*(W+1)*(Z+1)+k];
                }
            }
        }
        if (ConditionFaceYm==0)
        {

#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(k=0; k<(Z); k++)
                {
                    Vi_BufferFaceAreteJm[i*(Z)+k] =	Vx_XpYm_X[i*(W+1)*(Z)+k]	+
                                                    Vx_XpYm_Y[i*(W+1)*(Z)+k]	+
                                                    Vx_XpYm_Z[i*(W+1)*(Z)+k];
                }
            }

#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(k=0; k<(Z+1); k++)
                {
                    Vk_BufferFaceAreteJm[i*(Z+1)+k] =	Vz_XpYm_X[i*(W+1)*(Z+1)+k]	+
                                                        Vz_XpYm_Y[i*(W+1)*(Z+1)+k]	+
                                                        Vz_XpYm_Z[i*(W+1)*(Z+1)+k];
                }
            }
        }
        /************************************************************************/
        /**************  Face  sur PML Arete ZX *********************************/
        /************************************************************************/

        if (ConditionFaceZp==0)
        {
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(Y); j++)
                {
                    Vi_BufferFaceAreteKp[i*(Y)+j] =	Vx_ZpXp_X[i*(Y)+j]	+
                                                    Vx_ZpXp_Y[i*(Y)+j]	+
                                                    Vx_ZpXp_Z[i*(Y)+j];
                }
            }

#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(Y+1); j++)
                {
                    Vj_BufferFaceAreteKp[i*(Y+1)+j] =	Vy_ZpXp_X[i*(Y+1)+j]	+
                                                        Vy_ZpXp_Y[i*(Y+1)+j]	+
                                                        Vy_ZpXp_Z[i*(Y+1)+j];
                }
            }
        }

        if (ConditionFaceZm==0)
        {
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(Y); j++)
                {
                    Vi_BufferFaceAreteKm[i*(Y)+j] =	Vx_ZmXp_X[i*(Y)+j]	+
                                                    Vx_ZmXp_Y[i*(Y)+j]	+
                                                    Vx_ZmXp_Z[i*(Y)+j];
                }
            }

#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(Y+1); j++)
                {
                    Vj_BufferFaceAreteKm[i*(Y+1)+j] =	Vy_ZmXp_X[i*(Y+1)+j]	+
                                                        Vy_ZmXp_Y[i*(Y+1)+j]	+
                                                        Vy_ZmXp_Z[i*(Y+1)+j];
                }
            }
        }
    }
    if (!strcmp(TypeFace,"Xm"))
    {

        TableCii = TableC11;
        TableCjj = TableC22;
        TableCkk = TableC33;
        TableCij = TableC12;
        TableCjk = TableC23;
        TableCki = TableC31;
        TableCjkjk = TableC44;
        TableCkiki = TableC55;
        TableCijij = TableC66;

#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<Y; j++)
        {
            for(k=0; k<Z; k++)
            {
                Vi_BufferFaceBlocI[j*(Z)+k] = Vx[j*(Z)+k];
                Carte_Tampon[j*(Z)+k] = Carte[j*(Z)+k];
            }
        }
        /************************************************************************/
        /**************  Face  sur PML Arete XY *********************************/
        /************************************************************************/

        if (ConditionFaceYp==0)
        {
#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(k=0; k<(Z); k++)
                {
                    Vi_BufferFaceAreteJp[i*(Z)+k] =	Vx_XmYp_X[i*(W+1)*(Z)+k]	+
                                                    Vx_XmYp_Y[i*(W+1)*(Z)+k]	+
                                                    Vx_XmYp_Z[i*(W+1)*(Z)+k];
                }
            }

#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(k=0; k<(Z+1); k++)
                {
                    Vk_BufferFaceAreteJp[i*(Z+1)+k] =	Vz_XmYp_X[i*(W+1)*(Z+1)+k]	+
                                                        Vz_XmYp_Y[i*(W+1)*(Z+1)+k]	+
                                                        Vz_XmYp_Z[i*(W+1)*(Z+1)+k];
                }
            }
        }
        if (ConditionFaceYm==0)
        {
#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(k=0; k<(Z); k++)
                {
                    Vi_BufferFaceAreteJm[i*(Z)+k] =	Vx_XmYm_X[i*(W+1)*(Z)+k]	+
                                                    Vx_XmYm_Y[i*(W+1)*(Z)+k]	+
                                                    Vx_XmYm_Z[i*(W+1)*(Z)+k];
                }
            }

#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(k=0; k<(Z+1); k++)
                {
                    Vk_BufferFaceAreteJm[i*(Z+1)+k] =	Vz_XmYm_X[i*(W+1)*(Z+1)+k]	+
                                                        Vz_XmYm_Y[i*(W+1)*(Z+1)+k]	+
                                                        Vz_XmYm_Z[i*(W+1)*(Z+1)+k];
                }
            }
        }
        /************************************************************************/
        /**************  Face  sur PML Arete ZX *********************************/
        /************************************************************************/

        if (ConditionFaceZp==0)
        {
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(Y); j++)
                {
                    Vi_BufferFaceAreteKp[i*(Y)+j] =	Vx_ZpXm_X[i*(Y)+j]	+
                                                    Vx_ZpXm_Y[i*(Y)+j]	+
                                                    Vx_ZpXm_Z[i*(Y)+j];
                }
            }

#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(Y+1); j++)
                {
                    Vj_BufferFaceAreteKp[i*(Y+1)+j] =	Vy_ZpXm_X[i*(Y+1)+j]	+
                                                        Vy_ZpXm_Y[i*(Y+1)+j]	+
                                                        Vy_ZpXm_Z[i*(Y+1)+j];
                }
            }
        }

        if (ConditionFaceZm==0)
        {
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(Y); j++)
                {
                    Vi_BufferFaceAreteKm[i*(Y)+j] =	Vx_ZmXm_X[i*(Y)+j]	+
                                                    Vx_ZmXm_Y[i*(Y)+j]	+
                                                    Vx_ZmXm_Z[i*(Y)+j];
                }
            }

#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(Y+1); j++)
                {
                    Vj_BufferFaceAreteKm[i*(Y+1)+j] =	Vy_ZmXm_X[i*(Y+1)+j]	+
                                                        Vy_ZmXm_Y[i*(Y+1)+j]	+
                                                        Vy_ZmXm_Z[i*(Y+1)+j];
                }
            }
        }
    }

    if (!strcmp(TypeFace,"Yp"))
    {

        TableCii = TableC22;
        TableCjj = TableC33;
        TableCkk = TableC11;
        TableCij = TableC23;
        TableCjk = TableC31;
        TableCki = TableC12;
        TableCjkjk = TableC55;
        TableCkiki = TableC66;
        TableCijij = TableC44;

#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<Z; j++)
        {
            for(k=0; k<X; k++)
            {
                Vi_BufferFaceBlocI[j*(X)+k] = Vy[k*(Y+1)*(Z)+(Y)*(Z)+j];
                Carte_Tampon[j*(X)+k] = Carte[k*(Y)*(Z)+(Y-1)*(Z)+j];
            }
        }

        /************************************************************************/
        /**************  Face  sur PML Arete YZ *********************************/
        /************************************************************************/

        if (ConditionFaceZp==0)
        {
#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(k=0; k<(X); k++)
                {
                    Vi_BufferFaceAreteJp[i*(X)+k] =	Vy_YpZp_X[i*(W+1)*(X)+k]	+
                                                    Vy_YpZp_Y[i*(W+1)*(X)+k]	+
                                                    Vy_YpZp_Z[i*(W+1)*(X)+k];
                }
            }
#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(k=0; k<(X+1); k++)
                {
                    Vk_BufferFaceAreteJp[i*(X+1)+k] =	Vx_YpZp_X[i*(W+1)*(X+1)+k]	+
                                                        Vx_YpZp_Y[i*(W+1)*(X+1)+k]	+
                                                        Vx_YpZp_Z[i*(W+1)*(X+1)+k];
                }
            }
        }
        if (ConditionFaceZm==0)
        {
#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(k=0; k<(X); k++)
                {
                    Vi_BufferFaceAreteJm[i*(X)+k] =	Vy_YpZm_X[i*(W+1)*(X)+k]	+
                                                    Vy_YpZm_Y[i*(W+1)*(X)+k]	+
                                                    Vy_YpZm_Z[i*(W+1)*(X)+k];
                }
            }
#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(k=0; k<(X+1); k++)
                {
                    Vk_BufferFaceAreteJm[i*(X+1)+k] =	Vx_YpZm_X[i*(W+1)*(X+1)+k]	+
                                                        Vx_YpZm_Y[i*(W+1)*(X+1)+k]	+
                                                        Vx_YpZm_Z[i*(W+1)*(X+1)+k];
                }
            }
        }
        /************************************************************************/
        /**************  Face  sur PML Arete XY *********************************/
        /************************************************************************/

        if (ConditionFaceXp==0)
        {
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(Z); j++)
                {
                    Vi_BufferFaceAreteKp[i*(Z)+j] =	Vy_XpYp_X[i*(Z)+j]	+
                                                    Vy_XpYp_Y[i*(Z)+j]	+
                                                    Vy_XpYp_Z[i*(Z)+j];
                }
            }

#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(Z+1); j++)
                {
                    Vj_BufferFaceAreteKp[i*(Z+1)+j] =	Vz_XpYp_X[i*(Z+1)+j]	+
                                                        Vz_XpYp_Y[i*(Z+1)+j]	+
                                                        Vz_XpYp_Z[i*(Z+1)+j];
                }
            }
        }

        if (ConditionFaceXm==0)
        {
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(Z); j++)
                {
                    Vi_BufferFaceAreteKm[i*(Z)+j] =	Vy_XmYp_X[i*(Z)+j]	+
                                                    Vy_XmYp_Y[i*(Z)+j]	+
                                                    Vy_XmYp_Z[i*(Z)+j];
                }
            }

#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(Z+1); j++)
                {
                    Vj_BufferFaceAreteKm[i*(Z+1)+j] =	Vz_XmYp_X[i*(Z+1)+j]	+
                                                        Vz_XmYp_Y[i*(Z+1)+j]	+
                                                        Vz_XmYp_Z[i*(Z+1)+j];
                }
            }
        }
    }

    if (!strcmp(TypeFace,"Ym"))
    {

        TableCii = TableC22;
        TableCjj = TableC33;
        TableCkk = TableC11;
        TableCij = TableC23;
        TableCjk = TableC31;
        TableCki = TableC12;
        TableCjkjk = TableC55;
        TableCkiki = TableC66;
        TableCijij = TableC44;

#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<Z; j++)
        {
            for(k=0; k<X; k++)
            {
                Vi_BufferFaceBlocI[j*(X)+k] = Vy[k*(Y+1)*(Z)+(0)*(Z)+j];
                Carte_Tampon[j*(X)+k] = Carte[k*(Y)*(Z)+(0)*(Z)+j];
            }
        }
        /************************************************************************/
        /**************  Face  sur PML Arete YZ *********************************/
        /************************************************************************/
        if (ConditionFaceZp==0)
        {
#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(k=0; k<(X); k++)
                {
                    Vi_BufferFaceAreteJp[i*(X)+k] =	Vy_YmZp_X[i*(W+1)*(X)+k]	+
                                                    Vy_YmZp_Y[i*(W+1)*(X)+k]	+
                                                    Vy_YmZp_Z[i*(W+1)*(X)+k];
                }
            }
#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(k=0; k<(X+1); k++)
                {
                    Vk_BufferFaceAreteJp[i*(X+1)+k] =	Vx_YmZp_X[i*(W+1)*(X+1)+k]	+
                                                        Vx_YmZp_Y[i*(W+1)*(X+1)+k]	+
                                                        Vx_YmZp_Z[i*(W+1)*(X+1)+k];
                }
            }
        }
        if (ConditionFaceZm==0)
        {
#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(k=0; k<(X); k++)
                {
                    Vi_BufferFaceAreteJm[i*(X)+k] =	Vy_YmZm_X[i*(W+1)*(X)+k]	+
                                                    Vy_YmZm_Y[i*(W+1)*(X)+k]	+
                                                    Vy_YmZm_Z[i*(W+1)*(X)+k];
                }
            }
#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(k=0; k<(X+1); k++)
                {
                    Vk_BufferFaceAreteJm[i*(X+1)+k] =	Vx_YmZm_X[i*(W+1)*(X+1)+k]	+
                                                        Vx_YmZm_Y[i*(W+1)*(X+1)+k]	+
                                                        Vx_YmZm_Z[i*(W+1)*(X+1)+k];
                }
            }
        }
        /************************************************************************/
        /**************  Face  sur PML Arete XY *********************************/
        /************************************************************************/
        if (ConditionFaceXp==0)
        {
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(Z); j++)
                {
                    Vi_BufferFaceAreteKp[i*(Z)+j] =	Vy_XpYm_X[i*(Z)+j]	+
                                                    Vy_XpYm_Y[i*(Z)+j]	+
                                                    Vy_XpYm_Z[i*(Z)+j];
                }
            }

#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(Z+1); j++)
                {
                    Vj_BufferFaceAreteKp[i*(Z+1)+j] =	Vz_XpYm_X[i*(Z+1)+j]	+
                                                        Vz_XpYm_Y[i*(Z+1)+j]	+
                                                        Vz_XpYm_Z[i*(Z+1)+j];
                }
            }
        }
        if (ConditionFaceXm==0)
        {
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W); i++)
            {
                for(j=0; j<(Z); j++)
                {
                    Vi_BufferFaceAreteKm[i*(Z)+j] =	Vy_XmYm_X[i*(Z)+j]	+
                                                    Vy_XmYm_Y[i*(Z)+j]	+
                                                    Vy_XmYm_Z[i*(Z)+j];
                }
            }

#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(Z+1); j++)
                {
                    Vj_BufferFaceAreteKm[i*(Z+1)+j] =	Vz_XmYm_X[i*(Z+1)+j]	+
                                                        Vz_XmYm_Y[i*(Z+1)+j]	+
                                                        Vz_XmYm_Z[i*(Z+1)+j];
                }
            }
        }
    }

    if (!strcmp(TypeFace,"Zp"))
    {

        TableCii = TableC33;
        TableCjj = TableC11;
        TableCkk = TableC22;
        TableCij = TableC31;
        TableCjk = TableC12;
        TableCki = TableC23;
        TableCjkjk = TableC66;
        TableCkiki = TableC44;
        TableCijij = TableC55;

#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<X; i++)
        {
            for(k=0; k<Y; k++)
            {
                Vi_BufferFaceBlocI[i*(Y)+k] = Vz[i*(Y)*(Z+1)+k*(Z+1)+(Z)];
                Carte_Tampon[i*(Y)+k] = Carte[i*(Y)*(Z)+k*(Z)+(Z-1)];
            }
        }
        /************************************************************************/
        /**************  Face  sur PML Arete ZX *********************************/
        /************************************************************************/
        if (ConditionFaceXp==0)
        {
#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<W; i++)
            {
                for(k=0; k<Y; k++)
                {
                    Vi_BufferFaceAreteJp[i*(Y)+k]=Vz_ZpXp_X[i*(W+1)*(Y)+k] +
                                                  Vz_ZpXp_Y[i*(W+1)*(Y)+k] +
                                                  Vz_ZpXp_Z[i*(W+1)*(Y)+k];
                }
            }
#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(k=0; k<(Y+1); k++)
                {
                    Vk_BufferFaceAreteJp[i*(Y+1)+k]=Vy_ZpXp_X[i*(W+1)*(Y+1)+k] +
                                                    Vy_ZpXp_Y[i*(W+1)*(Y+1)+k] +
                                                    Vy_ZpXp_Z[i*(W+1)*(Y+1)+k];
                }
            }
        }
        if (ConditionFaceXm==0)
        {
#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<W; i++)
            {
                for(k=0; k<Y; k++)
                {
                    Vi_BufferFaceAreteJm[i*(Y)+k]=Vz_ZpXm_X[i*(W+1)*(Y)+k] +
                                                  Vz_ZpXm_Y[i*(W+1)*(Y)+k] +
                                                  Vz_ZpXm_Z[i*(W+1)*(Y)+k];

                }
            }
#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(k=0; k<(Y+1); k++)
                {
                    Vk_BufferFaceAreteJm[i*(Y+1)+k]=Vy_ZpXm_X[i*(W+1)*(Y+1)+k] +
                                                    Vy_ZpXm_Y[i*(W+1)*(Y+1)+k] +
                                                    Vy_ZpXm_Z[i*(W+1)*(Y+1)+k];
                }
            }
        }
        /************************************************************************/
        /**************  Face  sur PML Arete YZ *********************************/
        /************************************************************************/
        if (ConditionFaceYp==0)
        {
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<W; i++)
            {
                for(j=0; j<X; j++)
                {
                    Vi_BufferFaceAreteKp[i*(X)+j]=Vz_YpZp_X[i*(X)+j]	+
                                                  Vz_YpZp_Y[i*(X)+j]	+
                                                  Vz_YpZp_Z[i*(X)+j];
                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(X+1); j++)
                {
                    Vj_BufferFaceAreteKp[i*(X+1)+j]=	Vx_YpZp_X[i*(X+1)+j]	+
                                                        Vx_YpZp_Y[i*(X+1)+j]	+
                                                        Vx_YpZp_Z[i*(X+1)+j];
                }
            }
        }
        if (ConditionFaceYm==0)
        {
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<W; i++)
            {
                for(j=0; j<X; j++)
                {
                    Vi_BufferFaceAreteKm[i*(X)+j]=Vz_YmZp_X[i*(X)+j]	+
                                                  Vz_YmZp_Y[i*(X)+j]	+
                                                  Vz_YmZp_Z[i*(X)+j];
                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(X+1); j++)
                {
                    Vj_BufferFaceAreteKm[i*(X+1)+j]=	Vx_YmZp_X[i*(X+1)+j]	+
                                                        Vx_YmZp_Y[i*(X+1)+j]	+
                                                        Vx_YmZp_Z[i*(X+1)+j];
                }
            }
        }
    }

    if (!strcmp(TypeFace,"Zm"))
    {

        TableCii = TableC33;
        TableCjj = TableC11;
        TableCkk = TableC22;
        TableCij = TableC31;
        TableCjk = TableC12;
        TableCki = TableC23;
        TableCjkjk = TableC66;
        TableCkiki = TableC44;
        TableCijij = TableC55;

#pragma omp parallel for private(j) schedule(guided)
        for(i=0; i<X; i++)
        {
            for(j=0; j<Y; j++)
            {
                Vi_BufferFaceBlocI[i*(Y)+j] = Vz[i*(Y)*(Z+1)+j*(Z+1)+(0)];
                Carte_Tampon[i*(Y)+j] = Carte[i*(Y)*(Z)+j*(Z)+(0)];
            }
        }
        /************************************************************************/
        /**************  Face  sur PML Arete ZX *********************************/
        /************************************************************************/
        if (ConditionFaceXp==0)
        {
#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<W; i++)
            {
                for(k=0; k<Y; k++)
                {
                    Vi_BufferFaceAreteJp[i*(Y)+k]=Vz_ZmXp_X[i*(W+1)*(Y)+k] +
                                                  Vz_ZmXp_Y[i*(W+1)*(Y)+k] +
                                                  Vz_ZmXp_Z[i*(W+1)*(Y)+k];

                }
            }
#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(k=0; k<(Y+1); k++)
                {
                    Vk_BufferFaceAreteJp[i*(Y+1)+k]=Vy_ZmXp_X[i*(W+1)*(Y+1)+k] +
                                                    Vy_ZmXp_Y[i*(W+1)*(Y+1)+k] +
                                                    Vy_ZmXp_Z[i*(W+1)*(Y+1)+k];
                }
            }
        }
        if (ConditionFaceXm==0)
        {
#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<W; i++)
            {
                for(k=0; k<Y; k++)
                {
                    Vi_BufferFaceAreteJm[i*(Y)+k]=Vz_ZmXm_X[i*(W+1)*(Y)+k] +
                                                  Vz_ZmXm_Y[i*(W+1)*(Y)+k] +
                                                  Vz_ZmXm_Z[i*(W+1)*(Y)+k];

                }
            }
#pragma omp parallel for private(k) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(k=0; k<(Y+1); k++)
                {
                    Vk_BufferFaceAreteJm[i*(Y+1)+k]=Vy_ZmXm_X[i*(W+1)*(Y+1)+k] +
                                                    Vy_ZmXm_Y[i*(W+1)*(Y+1)+k] +
                                                    Vy_ZmXm_Z[i*(W+1)*(Y+1)+k];
                }
            }
        }
        /************************************************************************/
        /**************  Face  sur PML Arete YZ *********************************/
        /************************************************************************/
        if (ConditionFaceYp==0)
        {
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<W; i++)
            {
                for(j=0; j<X; j++)
                {
                    Vi_BufferFaceAreteKp[i*(X)+j]=Vz_YpZm_X[i*(X)+j]	+
                                                  Vz_YpZm_Y[i*(X)+j]	+
                                                  Vz_YpZm_Z[i*(X)+j];
                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(X+1); j++)
                {
                    Vj_BufferFaceAreteKp[i*(X+1)+j]=	Vx_YpZm_X[i*(X+1)+j]	+
                                                        Vx_YpZm_Y[i*(X+1)+j]	+
                                                        Vx_YpZm_Z[i*(X+1)+j];
                }
            }
        }
        if (ConditionFaceYm==0)
        {
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<W; i++)
            {
                for(j=0; j<X; j++)
                {
                    Vi_BufferFaceAreteKm[i*(X)+j]=Vz_YmZm_X[i*(X)+j]	+
                                                  Vz_YmZm_Y[i*(X)+j]	+
                                                  Vz_YmZm_Z[i*(X)+j];
                }
            }
#pragma omp parallel for private(j) schedule(guided)
            for(i=0; i<(W+1); i++)
            {
                for(j=0; j<(X+1); j++)
                {
                    Vj_BufferFaceAreteKm[i*(X+1)+j]=	Vx_YmZm_X[i*(X+1)+j]	+
                                                        Vx_YmZm_Y[i*(X+1)+j]	+
                                                        Vx_YmZm_Z[i*(X+1)+j];
                }
            }
        }
    }

}

void CalculBordsVitesses(void)
{

    int i,j,k; /* indice (x,y,z)*/
    double Legerete,FacteurQModel;  /* Parametre effectif de lgeret*/

    /*(0: PML, 1:Symetrie, 2:Libre, 3:Rigide)*/

    /*Face Xp */
    switch(ConditionFaceXp)
    {
    case 0:

#pragma omp parallel for private(k,Legerete) schedule(guided)

        for (j=0; j<Y; j++)
        {
            for (k=0; k<Z; k++)
            {

                Legerete=TableLegerete[Carte[(X-1)*(Y)*(Z)+(j)*(Z)+(k)]];

                Vx[(X)*(Y)*(Z)+(j)*(Z)+(k)] += DeltaTsurH*Legerete*(
                                                   ((Txx_Xp_X[(j)*(Z)+(k)]+Txx_Xp_YZ[(j)*(Z)+(k)])	-	Txx[(X-1)*(Y)*(Z)+(j)*(Z)+(k)]		)	+
                                                   (Txy[(X)*(Y+1)*(Z)+(j+1)*(Z)+(k)]								-	Txy[(X)*(Y+1)*(Z)+(j)*(Z)+(k)]		)	+
                                                   (Tzx[(X)*(Y)*(Z+1)+(j)*(Z+1)+(k+1)]							-	Tzx[(X)*(Y)*(Z+1)+(j)*(Z+1)+(k)]	)	);
            }
        }
        break;
    case 1:/*implicite*/
        break;
    case 2:

#pragma omp parallel for private(k,Legerete) schedule(guided)

        for (j=0; j<Y; j++)
        {
            for (k=0; k<Z; k++)
            {

                Legerete=TableLegerete[Carte[(X-1)*(Y)*(Z)+(j)*(Z)+(k)]];

                Vx[(X)*(Y)*(Z)+(j)*(Z)+(k)] += DeltaTsurH*Legerete*(
                                                   (0	-	2*Txx[(X-1)*(Y)*(Z)+(j)*(Z)+(k)]		)	+
                                                   (Txy[(X)*(Y+1)*(Z)+(j+1)*(Z)+(k)]								-	Txy[(X)*(Y+1)*(Z)+(j)*(Z)+(k)]		)	+
                                                   (Tzx[(X)*(Y)*(Z+1)+(j)*(Z+1)+(k+1)]							-	Tzx[(X)*(Y)*(Z+1)+(j)*(Z+1)+(k)]	)	);
            }
        }
        break;
    case 3:
        break;
    }

    /*	 Face Xm */
    switch(ConditionFaceXm)
    {
    case 0:

#pragma omp parallel for private(k,Legerete) schedule(guided)

        for (j=0; j<Y; j++)
        {
            for (k=0; k<Z; k++)
            {

                Legerete=TableLegerete[Carte[(j)*(Z)+(k)]];

                Vx[(j)*(Z)+(k)] += DeltaTsurH*Legerete*(
                                       -((Txx_Xm_X[(j)*(Z)+(k)]+Txx_Xm_YZ[(j)*(Z)+(k)])	-	Txx[(j)*(Z)+(k)]									)	+
                                       (Txy[(j+1)*(Z)+(k)]															-	Txy[(j)*(Z)+(k)]		)	+
                                       (Tzx[(j)*(Z+1)+(k+1)]														-	Tzx[(j)*(Z+1)+(k)]	)	);
            }
        }
        break;
    case 1:/*implicite*/
        break;
    case 2:

#pragma omp parallel for private(k,Legerete) schedule(guided)

        for (j=0; j<Y; j++)
        {
            for (k=0; k<Z; k++)
            {

                Legerete=TableLegerete[Carte[(j)*(Z)+(k)]];

                Vx[(j)*(Z)+(k)] += DeltaTsurH*Legerete*(
                                       -(0	-	2*Txx[(j)*(Z)+(k)]									)	+
                                       (Txy[(j+1)*(Z)+(k)]															-	Txy[(j)*(Z)+(k)]		)	+
                                       (Tzx[(j)*(Z+1)+(k+1)]														-	Tzx[(j)*(Z+1)+(k)]	)	);
            }
        }
        break;
    case 3:
        break;
    }

    /*Face Yp */
    switch(ConditionFaceYp)
    {
    case 0:

#pragma omp parallel for private(k,Legerete) schedule(guided)

        for (i=0; i<X; i++)
        {
            for (k=0; k<Z; k++)
            {

                Legerete=TableLegerete[Carte[(i)*(Y)*(Z)+(Y-1)*(Z)+(k)]];

                Vy[(i)*(Y+1)*(Z)+(Y)*(Z)+(k)] += DeltaTsurH*Legerete*(
                                                     (Txy[(i+1)*(Y+1)*(Z)+(Y)*(Z)+(k)]									-	Txy[(i)*(Y+1)*(Z)+(Y)*(Z)+(k)]		)	+
                                                     ((Tyy_Yp_Y[(k)*(X)+(i)]+Tyy_Yp_ZX[(k)*(X)+(i)])		-	Tyy[(i)*(Y)*(Z)+(Y-1)*(Z)+(k)]		)	+
                                                     (Tyz[(i)*(Y+1)*(Z+1)+(Y)*(Z+1)+(k+1)]							-	Tyz[(i)*(Y+1)*(Z+1)+(Y)*(Z+1)+(k)])	);
            }
        }
        break;
    case 1:/*implicite*/
        break;
    case 2:

#pragma omp parallel for private(k,Legerete) schedule(guided)

        for (i=0; i<X; i++)
        {
            for (k=0; k<Z; k++)
            {

                Legerete=TableLegerete[Carte[(i)*(Y)*(Z)+(Y-1)*(Z)+(k)]];

                Vy[(i)*(Y+1)*(Z)+(Y)*(Z)+(k)] += DeltaTsurH*Legerete*(
                                                     (Txy[(i+1)*(Y+1)*(Z)+(Y)*(Z)+(k)]									-	Txy[(i)*(Y+1)*(Z)+(Y)*(Z)+(k)]		)	+
                                                     (0		-	2*Tyy[(i)*(Y)*(Z)+(Y-1)*(Z)+(k)]		)	+
                                                     (Tyz[(i)*(Y+1)*(Z+1)+(Y)*(Z+1)+(k+1)]							-	Tyz[(i)*(Y+1)*(Z+1)+(Y)*(Z+1)+(k)])	);
            }
        }
        break;
    case 3:
        break;
    }

    /*	 Face Ym */
    switch(ConditionFaceYm)
    {
    case 0:

#pragma omp parallel for private(k,Legerete) schedule(guided)

        for (i=0; i<X; i++)
        {
            for (k=0; k<Z; k++)
            {

                Legerete=TableLegerete[Carte[(i)*(Y)*(Z)+(k)]];

                Vy[(i)*(Y+1)*(Z)+(k)] += DeltaTsurH*Legerete*(
                                             (Txy[(i+1)*(Y+1)*(Z)+(k)]														-	Txy[(i)*(Y+1)*(Z)+(k)]		)	+
                                             -((Tyy_Ym_Y[(k)*(X)+(i)]+Tyy_Ym_ZX[(k)*(X)+(i)])		-	Tyy[(i)*(Y)*(Z)+(k)]		)	+
                                             (Tyz[(i)*(Y+1)*(Z+1)+(k+1)]													-	Tyz[(i)*(Y+1)*(Z+1)+(k)])	);
            }
        }
        break;
    case 1:/*implicite*/
        break;
    case 2:

#pragma omp parallel for private(k,Legerete) schedule(guided)

        for (i=0; i<X; i++)
        {
            for (k=0; k<Z; k++)
            {

                Legerete=TableLegerete[Carte[(i)*(Y)*(Z)+(k)]];

                Vy[(i)*(Y+1)*(Z)+(k)] += DeltaTsurH*Legerete*(
                                             (Txy[(i+1)*(Y+1)*(Z)+(k)]														-	Txy[(i)*(Y+1)*(Z)+(k)]		)	+
                                             -(0		-	2*Tyy[(i)*(Y)*(Z)+(k)]		)	+
                                             (Tyz[(i)*(Y+1)*(Z+1)+(k+1)]													-	Tyz[(i)*(Y+1)*(Z+1)+(k)])	);
            }
        }
        break;
    case 3:
        break;
    }
    /*	 Face Zp */
    switch(ConditionFaceZp)
    {
    case 0:

#pragma omp parallel for private(j,Legerete) schedule(guided)

        for (i=0; i<X; i++)
        {
            for (j=0; j<Y; j++)
            {

                Legerete=TableLegerete[Carte[(i)*(Y)*(Z)+(j)*(Z)+(Z-1)]];

                Vz[(i)*(Y)*(Z+1)+(j)*(Z+1)+(Z)] += DeltaTsurH*Legerete*(
                                                       (Tzx[(i+1)*(Y)*(Z+1)+(j)*(Z+1)+(Z)]									-	Tzx[(i)*(Y)*(Z+1)+(j)*(Z+1)+(Z)]	)	+
                                                       (Tyz[(i)*(Y+1)*(Z+1)+(j+1)*(Z+1)+(Z)]								-	Tyz[(i)*(Y+1)*(Z+1)+(j)*(Z+1)+(Z)])	+
                                                       ((Tzz_Zp_Z[(i)*(Y)+(j)]+Tzz_Zp_XY[(i)*(Y)+(j)])			-	Tzz[(i)*(Y)*(Z)+(j)*(Z)+(Z-1)]		)	);
            }
        }
        break;
    case 1:/*implicite*/
        break;
    case 2:

#pragma omp parallel for private(j,Legerete) schedule(guided)

        for (i=0; i<X; i++)
        {
            for (j=0; j<Y; j++)
            {

                Legerete=TableLegerete[Carte[(i)*(Y)*(Z)+(j)*(Z)+(Z-1)]];

                Vz[(i)*(Y)*(Z+1)+(j)*(Z+1)+(Z)] += DeltaTsurH*Legerete*(
                                                       (Tzx[(i+1)*(Y)*(Z+1)+(j)*(Z+1)+(Z)]									-	Tzx[(i)*(Y)*(Z+1)+(j)*(Z+1)+(Z)]	)	+
                                                       (Tyz[(i)*(Y+1)*(Z+1)+(j+1)*(Z+1)+(Z)]								-	Tyz[(i)*(Y+1)*(Z+1)+(j)*(Z+1)+(Z)])	+
                                                       (0			-	2*Tzz[(i)*(Y)*(Z)+(j)*(Z)+(Z-1)]		)	);
            }
        }
        break;
    case 3:
        break;
    }

    /*	 Face Zm */
    switch(ConditionFaceZm)
    {
    case 0:

#pragma omp parallel for private(j,Legerete) schedule(guided)

        for (i=0; i<X; i++)
        {
            for (j=0; j<Y; j++)
            {

                Legerete=TableLegerete[Carte[(i)*(Y)*(Z)+(j)*(Z)]];

                Vz[(i)*(Y)*(Z+1)+(j)*(Z+1)] += DeltaTsurH*Legerete*(
                                                   (Tzx[(i+1)*(Y)*(Z+1)+(j)*(Z+1)]											-		Tzx[(i)*(Y)*(Z+1)+(j)*(Z+1)]	)	+
                                                   (Tyz[(i)*(Y+1)*(Z+1)+(j+1)*(Z+1)]										-		Tyz[(i)*(Y+1)*(Z+1)+(j)*(Z+1)])	+
                                                   -((Tzz_Zm_Z[(i)*(Y)+(j)]+Tzz_Zm_XY[(i)*(Y)+(j)])		-		Tzz[(i)*(Y)*(Z)+(j)*(Z)]			)	);
            }
        }
        break;
    case 1:/*implicite*/
        break;
    case 2:

#pragma omp parallel for private(j,Legerete) schedule(guided)

        for (i=0; i<X; i++)
        {
            for (j=0; j<Y; j++)
            {

                Legerete=TableLegerete[Carte[(i)*(Y)*(Z)+(j)*(Z)]];

                Vz[(i)*(Y)*(Z+1)+(j)*(Z+1)] += DeltaTsurH*Legerete*(
                                                   (Tzx[(i+1)*(Y)*(Z+1)+(j)*(Z+1)]											-		Tzx[(i)*(Y)*(Z+1)+(j)*(Z+1)]	)	+
                                                   (Tyz[(i)*(Y+1)*(Z+1)+(j+1)*(Z+1)]										-		Tyz[(i)*(Y+1)*(Z+1)+(j)*(Z+1)])	+
                                                   -(0		-		2*Tzz[(i)*(Y)*(Z)+(j)*(Z)]			)	);
            }
        }
        break;
    case 3:
        break;
    }

    /**************************************************************************/
    /*********************** Attenuating Factor QModel ************************/
    /**************************************************************************/



    if(TypeAbsorption == 2)
    {

#pragma omp parallel for private(k,FacteurQModel) schedule(guided)

        for (j=0; j<Y; j++)
        {
            for (k=0; k<Z; k++)
            {
                FacteurQModel = TableQModel[Carte[(X-1)*(Y)*(Z)+(j)*(Z)+(k)]] ;
                Vx[(X)*(Y)*(Z)+(j)*(Z)+(k)] *= FacteurQModel;
            }
        }

#pragma omp parallel for private(k,FacteurQModel) schedule(guided)


        for (j=0; j<Y; j++)
        {
            for (k=0; k<Z; k++)
            {
                FacteurQModel = TableQModel[Carte[(j)*(Z)+(k)]] ;
                Vx[(j)*(Z)+(k)] *= FacteurQModel;
            }
        }

#pragma omp parallel for private(k,FacteurQModel) schedule(guided)

        for (i=0; i<X; i++)
        {
            for (k=0; k<Z; k++)
            {

                FacteurQModel = TableQModel[Carte[(i)*(Y)*(Z)+(Y-1)*(Z)+(k)]] ;
                Vy[(i)*(Y+1)*(Z)+(Y)*(Z)+(k)] *= FacteurQModel;
            }
        }

#pragma omp parallel for private(k,FacteurQModel) schedule(guided)

        for (i=0; i<X; i++)
        {
            for (k=0; k<Z; k++)
            {

                FacteurQModel = TableQModel[Carte[(i)*(Y)*(Z)+(k)]] ;
                Vy[(i)*(Y+1)*(Z)+(k)] *= FacteurQModel;
            }
        }

#pragma omp parallel for private(j,FacteurQModel) schedule(guided)

        for (i=0; i<X; i++)
        {
            for (j=0; j<Y; j++)
            {

                FacteurQModel = TableQModel[Carte[(i)*(Y)*(Z)+(j)*(Z)+(Z-1)]] ;
                Vz[(i)*(Y)*(Z+1)+(j)*(Z+1)+(Z)] *= FacteurQModel;
            }
        }

#pragma omp parallel for private(j,FacteurQModel) schedule(guided)

        for (i=0; i<X; i++)
        {
            for (j=0; j<Y; j++)
            {

                FacteurQModel = TableQModel[Carte[(i)*(Y)*(Z)+(j)*(Z)]] ;
                Vz[(i)*(Y)*(Z+1)+(j)*(Z+1)] *= FacteurQModel;
            }
        }
    }



    /**************************************************************************/
    /*********************** Calcul des Faces PML *****************************/
    /**************************************************************************/

    if(ConditionFaceXp==0)
    {
        ReMiseAZeroTampons();
        MiseAJourContraintesTamponFacePML("Xp");
        CalculPMLFaceVitesses(Vx_Xp_X,Vx_Xp_YZ,
                              Vy_Xp_X,Vy_Xp_YZ,
                              Vz_Xp_X,Vz_Xp_YZ,
                              Txx_Xp_X,Txx_Xp_YZ,
                              Tyy_Xp_X,Tyy_Xp_YZ,
                              Tzz_Xp_X,Tzz_Xp_YZ,
                              Txy_Xp_X,Txy_Xp_YZ,
                              Tzx_Xp_X,Tzx_Xp_YZ,
                              Tyz_Xp,
                              Y,Z,"Xp");
    }
    if(ConditionFaceXm==0)
    {
        ReMiseAZeroTampons();
        MiseAJourContraintesTamponFacePML("Xm");
        CalculPMLFaceVitesses(Vx_Xm_X,Vx_Xm_YZ,
                              Vy_Xm_X,Vy_Xm_YZ,
                              Vz_Xm_X,Vz_Xm_YZ,
                              Txx_Xm_X,Txx_Xm_YZ,
                              Tyy_Xm_X,Tyy_Xm_YZ,
                              Tzz_Xm_X,Tzz_Xm_YZ,
                              Txy_Xm_X,Txy_Xm_YZ,
                              Tzx_Xm_X,Tzx_Xm_YZ,
                              Tyz_Xm,
                              Y,Z,"Xm");
    }
    if(ConditionFaceYp==0)
    {
        ReMiseAZeroTampons();
        MiseAJourContraintesTamponFacePML("Yp");
        CalculPMLFaceVitesses(Vy_Yp_Y,Vy_Yp_ZX,
                              Vz_Yp_Y,Vz_Yp_ZX,
                              Vx_Yp_Y,Vx_Yp_ZX,
                              Tyy_Yp_Y,Tyy_Yp_ZX,
                              Tzz_Yp_Y,Tzz_Yp_ZX,
                              Txx_Yp_Y,Txx_Yp_ZX,
                              Tyz_Yp_Y,Tyz_Yp_ZX,
                              Txy_Yp_Y,Txy_Yp_ZX,
                              Tzx_Yp,
                              Z,X,"Yp");
    }
    if(ConditionFaceYm==0)
    {
        ReMiseAZeroTampons();
        MiseAJourContraintesTamponFacePML("Ym");
        CalculPMLFaceVitesses(Vy_Ym_Y,Vy_Ym_ZX,
                              Vz_Ym_Y,Vz_Ym_ZX,
                              Vx_Ym_Y,Vx_Ym_ZX,
                              Tyy_Ym_Y,Tyy_Ym_ZX,
                              Tzz_Ym_Y,Tzz_Ym_ZX,
                              Txx_Ym_Y,Txx_Ym_ZX,
                              Tyz_Ym_Y,Tyz_Ym_ZX,
                              Txy_Ym_Y,Txy_Ym_ZX,
                              Tzx_Ym,
                              Z,X,"Ym");
    }
    if(ConditionFaceZp==0)
    {
        ReMiseAZeroTampons();
        MiseAJourContraintesTamponFacePML("Zp");
        CalculPMLFaceVitesses(Vz_Zp_Z,Vz_Zp_XY,
                              Vx_Zp_Z,Vx_Zp_XY,
                              Vy_Zp_Z,Vy_Zp_XY,
                              Tzz_Zp_Z,Tzz_Zp_XY,
                              Txx_Zp_Z,Txx_Zp_XY,
                              Tyy_Zp_Z,Tyy_Zp_XY,
                              Tzx_Zp_Z,Tzx_Zp_XY,
                              Tyz_Zp_Z,Tyz_Zp_XY,
                              Txy_Zp,
                              X,Y,"Zp");
    }
    if(ConditionFaceZm==0)
    {
        ReMiseAZeroTampons();
        MiseAJourContraintesTamponFacePML("Zm");
        CalculPMLFaceVitesses(Vz_Zm_Z,Vz_Zm_XY,
                              Vx_Zm_Z,Vx_Zm_XY,
                              Vy_Zm_Z,Vy_Zm_XY,
                              Tzz_Zm_Z,Tzz_Zm_XY,
                              Txx_Zm_Z,Txx_Zm_XY,
                              Tyy_Zm_Z,Tyy_Zm_XY,
                              Tzx_Zm_Z,Tzx_Zm_XY,
                              Tyz_Zm_Z,Tyz_Zm_XY,
                              Txy_Zm,
                              X,Y,"Zm");
    }

    /**************************************************************************/
    /*********************** Calcul des aretes PML ****************************/
    /**************************************************************************/

    /**************************************************************************/
    /*********** Aretes perpendiculaire au plan XY ****************************/
    /**************************************************************************/
    if ((ConditionFaceXp==0)&&(ConditionFaceYp==0))
    {
        ReMiseAZeroTampons();
        MiseAJourContraintesTamponAretePML("XpYp");
        CalculPMLAreteVitesses(Vx_XpYp_X,Vx_XpYp_Y,Vx_XpYp_Z,
                               Vy_XpYp_X,Vy_XpYp_Y,Vy_XpYp_Z,
                               Vz_XpYp_X,Vz_XpYp_Y,Vz_XpYp_Z,
                               Txx_XpYp_X,Txx_XpYp_Y,Txx_XpYp_Z,
                               Tyy_XpYp_X,Tyy_XpYp_Y,Tyy_XpYp_Z,
                               Tzz_XpYp_X,Tzz_XpYp_Y,Tzz_XpYp_Z,
                               Txy_XpYp_X,Txy_XpYp_Y,
                               Tyz_XpYp_Y,Tyz_XpYp_Z,
                               Tzx_XpYp_Z,Tzx_XpYp_X,
                               Z,"XpYp");
    }
    if ((ConditionFaceXm==0)&&(ConditionFaceYp==0))
    {
        ReMiseAZeroTampons();
        MiseAJourContraintesTamponAretePML("XmYp");
        CalculPMLAreteVitesses(Vx_XmYp_X,Vx_XmYp_Y,Vx_XmYp_Z,
                               Vy_XmYp_X,Vy_XmYp_Y,Vy_XmYp_Z,
                               Vz_XmYp_X,Vz_XmYp_Y,Vz_XmYp_Z,
                               Txx_XmYp_X,Txx_XmYp_Y,Txx_XmYp_Z,
                               Tyy_XmYp_X,Tyy_XmYp_Y,Tyy_XmYp_Z,
                               Tzz_XmYp_X,Tzz_XmYp_Y,Tzz_XmYp_Z,
                               Txy_XmYp_X,Txy_XmYp_Y,
                               Tyz_XmYp_Y,Tyz_XmYp_Z,
                               Tzx_XmYp_Z,Tzx_XmYp_X,
                               Z,"XmYp");
    }
    if ((ConditionFaceXp==0)&&(ConditionFaceYm==0))
    {
        ReMiseAZeroTampons();
        MiseAJourContraintesTamponAretePML("XpYm");
        CalculPMLAreteVitesses(Vx_XpYm_X,Vx_XpYm_Y,Vx_XpYm_Z,
                               Vy_XpYm_X,Vy_XpYm_Y,Vy_XpYm_Z,
                               Vz_XpYm_X,Vz_XpYm_Y,Vz_XpYm_Z,
                               Txx_XpYm_X,Txx_XpYm_Y,Txx_XpYm_Z,
                               Tyy_XpYm_X,Tyy_XpYm_Y,Tyy_XpYm_Z,
                               Tzz_XpYm_X,Tzz_XpYm_Y,Tzz_XpYm_Z,
                               Txy_XpYm_X,Txy_XpYm_Y,
                               Tyz_XpYm_Y,Tyz_XpYm_Z,
                               Tzx_XpYm_Z,Tzx_XpYm_X,
                               Z,"XpYm");
    }
    if ((ConditionFaceXm==0)&&(ConditionFaceYm==0))
    {
        ReMiseAZeroTampons();
        MiseAJourContraintesTamponAretePML("XmYm");
        CalculPMLAreteVitesses(Vx_XmYm_X,Vx_XmYm_Y,Vx_XmYm_Z,
                               Vy_XmYm_X,Vy_XmYm_Y,Vy_XmYm_Z,
                               Vz_XmYm_X,Vz_XmYm_Y,Vz_XmYm_Z,
                               Txx_XmYm_X,Txx_XmYm_Y,Txx_XmYm_Z,
                               Tyy_XmYm_X,Tyy_XmYm_Y,Tyy_XmYm_Z,
                               Tzz_XmYm_X,Tzz_XmYm_Y,Tzz_XmYm_Z,
                               Txy_XmYm_X,Txy_XmYm_Y,
                               Tyz_XmYm_Y,Tyz_XmYm_Z,
                               Tzx_XmYm_Z,Tzx_XmYm_X,
                               Z,"XmYm");
    }
    /**************************************************************************/
    /*********** Aretes perpendiculaire au plan YZ ****************************/
    /**************************************************************************/
    if ((ConditionFaceYp==0)&&(ConditionFaceZp==0))
    {
        ReMiseAZeroTampons();
        MiseAJourContraintesTamponAretePML("YpZp");
        CalculPMLAreteVitesses(Vy_YpZp_Y,Vy_YpZp_Z,Vy_YpZp_X,
                               Vz_YpZp_Y,Vz_YpZp_Z,Vz_YpZp_X,
                               Vx_YpZp_Y,Vx_YpZp_Z,Vx_YpZp_X,
                               Tyy_YpZp_Y,Tyy_YpZp_Z,Tyy_YpZp_X,
                               Tzz_YpZp_Y,Tzz_YpZp_Z,Tzz_YpZp_X,
                               Txx_YpZp_Y,Txx_YpZp_Z,Txx_YpZp_X,
                               Tyz_YpZp_Y,Tyz_YpZp_Z,
                               Tzx_YpZp_Z,Tzx_YpZp_X,
                               Txy_YpZp_X,Txy_YpZp_Y,
                               X,"YpZp");
    }
    if ((ConditionFaceYp==0)&&(ConditionFaceZm==0))
    {
        ReMiseAZeroTampons();
        MiseAJourContraintesTamponAretePML("YpZm");
        CalculPMLAreteVitesses(Vy_YpZm_Y,Vy_YpZm_Z,Vy_YpZm_X,
                               Vz_YpZm_Y,Vz_YpZm_Z,Vz_YpZm_X,
                               Vx_YpZm_Y,Vx_YpZm_Z,Vx_YpZm_X,
                               Tyy_YpZm_Y,Tyy_YpZm_Z,Tyy_YpZm_X,
                               Tzz_YpZm_Y,Tzz_YpZm_Z,Tzz_YpZm_X,
                               Txx_YpZm_Y,Txx_YpZm_Z,Txx_YpZm_X,
                               Tyz_YpZm_Y,Tyz_YpZm_Z,
                               Tzx_YpZm_Z,Tzx_YpZm_X,
                               Txy_YpZm_X,Txy_YpZm_Y,
                               X,"YpZm");
    }
    if ((ConditionFaceYm==0)&&(ConditionFaceZp==0))
    {
        ReMiseAZeroTampons();
        MiseAJourContraintesTamponAretePML("YmZp");
        CalculPMLAreteVitesses(Vy_YmZp_Y,Vy_YmZp_Z,Vy_YmZp_X,
                               Vz_YmZp_Y,Vz_YmZp_Z,Vz_YmZp_X,
                               Vx_YmZp_Y,Vx_YmZp_Z,Vx_YmZp_X,
                               Tyy_YmZp_Y,Tyy_YmZp_Z,Tyy_YmZp_X,
                               Tzz_YmZp_Y,Tzz_YmZp_Z,Tzz_YmZp_X,
                               Txx_YmZp_Y,Txx_YmZp_Z,Txx_YmZp_X,
                               Tyz_YmZp_Y,Tyz_YmZp_Z,
                               Tzx_YmZp_Z,Tzx_YmZp_X,
                               Txy_YmZp_X,Txy_YmZp_Y,
                               X,"YmZp");
    }
    if ((ConditionFaceYm==0)&&(ConditionFaceZm==0))
    {
        ReMiseAZeroTampons();
        MiseAJourContraintesTamponAretePML("YmZm");
        CalculPMLAreteVitesses(Vy_YmZm_Y,Vy_YmZm_Z,Vy_YmZm_X,
                               Vz_YmZm_Y,Vz_YmZm_Z,Vz_YmZm_X,
                               Vx_YmZm_Y,Vx_YmZm_Z,Vx_YmZm_X,
                               Tyy_YmZm_Y,Tyy_YmZm_Z,Tyy_YmZm_X,
                               Tzz_YmZm_Y,Tzz_YmZm_Z,Tzz_YmZm_X,
                               Txx_YmZm_Y,Txx_YmZm_Z,Txx_YmZm_X,
                               Tyz_YmZm_Y,Tyz_YmZm_Z,
                               Tzx_YmZm_Z,Tzx_YmZm_X,
                               Txy_YmZm_X,Txy_YmZm_Y,
                               X,"YmZm");
    }
    /**************************************************************************/
    /*********** Aretes perpendiculaire au plan ZX ****************************/
    /**************************************************************************/

    if ((ConditionFaceZp==0)&&(ConditionFaceXp==0))
    {
        ReMiseAZeroTampons();
        MiseAJourContraintesTamponAretePML("ZpXp");
        CalculPMLAreteVitesses(Vz_ZpXp_Z,Vz_ZpXp_X,Vz_ZpXp_Y,
                               Vx_ZpXp_Z,Vx_ZpXp_X,Vx_ZpXp_Y,
                               Vy_ZpXp_Z,Vy_ZpXp_X,Vy_ZpXp_Y,
                               Tzz_ZpXp_Z,Tzz_ZpXp_X,Tzz_ZpXp_Y,
                               Txx_ZpXp_Z,Txx_ZpXp_X,Txx_ZpXp_Y,
                               Tyy_ZpXp_Z,Tyy_ZpXp_X,Tyy_ZpXp_Y,
                               Tzx_ZpXp_Z,Tzx_ZpXp_X,
                               Txy_ZpXp_X,Txy_ZpXp_Y,
                               Tyz_ZpXp_Y,Tyz_ZpXp_Z,
                               Y,"ZpXp");
    }
    if ((ConditionFaceZm==0)&&(ConditionFaceXp==0))
    {
        ReMiseAZeroTampons();
        MiseAJourContraintesTamponAretePML("ZmXp");
        CalculPMLAreteVitesses(Vz_ZmXp_Z,Vz_ZmXp_X,Vz_ZmXp_Y,
                               Vx_ZmXp_Z,Vx_ZmXp_X,Vx_ZmXp_Y,
                               Vy_ZmXp_Z,Vy_ZmXp_X,Vy_ZmXp_Y,
                               Tzz_ZmXp_Z,Tzz_ZmXp_X,Tzz_ZmXp_Y,
                               Txx_ZmXp_Z,Txx_ZmXp_X,Txx_ZmXp_Y,
                               Tyy_ZmXp_Z,Tyy_ZmXp_X,Tyy_ZmXp_Y,
                               Tzx_ZmXp_Z,Tzx_ZmXp_X,
                               Txy_ZmXp_X,Txy_ZmXp_Y,
                               Tyz_ZmXp_Y,Tyz_ZmXp_Z,
                               Y,"ZmXp");
    }
    if ((ConditionFaceZp==0)&&(ConditionFaceXm==0))
    {
        ReMiseAZeroTampons();
        MiseAJourContraintesTamponAretePML("ZpXm");
        CalculPMLAreteVitesses(Vz_ZpXm_Z,Vz_ZpXm_X,Vz_ZpXm_Y,
                               Vx_ZpXm_Z,Vx_ZpXm_X,Vx_ZpXm_Y,
                               Vy_ZpXm_Z,Vy_ZpXm_X,Vy_ZpXm_Y,
                               Tzz_ZpXm_Z,Tzz_ZpXm_X,Tzz_ZpXm_Y,
                               Txx_ZpXm_Z,Txx_ZpXm_X,Txx_ZpXm_Y,
                               Tyy_ZpXm_Z,Tyy_ZpXm_X,Tyy_ZpXm_Y,
                               Tzx_ZpXm_Z,Tzx_ZpXm_X,
                               Txy_ZpXm_X,Txy_ZpXm_Y,
                               Tyz_ZpXm_Y,Tyz_ZpXm_Z,
                               Y,"ZpXm");
    }
    if ((ConditionFaceZm==0)&&(ConditionFaceXm==0))
    {
        ReMiseAZeroTampons();
        MiseAJourContraintesTamponAretePML("ZmXm");
        CalculPMLAreteVitesses(Vz_ZmXm_Z,Vz_ZmXm_X,Vz_ZmXm_Y,
                               Vx_ZmXm_Z,Vx_ZmXm_X,Vx_ZmXm_Y,
                               Vy_ZmXm_Z,Vy_ZmXm_X,Vy_ZmXm_Y,
                               Tzz_ZmXm_Z,Tzz_ZmXm_X,Tzz_ZmXm_Y,
                               Txx_ZmXm_Z,Txx_ZmXm_X,Txx_ZmXm_Y,
                               Tyy_ZmXm_Z,Tyy_ZmXm_X,Tyy_ZmXm_Y,
                               Tzx_ZmXm_Z,Tzx_ZmXm_X,
                               Txy_ZmXm_X,Txy_ZmXm_Y,
                               Tyz_ZmXm_Y,Tyz_ZmXm_Z,
                               Y,"ZmXm");
    }

    /**************************************************************************/
    /******************* Calcul des Coins PML *********************************/
    /**************************************************************************/
    if ((ConditionFaceXp==0)&&(ConditionFaceYp==0)&&(ConditionFaceZp==0))
    {
        ReMiseAZeroTampons();
        MiseAJourContraintesTamponCoinPML("XpYpZp");
        CalculPMLCoinVitesses(Vx_XpYpZp_X,Vx_XpYpZp_Y,Vx_XpYpZp_Z,
                              Vy_XpYpZp_X,Vy_XpYpZp_Y,Vy_XpYpZp_Z,
                              Vz_XpYpZp_X,Vz_XpYpZp_Y,Vz_XpYpZp_Z,
                              Txx_XpYpZp_X,Txx_XpYpZp_Y,Txx_XpYpZp_Z,
                              Tyy_XpYpZp_X,Tyy_XpYpZp_Y,Tyy_XpYpZp_Z,
                              Tzz_XpYpZp_X,Tzz_XpYpZp_Y,Tzz_XpYpZp_Z,
                              Txy_XpYpZp_X,Txy_XpYpZp_Y,
                              Tyz_XpYpZp_Y,Tyz_XpYpZp_Z,
                              Tzx_XpYpZp_Z,Tzx_XpYpZp_X,
                              "XpYpZp");
    }
    if ((ConditionFaceXp==0)&&(ConditionFaceYp==0)&&(ConditionFaceZm==0))
    {
        ReMiseAZeroTampons();
        MiseAJourContraintesTamponCoinPML("XpYpZm");
        CalculPMLCoinVitesses(Vx_XpYpZm_X,Vx_XpYpZm_Y,Vx_XpYpZm_Z,
                              Vy_XpYpZm_X,Vy_XpYpZm_Y,Vy_XpYpZm_Z,
                              Vz_XpYpZm_X,Vz_XpYpZm_Y,Vz_XpYpZm_Z,
                              Txx_XpYpZm_X,Txx_XpYpZm_Y,Txx_XpYpZm_Z,
                              Tyy_XpYpZm_X,Tyy_XpYpZm_Y,Tyy_XpYpZm_Z,
                              Tzz_XpYpZm_X,Tzz_XpYpZm_Y,Tzz_XpYpZm_Z,
                              Txy_XpYpZm_X,Txy_XpYpZm_Y,
                              Tyz_XpYpZm_Y,Tyz_XpYpZm_Z,
                              Tzx_XpYpZm_Z,Tzx_XpYpZm_X,
                              "XpYpZm");
    }
    if ((ConditionFaceXp==0)&&(ConditionFaceYm==0)&&(ConditionFaceZp==0))
    {
        ReMiseAZeroTampons();
        MiseAJourContraintesTamponCoinPML("XpYmZp");
        CalculPMLCoinVitesses(Vx_XpYmZp_X,Vx_XpYmZp_Y,Vx_XpYmZp_Z,
                              Vy_XpYmZp_X,Vy_XpYmZp_Y,Vy_XpYmZp_Z,
                              Vz_XpYmZp_X,Vz_XpYmZp_Y,Vz_XpYmZp_Z,
                              Txx_XpYmZp_X,Txx_XpYmZp_Y,Txx_XpYmZp_Z,
                              Tyy_XpYmZp_X,Tyy_XpYmZp_Y,Tyy_XpYmZp_Z,
                              Tzz_XpYmZp_X,Tzz_XpYmZp_Y,Tzz_XpYmZp_Z,
                              Txy_XpYmZp_X,Txy_XpYmZp_Y,
                              Tyz_XpYmZp_Y,Tyz_XpYmZp_Z,
                              Tzx_XpYmZp_Z,Tzx_XpYmZp_X,
                              "XpYmZp");
    }
    if ((ConditionFaceXp==0)&&(ConditionFaceYm==0)&&(ConditionFaceZm==0))
    {
        ReMiseAZeroTampons();
        MiseAJourContraintesTamponCoinPML("XpYmZm");
        CalculPMLCoinVitesses(Vx_XpYmZm_X,Vx_XpYmZm_Y,Vx_XpYmZm_Z,
                              Vy_XpYmZm_X,Vy_XpYmZm_Y,Vy_XpYmZm_Z,
                              Vz_XpYmZm_X,Vz_XpYmZm_Y,Vz_XpYmZm_Z,
                              Txx_XpYmZm_X,Txx_XpYmZm_Y,Txx_XpYmZm_Z,
                              Tyy_XpYmZm_X,Tyy_XpYmZm_Y,Tyy_XpYmZm_Z,
                              Tzz_XpYmZm_X,Tzz_XpYmZm_Y,Tzz_XpYmZm_Z,
                              Txy_XpYmZm_X,Txy_XpYmZm_Y,
                              Tyz_XpYmZm_Y,Tyz_XpYmZm_Z,
                              Tzx_XpYmZm_Z,Tzx_XpYmZm_X,
                              "XpYmZm");
    }
    if ((ConditionFaceXm==0)&&(ConditionFaceYp==0)&&(ConditionFaceZp==0))
    {
        ReMiseAZeroTampons();
        MiseAJourContraintesTamponCoinPML("XmYpZp");
        CalculPMLCoinVitesses(Vx_XmYpZp_X,Vx_XmYpZp_Y,Vx_XmYpZp_Z,
                              Vy_XmYpZp_X,Vy_XmYpZp_Y,Vy_XmYpZp_Z,
                              Vz_XmYpZp_X,Vz_XmYpZp_Y,Vz_XmYpZp_Z,
                              Txx_XmYpZp_X,Txx_XmYpZp_Y,Txx_XmYpZp_Z,
                              Tyy_XmYpZp_X,Tyy_XmYpZp_Y,Tyy_XmYpZp_Z,
                              Tzz_XmYpZp_X,Tzz_XmYpZp_Y,Tzz_XmYpZp_Z,
                              Txy_XmYpZp_X,Txy_XmYpZp_Y,
                              Tyz_XmYpZp_Y,Tyz_XmYpZp_Z,
                              Tzx_XmYpZp_Z,Tzx_XmYpZp_X,
                              "XmYpZp");
    }
    if ((ConditionFaceXm==0)&&(ConditionFaceYp==0)&&(ConditionFaceZm==0))
    {
        ReMiseAZeroTampons();
        MiseAJourContraintesTamponCoinPML("XmYpZm");
        CalculPMLCoinVitesses(Vx_XmYpZm_X,Vx_XmYpZm_Y,Vx_XmYpZm_Z,
                              Vy_XmYpZm_X,Vy_XmYpZm_Y,Vy_XmYpZm_Z,
                              Vz_XmYpZm_X,Vz_XmYpZm_Y,Vz_XmYpZm_Z,
                              Txx_XmYpZm_X,Txx_XmYpZm_Y,Txx_XmYpZm_Z,
                              Tyy_XmYpZm_X,Tyy_XmYpZm_Y,Tyy_XmYpZm_Z,
                              Tzz_XmYpZm_X,Tzz_XmYpZm_Y,Tzz_XmYpZm_Z,
                              Txy_XmYpZm_X,Txy_XmYpZm_Y,
                              Tyz_XmYpZm_Y,Tyz_XmYpZm_Z,
                              Tzx_XmYpZm_Z,Tzx_XmYpZm_X,
                              "XmYpZm");
    }
    if ((ConditionFaceXm==0)&&(ConditionFaceYm==0)&&(ConditionFaceZp==0))
    {
        ReMiseAZeroTampons();
        MiseAJourContraintesTamponCoinPML("XmYmZp");
        CalculPMLCoinVitesses(Vx_XmYmZp_X,Vx_XmYmZp_Y,Vx_XmYmZp_Z,
                              Vy_XmYmZp_X,Vy_XmYmZp_Y,Vy_XmYmZp_Z,
                              Vz_XmYmZp_X,Vz_XmYmZp_Y,Vz_XmYmZp_Z,
                              Txx_XmYmZp_X,Txx_XmYmZp_Y,Txx_XmYmZp_Z,
                              Tyy_XmYmZp_X,Tyy_XmYmZp_Y,Tyy_XmYmZp_Z,
                              Tzz_XmYmZp_X,Tzz_XmYmZp_Y,Tzz_XmYmZp_Z,
                              Txy_XmYmZp_X,Txy_XmYmZp_Y,
                              Tyz_XmYmZp_Y,Tyz_XmYmZp_Z,
                              Tzx_XmYmZp_Z,Tzx_XmYmZp_X,
                              "XmYmZp");
    }
    if ((ConditionFaceXm==0)&&(ConditionFaceYm==0)&&(ConditionFaceZm==0))
    {
        ReMiseAZeroTampons();
        MiseAJourContraintesTamponCoinPML("XmYmZm");
        CalculPMLCoinVitesses(Vx_XmYmZm_X,Vx_XmYmZm_Y,Vx_XmYmZm_Z,
                              Vy_XmYmZm_X,Vy_XmYmZm_Y,Vy_XmYmZm_Z,
                              Vz_XmYmZm_X,Vz_XmYmZm_Y,Vz_XmYmZm_Z,
                              Txx_XmYmZm_X,Txx_XmYmZm_Y,Txx_XmYmZm_Z,
                              Tyy_XmYmZm_X,Tyy_XmYmZm_Y,Tyy_XmYmZm_Z,
                              Tzz_XmYmZm_X,Tzz_XmYmZm_Y,Tzz_XmYmZm_Z,
                              Txy_XmYmZm_X,Txy_XmYmZm_Y,
                              Tyz_XmYmZm_Y,Tyz_XmYmZm_Z,
                              Tzx_XmYmZm_Z,Tzx_XmYmZm_X,
                              "XmYmZm");
    }
}

void CalculBordsContraintes(void)
{

    int i,j,k;
    double C44,C55,C66,FacteurQModel;



    /*Face Xp */
    switch(ConditionFaceXp)
    {
    case 0:
#pragma omp parallel for private(k,C66) schedule(guided)
        for (j=1; j<Y; j++)
        {
            for (k=0; k<Z; k++)
            {

                if (TableC66[Carte[(X-1)*(Y)*(Z)+(j-1)*(Z)+(k)]]*
                        TableC66[Carte[(X-1)*(Y)*(Z)+(j)*(Z)+(k)]])
                {

                    C66=2./(1./TableC66[Carte[(X-1)*(Y)*(Z)+(j-1)*(Z)+(k)]]+
                            1./TableC66[Carte[(X-1)*(Y)*(Z)+(j)*(Z)+(k)]]);
                }
                else
                {
                    C66=0.0;
                }

                Txy[(X)*(Y+1)*(Z)+(j)*(Z)+(k)] += DeltaTsurH*C66*(
                                                      (	Vx[(X)*(Y)*(Z)+(j)*(Z)+(k)]									-		Vx[(X)*(Y)*(Z)+(j-1)*(Z)+(k)]		)	+
                                                      ((Vy_Xp_X[(j)*(Z)+(k)]+Vy_Xp_YZ[(j)*(Z)+(k)])	-		Vy[(X-1)*(Y+1)*(Z)+(j)*(Z)+(k)]	)	);
            }
        }

#pragma omp parallel for private(k,C55) schedule(guided)
        for (j=0; j<Y; j++)
        {
            for (k=1; k<Z; k++)
            {

                if (TableC55[Carte[(X-1)*(Y)*(Z)+(j)*(Z)+(k-1)]]*
                        TableC55[Carte[(X-1)*(Y)*(Z)+(j)*(Z)+(k)]])
                {

                    C55=2./(1./TableC55[Carte[(X-1)*(Y)*(Z)+(j)*(Z)+(k-1)]]+
                            1./TableC55[Carte[(X-1)*(Y)*(Z)+(j)*(Z)+(k)]]);
                }
                else
                {
                    C55=0.0;
                }

                Tzx[(X)*(Y)*(Z+1)+(j)*(Z+1)+(k)] += DeltaTsurH*C55*(
                                                        (	Vx[(X)*(Y)*(Z)+(j)*(Z)+(k)]											-	Vx[(X)*(Y)*(Z)+(j)*(Z)+(k-1)]			)	+
                                                        ((Vz_Xp_X[(j)*(Z+1)+(k)]+Vz_Xp_YZ[(j)*(Z+1)+(k)])	-	Vz[(X-1)*(Y)*(Z+1)+(j)*(Z+1)+(k)]	)	);
            }
        }
        break;
    case 1: /*Symetrie, implicite*/
        break;
    case 2:
        break;
    case 3:
        break;
    }

    /*	Face Xm */
    switch(ConditionFaceXm)
    {
    case 0:
#pragma omp parallel for private(k,C66) schedule(guided)
        for (j=1; j<Y; j++)
        {
            for (k=0; k<Z; k++)
            {

                if (TableC66[Carte[(j-1)*(Z)+(k)]]*
                        TableC66[Carte[(j)*(Z)+(k)]])
                {

                    C66=2./(1./TableC66[Carte[(j-1)*(Z)+(k)]]+
                            1./TableC66[Carte[(j)*(Z)+(k)]]);
                }
                else
                {
                    C66=0.0;
                }

                Txy[(j)*(Z)+(k)] += DeltaTsurH*C66*(
                                        (Vx[(j)*(Z)+(k)]																-		Vx[(j-1)*(Z)+(k)]	)	+
                                        -((Vy_Xm_X[(j)*(Z)+(k)]+Vy_Xm_YZ[(j)*(Z)+(k)])	-		Vy[(j)*(Z)+(k)]		)	);
            }
        }

#pragma omp parallel for private(k,C55) schedule(guided)
        for (j=0; j<Y; j++)
        {
            for (k=1; k<Z; k++)
            {

                if (TableC55[Carte[(j)*(Z)+(k-1)]]*
                        TableC55[Carte[(j)*(Z)+(k)]])
                {

                    C55=2./(1./TableC55[Carte[(j)*(Z)+(k-1)]]+
                            1./TableC55[Carte[(j)*(Z)+(k)]]);
                }
                else
                {
                    C55=0.0;
                }

                Tzx[(j)*(Z+1)+(k)] += DeltaTsurH*C55*(
                                          (Vx[(j)*(Z)+(k)]																		-	Vx[(j)*(Z)+(k-1)]	)	+
                                          -((Vz_Xm_X[(j)*(Z+1)+(k)]+Vz_Xm_YZ[(j)*(Z+1)+(k)])	-	Vz[(j)*(Z+1)+(k)]	)	);
            }
        }
        break;
    case 1: /*Symetrie, implicite*/
        break;
    case 2:
        break;
    case 3:
        break;
    }

    /*Face Yp*/
    switch(ConditionFaceYp)
    {
    case 0:
#pragma omp parallel for private(k,C66) schedule(guided)
        for (i=1; i<X; i++)
        {
            for (k=0; k<Z; k++)
            {

                if (TableC66[Carte[(i-1)*(Y)*(Z)+(Y-1)*(Z)+(k)]]*
                        TableC66[Carte[(i)*(Y)*(Z)+(Y-1)*(Z)+(k)]])
                {

                    C66=2./(1./TableC66[Carte[(i-1)*(Y)*(Z)+(Y-1)*(Z)+(k)]]+
                            1./TableC66[Carte[(i)*(Y)*(Z)+(Y-1)*(Z)+(k)]]);
                }
                else
                {
                    C66=0.0;
                }

                Txy[(i)*(Y+1)*(Z)+(Y)*(Z)+(k)] += DeltaTsurH*C66*(
                                                      ((Vx_Yp_Y[k*(X+1)+(i)]+Vx_Yp_ZX[k*(X+1)+(i)])		-	Vx[(i)*(Y)*(Z)+(Y-1)*(Z)+(k)]		)	+
                                                      (Vy[(i)*(Y+1)*(Z)+(Y)*(Z)+(k)]									-	Vy[(i-1)*(Y+1)*(Z)+(Y)*(Z)+(k)]	)	);
            }
        }

#pragma omp parallel for private(k,C44) schedule(guided)
        for (i=0; i<X; i++)
        {
            for (k=1; k<Z; k++)
            {

                if (TableC44[Carte[(i)*(Y)*(Z)+(Y-1)*(Z)+(k-1)]]*
                        TableC44[Carte[(i)*(Y)*(Z)+(Y-1)*(Z)+(k)]])
                {

                    C44=2./(1./TableC44[Carte[(i)*(Y)*(Z)+(Y-1)*(Z)+(k-1)]]+
                            1./TableC44[Carte[(i)*(Y)*(Z)+(Y-1)*(Z)+(k)]]);
                }
                else
                {
                    C44=0.0;
                }

                Tyz[(i)*(Y+1)*(Z+1)+(Y)*(Z+1)+(k)] += DeltaTsurH*C44*(
                        (Vy[(i)*(Y+1)*(Z)+(Y)*(Z)+(k)]						-		Vy[(i)*(Y+1)*(Z)+(Y)*(Z)+(k-1)]		)	+
                        ((Vz_Yp_Y[k*(X)+(i)]+Vz_Yp_ZX[k*(X)+(i)])	-		Vz[(i)*(Y)*(Z+1)+(Y-1)*(Z+1)+(k)]	)	);
            }
        }
        break;
    case 1: /*Symetrie, implicite*/
        break;
    case 2:
        break;
    case 3:
        break;
    }

    /* Face Ym */
    switch(ConditionFaceYm)
    {
    case 0:
#pragma omp parallel for private(k,C66) schedule(guided)
        for (i=1; i<X; i++)
        {
            for (k=0; k<Z; k++)
            {

                if (TableC66[Carte[(i-1)*(Y)*(Z)+(k)]]*
                        TableC66[Carte[(i)*(Y)*(Z)+(Y-1)*(Z)+(k)]])
                {

                    C66=2./(1./TableC66[Carte[(i-1)*(Y)*(Z)+(k)]]+
                            1./TableC66[Carte[(i)*(Y)*(Z)+(k)]]);
                }
                else
                {
                    C66=0.0;
                }

                Txy[(i)*(Y+1)*(Z)+(k)] += DeltaTsurH*C66*(
                                              -((Vx_Ym_Y[k*(X+1)+(i)]+Vx_Ym_ZX[k*(X+1)+(i)])	-		Vx[(i)*(Y)*(Z)+(k)]		)	+
                                              (Vy[(i)*(Y+1)*(Z)+(k)]							-		Vy[(i-1)*(Y+1)*(Z)+(k)]	)	);
            }
        }

#pragma omp parallel for private(k,C44) schedule(guided)
        for (i=0; i<X; i++)
        {
            for (k=1; k<Z; k++)
            {

                if (TableC44[Carte[(i)*(Y)*(Z)+(k-1)]]*
                        TableC44[Carte[(i)*(Y)*(Z)+(k)]])
                {

                    C44=2./(1./TableC44[Carte[(i)*(Y)*(Z)+(k-1)]]+
                            1./TableC44[Carte[(i)*(Y)*(Z)+(k)]]);
                }
                else
                {
                    C44=0.0;
                }

                Tyz[(i)*(Y+1)*(Z+1)+(k)] += DeltaTsurH*C44*(
                                                (Vy[(i)*(Y+1)*(Z)+(k)]						-		Vy[(i)*(Y+1)*(Z)+(k-1)]		)	+
                                                -((Vz_Ym_Y[k*(X)+(i)]+Vz_Ym_ZX[k*(X)+(i)])	-		Vz[(i)*(Y)*(Z+1)+(k)]			)	);
            }
        }
        break;
    case 1: /*Symetrie, implicite*/
        break;
    case 2:
        break;
    case 3:
        break;
    }
    /*Face Zp */
    switch(ConditionFaceZp)
    {
    case 0:
#pragma omp parallel for private(j,C44) schedule(guided)
        for (i=0; i<X; i++)
        {
            for (j=1; j<Y; j++)
            {

                if (TableC44[Carte[(i)*(Y)*(Z)+(j-1)*(Z)+(Z-1)]]*
                        TableC44[Carte[(i)*(Y)*(Z)+(j)*(Z)+(Z-1)]])
                {

                    C44=2./(1./TableC44[Carte[(i)*(Y)*(Z)+(j-1)*(Z)+(Z-1)]]+
                            1./TableC44[Carte[(i)*(Y)*(Z)+(j)*(Z)+(Z-1)]]);
                }
                else
                {
                    C44=0.0;
                }

                Tyz[(i)*(Y+1)*(Z+1)+(j)*(Z+1)+(Z)] += DeltaTsurH*C44*(
                        ((Vy_Zp_Z[i*(Y+1)+(j)]+Vy_Zp_XY[i*(Y+1)+(j)])		-		Vy[(i)*(Y+1)*(Z)+(j)*(Z)+(Z-1)]		)	+
                        (Vz[(i)*(Y)*(Z+1)+(j)*(Z+1)+(Z)]								-		Vz[(i)*(Y)*(Z+1)+(j-1)*(Z+1)+(Z)]	)	);
            }
        }

#pragma omp parallel for private(j,C55) schedule(guided)
        for (i=1; i<X; i++)
        {
            for (j=0; j<Y; j++)
            {

                if (TableC55[Carte[(i-1)*(Y)*(Z)+(j)*(Z)+(Z-1)]]*
                        TableC55[Carte[(i)*(Y)*(Z)+(j)*(Z)+(Z-1)]])
                {

                    C55=2./(1./TableC55[Carte[(i-1)*(Y)*(Z)+(j)*(Z)+(Z-1)]]+
                            1./TableC55[Carte[(i)*(Y)*(Z)+(j)*(Z)+(Z-1)]]);
                }
                else
                {
                    C55=0.0;
                }

                Tzx[(i)*(Y)*(Z+1)+(j)*(Z+1)+(Z)] += DeltaTsurH*C55*(
                                                        ((Vx_Zp_Z[i*(Y)+(j)]+Vx_Zp_XY[i*(Y)+(j)])			-		Vx[(i)*(Y)*(Z)+(j)*(Z)+(Z-1)]			)	+
                                                        (Vz[(i)*(Y)*(Z+1)+(j)*(Z+1)+(Z)]					-		Vz[(i-1)*(Y)*(Z+1)+(j)*(Z+1)+(Z)]	)	);
            }
        }

        break;
    case 1: /*Symetrie, implicite*/
        break;
    case 2:
        break;
    case 3:
        break;
    }

    /*	 Face Zm */
    switch(ConditionFaceZm)
    {
    case 0:
#pragma omp parallel for private(j,C44) schedule(guided)
        for (i=0; i<X; i++)
        {
            for (j=1; j<Y; j++)
            {

                if (TableC44[Carte[(i)*(Y)*(Z)+(j-1)*(Z)]]*
                        TableC44[Carte[(i)*(Y)*(Z)+(j)*(Z)]])
                {

                    C44=2./(1./TableC44[Carte[(i)*(Y)*(Z)+(j-1)*(Z)]]+
                            1./TableC44[Carte[(i)*(Y)*(Z)+(j)*(Z)]]);
                }
                else
                {
                    C44=0.0;
                }

                Tyz[(i)*(Y+1)*(Z+1)+(j)*(Z+1)] += DeltaTsurH*C44*(
                                                      -((Vy_Zm_Z[i*(Y+1)+(j)]+Vy_Zm_XY[i*(Y+1)+(j)])		-		Vy[(i)*(Y+1)*(Z)+(j)*(Z)]		)	+
                                                      (Vz[(i)*(Y)*(Z+1)+(j)*(Z+1)]						-		Vz[(i)*(Y)*(Z+1)+(j-1)*(Z+1)]	)	);
            }
        }

#pragma omp parallel for private(j,C55) schedule(guided)
        for (i=1; i<X; i++)
        {
            for (j=0; j<Y; j++)
            {

                if (TableC55[Carte[(i-1)*(Y)*(Z)+(j)*(Z)]]*
                        TableC55[Carte[(i)*(Y)*(Z)+(j)*(Z)]])
                {

                    C55=2./(1./TableC55[Carte[(i-1)*(Y)*(Z)+(j)*(Z)]]+
                            1./TableC55[Carte[(i)*(Y)*(Z)+(j)*(Z)]]);
                }
                else
                {
                    C55=0.0;
                }

                Tzx[(i)*(Y)*(Z+1)+(j)*(Z+1)] += DeltaTsurH*C55*(
                                                    -((Vx_Zm_Z[i*(Y)+(j)]+Vx_Zm_XY[i*(Y)+(j)])			-			Vx[(i)*(Y)*(Z)+(j)*(Z)]			)	+
                                                    (Vz[(i)*(Y)*(Z+1)+(j)*(Z+1)]						-			Vz[(i-1)*(Y)*(Z+1)+(j)*(Z+1)]	)	);
            }
        }
        break;
    case 1: /*Symetrie, implicite*/
        break;
    case 2:
        break;
    case 3:
        break;
    }


    /***************************************** Arete Xp/Yp ************************************/
    if ((ConditionFaceXp==0)&&(ConditionFaceYp==0))
    {
#pragma omp parallel for private(C66) schedule(guided)
        for (k=0; k<Z; k++)
        {

            C66=TableC66[Carte[(X-1)*(Y)*(Z)+(Y-1)*(Z)+(k)]];

            Txy[(X)*(Y+1)*(Z)+(Y)*(Z)+(k)] += DeltaTsurH*C66*(
                                                  ((Vx_Yp_Y[(k)*(X+1)+X]+Vx_Yp_ZX[(k)*(X+1)+X])	-		Vx[(X)*(Y)*(Z)+(Y-1)*(Z)+(k)]		)	+
                                                  ((Vy_Xp_X[(Y)*(Z)+(k)]+Vy_Xp_YZ[(Y)*(Z)+(k)])	-		Vy[(X-1)*(Y+1)*(Z)+(Y)*(Z)+(k)]	)	);
        }
    }
    /* Symetries Traites implicitement...*/

    /***************************************** Arete Xp/Ym ************************************/
    if ((ConditionFaceXp==0)&&(ConditionFaceYm==0))
    {
#pragma omp parallel for private(C66) schedule(guided)
        for (k=0; k<Z; k++)
        {

            C66=TableC66[Carte[(X-1)*(Y)*(Z)+(0)*(Z)+(k)]];


            Txy[(X)*(Y+1)*(Z)+(0)*(Z)+(k)] += DeltaTsurH*C66*(
                                                  -((Vx_Ym_Y[(k)*(X+1)+X]+Vx_Ym_ZX[(k)*(X+1)+X])	-		Vx[(X)*(Y)*(Z)+(0)*(Z)+(k)]		)	+
                                                  ((Vy_Xp_X[(0)*(Z)+(k)]+Vy_Xp_YZ[(0)*(Z)+(k)])	-		Vy[(X-1)*(Y+1)*(Z)+(0)*(Z)+(k)]	)	);
        }
    }
    /* Symetries Traites implicitement...*/
    /***************************************** Arete Xm/Yp ************************************/
    if ((ConditionFaceXm==0)&&(ConditionFaceYp==0))
    {
#pragma omp parallel for private(C66) schedule(guided)
        for (k=0; k<Z; k++)
        {

            C66=TableC66[Carte[(0)*(Y)*(Z)+(Y-1)*(Z)+(k)]];


            Txy[(0)*(Y+1)*(Z)+(Y)*(Z)+(k)] += DeltaTsurH*C66*(
                                                  ((Vx_Yp_Y[(k)*(X+1)+0]+Vx_Yp_ZX[(k)*(X+1)+0])	-		Vx[(0)*(Y)*(Z)+(Y-1)*(Z)+(k)]		)	+
                                                  -((Vy_Xm_X[(Y)*(Z)+(k)]+Vy_Xm_YZ[(Y)*(Z)+(k)])	-		Vy[(0)*(Y+1)*(Z)+(Y)*(Z)+(k)]	)	);
        }
    }
    /* Symetries Traites implicitement...*/
    /***************************************** Arete Xm/Ym ************************************/
    if ((ConditionFaceXm==0)&&(ConditionFaceYm==0))
    {
#pragma omp parallel for private(C66) schedule(guided)
        for (k=0; k<Z; k++)
        {

            C66=TableC66[Carte[(0)*(Y)*(Z)+(0)*(Z)+(k)]];


            Txy[(0)*(Y+1)*(Z)+(0)*(Z)+(k)] += DeltaTsurH*C66*(
                                                  -((Vx_Ym_Y[(k)*(X+1)+0]+Vx_Ym_ZX[(k)*(X+1)+0])	-		Vx[(0)*(Y)*(Z)+(0)*(Z)+(k)]		)	+
                                                  -((Vy_Xm_X[(0)*(Z)+(k)]+Vy_Xm_YZ[(0)*(Z)+(k)])	-		Vy[(0)*(Y+1)*(Z)+(0)*(Z)+(k)]	)	);
        }
    }
    /* Symetries Traites implicitement...*/
    /***************************************** Arete Xp/Zp ************************************/
    if ((ConditionFaceXp==0)&&(ConditionFaceZp==0))
    {
#pragma omp parallel for private(C55) schedule(guided)
        for (j=0; j<Y; j++)
        {

            C55=TableC55[Carte[(X-1)*(Y)*(Z)+(j)*(Z)+(Z-1)]];

            Tzx[(X)*(Y)*(Z+1)+(j)*(Z+1)+(Z)] += DeltaTsurH*C55*(
                                                    ((Vx_Zp_Z[X*(Y)+(j)]		+Vx_Zp_XY[X*(Y)+(j)])				-		Vx[(X)*(Y)*(Z)+(j)*(Z)+(Z-1)]			)	+
                                                    ((Vz_Xp_X[(j)*(Z+1)+(Z)]+Vz_Xp_YZ[(j)*(Z+1)+(Z)])		-		Vz[(X-1)*(Y)*(Z+1)+(j)*(Z+1)+(Z)]	)	);
        }
    }
    /* Symetries Traites implicitement...*/
    /***************************************** Arete Xp/Zm ************************************/
    if ((ConditionFaceXp==0)&&(ConditionFaceZm==0))
    {
#pragma omp parallel for private(C55) schedule(guided)
        for (j=0; j<Y; j++)
        {

            C55=TableC55[Carte[(X-1)*(Y)*(Z)+(j)*(Z)+(0)]];

            Tzx[(X)*(Y)*(Z+1)+(j)*(Z+1)+(0)] += DeltaTsurH*C55*(
                                                    -((Vx_Zm_Z[X*(Y)+(j)]		+Vx_Zm_XY[X*(Y)+(j)])				-		Vx[(X)*(Y)*(Z)+(j)*(Z)+(0)]			)	+
                                                    ((Vz_Xp_X[(j)*(Z+1)+(0)]+Vz_Xp_YZ[(j)*(Z+1)+(0)])		-		Vz[(X-1)*(Y)*(Z+1)+(j)*(Z+1)+(0)]	)	);
        }
    }
    /* Symetries Traites implicitement...*/
    /***************************************** Arete Xm/Zp ************************************/
    if ((ConditionFaceXm==0)&&(ConditionFaceZp==0))
    {
#pragma omp parallel for private(C55) schedule(guided)
        for (j=0; j<Y; j++)
        {

            C55=TableC55[Carte[(0)*(Y)*(Z)+(j)*(Z)+(Z-1)]];

            Tzx[(0)*(Y)*(Z+1)+(j)*(Z+1)+(Z)] += DeltaTsurH*C55*(
                                                    ((Vx_Zp_Z[0*(Y)+(j)]		+Vx_Zp_XY[0*(Y)+(j)])				-		Vx[(0)*(Y)*(Z)+(j)*(Z)+(Z-1)]			)	+
                                                    -((Vz_Xm_X[(j)*(Z+1)+(Z)]+Vz_Xm_YZ[(j)*(Z+1)+(Z)])		-		Vz[(0)*(Y)*(Z+1)+(j)*(Z+1)+(Z)]	)	);
        }
    }
    /* Symetries Traites implicitement...*/
    /***************************************** Arete Xm/Zm ************************************/
    if ((ConditionFaceXm==0)&&(ConditionFaceZm==0))
    {
#pragma omp parallel for private(C55) schedule(guided)
        for (j=0; j<Y; j++)
        {

            C55=TableC55[Carte[(0)*(Y)*(Z)+(j)*(Z)+(0)]];

            Tzx[(0)*(Y)*(Z+1)+(j)*(Z+1)+(0)] += DeltaTsurH*C55*(
                                                    -((Vx_Zm_Z[0*(Y)+(j)]		+Vx_Zm_XY[0*(Y)+(j)])				-		Vx[(0)*(Y)*(Z)+(j)*(Z)+(0)]			)	+
                                                    -((Vz_Xm_X[(j)*(Z+1)+(0)]+Vz_Xm_YZ[(j)*(Z+1)+(0)])		-		Vz[(0)*(Y)*(Z+1)+(j)*(Z+1)+(0)]	)	);
        }
    }
    /* Symetries Traites implicitement...*/
    /***************************************** Arete Yp/Zp ************************************/
    if ((ConditionFaceYp==0)&&(ConditionFaceZp==0))
    {
#pragma omp parallel for private(C44) schedule(guided)
        for (i=0; i<X; i++)
        {

            C44=TableC44[Carte[(i)*(Y)*(Z)+(Y-1)*(Z)+(Z-1)]];

            Tyz[(i)*(Y+1)*(Z+1)+(Y)*(Z+1)+(Z)] += DeltaTsurH*C44*(
                    ((Vy_Zp_Z[(i)*(Y+1)+Y]+Vy_Zp_XY[(i)*(Y+1)+Y])	-		Vy[(i)*(Y+1)*(Z)+(Y)*(Z)+(Z-1)]		)	+
                    ((Vz_Yp_Y[Z*(X)+(i)]	+Vz_Yp_ZX[Z*(X)+(i)])		-		Vz[(i)*(Y)*(Z+1)+(Y-1)*(Z+1)+(Z)]	)	);
        }
    }
    /* Symetries Traites implicitement...*/
    /***************************************** Arete Yp/Zm ************************************/
    if ((ConditionFaceYp==0)&&(ConditionFaceZm==0))
    {
#pragma omp parallel for private(C44) schedule(guided)
        for (i=0; i<X; i++)
        {

            C44=TableC44[Carte[(i)*(Y)*(Z)+(Y-1)*(Z)+0]];

            Tyz[(i)*(Y+1)*(Z+1)+(Y)*(Z+1)+(0)] += DeltaTsurH*C44*(
                    -((Vy_Zm_Z[(i)*(Y+1)+Y]+Vy_Zm_XY[(i)*(Y+1)+Y])	-		Vy[(i)*(Y+1)*(Z)+(Y)*(Z)+(0)]		)	+
                    ((Vz_Yp_Y[0*(X)+(i)]	+Vz_Yp_ZX[0*(X)+(i)])		-		Vz[(i)*(Y)*(Z+1)+(Y-1)*(Z+1)+(0)]	)	);
        }
    }

    /* Symetries Traites implicitement...*/
    /***************************************** Arete Ym/Zp ************************************/
    if ((ConditionFaceYm==0)&&(ConditionFaceZp==0))
    {
#pragma omp parallel for private(C44) schedule(guided)
        for (i=0; i<X; i++)
        {

            C44=TableC44[Carte[(i)*(Y)*(Z)+(0)*(Z)+(Z-1)]];

            Tyz[(i)*(Y+1)*(Z+1)+(0)*(Z+1)+(Z)] += DeltaTsurH*C44*(
                    ((Vy_Zp_Z[(i)*(Y+1)+0]+Vy_Zp_XY[(i)*(Y+1)+0])	-		Vy[(i)*(Y+1)*(Z)+(0)*(Z)+(Z-1)]		)	+
                    -((Vz_Ym_Y[Z*(X)+(i)]	+Vz_Ym_ZX[Z*(X)+(i)])		-		Vz[(i)*(Y)*(Z+1)+(0)*(Z+1)+(Z)]	)	);
        }
    }
    /* Symetries Traites implicitement...*/
    /***************************************** Arete Ym/Zm ************************************/
    if ((ConditionFaceYm==0)&&(ConditionFaceZm==0))
    {
#pragma omp parallel for private(C44) schedule(guided)
        for (i=0; i<X; i++)
        {

            C44=TableC44[Carte[(i)*(Y)*(Z)+(0)*(Z)+0]];

            Tyz[(i)*(Y+1)*(Z+1)+(0)*(Z+1)+(0)] += DeltaTsurH*C44*(
                    -((Vy_Zm_Z[(i)*(Y+1)+0]+Vy_Zm_XY[(i)*(Y+1)+0])	-		Vy[(i)*(Y+1)*(Z)+(0)*(Z)+(0)]		)	+
                    -((Vz_Ym_Y[0*(X)+(i)]	+Vz_Ym_ZX[0*(X)+(i)])		-		Vz[(i)*(Y)*(Z+1)+(0)*(Z+1)+(0)]	)	);
        }
    }
    /* Symetries Traites implicitement...*/

    /**************************************************************************/
    /*********************** Attenuating Factor QModel ************************/
    /**************************************************************************/

    if(TypeAbsorption == 2)
    {

        /*********************** FACES ************************/

#pragma omp parallel for private(k,FacteurQModel) schedule(guided)
        for (j=1; j<Y; j++)
        {
            for (k=0; k<Z; k++)
            {
                FacteurQModel = sqrt(TableQModel[Carte[(X-1)*(Y)*(Z)+(j-1)*(Z)+(k)]]*TableQModel[Carte[(X-1)*(Y)*(Z)+(j)*(Z)+(k)]]) ;
                Txy[(X)*(Y+1)*(Z)+(j)*(Z)+(k)] *= FacteurQModel;
            }
        }
#pragma omp parallel for private(k,FacteurQModel) schedule(guided)
        for (j=1; j<Y; j++)
        {
            for (k=0; k<Z; k++)
            {
                FacteurQModel = sqrt(TableQModel[Carte[(j-1)*(Z)+(k)]]*TableQModel[Carte[(j)*(Z)+(k)]]) ;
                Txy[(j)*(Z)+(k)] *= FacteurQModel;
            }
        }
#pragma omp parallel for private(k,FacteurQModel) schedule(guided)
        for (j=0; j<Y; j++)
        {
            for (k=1; k<Z; k++)
            {
                FacteurQModel = sqrt(TableQModel[Carte[(X-1)*(Y)*(Z)+(j)*(Z)+(k-1)]]*TableQModel[Carte[(X-1)*(Y)*(Z)+(j)*(Z)+(k)]]) ;
                Tzx[(X)*(Y)*(Z+1)+(j)*(Z+1)+(k)] *= FacteurQModel;
            }
        }
#pragma omp parallel for private(k,FacteurQModel) schedule(guided)
        for (j=0; j<Y; j++)
        {
            for (k=1; k<Z; k++)
            {
                FacteurQModel = sqrt(TableQModel[Carte[(j)*(Z)+(k-1)]]*TableQModel[Carte[(j)*(Z)+(k)]]) ;
                Tzx[(j)*(Z+1)+(k)] *= FacteurQModel;
            }
        }
#pragma omp parallel for private(k,FacteurQModel) schedule(guided)
        for (i=1; i<X; i++)
        {
            for (k=0; k<Z; k++)
            {
                FacteurQModel = sqrt(TableQModel[Carte[(i-1)*(Y)*(Z)+(Y-1)*(Z)+(k)]]*TableQModel[Carte[(i)*(Y)*(Z)+(Y-1)*(Z)+(k)]]) ;
                Txy[(i)*(Y+1)*(Z)+(Y)*(Z)+(k)] *= FacteurQModel;
            }
        }
#pragma omp parallel for private(k,FacteurQModel) schedule(guided)
        for (i=1; i<X; i++)
        {
            for (k=0; k<Z; k++)
            {
                FacteurQModel = sqrt(TableQModel[Carte[(i-1)*(Y)*(Z)+(k)]]*TableQModel[Carte[(i)*(Y)*(Z)+(k)]]) ;
                Txy[(i)*(Y+1)*(Z)+(k)] *= FacteurQModel;
            }
        }
#pragma omp parallel for private(k,FacteurQModel) schedule(guided)
        for (i=0; i<X; i++)
        {
            for (k=1; k<Z; k++)
            {
                FacteurQModel = sqrt(TableQModel[Carte[(i)*(Y)*(Z)+(Y-1)*(Z)+(k-1)]]*TableQModel[Carte[(i)*(Y)*(Z)+(Y-1)*(Z)+(k)]]) ;
                Tyz[(i)*(Y+1)*(Z+1)+(Y)*(Z+1)+(k)] *= FacteurQModel;
            }
        }
#pragma omp parallel for private(k,FacteurQModel) schedule(guided)
        for (i=0; i<X; i++)
        {
            for (k=1; k<Z; k++)
            {
                FacteurQModel = sqrt(TableQModel[Carte[(i)*(Y)*(Z)+(k-1)]]*TableQModel[Carte[(i)*(Y)*(Z)+(k)]]) ;
                Tyz[(i)*(Y+1)*(Z+1)+(k)] *= FacteurQModel;
            }
        }
#pragma omp parallel for private(j,FacteurQModel) schedule(guided)
        for (i=0; i<X; i++)
        {
            for (j=1; j<Y; j++)
            {
                FacteurQModel = sqrt(TableQModel[Carte[(i)*(Y)*(Z)+(j-1)*(Z)+(Z-1)]]*TableQModel[Carte[(i)*(Y)*(Z)+(j)*(Z)+(Z-1)]]) ;
                Tyz[(i)*(Y+1)*(Z+1)+(j)*(Z+1)+(Z)] *= FacteurQModel;
            }
        }
#pragma omp parallel for private(j,FacteurQModel) schedule(guided)
        for (i=0; i<X; i++)
        {
            for (j=1; j<Y; j++)
            {
                FacteurQModel = sqrt(TableQModel[Carte[(i)*(Y)*(Z)+(j-1)*(Z)]]*TableQModel[Carte[(i)*(Y)*(Z)+(j)*(Z)]]) ;
                Tyz[(i)*(Y+1)*(Z+1)+(j)*(Z+1)] *= FacteurQModel;
            }
        }
#pragma omp parallel for private(j,FacteurQModel) schedule(guided)
        for (i=1; i<X; i++)
        {
            for (j=0; j<Y; j++)
            {
                FacteurQModel = sqrt(TableQModel[Carte[(i-1)*(Y)*(Z)+(j)*(Z)+(Z-1)]]*TableQModel[Carte[(i)*(Y)*(Z)+(j)*(Z)+(Z-1)]]) ;
                Tzx[(i)*(Y)*(Z+1)+(j)*(Z+1)+(Z)] *= FacteurQModel;
            }
        }
#pragma omp parallel for private(j,FacteurQModel) schedule(guided)
        for (i=1; i<X; i++)
        {
            for (j=0; j<Y; j++)
            {
                FacteurQModel = sqrt(TableQModel[Carte[(i-1)*(Y)*(Z)+(j)*(Z)]]*TableQModel[Carte[(i)*(Y)*(Z)+(j)*(Z)]]) ;
                Tzx[(i)*(Y)*(Z+1)+(j)*(Z+1)] *= FacteurQModel;
            }
        }

        /*********************** ARETES ************************/

        /***************************************** Arete Xp/Yp ************************************/
#pragma omp parallel for private(FacteurQModel) schedule(guided)
        for (k=0; k<Z; k++)
        {

            FacteurQModel=TableQModel[Carte[(X-1)*(Y)*(Z)+(Y-1)*(Z)+(k)]];

            Txy[(X)*(Y+1)*(Z)+(Y)*(Z)+(k)] *= FacteurQModel;
        }

        /***************************************** Arete Xp/Ym ************************************/
#pragma omp parallel for private(FacteurQModel) schedule(guided)
        for (k=0; k<Z; k++)
        {

            FacteurQModel=TableQModel[Carte[(X-1)*(Y)*(Z)+(0)*(Z)+(k)]];


            Txy[(X)*(Y+1)*(Z)+(0)*(Z)+(k)] *= FacteurQModel;
        }
        /***************************************** Arete Xm/Yp ************************************/
#pragma omp parallel for private(FacteurQModel) schedule(guided)
        for (k=0; k<Z; k++)
        {

            FacteurQModel=TableQModel[Carte[(0)*(Y)*(Z)+(Y-1)*(Z)+(k)]];


            Txy[(0)*(Y+1)*(Z)+(Y)*(Z)+(k)] *= FacteurQModel;
        }
        /***************************************** Arete Xm/Ym ************************************/
#pragma omp parallel for private(FacteurQModel) schedule(guided)
        for (k=0; k<Z; k++)
        {

            FacteurQModel=TableQModel[Carte[(0)*(Y)*(Z)+(0)*(Z)+(k)]];


            Txy[(0)*(Y+1)*(Z)+(0)*(Z)+(k)] *= FacteurQModel;
        }
        /***************************************** Arete Xp/Zp ************************************/
#pragma omp parallel for private(FacteurQModel) schedule(guided)
        for (j=0; j<Y; j++)
        {

            FacteurQModel=TableQModel[Carte[(X-1)*(Y)*(Z)+(j)*(Z)+(Z-1)]];

            Tzx[(X)*(Y)*(Z+1)+(j)*(Z+1)+(Z)] *= FacteurQModel;
        }
        /***************************************** Arete Xp/Zm ************************************/
#pragma omp parallel for private(FacteurQModel) schedule(guided)
        for (j=0; j<Y; j++)
        {

            FacteurQModel=TableQModel[Carte[(X-1)*(Y)*(Z)+(j)*(Z)+(0)]];

            Tzx[(X)*(Y)*(Z+1)+(j)*(Z+1)+(0)] *= FacteurQModel;
        }
        /***************************************** Arete Xm/Zp ************************************/
#pragma omp parallel for private(FacteurQModel) schedule(guided)
        for (j=0; j<Y; j++)
        {

            FacteurQModel=TableQModel[Carte[(0)*(Y)*(Z)+(j)*(Z)+(Z-1)]];

            Tzx[(0)*(Y)*(Z+1)+(j)*(Z+1)+(Z)] *= FacteurQModel;
        }
        /***************************************** Arete Xm/Zm ************************************/
#pragma omp parallel for private(FacteurQModel) schedule(guided)
        for (j=0; j<Y; j++)
        {

            FacteurQModel=TableQModel[Carte[(0)*(Y)*(Z)+(j)*(Z)+(0)]];

            Tzx[(0)*(Y)*(Z+1)+(j)*(Z+1)+(0)] *= FacteurQModel;
        }
        /***************************************** Arete Yp/Zp ************************************/
#pragma omp parallel for private(FacteurQModel) schedule(guided)
        for (i=0; i<X; i++)
        {

            FacteurQModel=TableQModel[Carte[(i)*(Y)*(Z)+(Y-1)*(Z)+(Z-1)]];

            Tyz[(i)*(Y+1)*(Z+1)+(Y)*(Z+1)+(Z)] *= FacteurQModel;
        }
        /***************************************** Arete Yp/Zm ************************************/
#pragma omp parallel for private(FacteurQModel) schedule(guided)
        for (i=0; i<X; i++)
        {

            FacteurQModel=TableQModel[Carte[(i)*(Y)*(Z)+(Y-1)*(Z)+0]];

            Tyz[(i)*(Y+1)*(Z+1)+(Y)*(Z+1)+(0)] *= FacteurQModel;
        }
        /***************************************** Arete Ym/Zp ************************************/
#pragma omp parallel for private(FacteurQModel) schedule(guided)
        for (i=0; i<X; i++)
        {

            FacteurQModel=TableQModel[Carte[(i)*(Y)*(Z)+(0)*(Z)+(Z-1)]];

            Tyz[(i)*(Y+1)*(Z+1)+(0)*(Z+1)+(Z)] *= FacteurQModel;
        }
        /***************************************** Arete Ym/Zm ************************************/
#pragma omp parallel for private(FacteurQModel) schedule(guided)
        for (i=0; i<X; i++)
        {

            FacteurQModel=TableQModel[Carte[(i)*(Y)*(Z)+(0)*(Z)+0]];

            Tyz[(i)*(Y+1)*(Z+1)+(0)*(Z+1)+(0)] *= FacteurQModel;
        }

    }

    /*
    if(TypeAbsorption == 2){
    for(j = 1 ; j < Y ; j++){
    FacteurQModel = sqrt(TableQModel[Carte[(X-1)*Y+j-1]]*TableQModel[Carte[(X-1)*Y+j]]) ;
    Txy[X*(Y+1)+j] *= FacteurQModel ;
    FacteurQModel = sqrt(TableQModel[Carte[(0)*Y+j-1]]*TableQModel[Carte[(0)*Y+j]]) ;
    Txy[0*(Y+1)+j] *= FacteurQModel ;
    }
    for(j = 1 ; j < X ; j++){
    FacteurQModel = sqrt(TableQModel[Carte[(j-1)*Y+Y-1]]*TableQModel[Carte[(j)*Y+Y-1]]) ;
    Txy[j*(Y+1)+Y] *= FacteurQModel ;
    FacteurQModel = sqrt(TableQModel[Carte[(j-1)*Y+0]]*TableQModel[Carte[(j)*Y+0]]) ;
    Txy[j*(Y+1)+0] *= FacteurQModel ;
    	}
    	Txy[0*(Y+1)+0] *= TableQModel[Carte[(0)*Y+0]] ;
    	Txy[0*(Y+1)+Y] *= TableQModel[Carte[(0)*Y+Y-1]] ;
    	Txy[X*(Y+1)+0] *= TableQModel[Carte[(X-1)*Y+0]] ;
    	Txy[X*(Y+1)+Y] *= TableQModel[Carte[(X-1)*Y+Y-1]] ;
    }

    */



    /**************************************************************************/
    /******************************* Calcul des faces PML *********************/
    /**************************************************************************/
    if(ConditionFaceXp==0)
    {
        ReMiseAZeroTampons();
        MiseAJourVitessesTamponFacePML("Xp");
        CalculPMLFaceContraintes(Vx_Xp_X,Vx_Xp_YZ,
                                 Vy_Xp_X,Vy_Xp_YZ,
                                 Vz_Xp_X,Vz_Xp_YZ,
                                 Txx_Xp_X,Txx_Xp_YZ,
                                 Tyy_Xp_X,Tyy_Xp_YZ,
                                 Tzz_Xp_X,Tzz_Xp_YZ,
                                 Txy_Xp_X,Txy_Xp_YZ,
                                 Tzx_Xp_X,Tzx_Xp_YZ,
                                 Tyz_Xp,
                                 Y,Z,"Xp");
    }
    if(ConditionFaceXm==0)
    {
        ReMiseAZeroTampons();
        MiseAJourVitessesTamponFacePML("Xm");
        CalculPMLFaceContraintes(Vx_Xm_X,Vx_Xm_YZ,
                                 Vy_Xm_X,Vy_Xm_YZ,
                                 Vz_Xm_X,Vz_Xm_YZ,
                                 Txx_Xm_X,Txx_Xm_YZ,
                                 Tyy_Xm_X,Tyy_Xm_YZ,
                                 Tzz_Xm_X,Tzz_Xm_YZ,
                                 Txy_Xm_X,Txy_Xm_YZ,
                                 Tzx_Xm_X,Tzx_Xm_YZ,
                                 Tyz_Xm,
                                 Y,Z,"Xm");
    }
    if(ConditionFaceYp==0)
    {
        ReMiseAZeroTampons();
        MiseAJourVitessesTamponFacePML("Yp");
        CalculPMLFaceContraintes(Vy_Yp_Y,Vy_Yp_ZX,
                                 Vz_Yp_Y,Vz_Yp_ZX,
                                 Vx_Yp_Y,Vx_Yp_ZX,
                                 Tyy_Yp_Y,Tyy_Yp_ZX,
                                 Tzz_Yp_Y,Tzz_Yp_ZX,
                                 Txx_Yp_Y,Txx_Yp_ZX,
                                 Tyz_Yp_Y,Tyz_Yp_ZX,
                                 Txy_Yp_Y,Txy_Yp_ZX,
                                 Tzx_Yp,
                                 Z,X,"Yp");
    }
    if(ConditionFaceYm==0)
    {
        ReMiseAZeroTampons();
        MiseAJourVitessesTamponFacePML("Ym");
        CalculPMLFaceContraintes(Vy_Ym_Y,Vy_Ym_ZX,
                                 Vz_Ym_Y,Vz_Ym_ZX,
                                 Vx_Ym_Y,Vx_Ym_ZX,
                                 Tyy_Ym_Y,Tyy_Ym_ZX,
                                 Tzz_Ym_Y,Tzz_Ym_ZX,
                                 Txx_Ym_Y,Txx_Ym_ZX,
                                 Tyz_Ym_Y,Tyz_Ym_ZX,
                                 Txy_Ym_Y,Txy_Ym_ZX,
                                 Tzx_Ym,
                                 Z,X,"Ym");
    }
    if(ConditionFaceZp==0)
    {
        ReMiseAZeroTampons();
        MiseAJourVitessesTamponFacePML("Zp");
        CalculPMLFaceContraintes(Vz_Zp_Z,Vz_Zp_XY,
                                 Vx_Zp_Z,Vx_Zp_XY,
                                 Vy_Zp_Z,Vy_Zp_XY,
                                 Tzz_Zp_Z,Tzz_Zp_XY,
                                 Txx_Zp_Z,Txx_Zp_XY,
                                 Tyy_Zp_Z,Tyy_Zp_XY,
                                 Tzx_Zp_Z,Tzx_Zp_XY,
                                 Tyz_Zp_Z,Tyz_Zp_XY,
                                 Txy_Zp,
                                 X,Y,"Zp");
    }
    if(ConditionFaceZm==0)
    {
        ReMiseAZeroTampons();
        MiseAJourVitessesTamponFacePML("Zm");
        CalculPMLFaceContraintes(Vz_Zm_Z,Vz_Zm_XY,
                                 Vx_Zm_Z,Vx_Zm_XY,
                                 Vy_Zm_Z,Vy_Zm_XY,
                                 Tzz_Zm_Z,Tzz_Zm_XY,
                                 Txx_Zm_Z,Txx_Zm_XY,
                                 Tyy_Zm_Z,Tyy_Zm_XY,
                                 Tzx_Zm_Z,Tzx_Zm_XY,
                                 Tyz_Zm_Z,Tyz_Zm_XY,
                                 Txy_Zm,
                                 X,Y,"Zm");
    }
    /**************************************************************************/
    /******************* Calcul des Aretes PML ********************************/
    /**************************************************************************/

    /**************************************************************************/
    /*********** Aretes perpendiculaire au plan XY ****************************/
    /**************************************************************************/

    if ((ConditionFaceXp==0)&&(ConditionFaceYp==0))
    {
        ReMiseAZeroTampons();
        MiseAJourVitessesTamponAretePML("XpYp");
        CalculPMLAreteContraintes(Vx_XpYp_X,Vx_XpYp_Y,Vx_XpYp_Z,
                                  Vy_XpYp_X,Vy_XpYp_Y,Vy_XpYp_Z,
                                  Vz_XpYp_X,Vz_XpYp_Y,Vz_XpYp_Z,
                                  Txx_XpYp_X,Txx_XpYp_Y,Txx_XpYp_Z,
                                  Tyy_XpYp_X,Tyy_XpYp_Y,Tyy_XpYp_Z,
                                  Tzz_XpYp_X,Tzz_XpYp_Y,Tzz_XpYp_Z,
                                  Txy_XpYp_X,Txy_XpYp_Y,
                                  Tyz_XpYp_Y,Tyz_XpYp_Z,
                                  Tzx_XpYp_Z,Tzx_XpYp_X,
                                  Z,"XpYp");
    }
    if ((ConditionFaceXp==0)&&(ConditionFaceYm==0))
    {
        ReMiseAZeroTampons();
        MiseAJourVitessesTamponAretePML("XpYm");
        CalculPMLAreteContraintes(Vx_XpYm_X,Vx_XpYm_Y,Vx_XpYm_Z,
                                  Vy_XpYm_X,Vy_XpYm_Y,Vy_XpYm_Z,
                                  Vz_XpYm_X,Vz_XpYm_Y,Vz_XpYm_Z,
                                  Txx_XpYm_X,Txx_XpYm_Y,Txx_XpYm_Z,
                                  Tyy_XpYm_X,Tyy_XpYm_Y,Tyy_XpYm_Z,
                                  Tzz_XpYm_X,Tzz_XpYm_Y,Tzz_XpYm_Z,
                                  Txy_XpYm_X,Txy_XpYm_Y,
                                  Tyz_XpYm_Y,Tyz_XpYm_Z,
                                  Tzx_XpYm_Z,Tzx_XpYm_X,
                                  Z,"XpYm");
    }
    if ((ConditionFaceXm==0)&&(ConditionFaceYp==0))
    {
        ReMiseAZeroTampons();
        MiseAJourVitessesTamponAretePML("XmYp");
        CalculPMLAreteContraintes(Vx_XmYp_X,Vx_XmYp_Y,Vx_XmYp_Z,
                                  Vy_XmYp_X,Vy_XmYp_Y,Vy_XmYp_Z,
                                  Vz_XmYp_X,Vz_XmYp_Y,Vz_XmYp_Z,
                                  Txx_XmYp_X,Txx_XmYp_Y,Txx_XmYp_Z,
                                  Tyy_XmYp_X,Tyy_XmYp_Y,Tyy_XmYp_Z,
                                  Tzz_XmYp_X,Tzz_XmYp_Y,Tzz_XmYp_Z,
                                  Txy_XmYp_X,Txy_XmYp_Y,
                                  Tyz_XmYp_Y,Tyz_XmYp_Z,
                                  Tzx_XmYp_Z,Tzx_XmYp_X,
                                  Z,"XmYp");
    }
    if ((ConditionFaceXm==0)&&(ConditionFaceYm==0))
    {
        ReMiseAZeroTampons();
        MiseAJourVitessesTamponAretePML("XmYm");
        CalculPMLAreteContraintes(Vx_XmYm_X,Vx_XmYm_Y,Vx_XmYm_Z,
                                  Vy_XmYm_X,Vy_XmYm_Y,Vy_XmYm_Z,
                                  Vz_XmYm_X,Vz_XmYm_Y,Vz_XmYm_Z,
                                  Txx_XmYm_X,Txx_XmYm_Y,Txx_XmYm_Z,
                                  Tyy_XmYm_X,Tyy_XmYm_Y,Tyy_XmYm_Z,
                                  Tzz_XmYm_X,Tzz_XmYm_Y,Tzz_XmYm_Z,
                                  Txy_XmYm_X,Txy_XmYm_Y,
                                  Tyz_XmYm_Y,Tyz_XmYm_Z,
                                  Tzx_XmYm_Z,Tzx_XmYm_X,
                                  Z,"XmYm");
    }
    /**************************************************************************/
    /*********** Aretes perpendiculaire au plan YZ ****************************/
    /**************************************************************************/
    /**************************************************************************/
    if ((ConditionFaceYp==0)&&(ConditionFaceZp==0))
    {
        ReMiseAZeroTampons();
        MiseAJourVitessesTamponAretePML("YpZp");
        CalculPMLAreteContraintes(Vy_YpZp_Y,Vy_YpZp_Z,Vy_YpZp_X,
                                  Vz_YpZp_Y,Vz_YpZp_Z,Vz_YpZp_X,
                                  Vx_YpZp_Y,Vx_YpZp_Z,Vx_YpZp_X,
                                  Tyy_YpZp_Y,Tyy_YpZp_Z,Tyy_YpZp_X,
                                  Tzz_YpZp_Y,Tzz_YpZp_Z,Tzz_YpZp_X,
                                  Txx_YpZp_Y,Txx_YpZp_Z,Txx_YpZp_X,
                                  Tyz_YpZp_Y,Tyz_YpZp_Z,
                                  Tzx_YpZp_Z,Tzx_YpZp_X,
                                  Txy_YpZp_X,Txy_YpZp_Y,
                                  X,"YpZp");
    }
    if ((ConditionFaceYm==0)&&(ConditionFaceZp==0))
    {
        ReMiseAZeroTampons();
        MiseAJourVitessesTamponAretePML("YmZp");
        CalculPMLAreteContraintes(Vy_YmZp_Y,Vy_YmZp_Z,Vy_YmZp_X,
                                  Vz_YmZp_Y,Vz_YmZp_Z,Vz_YmZp_X,
                                  Vx_YmZp_Y,Vx_YmZp_Z,Vx_YmZp_X,
                                  Tyy_YmZp_Y,Tyy_YmZp_Z,Tyy_YmZp_X,
                                  Tzz_YmZp_Y,Tzz_YmZp_Z,Tzz_YmZp_X,
                                  Txx_YmZp_Y,Txx_YmZp_Z,Txx_YmZp_X,
                                  Tyz_YmZp_Y,Tyz_YmZp_Z,
                                  Tzx_YmZp_Z,Tzx_YmZp_X,
                                  Txy_YmZp_X,Txy_YmZp_Y,
                                  X,"YmZp");
    }
    if ((ConditionFaceYp==0)&&(ConditionFaceZm==0))
    {
        ReMiseAZeroTampons();
        MiseAJourVitessesTamponAretePML("YpZm");
        CalculPMLAreteContraintes(Vy_YpZm_Y,Vy_YpZm_Z,Vy_YpZm_X,
                                  Vz_YpZm_Y,Vz_YpZm_Z,Vz_YpZm_X,
                                  Vx_YpZm_Y,Vx_YpZm_Z,Vx_YpZm_X,
                                  Tyy_YpZm_Y,Tyy_YpZm_Z,Tyy_YpZm_X,
                                  Tzz_YpZm_Y,Tzz_YpZm_Z,Tzz_YpZm_X,
                                  Txx_YpZm_Y,Txx_YpZm_Z,Txx_YpZm_X,
                                  Tyz_YpZm_Y,Tyz_YpZm_Z,
                                  Tzx_YpZm_Z,Tzx_YpZm_X,
                                  Txy_YpZm_X,Txy_YpZm_Y,
                                  X,"YpZm");
    }
    if ((ConditionFaceYm==0)&&(ConditionFaceZm==0))
    {
        ReMiseAZeroTampons();
        MiseAJourVitessesTamponAretePML("YmZm");
        CalculPMLAreteContraintes(Vy_YmZm_Y,Vy_YmZm_Z,Vy_YmZm_X,
                                  Vz_YmZm_Y,Vz_YmZm_Z,Vz_YmZm_X,
                                  Vx_YmZm_Y,Vx_YmZm_Z,Vx_YmZm_X,
                                  Tyy_YmZm_Y,Tyy_YmZm_Z,Tyy_YmZm_X,
                                  Tzz_YmZm_Y,Tzz_YmZm_Z,Tzz_YmZm_X,
                                  Txx_YmZm_Y,Txx_YmZm_Z,Txx_YmZm_X,
                                  Tyz_YmZm_Y,Tyz_YmZm_Z,
                                  Tzx_YmZm_Z,Tzx_YmZm_X,
                                  Txy_YmZm_X,Txy_YmZm_Y,
                                  X,"YmZm");
    }

    /**************************************************************************/
    /*********** Aretes perpendiculaire au plan ZX ****************************/
    /**************************************************************************/
    /**************************************************************************/

    if ((ConditionFaceZp==0)&&(ConditionFaceXp==0))
    {
        ReMiseAZeroTampons();
        MiseAJourVitessesTamponAretePML("ZpXp");
        CalculPMLAreteContraintes(Vz_ZpXp_Z,Vz_ZpXp_X,Vz_ZpXp_Y,
                                  Vx_ZpXp_Z,Vx_ZpXp_X,Vx_ZpXp_Y,
                                  Vy_ZpXp_Z,Vy_ZpXp_X,Vy_ZpXp_Y,
                                  Tzz_ZpXp_Z,Tzz_ZpXp_X,Tzz_ZpXp_Y,
                                  Txx_ZpXp_Z,Txx_ZpXp_X,Txx_ZpXp_Y,
                                  Tyy_ZpXp_Z,Tyy_ZpXp_X,Tyy_ZpXp_Y,
                                  Tzx_ZpXp_Z,Tzx_ZpXp_X,
                                  Txy_ZpXp_X,Txy_ZpXp_Y,
                                  Tyz_ZpXp_Y,Tyz_ZpXp_Z,
                                  Y,"ZpXp");
    }
    if ((ConditionFaceZm==0)&&(ConditionFaceXp==0))
    {
        ReMiseAZeroTampons();
        MiseAJourVitessesTamponAretePML("ZmXp");
        CalculPMLAreteContraintes(Vz_ZmXp_Z,Vz_ZmXp_X,Vz_ZmXp_Y,
                                  Vx_ZmXp_Z,Vx_ZmXp_X,Vx_ZmXp_Y,
                                  Vy_ZmXp_Z,Vy_ZmXp_X,Vy_ZmXp_Y,
                                  Tzz_ZmXp_Z,Tzz_ZmXp_X,Tzz_ZmXp_Y,
                                  Txx_ZmXp_Z,Txx_ZmXp_X,Txx_ZmXp_Y,
                                  Tyy_ZmXp_Z,Tyy_ZmXp_X,Tyy_ZmXp_Y,
                                  Tzx_ZmXp_Z,Tzx_ZmXp_X,
                                  Txy_ZmXp_X,Txy_ZmXp_Y,
                                  Tyz_ZmXp_Y,Tyz_ZmXp_Z,
                                  Y,"ZmXp");
    }
    if ((ConditionFaceZp==0)&&(ConditionFaceXm==0))
    {
        ReMiseAZeroTampons();
        MiseAJourVitessesTamponAretePML("ZpXm");
        CalculPMLAreteContraintes(Vz_ZpXm_Z,Vz_ZpXm_X,Vz_ZpXm_Y,
                                  Vx_ZpXm_Z,Vx_ZpXm_X,Vx_ZpXm_Y,
                                  Vy_ZpXm_Z,Vy_ZpXm_X,Vy_ZpXm_Y,
                                  Tzz_ZpXm_Z,Tzz_ZpXm_X,Tzz_ZpXm_Y,
                                  Txx_ZpXm_Z,Txx_ZpXm_X,Txx_ZpXm_Y,
                                  Tyy_ZpXm_Z,Tyy_ZpXm_X,Tyy_ZpXm_Y,
                                  Tzx_ZpXm_Z,Tzx_ZpXm_X,
                                  Txy_ZpXm_X,Txy_ZpXm_Y,
                                  Tyz_ZpXm_Y,Tyz_ZpXm_Z,
                                  Y,"ZpXm");
    }
    if ((ConditionFaceZm==0)&&(ConditionFaceXm==0))
    {
        ReMiseAZeroTampons();
        MiseAJourVitessesTamponAretePML("ZmXm");
        CalculPMLAreteContraintes(Vz_ZmXm_Z,Vz_ZmXm_X,Vz_ZmXm_Y,
                                  Vx_ZmXm_Z,Vx_ZmXm_X,Vx_ZmXm_Y,
                                  Vy_ZmXm_Z,Vy_ZmXm_X,Vy_ZmXm_Y,
                                  Tzz_ZmXm_Z,Tzz_ZmXm_X,Tzz_ZmXm_Y,
                                  Txx_ZmXm_Z,Txx_ZmXm_X,Txx_ZmXm_Y,
                                  Tyy_ZmXm_Z,Tyy_ZmXm_X,Tyy_ZmXm_Y,
                                  Tzx_ZmXm_Z,Tzx_ZmXm_X,
                                  Txy_ZmXm_X,Txy_ZmXm_Y,
                                  Tyz_ZmXm_Y,Tyz_ZmXm_Z,
                                  Y,"ZmXm");
    }
    /**************************************************************************/
    /******************* Calcul des Coins PML *********************************/
    /**************************************************************************/
    if ((ConditionFaceXp==0)&&(ConditionFaceYp==0)&&(ConditionFaceZp==0))
    {
        ReMiseAZeroTampons();
        MiseAJourVitessesTamponCoinPML("XpYpZp");
        CalculPMLCoinContraintes(Vx_XpYpZp_X,Vx_XpYpZp_Y,Vx_XpYpZp_Z,
                                 Vy_XpYpZp_X,Vy_XpYpZp_Y,Vy_XpYpZp_Z,
                                 Vz_XpYpZp_X,Vz_XpYpZp_Y,Vz_XpYpZp_Z,
                                 Txx_XpYpZp_X,Txx_XpYpZp_Y,Txx_XpYpZp_Z,
                                 Tyy_XpYpZp_X,Tyy_XpYpZp_Y,Tyy_XpYpZp_Z,
                                 Tzz_XpYpZp_X,Tzz_XpYpZp_Y,Tzz_XpYpZp_Z,
                                 Txy_XpYpZp_X,Txy_XpYpZp_Y,
                                 Tyz_XpYpZp_Y,Tyz_XpYpZp_Z,
                                 Tzx_XpYpZp_Z,Tzx_XpYpZp_X,
                                 "XpYpZp");
    }
    if ((ConditionFaceXp==0)&&(ConditionFaceYp==0)&&(ConditionFaceZm==0))
    {
        ReMiseAZeroTampons();
        MiseAJourVitessesTamponCoinPML("XpYpZm");
        CalculPMLCoinContraintes(Vx_XpYpZm_X,Vx_XpYpZm_Y,Vx_XpYpZm_Z,
                                 Vy_XpYpZm_X,Vy_XpYpZm_Y,Vy_XpYpZm_Z,
                                 Vz_XpYpZm_X,Vz_XpYpZm_Y,Vz_XpYpZm_Z,
                                 Txx_XpYpZm_X,Txx_XpYpZm_Y,Txx_XpYpZm_Z,
                                 Tyy_XpYpZm_X,Tyy_XpYpZm_Y,Tyy_XpYpZm_Z,
                                 Tzz_XpYpZm_X,Tzz_XpYpZm_Y,Tzz_XpYpZm_Z,
                                 Txy_XpYpZm_X,Txy_XpYpZm_Y,
                                 Tyz_XpYpZm_Y,Tyz_XpYpZm_Z,
                                 Tzx_XpYpZm_Z,Tzx_XpYpZm_X,
                                 "XpYpZm");
    }
    if ((ConditionFaceXp==0)&&(ConditionFaceYm==0)&&(ConditionFaceZp==0))
    {
        ReMiseAZeroTampons();
        MiseAJourVitessesTamponCoinPML("XpYmZp");
        CalculPMLCoinContraintes(Vx_XpYmZp_X,Vx_XpYmZp_Y,Vx_XpYmZp_Z,
                                 Vy_XpYmZp_X,Vy_XpYmZp_Y,Vy_XpYmZp_Z,
                                 Vz_XpYmZp_X,Vz_XpYmZp_Y,Vz_XpYmZp_Z,
                                 Txx_XpYmZp_X,Txx_XpYmZp_Y,Txx_XpYmZp_Z,
                                 Tyy_XpYmZp_X,Tyy_XpYmZp_Y,Tyy_XpYmZp_Z,
                                 Tzz_XpYmZp_X,Tzz_XpYmZp_Y,Tzz_XpYmZp_Z,
                                 Txy_XpYmZp_X,Txy_XpYmZp_Y,
                                 Tyz_XpYmZp_Y,Tyz_XpYmZp_Z,
                                 Tzx_XpYmZp_Z,Tzx_XpYmZp_X,
                                 "XpYmZp");
    }
    if ((ConditionFaceXp==0)&&(ConditionFaceYm==0)&&(ConditionFaceZm==0))
    {
        ReMiseAZeroTampons();
        MiseAJourVitessesTamponCoinPML("XpYmZm");
        CalculPMLCoinContraintes(Vx_XpYmZm_X,Vx_XpYmZm_Y,Vx_XpYmZm_Z,
                                 Vy_XpYmZm_X,Vy_XpYmZm_Y,Vy_XpYmZm_Z,
                                 Vz_XpYmZm_X,Vz_XpYmZm_Y,Vz_XpYmZm_Z,
                                 Txx_XpYmZm_X,Txx_XpYmZm_Y,Txx_XpYmZm_Z,
                                 Tyy_XpYmZm_X,Tyy_XpYmZm_Y,Tyy_XpYmZm_Z,
                                 Tzz_XpYmZm_X,Tzz_XpYmZm_Y,Tzz_XpYmZm_Z,
                                 Txy_XpYmZm_X,Txy_XpYmZm_Y,
                                 Tyz_XpYmZm_Y,Tyz_XpYmZm_Z,
                                 Tzx_XpYmZm_Z,Tzx_XpYmZm_X,
                                 "XpYmZm");
    }
    if ((ConditionFaceXm==0)&&(ConditionFaceYp==0)&&(ConditionFaceZp==0))
    {
        ReMiseAZeroTampons();
        MiseAJourVitessesTamponCoinPML("XmYpZp");
        CalculPMLCoinContraintes(Vx_XmYpZp_X,Vx_XmYpZp_Y,Vx_XmYpZp_Z,
                                 Vy_XmYpZp_X,Vy_XmYpZp_Y,Vy_XmYpZp_Z,
                                 Vz_XmYpZp_X,Vz_XmYpZp_Y,Vz_XmYpZp_Z,
                                 Txx_XmYpZp_X,Txx_XmYpZp_Y,Txx_XmYpZp_Z,
                                 Tyy_XmYpZp_X,Tyy_XmYpZp_Y,Tyy_XmYpZp_Z,
                                 Tzz_XmYpZp_X,Tzz_XmYpZp_Y,Tzz_XmYpZp_Z,
                                 Txy_XmYpZp_X,Txy_XmYpZp_Y,
                                 Tyz_XmYpZp_Y,Tyz_XmYpZp_Z,
                                 Tzx_XmYpZp_Z,Tzx_XmYpZp_X,
                                 "XmYpZp");
    }
    if ((ConditionFaceXm==0)&&(ConditionFaceYp==0)&&(ConditionFaceZm==0))
    {
        ReMiseAZeroTampons();
        MiseAJourVitessesTamponCoinPML("XmYpZm");
        CalculPMLCoinContraintes(Vx_XmYpZm_X,Vx_XmYpZm_Y,Vx_XmYpZm_Z,
                                 Vy_XmYpZm_X,Vy_XmYpZm_Y,Vy_XmYpZm_Z,
                                 Vz_XmYpZm_X,Vz_XmYpZm_Y,Vz_XmYpZm_Z,
                                 Txx_XmYpZm_X,Txx_XmYpZm_Y,Txx_XmYpZm_Z,
                                 Tyy_XmYpZm_X,Tyy_XmYpZm_Y,Tyy_XmYpZm_Z,
                                 Tzz_XmYpZm_X,Tzz_XmYpZm_Y,Tzz_XmYpZm_Z,
                                 Txy_XmYpZm_X,Txy_XmYpZm_Y,
                                 Tyz_XmYpZm_Y,Tyz_XmYpZm_Z,
                                 Tzx_XmYpZm_Z,Tzx_XmYpZm_X,
                                 "XmYpZm");
    }
    if ((ConditionFaceXm==0)&&(ConditionFaceYm==0)&&(ConditionFaceZp==0))
    {
        ReMiseAZeroTampons();
        MiseAJourVitessesTamponCoinPML("XmYmZp");
        CalculPMLCoinContraintes(Vx_XmYmZp_X,Vx_XmYmZp_Y,Vx_XmYmZp_Z,
                                 Vy_XmYmZp_X,Vy_XmYmZp_Y,Vy_XmYmZp_Z,
                                 Vz_XmYmZp_X,Vz_XmYmZp_Y,Vz_XmYmZp_Z,
                                 Txx_XmYmZp_X,Txx_XmYmZp_Y,Txx_XmYmZp_Z,
                                 Tyy_XmYmZp_X,Tyy_XmYmZp_Y,Tyy_XmYmZp_Z,
                                 Tzz_XmYmZp_X,Tzz_XmYmZp_Y,Tzz_XmYmZp_Z,
                                 Txy_XmYmZp_X,Txy_XmYmZp_Y,
                                 Tyz_XmYmZp_Y,Tyz_XmYmZp_Z,
                                 Tzx_XmYmZp_Z,Tzx_XmYmZp_X,
                                 "XmYmZp");
    }
    if ((ConditionFaceXm==0)&&(ConditionFaceYm==0)&&(ConditionFaceZm==0))
    {
        ReMiseAZeroTampons();
        MiseAJourVitessesTamponCoinPML("XmYmZm");
        CalculPMLCoinContraintes(Vx_XmYmZm_X,Vx_XmYmZm_Y,Vx_XmYmZm_Z,
                                 Vy_XmYmZm_X,Vy_XmYmZm_Y,Vy_XmYmZm_Z,
                                 Vz_XmYmZm_X,Vz_XmYmZm_Y,Vz_XmYmZm_Z,
                                 Txx_XmYmZm_X,Txx_XmYmZm_Y,Txx_XmYmZm_Z,
                                 Tyy_XmYmZm_X,Tyy_XmYmZm_Y,Tyy_XmYmZm_Z,
                                 Tzz_XmYmZm_X,Tzz_XmYmZm_Y,Tzz_XmYmZm_Z,
                                 Txy_XmYmZm_X,Txy_XmYmZm_Y,
                                 Tyz_XmYmZm_Y,Tyz_XmYmZm_Z,
                                 Tzx_XmYmZm_Z,Tzx_XmYmZm_X,
                                 "XmYmZm");
    }
}

int DeuxiemeMax(int a, int b, int c)
{

    int Maximum;

    Maximum = max(max(a,b),c);

    if(Maximum == a)
        return max(b,c);
    if(Maximum == b)
        return max(a,c);
    if(Maximum == c)
        return max(a,b);

    return 0; /* ne doit jamais arriver jusqu'ici en principe*/

}
void AdressageBuffer(void)
{

    /**************************************************************************/
    /****************************** PML Faces *********************************/
    /**************************************************************************/

    Vi_BufferFaceBlocI=Buffer1AireMax;
    Tij_BufferFaceBlocI=Buffer1AireMax;
    Tki_BufferFaceBlocI=Buffer2AireMax;

    Vi_BufferFaceAreteJp=Buffer01AireIntermediaire;
    Vi_BufferFaceAreteJm=Buffer02AireIntermediaire;
    Vk_BufferFaceAreteJp=Buffer03AireIntermediaire;
    Vk_BufferFaceAreteJm=Buffer04AireIntermediaire;
    Vi_BufferFaceAreteKp=Buffer05AireIntermediaire;
    Vi_BufferFaceAreteKm=Buffer06AireIntermediaire;
    Vj_BufferFaceAreteKp=Buffer07AireIntermediaire;
    Vj_BufferFaceAreteKm=Buffer08AireIntermediaire;

    Tki_BufferFaceAreteJp=Buffer01AireIntermediaire;
    Tki_BufferFaceAreteJm=Buffer02AireIntermediaire;
    Tjj_BufferFaceAreteJp=Buffer03AireIntermediaire;
    Tjj_BufferFaceAreteJm=Buffer04AireIntermediaire;
    Tij_BufferFaceAreteKp=Buffer05AireIntermediaire;
    Tij_BufferFaceAreteKm=Buffer06AireIntermediaire;
    Tkk_BufferFaceAreteKp=Buffer07AireIntermediaire;
    Tkk_BufferFaceAreteKm=Buffer08AireIntermediaire;

    /**************************************************************************/
    /****************************** PML Aretes ********************************/
    /**************************************************************************/

    Vj_BufferAreteFaceI=Buffer01AireIntermediaire;
    Vi_BufferAreteFaceJ=Buffer02AireIntermediaire;
    Vi_BufferAreteCoinKp=Buffer03AireIntermediaire;
    Vi_BufferAreteCoinKm=Buffer04AireIntermediaire;
    Vj_BufferAreteCoinKp=Buffer05AireIntermediaire;
    Vj_BufferAreteCoinKm=Buffer06AireIntermediaire;

    Tjk_BufferAreteFaceI=Buffer01AireIntermediaire;
    Tij_BufferAreteFaceI=Buffer02AireIntermediaire;
    Tij_BufferAreteFaceJ=Buffer03AireIntermediaire;
    Tki_BufferAreteFaceJ=Buffer04AireIntermediaire;
    Tij_BufferAreteCoinKp=Buffer05AireIntermediaire;
    Tij_BufferAreteCoinKm=Buffer06AireIntermediaire;
    Tkk_BufferAreteCoinKp=Buffer07AireIntermediaire;
    Tkk_BufferAreteCoinKm=Buffer08AireIntermediaire;

    /**************************************************************************/
    /****************************** PML Aretes ********************************/
    /**************************************************************************/

    Vi_BufferCoinAreteI=Buffer01AireIntermediaire;
    Vj_BufferCoinAreteJ=Buffer02AireIntermediaire;
    Vk_BufferCoinAreteK=Buffer03AireIntermediaire;

    Tij_BufferCoinAreteI=Buffer01AireIntermediaire;
    Tki_BufferCoinAreteI=Buffer02AireIntermediaire;
    Tij_BufferCoinAreteJ=Buffer03AireIntermediaire;
    Tjk_BufferCoinAreteJ=Buffer04AireIntermediaire;
    Tjk_BufferCoinAreteK=Buffer05AireIntermediaire;
    Tki_BufferCoinAreteK=Buffer06AireIntermediaire;

}

void ReMiseAZeroTampons(void)
{

    int AireMax,AireIntermediaire,i;

    AireMax = (max(max(X,Y),Z)+1)*(DeuxiemeMax(X,Y,Z)+1);
    AireIntermediaire = (max(max(X,Y),Z)+1)*(W+1);
    AireMax = max(AireMax,(W+1)*(W+1));
    AireIntermediaire = max(AireIntermediaire,(W+1)*(W+1));

#pragma omp parallel for schedule(guided)
    for (i=0; i<AireMax; i++)
    {
        Carte_Tampon[i]=0;
        Buffer1AireMax[i]=0;
        Buffer2AireMax[i]=0;
    }
#pragma omp parallel for schedule(guided)
    for (i=0; i<AireIntermediaire; i++)
    {
        Buffer01AireIntermediaire[i]=0;
        Buffer02AireIntermediaire[i]=0;
        Buffer03AireIntermediaire[i]=0;
        Buffer04AireIntermediaire[i]=0;
        Buffer05AireIntermediaire[i]=0;
        Buffer06AireIntermediaire[i]=0;
        Buffer07AireIntermediaire[i]=0;
        Buffer08AireIntermediaire[i]=0;
    }
}

void MiseAJourContraintesTamponCoinPML(const char *TypeCoin)
{
    int i,j,k;

    if (!strcmp(TypeCoin,"XpYpZp"))
    {

        Carte_Tampon[0] = Carte[(X-1)*(Y)*(Z)+(Y-1)*(Z)+Z-1];

        /**************  Face Zm ************************************************/

#pragma omp parallel for private(j) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(j=0; j<(W); j++)
            {
                Tjk_BufferCoinAreteK[i*(W)+j] = Tyz_XpYp_Y[i*(W)*(Z+1)+j*(Z+1)+Z] +
                                                Tyz_XpYp_Z[i*(W)*(Z+1)+j*(Z+1)+Z];
            }
        }
#pragma omp parallel for private(j) schedule(guided)
        for(i=0; i<(W); i++)
        {
            for(j=0; j<(W+1); j++)
            {
                Tki_BufferCoinAreteK[i*(W+1)+j] = Tzx_XpYp_Z[i*(W+1)*(Z+1)+j*(Z+1)+Z] +
                                                  Tzx_XpYp_X[i*(W+1)*(Z+1)+j*(Z+1)+Z];
            }
        }
        /**************  Face Xm ************************************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(W); k++)
            {
                Tki_BufferCoinAreteI[j*(W)+k] = Tzx_YpZp_Z[j*(W)*(X+1)+k*(X+1)+X] +
                                                Tzx_YpZp_X[j*(W)*(X+1)+k*(X+1)+X];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W); j++)
        {
            for(k=0; k<(W+1); k++)
            {
                Tij_BufferCoinAreteI[j*(W+1)+k] = Txy_YpZp_X[j*(W+1)*(X+1)+k*(X+1)+X] +
                                                  Txy_YpZp_Y[j*(W+1)*(X+1)+k*(X+1)+X];
            }
        }
        /**************  Face Ym ************************************************/
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W); i++)
        {
            for(k=0; k<(W+1); k++)
            {
                Tij_BufferCoinAreteJ[i*(W+1)+k] = Txy_ZpXp_X[k*(W)*(Y+1)+i*(Y+1)+Y] +
                                                  Txy_ZpXp_Y[k*(W)*(Y+1)+i*(Y+1)+Y];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(W); k++)
            {
                Tjk_BufferCoinAreteJ[i*(W)+k] = Tyz_ZpXp_Y[k*(W+1)*(Y+1)+i*(Y+1)+Y] +
                                                Tyz_ZpXp_Z[k*(W+1)*(Y+1)+i*(Y+1)+Y];
            }
        }
    }
    if (!strcmp(TypeCoin,"XpYpZm"))
    {

        Carte_Tampon[0] = Carte[(X-1)*(Y)*(Z)+(Y-1)*(Z)+0];

        /**************  Face Zm ************************************************/

#pragma omp parallel for private(j) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(j=0; j<(W); j++)
            {
                Tjk_BufferCoinAreteK[i*(W)+j] = Tyz_XpYp_Y[i*(W)*(Z+1)+j*(Z+1)+0] +
                                                Tyz_XpYp_Z[i*(W)*(Z+1)+j*(Z+1)+0];
            }
        }
#pragma omp parallel for private(j) schedule(guided)
        for(i=0; i<(W); i++)
        {
            for(j=0; j<(W+1); j++)
            {
                Tki_BufferCoinAreteK[i*(W+1)+j] = Tzx_XpYp_Z[i*(W+1)*(Z+1)+j*(Z+1)+0] +
                                                  Tzx_XpYp_X[i*(W+1)*(Z+1)+j*(Z+1)+0];
            }
        }
        /**************  Face Xm ************************************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(W); k++)
            {
                Tki_BufferCoinAreteI[j*(W)+k] = Tzx_YpZm_Z[j*(W)*(X+1)+k*(X+1)+X] +
                                                Tzx_YpZm_X[j*(W)*(X+1)+k*(X+1)+X];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W); j++)
        {
            for(k=0; k<(W+1); k++)
            {
                Tij_BufferCoinAreteI[j*(W+1)+k] = Txy_YpZm_X[j*(W+1)*(X+1)+k*(X+1)+X] +
                                                  Txy_YpZm_Y[j*(W+1)*(X+1)+k*(X+1)+X];
            }
        }
        /**************  Face Ym ************************************************/
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W); i++)
        {
            for(k=0; k<(W+1); k++)
            {
                Tij_BufferCoinAreteJ[i*(W+1)+k] = Txy_ZmXp_X[k*(W)*(Y+1)+i*(Y+1)+Y] +
                                                  Txy_ZmXp_Y[k*(W)*(Y+1)+i*(Y+1)+Y];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(W); k++)
            {
                Tjk_BufferCoinAreteJ[i*(W)+k] = Tyz_ZmXp_Y[k*(W+1)*(Y+1)+i*(Y+1)+Y] +
                                                Tyz_ZmXp_Z[k*(W+1)*(Y+1)+i*(Y+1)+Y];
            }
        }
    }
    if (!strcmp(TypeCoin,"XmYpZp"))
    {

        Carte_Tampon[0] = Carte[(0)*(Y)*(Z)+(Y-1)*(Z)+Z-1];

        /**************  Face Zm ************************************************/

#pragma omp parallel for private(j) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(j=0; j<(W); j++)
            {
                Tjk_BufferCoinAreteK[i*(W)+j] = Tyz_XmYp_Y[i*(W)*(Z+1)+j*(Z+1)+Z] +
                                                Tyz_XmYp_Z[i*(W)*(Z+1)+j*(Z+1)+Z];
            }
        }
#pragma omp parallel for private(j) schedule(guided)
        for(i=0; i<(W); i++)
        {
            for(j=0; j<(W+1); j++)
            {
                Tki_BufferCoinAreteK[i*(W+1)+j] = Tzx_XmYp_Z[i*(W+1)*(Z+1)+j*(Z+1)+Z] +
                                                  Tzx_XmYp_X[i*(W+1)*(Z+1)+j*(Z+1)+Z];
            }
        }
        /**************  Face Xm ************************************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(W); k++)
            {
                Tki_BufferCoinAreteI[j*(W)+k] = Tzx_YpZp_Z[j*(W)*(X+1)+k*(X+1)+0] +
                                                Tzx_YpZp_X[j*(W)*(X+1)+k*(X+1)+0];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W); j++)
        {
            for(k=0; k<(W+1); k++)
            {
                Tij_BufferCoinAreteI[j*(W+1)+k] = Txy_YpZp_X[j*(W+1)*(X+1)+k*(X+1)+0] +
                                                  Txy_YpZp_Y[j*(W+1)*(X+1)+k*(X+1)+0];
            }
        }
        /**************  Face Ym ************************************************/
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W); i++)
        {
            for(k=0; k<(W+1); k++)
            {
                Tij_BufferCoinAreteJ[i*(W+1)+k] = Txy_ZpXm_X[k*(W)*(Y+1)+i*(Y+1)+Y] +
                                                  Txy_ZpXm_Y[k*(W)*(Y+1)+i*(Y+1)+Y];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(W); k++)
            {
                Tjk_BufferCoinAreteJ[i*(W)+k] = Tyz_ZpXm_Y[k*(W+1)*(Y+1)+i*(Y+1)+Y] +
                                                Tyz_ZpXm_Z[k*(W+1)*(Y+1)+i*(Y+1)+Y];
            }
        }
    }
    if (!strcmp(TypeCoin,"XmYpZm"))
    {

        Carte_Tampon[0] = Carte[(0)*(Y)*(Z)+(Y-1)*(Z)+0];

        /**************  Face Zm ************************************************/

#pragma omp parallel for private(j) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(j=0; j<(W); j++)
            {
                Tjk_BufferCoinAreteK[i*(W)+j] = Tyz_XmYp_Y[i*(W)*(Z+1)+j*(Z+1)+0] +
                                                Tyz_XmYp_Z[i*(W)*(Z+1)+j*(Z+1)+0];
            }
        }
#pragma omp parallel for private(j) schedule(guided)
        for(i=0; i<(W); i++)
        {
            for(j=0; j<(W+1); j++)
            {
                Tki_BufferCoinAreteK[i*(W+1)+j] = Tzx_XmYp_Z[i*(W+1)*(Z+1)+j*(Z+1)+0] +
                                                  Tzx_XmYp_X[i*(W+1)*(Z+1)+j*(Z+1)+0];
            }
        }
        /**************  Face Xm ************************************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(W); k++)
            {
                Tki_BufferCoinAreteI[j*(W)+k] = Tzx_YpZm_Z[j*(W)*(X+1)+k*(X+1)+0] +
                                                Tzx_YpZm_X[j*(W)*(X+1)+k*(X+1)+0];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W); j++)
        {
            for(k=0; k<(W+1); k++)
            {
                Tij_BufferCoinAreteI[j*(W+1)+k] = Txy_YpZm_X[j*(W+1)*(X+1)+k*(X+1)+0] +
                                                  Txy_YpZm_Y[j*(W+1)*(X+1)+k*(X+1)+0];
            }
        }
        /**************  Face Ym ************************************************/
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W); i++)
        {
            for(k=0; k<(W+1); k++)
            {
                Tij_BufferCoinAreteJ[i*(W+1)+k] = Txy_ZmXm_X[k*(W)*(Y+1)+i*(Y+1)+Y] +
                                                  Txy_ZmXm_Y[k*(W)*(Y+1)+i*(Y+1)+Y];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(W); k++)
            {
                Tjk_BufferCoinAreteJ[i*(W)+k] = Tyz_ZmXm_Y[k*(W+1)*(Y+1)+i*(Y+1)+Y] +
                                                Tyz_ZmXm_Z[k*(W+1)*(Y+1)+i*(Y+1)+Y];
            }
        }
    }
    if (!strcmp(TypeCoin,"XpYmZp"))
    {

        Carte_Tampon[0] = Carte[(X-1)*(Y)*(Z)+(0)*(Z)+Z-1];

        /**************  Face Zm ************************************************/

#pragma omp parallel for private(j) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(j=0; j<(W); j++)
            {
                Tjk_BufferCoinAreteK[i*(W)+j] = Tyz_XpYm_Y[i*(W)*(Z+1)+j*(Z+1)+Z] +
                                                Tyz_XpYm_Z[i*(W)*(Z+1)+j*(Z+1)+Z];
            }
        }
#pragma omp parallel for private(j) schedule(guided)
        for(i=0; i<(W); i++)
        {
            for(j=0; j<(W+1); j++)
            {
                Tki_BufferCoinAreteK[i*(W+1)+j] = Tzx_XpYm_Z[i*(W+1)*(Z+1)+j*(Z+1)+Z] +
                                                  Tzx_XpYm_X[i*(W+1)*(Z+1)+j*(Z+1)+Z];
            }
        }
        /**************  Face Xm ************************************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(W); k++)
            {
                Tki_BufferCoinAreteI[j*(W)+k] = Tzx_YmZp_Z[j*(W)*(X+1)+k*(X+1)+X] +
                                                Tzx_YmZp_X[j*(W)*(X+1)+k*(X+1)+X];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W); j++)
        {
            for(k=0; k<(W+1); k++)
            {
                Tij_BufferCoinAreteI[j*(W+1)+k] = Txy_YmZp_X[j*(W+1)*(X+1)+k*(X+1)+X] +
                                                  Txy_YmZp_Y[j*(W+1)*(X+1)+k*(X+1)+X];
            }
        }
        /**************  Face Ym ************************************************/
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W); i++)
        {
            for(k=0; k<(W+1); k++)
            {
                Tij_BufferCoinAreteJ[i*(W+1)+k] = Txy_ZpXp_X[k*(W)*(Y+1)+i*(Y+1)+0] +
                                                  Txy_ZpXp_Y[k*(W)*(Y+1)+i*(Y+1)+0];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(W); k++)
            {
                Tjk_BufferCoinAreteJ[i*(W)+k] = Tyz_ZpXp_Y[k*(W+1)*(Y+1)+i*(Y+1)+0] +
                                                Tyz_ZpXp_Z[k*(W+1)*(Y+1)+i*(Y+1)+0];
            }
        }
    }
    if (!strcmp(TypeCoin,"XpYmZm"))
    {

        Carte_Tampon[0] = Carte[(X-1)*(Y)*(Z)+(0)*(Z)+0];

        /**************  Face Zm ************************************************/

#pragma omp parallel for private(j) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(j=0; j<(W); j++)
            {
                Tjk_BufferCoinAreteK[i*(W)+j] = Tyz_XpYm_Y[i*(W)*(Z+1)+j*(Z+1)+0] +
                                                Tyz_XpYm_Z[i*(W)*(Z+1)+j*(Z+1)+0];
            }
        }
#pragma omp parallel for private(j) schedule(guided)
        for(i=0; i<(W); i++)
        {
            for(j=0; j<(W+1); j++)
            {
                Tki_BufferCoinAreteK[i*(W+1)+j] = Tzx_XpYm_Z[i*(W+1)*(Z+1)+j*(Z+1)+0] +
                                                  Tzx_XpYm_X[i*(W+1)*(Z+1)+j*(Z+1)+0];
            }
        }
        /**************  Face Xm ************************************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(W); k++)
            {
                Tki_BufferCoinAreteI[j*(W)+k] = Tzx_YmZm_Z[j*(W)*(X+1)+k*(X+1)+X] +
                                                Tzx_YmZm_X[j*(W)*(X+1)+k*(X+1)+X];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W); j++)
        {
            for(k=0; k<(W+1); k++)
            {
                Tij_BufferCoinAreteI[j*(W+1)+k] = Txy_YmZm_X[j*(W+1)*(X+1)+k*(X+1)+X] +
                                                  Txy_YmZm_Y[j*(W+1)*(X+1)+k*(X+1)+X];
            }
        }
        /**************  Face Ym ************************************************/
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W); i++)
        {
            for(k=0; k<(W+1); k++)
            {
                Tij_BufferCoinAreteJ[i*(W+1)+k] = Txy_ZmXp_X[k*(W)*(Y+1)+i*(Y+1)+0] +
                                                  Txy_ZmXp_Y[k*(W)*(Y+1)+i*(Y+1)+0];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(W); k++)
            {
                Tjk_BufferCoinAreteJ[i*(W)+k] = Tyz_ZmXp_Y[k*(W+1)*(Y+1)+i*(Y+1)+0] +
                                                Tyz_ZmXp_Z[k*(W+1)*(Y+1)+i*(Y+1)+0];
            }
        }
    }
    if (!strcmp(TypeCoin,"XmYmZp"))
    {

        Carte_Tampon[0] = Carte[(0)*(Y)*(Z)+(0)*(Z)+Z-1];

        /**************  Face Zm ************************************************/

#pragma omp parallel for private(j) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(j=0; j<(W); j++)
            {
                Tjk_BufferCoinAreteK[i*(W)+j] = Tyz_XmYm_Y[i*(W)*(Z+1)+j*(Z+1)+Z] +
                                                Tyz_XmYm_Z[i*(W)*(Z+1)+j*(Z+1)+Z];
            }
        }
#pragma omp parallel for private(j) schedule(guided)
        for(i=0; i<(W); i++)
        {
            for(j=0; j<(W+1); j++)
            {
                Tki_BufferCoinAreteK[i*(W+1)+j] = Tzx_XmYm_Z[i*(W+1)*(Z+1)+j*(Z+1)+Z] +
                                                  Tzx_XmYm_X[i*(W+1)*(Z+1)+j*(Z+1)+Z];
            }
        }
        /**************  Face Xm ************************************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(W); k++)
            {
                Tki_BufferCoinAreteI[j*(W)+k] = Tzx_YmZp_Z[j*(W)*(X+1)+k*(X+1)+0] +
                                                Tzx_YmZp_X[j*(W)*(X+1)+k*(X+1)+0];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W); j++)
        {
            for(k=0; k<(W+1); k++)
            {
                Tij_BufferCoinAreteI[j*(W+1)+k] = Txy_YmZp_X[j*(W+1)*(X+1)+k*(X+1)+0] +
                                                  Txy_YmZp_Y[j*(W+1)*(X+1)+k*(X+1)+0];
            }
        }
        /**************  Face Ym ************************************************/
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W); i++)
        {
            for(k=0; k<(W+1); k++)
            {
                Tij_BufferCoinAreteJ[i*(W+1)+k] = Txy_ZpXm_X[k*(W)*(Y+1)+i*(Y+1)+0] +
                                                  Txy_ZpXm_Y[k*(W)*(Y+1)+i*(Y+1)+0];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(W); k++)
            {
                Tjk_BufferCoinAreteJ[i*(W)+k] = Tyz_ZpXm_Y[k*(W+1)*(Y+1)+i*(Y+1)+0] +
                                                Tyz_ZpXm_Z[k*(W+1)*(Y+1)+i*(Y+1)+0];
            }
        }
    }
    if (!strcmp(TypeCoin,"XmYmZm"))
    {

        Carte_Tampon[0] = Carte[(0)*(Y)*(Z)+(0)*(Z)+0];

        /**************  Face Zm ************************************************/

#pragma omp parallel for private(j) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(j=0; j<(W); j++)
            {
                Tjk_BufferCoinAreteK[i*(W)+j] = Tyz_XmYm_Y[i*(W)*(Z+1)+j*(Z+1)+0] +
                                                Tyz_XmYm_Z[i*(W)*(Z+1)+j*(Z+1)+0];
            }
        }
#pragma omp parallel for private(j) schedule(guided)
        for(i=0; i<(W); i++)
        {
            for(j=0; j<(W+1); j++)
            {
                Tki_BufferCoinAreteK[i*(W+1)+j] = Tzx_XmYm_Z[i*(W+1)*(Z+1)+j*(Z+1)+0] +
                                                  Tzx_XmYm_X[i*(W+1)*(Z+1)+j*(Z+1)+0];
            }
        }
        /**************  Face Xm ************************************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(W); k++)
            {
                Tki_BufferCoinAreteI[j*(W)+k] = Tzx_YmZm_Z[j*(W)*(X+1)+k*(X+1)+0] +
                                                Tzx_YmZm_X[j*(W)*(X+1)+k*(X+1)+0];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W); j++)
        {
            for(k=0; k<(W+1); k++)
            {
                Tij_BufferCoinAreteI[j*(W+1)+k] = Txy_YmZm_X[j*(W+1)*(X+1)+k*(X+1)+0] +
                                                  Txy_YmZm_Y[j*(W+1)*(X+1)+k*(X+1)+0];
            }
        }
        /**************  Face Ym ************************************************/
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W); i++)
        {
            for(k=0; k<(W+1); k++)
            {
                Tij_BufferCoinAreteJ[i*(W+1)+k] = Txy_ZmXm_X[k*(W)*(Y+1)+i*(Y+1)+0] +
                                                  Txy_ZmXm_Y[k*(W)*(Y+1)+i*(Y+1)+0];
            }
        }
#pragma omp parallel for private(k) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(k=0; k<(W); k++)
            {
                Tjk_BufferCoinAreteJ[i*(W)+k] = Tyz_ZmXm_Y[k*(W+1)*(Y+1)+i*(Y+1)+0] +
                                                Tyz_ZmXm_Z[k*(W+1)*(Y+1)+i*(Y+1)+0];
            }
        }
    }
}
void MiseAJourVitessesTamponCoinPML(const char *TypeCoin)
{
    int i,j,k;

    TableCii = TableC11;
    TableCjj = TableC22;
    TableCkk = TableC33;
    TableCij = TableC12;
    TableCjk = TableC23;
    TableCki = TableC31;
    TableCjkjk = TableC44;
    TableCkiki = TableC55;
    TableCijij = TableC66;

    if (!strcmp(TypeCoin,"XpYpZp"))
    {

        Carte_Tampon[0] = Carte[(X-1)*(Y)*(Z)+(Y-1)*(Z)+Z-1];

        /**************  Face Zm ************************************************/

#pragma omp parallel for private(j) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(j=0; j<(W+1); j++)
            {
                Vk_BufferCoinAreteK[i*(W+1)+j] =	Vz_XpYp_X[i*(W+1)*(Z+1)+j*(Z+1)+Z] +
                                                    Vz_XpYp_Y[i*(W+1)*(Z+1)+j*(Z+1)+Z] +
                                                    Vz_XpYp_Z[i*(W+1)*(Z+1)+j*(Z+1)+Z];
            }
        }
        /**************  Face Xm ************************************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(W+1); k++)
            {
                Vi_BufferCoinAreteI[j*(W+1)+k] =	Vx_YpZp_Y[j*(W+1)*(X+1)+k*(X+1)+X] +
                                                    Vx_YpZp_Z[j*(W+1)*(X+1)+k*(X+1)+X] +
                                                    Vx_YpZp_X[j*(W+1)*(X+1)+k*(X+1)+X];
            }
        }
        /**************  Face Ym ************************************************/
#pragma omp parallel for private(i) schedule(guided)
        for(k=0; k<(W+1); k++)
        {
            for(i=0; i<(W+1); i++)
            {
                Vj_BufferCoinAreteJ[k*(W+1)+i] =	Vy_ZpXp_Z[k*(W+1)*(Y+1)+i*(Y+1)+Y] +
                                                    Vy_ZpXp_X[k*(W+1)*(Y+1)+i*(Y+1)+Y] +
                                                    Vy_ZpXp_Y[k*(W+1)*(Y+1)+i*(Y+1)+Y];
            }
        }
    }
    if (!strcmp(TypeCoin,"XpYpZm"))
    {

        Carte_Tampon[0] = Carte[(X-1)*(Y)*(Z)+(Y-1)*(Z)+0];

        /**************  Face Zm ************************************************/

#pragma omp parallel for private(j) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(j=0; j<(W+1); j++)
            {
                Vk_BufferCoinAreteK[i*(W+1)+j] =	Vz_XpYp_X[i*(W+1)*(Z+1)+j*(Z+1)+0] +
                                                    Vz_XpYp_Y[i*(W+1)*(Z+1)+j*(Z+1)+0] +
                                                    Vz_XpYp_Z[i*(W+1)*(Z+1)+j*(Z+1)+0];
            }
        }
        /**************  Face Xm ************************************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(W+1); k++)
            {
                Vi_BufferCoinAreteI[j*(W+1)+k] =	Vx_YpZm_Y[j*(W+1)*(X+1)+k*(X+1)+X] +
                                                    Vx_YpZm_Z[j*(W+1)*(X+1)+k*(X+1)+X] +
                                                    Vx_YpZm_X[j*(W+1)*(X+1)+k*(X+1)+X];
            }
        }
        /**************  Face Ym ************************************************/
#pragma omp parallel for private(i) schedule(guided)
        for(k=0; k<(W+1); k++)
        {
            for(i=0; i<(W+1); i++)
            {
                Vj_BufferCoinAreteJ[k*(W+1)+i] =	Vy_ZmXp_Z[k*(W+1)*(Y+1)+i*(Y+1)+Y] +
                                                    Vy_ZmXp_X[k*(W+1)*(Y+1)+i*(Y+1)+Y] +
                                                    Vy_ZmXp_Y[k*(W+1)*(Y+1)+i*(Y+1)+Y];
            }
        }
    }
    if (!strcmp(TypeCoin,"XmYpZp"))
    {

        Carte_Tampon[0] = Carte[(0)*(Y)*(Z)+(Y-1)*(Z)+Z-1];

        /**************  Face Zm ************************************************/

#pragma omp parallel for private(j) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(j=0; j<(W+1); j++)
            {
                Vk_BufferCoinAreteK[i*(W+1)+j] =	Vz_XmYp_X[i*(W+1)*(Z+1)+j*(Z+1)+Z] +
                                                    Vz_XmYp_Y[i*(W+1)*(Z+1)+j*(Z+1)+Z] +
                                                    Vz_XmYp_Z[i*(W+1)*(Z+1)+j*(Z+1)+Z];
            }
        }
        /**************  Face Xm ************************************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(W+1); k++)
            {
                Vi_BufferCoinAreteI[j*(W+1)+k] =	Vx_YpZp_Y[j*(W+1)*(X+1)+k*(X+1)+0] +
                                                    Vx_YpZp_Z[j*(W+1)*(X+1)+k*(X+1)+0] +
                                                    Vx_YpZp_X[j*(W+1)*(X+1)+k*(X+1)+0];
            }
        }
        /**************  Face Ym ************************************************/
#pragma omp parallel for private(i) schedule(guided)
        for(k=0; k<(W+1); k++)
        {
            for(i=0; i<(W+1); i++)
            {
                Vj_BufferCoinAreteJ[k*(W+1)+i] =	Vy_ZpXm_Z[k*(W+1)*(Y+1)+i*(Y+1)+Y] +
                                                    Vy_ZpXm_X[k*(W+1)*(Y+1)+i*(Y+1)+Y] +
                                                    Vy_ZpXm_Y[k*(W+1)*(Y+1)+i*(Y+1)+Y];
            }
        }
    }
    if (!strcmp(TypeCoin,"XmYpZm"))
    {

        Carte_Tampon[0] = Carte[(0)*(Y)*(Z)+(Y-1)*(Z)+0];

        /**************  Face Zm ************************************************/

#pragma omp parallel for private(j) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(j=0; j<(W+1); j++)
            {
                Vk_BufferCoinAreteK[i*(W+1)+j] =	Vz_XmYp_X[i*(W+1)*(Z+1)+j*(Z+1)+0] +
                                                    Vz_XmYp_Y[i*(W+1)*(Z+1)+j*(Z+1)+0] +
                                                    Vz_XmYp_Z[i*(W+1)*(Z+1)+j*(Z+1)+0];
            }
        }
        /**************  Face Xm ************************************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(W+1); k++)
            {
                Vi_BufferCoinAreteI[j*(W+1)+k] =	Vx_YpZm_Y[j*(W+1)*(X+1)+k*(X+1)+0] +
                                                    Vx_YpZm_Z[j*(W+1)*(X+1)+k*(X+1)+0] +
                                                    Vx_YpZm_X[j*(W+1)*(X+1)+k*(X+1)+0];
            }
        }
        /**************  Face Ym ************************************************/
#pragma omp parallel for private(i) schedule(guided)
        for(k=0; k<(W+1); k++)
        {
            for(i=0; i<(W+1); i++)
            {
                Vj_BufferCoinAreteJ[k*(W+1)+i] =	Vy_ZmXm_Z[k*(W+1)*(Y+1)+i*(Y+1)+Y] +
                                                    Vy_ZmXm_X[k*(W+1)*(Y+1)+i*(Y+1)+Y] +
                                                    Vy_ZmXm_Y[k*(W+1)*(Y+1)+i*(Y+1)+Y];
            }
        }
    }
    if (!strcmp(TypeCoin,"XpYmZp"))
    {

        Carte_Tampon[0] = Carte[(X-1)*(Y)*(Z)+(0)*(Z)+Z-1];

        /**************  Face Zm ************************************************/

#pragma omp parallel for private(j) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(j=0; j<(W+1); j++)
            {
                Vk_BufferCoinAreteK[i*(W+1)+j] =	Vz_XpYm_X[i*(W+1)*(Z+1)+j*(Z+1)+Z] +
                                                    Vz_XpYm_Y[i*(W+1)*(Z+1)+j*(Z+1)+Z] +
                                                    Vz_XpYm_Z[i*(W+1)*(Z+1)+j*(Z+1)+Z];
            }
        }
        /**************  Face Xm ************************************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(W+1); k++)
            {
                Vi_BufferCoinAreteI[j*(W+1)+k] =	Vx_YmZp_Y[j*(W+1)*(X+1)+k*(X+1)+X] +
                                                    Vx_YmZp_Z[j*(W+1)*(X+1)+k*(X+1)+X] +
                                                    Vx_YmZp_X[j*(W+1)*(X+1)+k*(X+1)+X];
            }
        }
        /**************  Face Ym ************************************************/
#pragma omp parallel for private(i) schedule(guided)
        for(k=0; k<(W+1); k++)
        {
            for(i=0; i<(W+1); i++)
            {
                Vj_BufferCoinAreteJ[k*(W+1)+i] =	Vy_ZpXp_Z[k*(W+1)*(Y+1)+i*(Y+1)+0] +
                                                    Vy_ZpXp_X[k*(W+1)*(Y+1)+i*(Y+1)+0] +
                                                    Vy_ZpXp_Y[k*(W+1)*(Y+1)+i*(Y+1)+0];
            }
        }
    }
    if (!strcmp(TypeCoin,"XpYmZm"))
    {

        Carte_Tampon[0] = Carte[(X-1)*(Y)*(Z)+(0)*(Z)+0];

        /**************  Face Zm ************************************************/

#pragma omp parallel for private(j) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(j=0; j<(W+1); j++)
            {
                Vk_BufferCoinAreteK[i*(W+1)+j] =	Vz_XpYm_X[i*(W+1)*(Z+1)+j*(Z+1)+0] +
                                                    Vz_XpYm_Y[i*(W+1)*(Z+1)+j*(Z+1)+0] +
                                                    Vz_XpYm_Z[i*(W+1)*(Z+1)+j*(Z+1)+0];
            }
        }
        /**************  Face Xm ************************************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(W+1); k++)
            {
                Vi_BufferCoinAreteI[j*(W+1)+k] =	Vx_YmZm_Y[j*(W+1)*(X+1)+k*(X+1)+X] +
                                                    Vx_YmZm_Z[j*(W+1)*(X+1)+k*(X+1)+X] +
                                                    Vx_YmZm_X[j*(W+1)*(X+1)+k*(X+1)+X];
            }
        }
        /**************  Face Ym ************************************************/
#pragma omp parallel for private(i) schedule(guided)
        for(k=0; k<(W+1); k++)
        {
            for(i=0; i<(W+1); i++)
            {
                Vj_BufferCoinAreteJ[k*(W+1)+i] =	Vy_ZmXp_Z[k*(W+1)*(Y+1)+i*(Y+1)+0] +
                                                    Vy_ZmXp_X[k*(W+1)*(Y+1)+i*(Y+1)+0] +
                                                    Vy_ZmXp_Y[k*(W+1)*(Y+1)+i*(Y+1)+0];
            }
        }
    }
    if (!strcmp(TypeCoin,"XmYmZp"))
    {

        Carte_Tampon[0] = Carte[(0)*(Y)*(Z)+(0)*(Z)+Z-1];

        /**************  Face Zm ************************************************/

#pragma omp parallel for private(j) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(j=0; j<(W+1); j++)
            {
                Vk_BufferCoinAreteK[i*(W+1)+j] =	Vz_XmYm_X[i*(W+1)*(Z+1)+j*(Z+1)+Z] +
                                                    Vz_XmYm_Y[i*(W+1)*(Z+1)+j*(Z+1)+Z] +
                                                    Vz_XmYm_Z[i*(W+1)*(Z+1)+j*(Z+1)+Z];
            }
        }
        /**************  Face Xm ************************************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(W+1); k++)
            {
                Vi_BufferCoinAreteI[j*(W+1)+k] =	Vx_YmZp_Y[j*(W+1)*(X+1)+k*(X+1)+0] +
                                                    Vx_YmZp_Z[j*(W+1)*(X+1)+k*(X+1)+0] +
                                                    Vx_YmZp_X[j*(W+1)*(X+1)+k*(X+1)+0];
            }
        }
        /**************  Face Ym ************************************************/
#pragma omp parallel for private(i) schedule(guided)
        for(k=0; k<(W+1); k++)
        {
            for(i=0; i<(W+1); i++)
            {
                Vj_BufferCoinAreteJ[k*(W+1)+i] =	Vy_ZpXm_Z[k*(W+1)*(Y+1)+i*(Y+1)+0] +
                                                    Vy_ZpXm_X[k*(W+1)*(Y+1)+i*(Y+1)+0] +
                                                    Vy_ZpXm_Y[k*(W+1)*(Y+1)+i*(Y+1)+0];
            }
        }
    }
    if (!strcmp(TypeCoin,"XmYmZm"))
    {

        Carte_Tampon[0] = Carte[(0)*(Y)*(Z)+(0)*(Z)+0];

        /**************  Face Zm ************************************************/

#pragma omp parallel for private(j) schedule(guided)
        for(i=0; i<(W+1); i++)
        {
            for(j=0; j<(W+1); j++)
            {
                Vk_BufferCoinAreteK[i*(W+1)+j] =	Vz_XmYm_X[i*(W+1)*(Z+1)+j*(Z+1)+0] +
                                                    Vz_XmYm_Y[i*(W+1)*(Z+1)+j*(Z+1)+0] +
                                                    Vz_XmYm_Z[i*(W+1)*(Z+1)+j*(Z+1)+0];
            }
        }
        /**************  Face Xm ************************************************/
#pragma omp parallel for private(k) schedule(guided)
        for(j=0; j<(W+1); j++)
        {
            for(k=0; k<(W+1); k++)
            {
                Vi_BufferCoinAreteI[j*(W+1)+k] =	Vx_YmZm_Y[j*(W+1)*(X+1)+k*(X+1)+0] +
                                                    Vx_YmZm_Z[j*(W+1)*(X+1)+k*(X+1)+0] +
                                                    Vx_YmZm_X[j*(W+1)*(X+1)+k*(X+1)+0];
            }
        }
        /**************  Face Ym ************************************************/
#pragma omp parallel for private(i) schedule(guided)
        for(k=0; k<(W+1); k++)
        {
            for(i=0; i<(W+1); i++)
            {
                Vj_BufferCoinAreteJ[k*(W+1)+i] =	Vy_ZmXm_Z[k*(W+1)*(Y+1)+i*(Y+1)+0] +
                                                    Vy_ZmXm_X[k*(W+1)*(Y+1)+i*(Y+1)+0] +
                                                    Vy_ZmXm_Y[k*(W+1)*(Y+1)+i*(Y+1)+0];
            }
        }
    }
}


void MAJMatricesReception(void)
{

    int NumeroMatrice;

    for (NumeroMatrice=0; NumeroMatrice<NbMatricesReceptionTxx; NumeroMatrice++)
    {
        EnregistrementMatriceReception(Txx,&ReceptionTxx[NumeroMatrice],X,Y,Z);
    }

    for (NumeroMatrice=0; NumeroMatrice<NbMatricesReceptionTyy; NumeroMatrice++)
    {
        EnregistrementMatriceReception(Tyy,&ReceptionTyy[NumeroMatrice],X,Y,Z);
    }

    for (NumeroMatrice=0; NumeroMatrice<NbMatricesReceptionTzz; NumeroMatrice++)
    {
        EnregistrementMatriceReception(Tzz,&ReceptionTzz[NumeroMatrice],X,Y,Z);
    }

    for (NumeroMatrice=0; NumeroMatrice<NbMatricesReceptionTxy; NumeroMatrice++)
    {
        EnregistrementMatriceReception(Txy,&ReceptionTxy[NumeroMatrice],X+1,Y+1,Z);
    }

    for (NumeroMatrice=0; NumeroMatrice<NbMatricesReceptionTyz; NumeroMatrice++)
    {
        EnregistrementMatriceReception(Tyz,&ReceptionTyz[NumeroMatrice],X,Y+1,Z+1);
    }

    for (NumeroMatrice=0; NumeroMatrice<NbMatricesReceptionTzx; NumeroMatrice++)
    {
        EnregistrementMatriceReception(Tzx,&ReceptionTzx[NumeroMatrice],X+1,Y,Z+1);
    }

    for (NumeroMatrice=0; NumeroMatrice<NbMatricesReceptionVx; NumeroMatrice++)
    {
        EnregistrementMatriceReception(Vx,&ReceptionVx[NumeroMatrice],X+1,Y,Z);
    }

    for (NumeroMatrice=0; NumeroMatrice<NbMatricesReceptionVy; NumeroMatrice++)
    {
        EnregistrementMatriceReception(Vy,&ReceptionVy[NumeroMatrice],X,Y+1,Z);
    }

    for (NumeroMatrice=0; NumeroMatrice<NbMatricesReceptionVz; NumeroMatrice++)
    {
        EnregistrementMatriceReception(Vz,&ReceptionVz[NumeroMatrice],X,Y,Z+1);
    }
}

void EnregistrementMatriceReception(PREC_GRILLE *GrilleCourante,
                                    struct MATRICE_RECEPTION *MatriceCourante,int DimX,int DimY,int DimZ)
{

    int NJ,NK,j,k;

    int IndiceTempsMaxRcv=1+((int)((IndiceTempsMax-1)/RcvDwnSmplng));
    int	IndiceTempsRcv=IndiceTemps/RcvDwnSmplng;

    int count=0;


    switch (MatriceCourante->PlanI)
    {
    case 'X':
    case '1' :

        for(NJ=0; NJ<MatriceCourante->NbElementsJ; NJ++)
        {
            for(NK=0; NK<MatriceCourante->NbElementsK; NK++)
            {
                for(j=0; j<MatriceCourante->LargeurJ; j++)
                {
                    for(k=0; k<MatriceCourante->LargeurK; k++)
                    {
                        MatriceCourante->Signaux[(NJ*MatriceCourante->NbElementsK+NK)*IndiceTempsMaxRcv+IndiceTempsRcv]+=
                            GrilleCourante[(MatriceCourante->XMat)*DimY*DimZ+
                                           (MatriceCourante->YMat+NJ*MatriceCourante->PitchJ+j)*DimZ+
                                           (MatriceCourante->ZMat+NK*MatriceCourante->PitchK+k)];
                    }
                }
            }
        }
        break;
    case 'Y':
    case '2' :
        for(NJ=0; NJ<MatriceCourante->NbElementsJ; NJ++)
        {
            for(NK=0; NK<MatriceCourante->NbElementsK; NK++)
            {
                for(j=0; j<MatriceCourante->LargeurJ; j++)
                {
                    for(k=0; k<MatriceCourante->LargeurK; k++)
                    {
                        MatriceCourante->Signaux[(NJ*MatriceCourante->NbElementsK+NK)*IndiceTempsMaxRcv+IndiceTempsRcv]+=
                            GrilleCourante[(MatriceCourante->XMat+NK*MatriceCourante->PitchK+k)*DimY*DimZ+
                                           (MatriceCourante->YMat)*DimZ+
                                           (MatriceCourante->ZMat+NJ*MatriceCourante->PitchJ+j)];
                    }
                }
            }
        }
        break;
    case 'Z':
    case '3' :

        for(NJ=0; NJ<MatriceCourante->NbElementsJ; NJ++)
        {
            for(NK=0; NK<MatriceCourante->NbElementsK; NK++)
            {
                for(j=0; j<MatriceCourante->LargeurJ; j++)
                {
                    for(k=0; k<MatriceCourante->LargeurK; k++)
                    {
                        MatriceCourante->Signaux[(NJ*MatriceCourante->NbElementsK+NK)*IndiceTempsMaxRcv+IndiceTempsRcv]+=
                            GrilleCourante[(MatriceCourante->XMat+NJ*MatriceCourante->PitchJ+j)*DimY*DimZ+
                                           (MatriceCourante->YMat+NK*MatriceCourante->PitchK+k)*DimZ+
                                           (MatriceCourante->ZMat)];
                    }
                }
            }
        }
        break;
    case 'A':

        for(NJ=0; NJ<MatriceCourante->NbElementsJ; NJ++)
        {
            for(NK=0; NK<MatriceCourante->NbElementsK; NK++)
            {
                for(j=0; j<MatriceCourante->LargeurJ; j++)
                {
                    for(k=0; k<MatriceCourante->LargeurK; k++)
                    {
                        MatriceCourante->Signaux[(NJ*MatriceCourante->NbElementsK+NK)*IndiceTempsMaxRcv+IndiceTempsRcv]+=
                            GrilleCourante[(MatriceCourante->XMat+NJ*MatriceCourante->PitchJ)*DimY*DimZ+
                                           (MatriceCourante->YMat+j)*DimZ+
                                           (MatriceCourante->ZMat+k)];
                    }
                }
            }
        }
        break;
    case 'B':

        for(NJ=0; NJ<MatriceCourante->NbElementsJ; NJ++)
        {
            for(NK=0; NK<MatriceCourante->NbElementsK; NK++)
            {
                for(j=0; j<MatriceCourante->LargeurJ; j++)
                {
                    for(k=0; k<MatriceCourante->LargeurK; k++)
                    {
                        MatriceCourante->Signaux[(NJ*MatriceCourante->NbElementsK+NK)*IndiceTempsMaxRcv+IndiceTempsRcv]+=
                            GrilleCourante[(MatriceCourante->XMat+k)*DimY*DimZ+
                                           (MatriceCourante->YMat+NJ*MatriceCourante->PitchJ)*DimZ+
                                           (MatriceCourante->ZMat+j)];
                    }
                }
            }
        }
        break;
    case 'C':

        for(NJ=0; NJ<MatriceCourante->NbElementsJ; NJ++)
        {
            for(NK=0; NK<MatriceCourante->NbElementsK; NK++)
            {
                for(j=0; j<MatriceCourante->LargeurJ; j++)
                {
                    for(k=0; k<MatriceCourante->LargeurK; k++)
                    {

                        MatriceCourante->Signaux[(NJ*MatriceCourante->NbElementsK+NK)*IndiceTempsMaxRcv+IndiceTempsRcv]+=
                            GrilleCourante[(MatriceCourante->XMat+j)*DimY*DimZ+
                                           (MatriceCourante->YMat+k)*DimZ+
                                           (MatriceCourante->ZMat+NJ*MatriceCourante->PitchJ)];
                    }
                }
            }
        }
        break;
    case 'U':

        for(NJ=0; NJ<MatriceCourante->NbElementsJ; NJ++)
        {
            for(NK=0; NK<MatriceCourante->NbElementsK; NK++)
            {

                count=0;

                for(j=0; j<MatriceCourante->LargeurJ; j++)
                {
                    for(k=0; k<MatriceCourante->LargeurK; k++)
                    {

                        if (Carte[(MatriceCourante->XMat+NJ*MatriceCourante->PitchJ)*Y*Z+
                                  (MatriceCourante->YMat+j)*Z+
                                  (MatriceCourante->ZMat+k)]==(MatriceCourante->ActiveMedium))
                        {

                            count+=1;
                            MatriceCourante->Signaux[(NJ*MatriceCourante->NbElementsK+NK)*IndiceTempsMaxRcv+IndiceTempsRcv]+=
                                GrilleCourante[(MatriceCourante->XMat+NJ*MatriceCourante->PitchJ)*DimY*DimZ+
                                               (MatriceCourante->YMat+j)*DimZ+
                                               (MatriceCourante->ZMat+k)];
                        }
                    }
                }
                if(count>0)
                {
                    MatriceCourante->Signaux[(NJ*MatriceCourante->NbElementsK+NK)*IndiceTempsMaxRcv+IndiceTempsRcv]/=(double)count;
                }

            }
        }

        break;
    case 'V':

        for(NJ=0; NJ<MatriceCourante->NbElementsJ; NJ++)
        {
            for(NK=0; NK<MatriceCourante->NbElementsK; NK++)
            {
                count=0;

                for(j=0; j<MatriceCourante->LargeurJ; j++)
                {
                    for(k=0; k<MatriceCourante->LargeurK; k++)
                    {
                        if (Carte[(MatriceCourante->XMat+k)*Y*Z+
                                  (MatriceCourante->YMat+NJ*MatriceCourante->PitchJ)*Z+
                                  (MatriceCourante->ZMat+j)]==(MatriceCourante->ActiveMedium))
                        {

                            count+=1;
                            MatriceCourante->Signaux[(NJ*MatriceCourante->NbElementsK+NK)*IndiceTempsMaxRcv+IndiceTempsRcv]+=
                                GrilleCourante[(MatriceCourante->XMat+k)*DimY*DimZ+
                                               (MatriceCourante->YMat+NJ*MatriceCourante->PitchJ)*DimZ+
                                               (MatriceCourante->ZMat+j)];
                        }
                    }
                }
                if(count>0)
                {
                    MatriceCourante->Signaux[(NJ*MatriceCourante->NbElementsK+NK)*IndiceTempsMaxRcv+IndiceTempsRcv]/=(double)count;
                }
            }
        }
        break;
    case 'W':

        for(NJ=0; NJ<MatriceCourante->NbElementsJ; NJ++)
        {
            for(NK=0; NK<MatriceCourante->NbElementsK; NK++)
            {
                count=0;

                for(j=0; j<MatriceCourante->LargeurJ; j++)
                {
                    for(k=0; k<MatriceCourante->LargeurK; k++)
                    {
                        if (Carte[(MatriceCourante->XMat+j)*Y*Z+
                                  (MatriceCourante->YMat+k)*Z+
                                  (MatriceCourante->ZMat+NJ*MatriceCourante->PitchJ)]==(MatriceCourante->ActiveMedium))
                        {
                            count+=1;
                            MatriceCourante->Signaux[(NJ*MatriceCourante->NbElementsK+NK)*IndiceTempsMaxRcv+IndiceTempsRcv]+=
                                GrilleCourante[(MatriceCourante->XMat+j)*DimY*DimZ+
                                               (MatriceCourante->YMat+k)*DimZ+
                                               (MatriceCourante->ZMat+NJ*MatriceCourante->PitchJ)];
                        }
                    }
                }
                if(count>0)
                {
                    MatriceCourante->Signaux[(NJ*MatriceCourante->NbElementsK+NK)*IndiceTempsMaxRcv+IndiceTempsRcv]/=(double)count;
                }
            }
        }
        break;
    }
}

void ActivationMatriceSourceFile(PREC_GRILLE *GrilleCourante, struct MATRICE_RECEPTION *MatriceCourante,int DimX,int DimY,int DimZ)
{
    /* DimX useless, just here for better readability of the arguments... */

    int NbElementsJ  =MatriceCourante->NbElementsJ;
    int NbElementsK  =MatriceCourante->NbElementsK;
    char PlanI      =MatriceCourante->PlanI;
    int XMat        =MatriceCourante->XMat;
    int YMat        =MatriceCourante->YMat;
    int ZMat        =MatriceCourante->ZMat;
    int PitchJ      =MatriceCourante->PitchJ;
    int PitchK      =MatriceCourante->PitchK;
    int LargeurJ     =MatriceCourante->LargeurJ;
    int LargeurK     =MatriceCourante->LargeurK;
    double* Signaux =MatriceCourante->Signaux;
    int DurationPt  =MatriceCourante->DurationPt;

    int NJ,NK,j,k;

    if (IndiceTemps<DurationPt)
    {
        for(NJ=0; NJ<MatriceCourante->NbElementsJ; NJ++)
        {
            for(NK=0; NK<MatriceCourante->NbElementsK; NK++)
            {
                for(j=0; j<MatriceCourante->LargeurJ; j++)
                {
                    for(k=0; k<MatriceCourante->LargeurK; k++)
                    {

                        switch (MatriceCourante->PlanI)
                        {
                        case 'X':
                        case '1' :
                            if(SourceType==1)
                                GrilleCourante[(XMat)*DimY*DimZ+
                                               (YMat+NJ*PitchJ+j)*DimZ+
                                               (ZMat+NK*PitchK+k)]+=
                                                   Signaux[(NJ*NbElementsK+NK)*DurationPt+IndiceTemps];

                            else if(SourceType==2)
                                GrilleCourante[(XMat)*DimY*DimZ+
                                               (YMat+NJ*PitchJ+j)*DimZ+
                                               (ZMat+NK*PitchK+k)]=
                                                   Signaux[(NJ*NbElementsK+NK)*DurationPt+IndiceTemps];
                            break;
                        case 'Y':
                        case '2' :
                            if(SourceType==1)
                                GrilleCourante[(XMat+NK*PitchK+k)*DimY*DimZ+
                                               (YMat)*DimZ+
                                               (ZMat+NJ*PitchJ+j)]+=
                                                   Signaux[(NJ*NbElementsK+NK)*DurationPt+IndiceTemps];

                            else if(SourceType==2)
                                GrilleCourante[(XMat+NK*PitchK+k)*DimY*DimZ+
                                               (YMat)*DimZ+
                                               (ZMat+NJ*PitchJ+j)]=
                                                   Signaux[(NJ*NbElementsK+NK)*DurationPt+IndiceTemps];
                            break;
                        case 'Z':
                        case '3' :
                            if(SourceType==1)
                                GrilleCourante[(XMat+NJ*PitchJ+j)*DimY*DimZ+
                                               (YMat+NK*PitchK+k)*DimZ+
                                               (ZMat)]+=
                                                   Signaux[(NJ*NbElementsK+NK)*DurationPt+IndiceTemps];

                            else if(SourceType==2)
                                GrilleCourante[(XMat+NJ*PitchJ+j)*DimY*DimZ+
                                               (YMat+NK*PitchK+k)*DimZ+
                                               (ZMat)]=
                                                   Signaux[(NJ*NbElementsK+NK)*DurationPt+IndiceTemps];
                            break;
                        }
                    }
                }
            }
        }
    }
}

void ActivationMatriceEmission(PREC_GRILLE *GrilleCourante,
                               struct MATRICE_EMISSION *MatriceCourante,int DimX,int DimY,int DimZ )
{

    double RetardMaxFocalisationJ=0,RetardMaxDeviationJ=0,RetardMaxFocalisationK=0,RetardMaxDeviationK=0;
    int NJ,NK,j,k;
    int Retard=0;
    double *CoeffApodisationJ,*CoeffApodisationK;

    /*simplement pour faciliter la relecture du code...*/
    int	NbElementsJ=MatriceCourante->NbElementsJ;
    int	NbElementsK=MatriceCourante->NbElementsK;
    char PlanI=MatriceCourante->PlanI;
    int	XMat=MatriceCourante->XMat;
    int YMat=MatriceCourante->YMat;
    int	ZMat=MatriceCourante->ZMat;
    int PitchJ=MatriceCourante->PitchJ;
    int LargeurJ=MatriceCourante->LargeurK;
    int PitchK=MatriceCourante->PitchJ;
    int LargeurK=MatriceCourante->LargeurK;
    int ApodisationJ=MatriceCourante->ApodisationJ;
    int ApodisationK=MatriceCourante->ApodisationK;
    double Focale=MatriceCourante->Focale;
    double DirectionJ=MatriceCourante->DirectionJ;
    double DirectionK=MatriceCourante->DirectionK;
    int FocalisationJ=MatriceCourante->FocalisationJ;
    int FocalisationK=MatriceCourante->FocalisationK;
    double VMilieu=MatriceCourante->VitesseMilieuEmission;
    double* Signal=MatriceCourante->Signal;

    /*	printf("%i\n",ApodisationJ);
    	printf("%i\n",NbElementsJ);*/

    if (FocalisationJ)
        RetardMaxFocalisationJ=(double)(((NbElementsJ-1)*PitchJ*H)*((NbElementsJ-1)*PitchJ*H))/(4*2*VMilieu*Focale);
    else
        RetardMaxFocalisationJ=0.;
    if (FocalisationK)
        RetardMaxFocalisationK=(double)(((NbElementsK-1)*PitchK*H)*((NbElementsK-1)*PitchK*H))/(4*2*VMilieu*Focale);
    else
        RetardMaxFocalisationK=0.;

    if (VMilieu)
    {
        RetardMaxDeviationJ = (double)((NbElementsJ-1)*PitchJ*H)/VMilieu*sin(2*3.1415/360.0*DirectionJ);
        RetardMaxDeviationK = (double)((NbElementsK-1)*PitchK*H)/VMilieu*sin(2*3.1415/360.0*DirectionK);
    }

    CoeffApodisationJ = (double *) calloc (NbElementsJ,sizeof(double));
    CoeffApodisationK = (double *) calloc (NbElementsK,sizeof(double));

    switch(ApodisationJ)
    {
    case 0:
        for(NJ=0; NJ<NbElementsJ; NJ++)
        {

            CoeffApodisationJ[NJ]=1;
        }
        break;
    case 1:
        for(NJ=0; NJ<NbElementsJ; NJ++)
        {

            CoeffApodisationJ[NJ]=(1-cos(2*3.141593*((double)(NJ))/((double)(NbElementsJ-1))));

        }
        break;
    case 2: /*Code rajoute a la va vite avant Noel 2003: si le transducteur est coupe symetriquement...*/
        for(NJ=0; NJ<NbElementsJ; NJ++)
        {

            CoeffApodisationJ[NJ]=(cos(0.5*3.141593*((double)(NJ))/((double)(NbElementsJ-1))));

        }
        break;
    }
    switch(ApodisationK)
    {
    case 0:
        for(NK=0; NK<NbElementsK; NK++)
        {

            CoeffApodisationK[NK]=1;
        }
        break;
    case 1:
        for(NK=0; NK<NbElementsK; NK++)
        {

            CoeffApodisationK[NK]=(1-cos(2*3.141593*((double)(NK))/((double)(NbElementsK-1))));
        }
        break;
    case 2:/*Code rajoute a la va vite avant Noel 2003: si le transducteur est coupe symetriquement...*/
        for(NK=0; NK<NbElementsK; NK++)
        {

            CoeffApodisationK[NK]=(cos(0.5*3.141593*((double)(NK))/((double)(NbElementsK-1))));
        }
        break;
    }

    for(NJ=0; NJ<MatriceCourante->NbElementsJ; NJ++)
    {
        for(NK=0; NK<MatriceCourante->NbElementsK; NK++)
        {
            for(j=0; j<MatriceCourante->LargeurJ; j++)
            {
                for(k=0; k<MatriceCourante->LargeurK; k++)
                {


                    /* ajout du retard de focalisation (quadratique...)*/
                    if(Focale>0)
                    {
                        Retard=(int)(4./DeltaT*RetardMaxFocalisationJ*(
                                         ((double)(NJ)/(double)(NbElementsJ-1)-0.5)*((double)(NJ)/(double)(NbElementsJ-1)-0.5)
                                         -0.25))
                               +(int)(4./DeltaT*RetardMaxFocalisationK*(
                                          ((double)(NK)/(double)(NbElementsK-1)-0.5)*((double)(NK)/(double)(NbElementsK-1)-0.5)
                                          -0.25));
                    }
                    else
                    {
                        Retard=(int)(4./DeltaT*RetardMaxFocalisationJ*(
                                         ((double)(NJ)/(double)(NbElementsJ-1)-0.5)*((double)(NJ)/(double)(NbElementsJ-1)-0.5)))
                               +(int)(4./DeltaT*RetardMaxFocalisationK*(
                                          ((double)(NK)/(double)(NbElementsK-1)-0.5)*((double)(NK)/(double)(NbElementsK-1)-0.5)));
                    }
                    /*ajout du retard de direction (linaire...)*/
                    if(DirectionJ<0)
                    {
                        Retard += (int)(1./DeltaT*RetardMaxDeviationJ*(1-(double)(NJ)/(double)(NbElementsJ-1)));
                    }
                    else
                    {
                        Retard += -(int)(1./DeltaT*RetardMaxDeviationJ*((double)(NJ)/(double)(NbElementsJ-1)));
                    }
                    if(DirectionK<0)
                    {
                        Retard += (int)(1./DeltaT*RetardMaxDeviationK*(1-(double)(NK)/(double)(NbElementsK-1)));
                    }
                    else
                    {
                        Retard += -(int)(1./DeltaT*RetardMaxDeviationK*((double)(NK)/(double)(NbElementsK-1)));
                    }


                    if(((IndiceTemps+Retard)>=0)&((IndiceTemps+Retard)<IndiceTempsMax))
                    {

                        switch (MatriceCourante->PlanI)
                        {
                        case 'X':
                        case '1':
                            if (SourceType==1)/*20050930*/
                                GrilleCourante[
                                    (MatriceCourante->XMat)*DimY*DimZ+
                                    (MatriceCourante->YMat+NJ*MatriceCourante->PitchJ+j)*DimZ+
                                    (MatriceCourante->ZMat+NK*MatriceCourante->PitchK+k)]
                                += Signal[IndiceTemps+Retard]*CoeffApodisationJ[NJ]*CoeffApodisationK[NK];
                            else if (SourceType==2)
                                if (IndiceTemps<MatriceCourante->DurationPt)
                                    GrilleCourante[
                                        (MatriceCourante->XMat)*DimY*DimZ+
                                        (MatriceCourante->YMat+NJ*MatriceCourante->PitchJ+j)*DimZ+
                                        (MatriceCourante->ZMat+NK*MatriceCourante->PitchK+k)]
                                    = Signal[IndiceTemps+Retard]*CoeffApodisationJ[NJ]*CoeffApodisationK[NK];
                            break;

                        case 'Y':
                        case '2':
                            if (SourceType==1)/*20050930*/
                                GrilleCourante[
                                    (MatriceCourante->XMat+NK*MatriceCourante->PitchK+k)*DimY*DimZ+
                                    (MatriceCourante->YMat)*DimZ+
                                    (MatriceCourante->ZMat+NJ*MatriceCourante->PitchJ+j)]
                                += Signal[IndiceTemps+Retard]*CoeffApodisationJ[NJ]*CoeffApodisationK[NK];
                            else if (SourceType==2)
                                if (IndiceTemps<MatriceCourante->DurationPt)
                                    GrilleCourante[
                                        (MatriceCourante->XMat+NK*MatriceCourante->PitchK+k)*DimY*DimZ+
                                        (MatriceCourante->YMat)*DimZ+
                                        (MatriceCourante->ZMat+NJ*MatriceCourante->PitchJ+j)]
                                    = Signal[IndiceTemps+Retard]*CoeffApodisationJ[NJ]*CoeffApodisationK[NK];
                            break;
                        case 'Z':
                        case '3':
                            if (SourceType==1)/*20050930*/
                                GrilleCourante[
                                    (MatriceCourante->XMat+NJ*MatriceCourante->PitchJ+j)*DimY*DimZ+
                                    (MatriceCourante->YMat+NK*MatriceCourante->PitchK+k)*DimZ+
                                    (MatriceCourante->ZMat)]
                                += Signal[IndiceTemps+Retard]*CoeffApodisationJ[NJ]*CoeffApodisationK[NK];
                            else if (SourceType==2)
                                if (IndiceTemps<MatriceCourante->DurationPt)
                                    GrilleCourante[
                                        (MatriceCourante->XMat+NJ*MatriceCourante->PitchJ+j)*DimY*DimZ+
                                        (MatriceCourante->YMat+NK*MatriceCourante->PitchK+k)*DimZ+
                                        (MatriceCourante->ZMat)]
                                    = Signal[IndiceTemps+Retard]*CoeffApodisationJ[NJ]*CoeffApodisationK[NK];

                            break;
                        }
                    }
                }
            }
        }
    }
    free(CoeffApodisationK);
    free(CoeffApodisationJ);
}

void LectureConditionsInitiales(char *NomFichCondIni,PREC_GRILLE *GrilleLue,int DimX, int DimY, int DimZ)
{

    int Entete,i;
    FILE *Fich=NULL;

    Fich = fopen(NomFichCondIni, "rb" );
    Entete=3*sizeof(int)+3*sizeof(double);

    fseek(Fich,Entete, SEEK_SET);

    for(i=0; i<DimX*DimY*DimZ; i++)
    {

        fread(GrilleLue+i,sizeof(float),1,Fich);
    }
    fclose( Fich );
}
